@@ -268,7 +268,8 @@ void ai_eval_buildings(struct city *pcity)
 /* rationale: If cost is N and benefit is N gold per MORT turns, want is
 TRADE_WEIGHTING * 100 / MORT.  This is comparable, thus the same weight -- Syela */
 
-  if (could_build_improvement(pcity, B_CATHEDRAL) && !built_elsewhere(pcity, B_MICHELANGELO))
+  if (could_build_improvement(pcity, B_CATHEDRAL) && 
+      (improvement_variant(B_MICHELANGELO)==1 || !built_elsewhere(pcity, B_MICHELANGELO)))
     values[B_CATHEDRAL] = building_value(get_cathedral_power(pcity), pcity, val);
   else if (tech_exists(game.rtech.cathedral_plus) &&
 	   get_invention(pplayer, game.rtech.cathedral_plus) != TECH_KNOWN)
@@ -382,7 +383,8 @@ TRADE_WEIGHTING * 100 / MORT.  This is comparable, thus the same weight -- Syela
   if (could_build_improvement(pcity, B_SUPERHIGHWAYS))
     values[B_SUPERHIGHWAYS] = road_trade(pcity) * t;
 
-  if (could_build_improvement(pcity, B_SUPERMARKET))
+  if (could_build_improvement(pcity, B_SUPERMARKET) &&
+      player_knows_techs_with_flag(pplayer, TF_FARMLAND))
     values[B_SUPERMARKET] = farmland_food(pcity) * food * hunger;
 
   if (could_build_improvement(pcity, B_TEMPLE))
@@ -448,9 +450,19 @@ someone learning Metallurgy, and the AI collapsing.  I hate the WALL. -- Syela *
         values[i] = building_value(2, pcity, val);
       if (i == B_RICHARDS) /* ignoring pollu, I don't think it matters here -- Syela */
         values[i] = (pcity->size + 1) * SHIELD_WEIGHTING;
-      if (i == B_MICHELANGELO && !city_got_building(pcity, B_CATHEDRAL))
-        values[i] = building_value(get_cathedral_power(pcity), pcity, val);
-      
+      if (i == B_MICHELANGELO) {
+	/* Note: Mich not built, so get_cathedral_power() doesn't include its effect. */
+        if (improvement_variant(B_MICHELANGELO)==0 &&
+	    !city_got_building(pcity, B_CATHEDRAL)) {
+	  /* Assumes Mich will act as the Cath that is not in this city. */
+          values[i] = building_value(get_cathedral_power(pcity), pcity, val);
+	} else if (improvement_variant(B_MICHELANGELO)==1 &&
+		   city_got_building(pcity, B_CATHEDRAL)) {
+	  /* Assumes Mich will double the power of the Cath that is in this city. */
+          values[i] = building_value(get_cathedral_power(pcity), pcity, val);
+	}
+      }
+
       /* The following is probably wrong if B_ORACLE req is
 	 not the same as game.rtech.temple_plus (was A_MYSTICISM)
 	 --dwp */
@@ -517,7 +529,7 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
 				   struct ai_choice *choice)
 {
   struct government *g = get_gov_pplayer(pplayer);
-  int set, con, i, want, iunit;
+  int set, con, utid, want, iunit, dw;
   struct ai_choice cur;
   int est_food = pcity->food_surplus + 2 * pcity->ppl_scientist + 2 * pcity->ppl_taxman; 
   int vans = 0;
@@ -529,9 +541,9 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
   set = city_get_settlers(pcity);
   con = map_get_continent(pcity->x, pcity->y);
 
-  i = best_role_unit(pcity, F_SETTLERS);
+  utid = best_role_unit(pcity, F_SETTLERS);
 
-  if (est_food > utype_food_cost(get_unit_type(i), g)) {
+  if (est_food > utype_food_cost(get_unit_type(utid), g)) {
 /* allowing multiple settlers per city now.  I think this is correct. -- Syela */
 /* settlers are an option */
 /* settler_want calculated in settlers.c, called from ai_manage_city */
@@ -551,6 +563,31 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
     }
   }
 
+  if (!(unit_flag(utid, F_CITIES))) {
+/* basically, copied from above and adjusted to handle city founders -- jjm */
+/* founder_want calculated in settlers.c, called from ai_manage_city */
+
+    utid = best_role_unit(pcity, F_CITIES);
+
+    if (est_food > utype_food_cost(get_unit_type(utid), g)) {
+      want = pcity->ai.founder_want;
+
+      if (want > choice->want) {
+	freelog(LOG_DEBUG, "%s (%d, %d) desires founders with passion %d",
+		pcity->name, pcity->x, pcity->y, want);
+	choice->want = want;
+	choice->type = 1;
+	ai_choose_role_unit(pplayer, pcity, choice, F_CITIES, want);
+      } else if (want < -choice->want) { /* need boats to colonize! */
+	choice->want = 0 - want;
+	choice->type = 1;
+	choice->choice = best_role_unit(pcity, F_CITIES); /* default */
+	ai_choose_ferryboat(pplayer, pcity, choice);
+      }
+    }
+
+  }
+
   unit_list_iterate(pplayer->units, punit)
     if (unit_flag(punit->type, F_CARAVAN) &&
         map_get_continent(punit->x, punit->y) == con) vans++;
@@ -571,9 +608,9 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
 /* notify comes equipped with an update.  It calls generate_warmap, but this */
 /* is a lot less warmap generation than there would be otherwise. -- Syela */
       iunit = best_role_unit(pcity, F_CARAVAN);
-      i = pcity->ai.distance_to_wonder_city * 8 /
+      dw = pcity->ai.distance_to_wonder_city * 8 /
 	((iunit==U_LAST) ? 3 : get_unit_type(iunit)->move_rate);
-      want -= i;
+      want -= dw;
       /* value of 8 is a total guess and could be wrong,
        * but it's better than 0 -- Syela */
       iunit = get_role_unit(F_CARAVAN, 0);
@@ -581,7 +618,7 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
         if (want > choice->want) {
           choice->want = want;
           choice->type = 1;
-	  ai_choose_role_unit(pplayer, pcity, choice, F_CARAVAN, i/2);
+	  ai_choose_role_unit(pplayer, pcity, choice, F_CARAVAN, dw/2);
         }
       } else
 	pplayer->ai.tech_want[get_unit_type(iunit)->tech_requirement] += want;
