@@ -187,6 +187,21 @@ static int assess_distance(struct city *pcity, struct unit *punit, int m,
   return(dist);
 }
 
+/********************************************************************** 
+  Call assess_danger() for all cities owned by pplayer.
+  This is necessary to initialize ... <some ai data>
+  before ... <some ai calculations> ...
+***********************************************************************/
+void assess_danger_player(struct player *pplayer)
+{
+  city_list_iterate(pplayer->cities, pcity)
+    assess_danger(pcity);
+  city_list_iterate_end;
+}
+	  
+/********************************************************************** 
+...
+***********************************************************************/
 int assess_danger(struct city *pcity)
 {
   int i, danger = 0, v, dist, con, m;
@@ -460,6 +475,9 @@ static void process_defender_want(struct player *pplayer, struct city *pcity,
     best /= 10;
   } /* was getting four-figure desire for battleships otherwise. -- Syela */
 /* Phalanx would be 16 * danger / 20.  Pikemen would be 36 * danger / (20 + l) */
+
+  if (best == 0) best = 1;   /* avoid divide-by-zero below */
+
 /* multiply by unit_types[bestid].build_cost / best */
   for (i = 0; i < game.num_unit_types; i++) {
     if (desire[i] && is_ai_simple_military(i)) {
