@@ -180,21 +180,51 @@ static void ai_city_choose_build(struct player *pplayer, struct city *pcity)
   bestchoice.want   = 0;
   bestchoice.type   = 0;
 
+  if( is_barbarian(pplayer) ) {    /* always build best attack unit */
+    int i, iunit, bestunit = -1, bestattack = 0;
+    for(i = 0; i < num_role_units(L_BARBARIAN_BUILD); i++) {
+      iunit = get_role_unit(L_BARBARIAN_BUILD, i);
+      if (get_unit_type(iunit)->attack_strength > bestattack) {
+	bestunit = iunit;
+	bestattack = get_unit_type(iunit)->attack_strength;
+      }
+    }
+    for(i = 0; i < num_role_units(L_BARBARIAN_BUILD_TECH); i++) {
+      iunit = get_role_unit(L_BARBARIAN_BUILD_TECH, i);
+      if (game.global_advances[get_unit_type(iunit)->tech_requirement]
+	  && get_unit_type(iunit)->attack_strength > bestattack) {
+	bestunit = iunit;
+	bestattack = get_unit_type(iunit)->attack_strength;
+      }
+    }
+    if (bestunit != -1) {
+      bestchoice.choice = bestunit;
+      bestchoice.want   = 101;
+      bestchoice.type   = 2;
+    }
+    else {
+      freelog(LOG_VERBOSE, "Barbarians don't know what to build!\n");
+    }
+  }
+  else {
+
 /* obsolete code destroyed */
 /*  military_advisor_choose_build(pplayer, pcity, &curchoice); */
 /* this is now handled in manage_city thanks to our friend ->ai.choice */
 /*  copy_if_better_choice(&curchoice, &bestchoice); */
-  copy_if_better_choice(&pcity->ai.choice, &bestchoice);
 
-  if (bestchoice.want <= 100 || !pcity->ai.urgency) { /* soldier at 101 cannot be denied */
-    domestic_advisor_choose_build(pplayer, pcity, &curchoice);
-    copy_if_better_choice(&curchoice, &bestchoice);
+    copy_if_better_choice(&pcity->ai.choice, &bestchoice);
+
+    if (bestchoice.want <= 100 || !pcity->ai.urgency) { /* soldier at 101 cannot be denied */
+      domestic_advisor_choose_build(pplayer, pcity, &curchoice);
+      copy_if_better_choice(&curchoice, &bestchoice);
+    }
+
+    pcity->ai.choice.choice = bestchoice.choice; /* we want to spend gold later */
+    pcity->ai.choice.want = bestchoice.want; /* so that we spend it in the right city */
+    pcity->ai.choice.type = bestchoice.type; /* instead of the one atop the list */
   }
 
-  pcity->ai.choice.choice = bestchoice.choice; /* we want to spend gold later */
-  pcity->ai.choice.want = bestchoice.want; /* so that we spend it in the right city */
-  pcity->ai.choice.type = bestchoice.type; /* instead of the one atop the list */
-
 /* type 0 = building, 1 = non-mil unit, 2 = attacker, 3 = defender */
 
   if (bestchoice.want) { /* Note - on fallbacks, will NOT get stopped building msg */
@@ -370,7 +400,8 @@ static void ai_new_spend_gold(struct player *pplayer)
                   pcity->shield_stock + pcity->shield_surplus, 0);
       }
 #endif
-      else if (pplayer->economic.gold >= buycost && (!frugal || bestchoice.want > 200)) {
+      else if ((pplayer->economic.gold-pplayer->ai.est_upkeep) >= buycost
+	      && (!frugal || bestchoice.want > 200)) {
 	if (ai_fuzzy(pplayer,1) || (pcity->ai.grave_danger
 				    && !assess_defense(pcity))) {
 	  really_handle_city_buy(pplayer, pcity); /* adequately tested now */
@@ -384,7 +415,7 @@ static void ai_new_spend_gold(struct player *pplayer)
         if (bestchoice.type && !unit_flag(bestchoice.choice, F_NONMIL)) {
           if (pcity->ai.grave_danger && !assess_defense(pcity)) { /* oh dear */
             try_to_sell_stuff(pplayer, pcity);
-            if (pplayer->economic.gold >= buycost) /* Phew! */
+            if ((pplayer->economic.gold-pplayer->ai.est_upkeep) >= buycost)
               really_handle_city_buy(pplayer, pcity);
 /* don't need to waste gold here, but may as well spend our production */
             else pcity->currently_building = ai_choose_defender_limited(pcity,
