@@ -273,6 +273,8 @@ static void ai_manage_taxes(struct player *pplayer)
       if (city_got_building(pcity, i)) expense += improvement_upkeep(pcity,i);
   city_list_iterate_end;
 
+  pplayer->ai.est_upkeep = expense;
+
   if (!trade) { /* can't return right away - thanks for the evidence, Muzz */
     city_list_iterate(pplayer->cities, pcity) 
       if (ai_fix_unhappy(pcity) && ai_fuzzy(pplayer,1))
@@ -288,9 +290,8 @@ static void ai_manage_taxes(struct player *pplayer)
     /* this code must be ABOVE the elvises[] if SIMPLISTIC is off */
     freelog(LOG_DEBUG, "Does %s want to be bigger? %d",
 		  pcity->name, wants_to_be_bigger(pcity));
-    if (g->rapture_size && pcity->size >= g->rapture_size  &&
-        pcity->food_surplus > 0 &&
-        !pcity->ppl_unhappy[4] && wants_to_be_bigger(pcity) &&
+    if (government_has_flag(g, G_RAPTURE_CITY_GROWTH) && pcity->size >= g->rapture_size  &&
+        pcity->food_surplus > 0 && !pcity->ppl_unhappy[4] && wants_to_be_bigger(pcity) &&
 	ai_fuzzy(pplayer,1)) {
       freelog(LOG_DEBUG, "%d happy people in %s",
 		    pcity->ppl_happy[4], pcity->name);
