@@ -23,10 +23,12 @@
 #include "map.h"
 #include "packets.h"
 #include "player.h"
+#include "rand.h"
 #include "shared.h"
 #include "timing.h"
 #include "unit.h"
 
+#include "barbarian.h"
 #include "cityhand.h"
 #include "citytools.h"
 #include "cityturn.h"
@@ -46,6 +48,9 @@
 
 extern struct move_cost_map warmap;
 
+static void ai_manage_barbarian_leader(struct player *pplayer, struct unit *leader);
+static void ai_manage_diplomat(struct player *pplayer, struct unit *pdiplomat);
+
 int unit_move_turns(struct unit *punit, int x, int y)
 {
   int m, d;
@@ -110,32 +115,34 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
   generate_warmap(map_get_city(x, y), punit); /* CPU-expensive but worth it -- Syela */
 /* otherwise did not understand about bays and did really stupid things! */
 
-/* BEGIN PART ONE: Look for huts.  Ground units ONLY. */
-  if (is_ground_unit(punit)) { /* boats don't hunt huts */
-    for (d = 1; d <= lmt; d++) {
-      for (i = 0 - d; i <= d; i++) {
-        f = 1;
-        if (i != 0 - d && i != d) f = d * 2; /* I was an idiot to forget this */
-        for (j = 0 - d; j <= d; j += f) {
-          if (map_get_tile(x + i, y + j)->special & S_HUT &&
-          warmap.cost[map_adjust_x(x + i)][y + j] < best &&
-          (!ai_handicap(pplayer, H_HUTS) || map_get_known(x+i, y+j, pplayer)) &&
-          map_get_continent(x + i, y + j) == con &&
-          tile_is_accessible(punit, x+i, y+j) && ai_fuzzy(pplayer,1)) {
-            dest_x = map_adjust_x(x + i);
-            dest_y = y + j;
-            best = warmap.cost[dest_x][dest_y];
-          }
-        }
-      }
-    } /* end for D */
-  }
-  if (best <= lmt * 3) {
-    punit->goto_dest_x = dest_x;
-    punit->goto_dest_y = dest_y;
-    set_unit_activity(punit, ACTIVITY_GOTO);
-    do_unit_goto(pplayer, punit);
-    return; /* maybe have moves left but I want to return anyway. */
+/* BEGIN PART ONE: Look for huts.  Non-Barbarian Ground units ONLY. */
+  if (!is_barbarian(pplayer)) {
+    if (is_ground_unit(punit)) { /* boats don't hunt huts */
+      for (d = 1; d <= lmt; d++) {
+	for (i = 0 - d; i <= d; i++) {
+	  f = 1;
+	  if (i != 0 - d && i != d) f = d * 2; /* I was an idiot to forget this */
+	  for (j = 0 - d; j <= d; j += f) {
+	    if (map_get_tile(x + i, y + j)->special & S_HUT &&
+		warmap.cost[map_adjust_x(x + i)][y + j] < best &&
+		(!ai_handicap(pplayer, H_HUTS) || map_get_known(x+i, y+j, pplayer)) &&
+		map_get_continent(x + i, y + j) == con &&
+		tile_is_accessible(punit, x+i, y+j) && ai_fuzzy(pplayer,1)) {
+	      dest_x = map_adjust_x(x + i);
+	      dest_y = y + j;
+	      best = warmap.cost[dest_x][dest_y];
+	    }
+	  }
+	}
+      } /* end for D */
+    }
+    if (best <= lmt * 3) {
+      punit->goto_dest_x = dest_x;
+      punit->goto_dest_y = dest_y;
+      set_unit_activity(punit, ACTIVITY_GOTO);
+      do_unit_goto(pplayer, punit, GOTO_MOVE_ANY);
+      return; /* maybe have moves left but I want to return anyway. */
+    }
   }
 
 /* END PART ONE */
@@ -152,8 +159,8 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
       for (j = -1; j <= 1; j++) {
         ok = (unit_flag(punit->type, F_TRIREME) ? 0 : 1);
         if (map_get_continent(x + i, y + j) == con &&
-        !is_enemy_unit_tile(x + i, y + j, punit->owner) &&
-        !map_get_city(x + i, y + j)) {
+	    !is_enemy_unit_tile(x + i, y + j, punit->owner) &&
+	    !map_get_city(x + i, y + j)) {
           cur = 0;
           for (a = i - 1; a <= i + 1; a++) {
             for (b = j - 1; b <= j + 1; b++) {
@@ -167,7 +174,9 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
             }
           }
           if (ok && (cur > best || (cur == best && myrand(2))) &&
-              could_unit_move_to_tile(punit, punit->x, punit->y, x+i, y+j)) {
+              could_unit_move_to_tile(punit, punit->x, punit->y, x+i, y+j) &&
+	      !(is_barbarian(pplayer) &&
+		(map_get_tile(x+i, y+j)->special & S_HUT))) {
             dest_x = map_adjust_x(x + i);
             dest_y = y + j;
             best = cur;
@@ -176,7 +185,7 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
       } /* end j */
     } /* end i */
     if (best) {
-      handle_unit_move_request(pplayer, punit, dest_x, dest_y);
+      handle_unit_move_request(pplayer, punit, dest_x, dest_y, FALSE);
       if(!unit_list_find(&pplayer->units, id)) return; /* died */
     }
     ct--; /* trying to avoid loops */
@@ -190,15 +199,17 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
   for (x = 0; x < map.xsize; x++) {
     for (y = 0; y < map.ysize; y++) {
       if (map_get_continent(x, y) == con && map_get_known(x, y, pplayer) &&
-         !is_enemy_unit_tile(x, y, punit->owner) && !map_get_city(x, y) &&
-         tile_is_accessible(punit, x, y)) {
+	  !is_enemy_unit_tile(x, y, punit->owner) && !map_get_city(x, y) &&
+	  tile_is_accessible(punit, x, y)) {
         cur = 0;
         for (a = -1; a <= 1; a++)
           for (b = -1; b <= 1; b++)
             if (!map_get_known(x + a, y + b, pplayer)) cur++;
         if (cur) {
           cur += 9 * (THRESHOLD - unit_move_turns(punit, x, y));
-          if (cur > best || (cur == best && myrand(2))) {
+          if ((cur > best || (cur == best && myrand(2))) &&
+	      !(is_barbarian(pplayer) &&
+		(map_get_tile(x, y)->special & S_HUT))) {
             dest_x = map_adjust_x(x);
             dest_y = y;
             best = cur;
@@ -211,7 +222,7 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
     punit->goto_dest_x = dest_x;
     punit->goto_dest_y = dest_y;
     set_unit_activity(punit, ACTIVITY_GOTO);
-    do_unit_goto(pplayer, punit);
+    do_unit_goto(pplayer, punit, GOTO_MOVE_ANY);
     return;
   }
 /* END PART THREE */
@@ -229,7 +240,7 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
       else {
         punit->goto_dest_x = x;
         punit->goto_dest_y = y;
-        do_unit_goto(pplayer, punit);
+        do_unit_goto(pplayer, punit, GOTO_MOVE_ANY);
       }
     }
   }
@@ -376,10 +387,40 @@ static int reinforcements_value(struct unit *punit, int x, int y)
   return(val);
 }
 
+static int city_reinforcements_cost_and_value(struct city *pcity, struct unit *punit)
+{ 
+  int val = 0, val2 = 0, val3 = 0, i, j, x, y;
+  struct tile *ptile;
+
+  if (!pcity) return 0;
+
+  x=pcity->x;
+  y=pcity->y;
+  for (j = y - 1; j <= y + 1; j++) {
+    if (j < 0 || j >= map.ysize) continue;
+    for (i = x - 1; i <= x + 1; i++) {
+      ptile = map_get_tile(i, j);
+      unit_list_iterate(ptile->units, aunit)
+        if (aunit == punit || aunit->owner != punit->owner) continue;
+        val = (unit_belligerence_basic(aunit));
+        if (val) {
+          val2 += val;
+          val3 += unit_types[aunit->type].build_cost;
+        }
+      unit_list_iterate_end;
+    }
+  }
+  pcity->ai.a = val2;
+  pcity->ai.f = val3;
+  return(val2);
+}
+
+#ifdef UNUSED
 static int reinforcements_cost(struct unit *punit, int x, int y)
 { /* I might rather have one function which does this and the above -- Syela */
   int val = 0, i, j;
   struct tile *ptile;
+
   for (j = y - 1; j <= y + 1; j++) {
     if (j < 0 || j >= map.ysize) continue;
     for (i = x - 1; i <= x + 1; i++) {
@@ -393,6 +434,7 @@ static int reinforcements_cost(struct unit *punit, int x, int y)
   }
   return(val);
 }
+#endif
 
 static int is_my_turn(struct unit *punit, struct unit *pdef)
 {
@@ -416,6 +458,16 @@ static int is_my_turn(struct unit *punit, struct unit *pdef)
   return(1);
 }
 
+/*************************************************************************
+This looks at tiles neighbouring the unit to find something to kill or
+explore. It prefers tiles in the following order:
+1. Undefended cities
+2. Huts
+3. Enemy units weaker than the unit
+4. Land barbarians also like unfrastructure tiles (for later pillage)
+If none of the following is there, nothing is chosen.
+**************************************************************************/
+
 /* work of Syela - mostly to fix the ZOC/goto strangeness */
 static int ai_military_findvictim(struct player *pplayer, struct unit *punit,
 				  int *dest_x, int *dest_y)
@@ -498,12 +550,18 @@ bodyguarding catapult - patt will resolve this bug nicely -- Syela */
       }
       if (map_get_tile(x1, y1)->special & S_HUT && best < 99999 &&
           could_unit_move_to_tile(punit, punit->x, punit->y, x1, y1) &&
+          !is_barbarian(&game.players[punit->owner]) &&
 /*          zoc_ok_move(punit, x1, y1) && !is_sailing_unit(punit) &&*/
           punit->ai.ai_role != AIUNIT_ESCORT && /* makes life easier */
           !punit->ai.charge && /* above line seems not to work. :( */
           punit->ai.ai_role != AIUNIT_DEFEND_HOME) { /* Oops! -- Syela */
         best = 99998; *dest_y = y1; *dest_x = x1;
       }
+      if( is_land_barbarian(pplayer) && best == 0 &&
+          get_tile_infrastructure_set(map_get_tile(x1, y1)) &&
+          could_unit_move_to_tile(punit, punit->x, punit->y, x1, y1) ) {
+        best = 1; *dest_y = y1; *dest_x = x1;
+      } /* next to nothing is better than nothing */
     }
   }
   return(best);
@@ -533,7 +591,7 @@ static void ai_military_bodyguard(struct player *pplayer, struct unit *punit)
     if (goto_is_sane(pplayer, punit, x, y, 1)) {
       punit->goto_dest_x = x;
       punit->goto_dest_y = y;
-      do_unit_goto(pplayer, punit);
+      do_unit_goto(pplayer, punit, GOTO_MOVE_ANY);
     } else punit->ai.charge = 0; /* can't possibly get there to help */
   } else { /* I had these guys set to just fortify, which is so dumb. -- Syela */
     i = ai_military_findvictim(pplayer, punit, &x, &y);
@@ -541,7 +599,7 @@ static void ai_military_bodyguard(struct player *pplayer, struct unit *punit)
 	    "Stationary escort @(%d,%d) received %d best @(%d,%d)",
 	    punit->x, punit->y, i, x, y);
     if (i >= 40 * SHIELD_WEIGHTING)
-      handle_unit_move_request(pplayer, punit, x, y);
+      handle_unit_move_request(pplayer, punit, x, y, FALSE);
 /* otherwise don't bother, but free cities are free cities and must be snarfed. -- Syela */
   }
   if (aunit && unit_list_find(&map_get_tile(x, y)->units, id) && aunit->ai.bodyguard)
@@ -552,8 +610,8 @@ int find_beachhead(struct unit *punit, int dest_x, int dest_y, int *x, int *y)
 {
   int ii[8] = { -1, 0, 1, -1, 1, -1, 0, 1 };
   int jj[8] = { -1, -1, -1, 0, 0, 1, 1, 1 };
-  int i, j, k, l, ok, t, fu;
-  int best = 0;
+  int i, j, k, l, ok, t, fu, best = 0;
+
   for (k = 0; k < 8; k++) {
     i = map_adjust_x(dest_x + ii[k]);
     j = map_adjust_y(dest_y + jj[k]);
@@ -561,9 +619,10 @@ int find_beachhead(struct unit *punit, int dest_x, int dest_y, int *x, int *y)
     t = map_get_terrain(i, j);
     if (warmap.seacost[i][j] <= 6 * THRESHOLD && t != T_OCEAN) { /* accessible beachhead */
       for (l = 0; l < 8 && !ok; l++) {
-        if (map_get_terrain(i + ii[l], j + jj[l]) == T_OCEAN &&
-            is_my_zoc(punit, i + ii[l], j + jj[l])) ok++;
-        if (map_get_terrain(i + ii[l], j + jj[l]) == T_OCEAN) fu++;
+        if (map_get_terrain(i + ii[l], j + jj[l]) == T_OCEAN) {
+          fu++;
+          if (is_my_zoc(punit, i + ii[l], j + jj[l])) ok++;
+        }
       }
       if (ok) { /* accessible beachhead with zoc-ok water tile nearby */
         ok = get_tile_type(t)->defense_bonus;
@@ -580,6 +639,37 @@ int find_beachhead(struct unit *punit, int dest_x, int dest_y, int *x, int *y)
   return(best);
 }
 
+/**************************************************************************
+find_beachhead() works only when city is not further that 1 tile from
+the sea. But Sea Raiders might want to attack cities inland.
+So this finds the nearest land tile on the same continent as the city.
+**************************************************************************/
+
+static void find_city_beach( struct city *pc, struct unit *punit, int *x, int *y)
+{
+  int i, j;
+  int xx, yy, best_xx = punit->x, best_yy = punit->y;
+  int dist = 100;
+  int far = real_map_distance( pc->x, pc->y, punit->x, punit->y );
+
+  for( i = 1-far; i < far; i++ )
+    for( j = 1-far; j < far; j++ ) {
+      xx = map_adjust_x(punit->x + i);
+      yy = map_adjust_y(punit->y + j);
+      if( map_same_continent( xx, yy, pc->x, pc->y ) &&
+          real_map_distance( punit->x, punit->y, xx, yy ) < dist ) {
+        dist = real_map_distance( punit->x, punit->y, xx, yy );
+        best_xx = xx; best_yy = yy;
+      }
+    }
+  *x = best_xx;
+  *y = best_yy;
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+
 static int ai_military_gothere(struct player *pplayer, struct unit *punit,
 			       int dest_x, int dest_y)
 {
@@ -620,6 +710,9 @@ static int ai_military_gothere(struct player *pplayer, struct unit *punit,
         punit->ai.bodyguard = -1;
     } else if (!unit_list_find(&ptile->units, punit->ai.bodyguard))
       punit->ai.bodyguard = 0;
+
+    if( is_barbarian(pplayer) )  /* barbarians must have more currage */
+      punit->ai.bodyguard = 0;
 /* end protection subroutine */
 
     if (!goto_is_sane(pplayer, punit, dest_x, dest_y, 1) ||
@@ -631,7 +724,7 @@ static int ai_military_gothere(struct player *pplayer, struct unit *punit,
         punit->goto_dest_x = bx;
         punit->goto_dest_y = by;
 	set_unit_activity(punit, ACTIVITY_GOTO);
-        do_unit_goto(pplayer,punit);
+        do_unit_goto(pplayer,punit, GOTO_MOVE_ANY);
         if (!unit_list_find(&pplayer->units, id)) return(-1); /* died */
       }
       ptile = map_get_tile(punit->x, punit->y);
@@ -660,7 +753,7 @@ static int ai_military_gothere(struct player *pplayer, struct unit *punit,
                 if (def) set_unit_activity(def, ACTIVITY_SENTRY);
               }
             unit_list_iterate_end; /* passengers are safely stowed away */
-            do_unit_goto(pplayer, ferryboat);
+            do_unit_goto(pplayer, ferryboat, GOTO_MOVE_ANY);
             set_unit_activity(punit, ACTIVITY_IDLE);
           } /* else wait, we can GOTO later. */
         }
@@ -694,7 +787,7 @@ handled properly.  There should be a way to do it with dir_ok but I'm tired now.
 			"Bodyguard at (%d, %d) is adjacent to (%d, %d)",
 			i, j, punit->x, punit->y);
                 if (aunit->moves_left) return(0);
-                else return(handle_unit_move_request(pplayer, punit, i, j));
+                else return(handle_unit_move_request(pplayer, punit, i, j, FALSE));
               }
             unit_list_iterate_end;
           } /* end j */
@@ -708,7 +801,7 @@ handled properly.  There should be a way to do it with dir_ok but I'm tired now.
 		punit->x, punit->y, dest_x, dest_y);
       }
       set_unit_activity(punit, ACTIVITY_GOTO);
-      do_unit_goto(pplayer,punit);
+      do_unit_goto(pplayer,punit,GOTO_MOVE_ANY);
       /* liable to bump into someone that will kill us.  Should avoid? */
     } else {
       freelog(LOG_DEBUG, "%s#%d@(%d,%d) not moving -> (%d, %d)",
@@ -815,6 +908,16 @@ void ai_military_findjob(struct player *pplayer,struct unit *punit)
     } /* end if home is in danger */
   } /* end if we have a home */
 
+  /* keep barbarians aggresive and primitive */
+  if( is_barbarian(pplayer) ) {
+     if( get_tile_infrastructure_set(map_get_tile(punit->x,punit->y)) &&
+         is_land_barbarian(pplayer) )
+       punit->ai.ai_role = AIUNIT_PILLAGE;  /* land barbarians pillage */
+     else
+       punit->ai.ai_role = AIUNIT_ATTACK;
+     return;
+  }
+
   if (punit->ai.charge) { /* I am a bodyguard */
     aunit = unit_list_find(&pplayer->units, punit->ai.charge);
     acity = find_city_by_id(punit->ai.charge);
@@ -897,7 +1000,7 @@ void ai_military_gohome(struct player *pplayer,struct unit *punit)
       /* aggro defense goes here -- Syela */
       ai_military_findvictim(pplayer, punit, &dest_x, &dest_y);
       punit->ai.ai_role=AIUNIT_NONE;
-      handle_unit_move_request(pplayer, punit, dest_x, dest_y);
+      handle_unit_move_request(pplayer, punit, dest_x, dest_y, FALSE);
                                        /* might bash someone */
     } else {
       freelog(LOG_DEBUG, "GOHOME(%d,%d)",
@@ -905,7 +1008,7 @@ void ai_military_gohome(struct player *pplayer,struct unit *punit)
       punit->goto_dest_x=pcity->x;
       punit->goto_dest_y=pcity->y;
       set_unit_activity(punit, ACTIVITY_GOTO);
-      do_unit_goto(pplayer,punit);
+      do_unit_goto(pplayer,punit,GOTO_MOVE_ANY);
     }
   } else {
     handle_unit_activity_request(pplayer, punit, ACTIVITY_FORTIFY);
@@ -934,8 +1037,7 @@ learning steam engine, even though ironclads would be very useful. -- Syela */
   for (i = 0; i < game.nplayers; i++) {
     aplayer = &game.players[i];
     city_list_iterate(aplayer->cities, acity)
-      acity->ai.a = reinforcements_value(punit, acity->x, acity->y);
-      acity->ai.f = reinforcements_cost(punit, acity->x, acity->y);
+      city_reinforcements_cost_and_value(acity, punit);
       acity->ai.invasion = 0;
     city_list_iterate_end;
   }
@@ -1216,6 +1318,12 @@ static int find_nearest_friendly_port(struct unit *punit)
   return 1;
 }
 
+/*************************************************************************
+This seems to do the attack. First find victim on the neighbouring tiles.
+If no enemies nearby find_something_to_kill() anywhere else. If there is
+nothing to kill, sailing units go home, others explore.
+**************************************************************************/
+
 void ai_military_attack(struct player *pplayer,struct unit *punit)
 { /* rewritten by Syela - old way was crashy and not smart (nor is this) */
   int dest_x, dest_y; 
@@ -1232,8 +1340,32 @@ void ai_military_attack(struct player *pplayer,struct unit *punit)
         if (same_pos(punit->x, punit->y, dest_x, dest_y)) {
 /* nothing to kill.  Adjacency is something for us to kill later. */
           if (is_sailing_unit(punit)) {
-            if (find_nearest_friendly_port(punit)) do_unit_goto(pplayer, punit);
-          } else ai_manage_explorer(pplayer, punit); /* nothing else to do */
+            if (find_nearest_friendly_port(punit))
+	      do_unit_goto(pplayer, punit, GOTO_MOVE_ANY);
+          } else {
+            ai_manage_explorer(pplayer, punit); /* nothing else to do */
+            /* you can still have some moves left here, but barbarians should
+               not sit helplessly, but advance towards nearest known enemy city */
+	    punit = find_unit_by_id(id);   /* unit might die while exploring */
+            if( punit && punit->moves_left && is_barbarian(pplayer) ) {
+              struct city *pc;
+              int fx, fy;
+              freelog(LOG_DEBUG,"Barbarians looking for target");
+              if( (pc = dist_nearest_city(pplayer, punit->x, punit->y, 0, 1)) ) {
+                if( map_get_terrain(punit->x,punit->y) != T_OCEAN ) {
+                  freelog(LOG_DEBUG,"Marching to city");
+                  ai_military_gothere(pplayer, punit, pc->x, pc->y);
+                }
+                else {
+                  /* sometimes find_beachhead is not enough */
+                  if( !find_beachhead(punit, pc->x, pc->y, &fx, &fy) )
+                    find_city_beach(pc, punit, &fx, &fy);           
+                  freelog(LOG_DEBUG,"Sailing to city");
+                  ai_military_gothere(pplayer, punit, fx, fy);
+                }
+              }
+            }
+          }
           return; /* Jane, stop this crazy thing! */
         } else if (!is_tiles_adjacent(punit->x, punit->y, dest_x, dest_y)) {
 /* if what we want to kill is adjacent, and findvictim didn't want it, WAIT! */
@@ -1246,7 +1378,7 @@ void ai_military_attack(struct player *pplayer,struct unit *punit)
 	freelog(LOG_DEBUG, "%s's %s at (%d, %d) bashing (%d, %d)",
 		      pplayer->name, unit_types[punit->type].name,
 		      punit->x, punit->y, dest_x, dest_y); 
-        handle_unit_move_request(pplayer, punit, dest_x, dest_y);
+        handle_unit_move_request(pplayer, punit, dest_x, dest_y, FALSE);
         punit = find_unit_by_id(id);
         if (punit) flag = punit->moves_left; else flag = 0;
       }
@@ -1315,6 +1447,13 @@ void ai_manage_caravan(struct player *pplayer, struct unit *punit)
   }
 }
 
+/**************************************************************************
+This seems to manage the ferryboat. When it carries units on their way
+to invade something, it goes there. If it carries other units, it returns home.
+When empty, it tries to find some units to carry or goes home or explores.
+Military units handled by ai_manage_military()
+**************************************************************************/
+
 static void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
 { /* It's about 12 feet square and has a capacity of almost 1000 pounds.
      It is well constructed of teak, and looks seaworthy. */
@@ -1349,19 +1488,30 @@ static void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
   if (p) {
     freelog(LOG_DEBUG, "%s#%d@(%d,%d), p=%d, n=%d",
 		  unit_name(punit->type), punit->id, punit->x, punit->y, p, n);
-    if (punit->moves_left && n) do_unit_goto(pplayer, punit);
+    if (punit->moves_left && n)
+      do_unit_goto(pplayer, punit, GOTO_MOVE_ANY);
     else if (!n && !map_get_city(punit->x, punit->y)) { /* rest in a city, for unhap */
       x = punit->goto_dest_x; y = punit->goto_dest_y;
-      if (find_nearest_friendly_port(punit)) do_unit_goto(pplayer, punit);
+      if (find_nearest_friendly_port(punit))
+	do_unit_goto(pplayer, punit, GOTO_MOVE_ANY);
       if (!unit_list_find(&pplayer->units, id)) return; /* oops! */
       punit->goto_dest_x = x; punit->goto_dest_y = y;
       send_unit_info(pplayer, punit, 0); /* to get the crosshairs right -- Syela */
     } else {
       freelog(LOG_DEBUG, "Ferryboat %d@(%d,%d) stalling.",
 		    punit->id, punit->x, punit->y);
+      if(is_barbarian(pplayer)) /* just in case */
+        ai_manage_explorer(pplayer, punit);
     }
     return;
   }
+
+  /* check if barbarian boat is empty and so not needed - the crew has landed */
+  if( is_barbarian(pplayer) && unit_list_size(&map_get_tile(punit->x, punit->y)->units)<2 ) {
+    wipe_unit(pplayer,punit);
+    return;
+  }
+
 /* ok, not carrying anyone, even the ferryman */
   punit->ai.passenger = 0;
   freelog(LOG_DEBUG, "Ferryboat %d@(%d, %d) is lonely.",
@@ -1395,7 +1545,7 @@ static void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
     punit->goto_dest_x = x;
     punit->goto_dest_y = y;
     set_unit_activity(punit, ACTIVITY_GOTO);
-    do_unit_goto(pplayer,punit);
+    do_unit_goto(pplayer,punit,GOTO_MOVE_ANY);
     if ((punit = unit_list_find(&pplayer->units, id)))
       set_unit_activity(punit, ACTIVITY_IDLE);
     return;
@@ -1412,7 +1562,7 @@ static void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
       punit->goto_dest_x = pcity->x;
       punit->goto_dest_y = pcity->y;
       set_unit_activity(punit, ACTIVITY_GOTO);
-      do_unit_goto(pplayer,punit);
+      do_unit_goto(pplayer,punit,GOTO_MOVE_ANY);
       return;
     }
   }
@@ -1438,7 +1588,7 @@ void ai_manage_military(struct player *pplayer,struct unit *punit)
   /* was getting a bad bug where a settlers caused a defender to leave home */
   /* and then all other supported units went on DEFEND_HOME/goto */
   ai_military_findjob(pplayer, punit);
-  
+
 #ifdef DEBUG
   {
     struct city *pcity;
@@ -1471,6 +1621,10 @@ void ai_manage_military(struct player *pplayer,struct unit *punit)
     case AIUNIT_ESCORT: 
       ai_military_bodyguard(pplayer, punit);
       break;
+    case AIUNIT_PILLAGE:
+      handle_unit_activity_request(pplayer, punit, ACTIVITY_PILLAGE);
+      return; /* when you pillage, you have moves left, avoid later fortify */
+      break;
     case AIUNIT_EXPLORE:
       ai_manage_explorer(pplayer, punit);
       break;
@@ -1485,37 +1639,100 @@ void ai_manage_military(struct player *pplayer,struct unit *punit)
       if (unit_list_find(&(map_get_tile(punit->x, punit->y)->units),
           punit->ai.ferryboat))
         handle_unit_activity_request(pplayer, punit, ACTIVITY_SENTRY);
-      else handle_unit_activity_request(pplayer, punit, ACTIVITY_FORTIFY);
+      else 
+        handle_unit_activity_request(pplayer, punit, ACTIVITY_FORTIFY);
     } /* better than doing nothing */
   }
 }
 
-void ai_manage_unit(struct player *pplayer, struct unit *punit) 
+/**************************************************************************
+  Barbarian units may disband spontaneously if their age is more then 5,
+  they are not in cities, and they are far from any enemy units. It is to 
+  remove barbarians that do not engage into any activity for a long time.
+**************************************************************************/
+
+static int unit_can_be_retired(struct unit *punit)
 {
+  int x, y, x1;
+
+  if( punit->fuel ) {   /* fuel abused for barbarian life span */
+    punit->fuel--;
+    return 0;
+  }
+
+  if( is_friendly_city_tile( punit->x, punit->y, punit->owner) )
+    return 0;
+
+  /* check if there is enemy nearby */
+  for(x = punit->x - 3; x < punit->x + 4; x++)
+    for(y = punit->y - 3; y < punit->y + 4; y++) { 
+      if( y < 0 || y > map.ysize )
+        continue;
+      x1 = map_adjust_x(x);
+      if( is_enemy_city_tile(x1,y,punit->owner) ||
+          is_enemy_unit_tile(x1,y,punit->owner) )
+        return 0;
+    }
+
+  return 1;
+}
+
+/**************************************************************************
+ manage one unit
+ Careful: punit may have been destroyed upon return from this routine!
+**************************************************************************/
+
+static void ai_manage_unit(struct player *pplayer, struct unit *punit,
+			   struct genlist_iterator *iter)
+{
+  /* retire useless barbarian units here, before calling the management
+     function */
+  if( is_barbarian(pplayer) ) {
+    if( unit_can_be_retired(punit) && myrand(100) > 90 ) {
+      wipe_unit_safe(0, punit, iter);
+      return;
+    }
+    if( !is_military_unit(punit)
+	&& !unit_has_role(punit->type, L_BARBARIAN_LEADER)) {
+      freelog(LOG_VERBOSE, "Barbarians picked up non-military unit.");
+      return;
+    }
+  }
+
   if ((unit_flag(punit->type, F_DIPLOMAT))
       || (unit_flag(punit->type, F_SPY))) {
-    return;
+    ai_manage_diplomat(pplayer, punit);
     /* the right test if the unit is in a city and 
        there is no other diplomat it musn't move.
        This unit is a bodyguard against enemy diplomats.
-       Rigth now I don't know how to use bodyguards! (17/12/98) (--NB)
+       Right now I don't know how to use bodyguards! (17/12/98) (--NB)
     */
-  }
-  else if (unit_flag(punit->type, F_SETTLERS)) {
+    return;
+  } else if (unit_flag(punit->type, F_SETTLERS)
+	     ||unit_flag(punit->type, F_CITIES)) {
     if (!punit->moves_left) return; /* can't do anything with no moves */
     ai_manage_settler(pplayer, punit);
+    return;
   } else if (unit_flag(punit->type, F_CARAVAN)) {
     ai_manage_caravan(pplayer, punit);
+    return;
+  } else if (unit_has_role(punit->type, L_BARBARIAN_LEADER)) {
+    ai_manage_barbarian_leader(pplayer, punit);
+    return;
   } else if (get_transporter_capacity(punit)) {
     ai_manage_ferryboat(pplayer, punit);
+    return;
   } else if (is_military_unit(punit)) {
     if (!punit->moves_left) return; /* can't do anything with no moves */
     ai_manage_military(pplayer,punit); 
+    return;
   } else {
     if (!punit->moves_left) return; /* can't do anything with no moves */
     ai_manage_explorer(pplayer, punit); /* what else could this be? -- Syela */
+    return;
   }
-  /* Careful Unit maybe void here */
+  /* should never get here */
+  freelog(LOG_DEBUG, "Error: At end of ai_manage_unit().");
 }
 
 /**************************************************************************
@@ -1532,7 +1749,7 @@ void ai_manage_units(struct player *pplayer)
   unit_list_iterate(pplayer->units, punit) {
     freelog(LOG_DEBUG, "Managing %s's %s %d@(%d,%d)", pplayer->name,
 		  unit_types[punit->type].name, punit->id, punit->x, punit->y);
-    ai_manage_unit(pplayer, punit);
+    ai_manage_unit(pplayer, punit, &myiter);
     /* Note punit might be gone!! */
     freelog(LOG_DEBUG, "Finished managing %s's unit", pplayer->name);
   }
@@ -1617,3 +1834,272 @@ int is_ai_simple_military(int type)
     && !unit_flag(type, F_SUBMARINE) /* not caught by capacity for civ1 */
     && !(get_unit_type(type)->transport_capacity >= 8);
 }
+
+/*
+ * If we are the only diplomat in a city, defend against enemy actions.
+ * The passive defense is set by game.diplchance.  The active defense is
+ * to bribe units which end their move nearby.
+ * Our next trick is to look for enemy units and cities on our continent.
+ * If we find a city, we look first to establish an embassy.  The
+ * information gained this way is assumed to be more useful than anything
+ * else we could do.  Making this come true is for future code.  -AJS
+ */
+static void ai_manage_diplomat(struct player *pplayer, struct unit *pdiplomat)
+{
+  int i, x, y, handicap, has_emb, continent, dist, rmd, oic, did;
+  struct packet_unit_request req;
+  struct packet_diplomat_action dact;
+  struct city *pcity, *ctarget;
+  struct tile *ptile;
+  struct unit *ptres;
+  struct player *aplayer = NULL;
+
+  if (pdiplomat->activity != ACTIVITY_IDLE)
+    handle_unit_activity_request(pplayer, pdiplomat, ACTIVITY_IDLE);
+
+  pcity = map_get_city(pdiplomat->x, pdiplomat->y);
+
+  if (pcity && count_diplomats_on_tile(pdiplomat->x, pdiplomat->y) == 1) {
+    /* We're the only diplomat in a city.  Defend it. */
+    if (pdiplomat->homecity != pcity->id) {
+      /* this may be superfluous, but I like it.  -AJS */
+      req.unit_id=pdiplomat->id;
+      req.city_id=pcity->id;
+      req.name[0]='\0';
+      handle_unit_change_homecity(pplayer, &req);
+    }
+  } else {
+    if (pcity) {
+      /*
+       * More than one diplomat in a city: may try to bribe trespassers.
+       * We may want this to be a city_map_iterate, or a warmap distance
+       * check, but this will suffice for now.  -AJS
+       */
+      for (x=-1; x<= 1; x++) {
+	for (y=-1; y<= 1; y++) {
+	  if (x == 0 && y == 0) continue;
+	  if (diplomat_can_do_action(pdiplomat, DIPLOMAT_BRIBE,
+				     pcity->x+x, pcity->y+y)) {
+	    /* A lone trespasser! Seize him! -AJS */
+	    ptile=map_get_tile(pcity->x+x, pcity->y+y);
+	    ptres = unit_list_get(&ptile->units, 0);
+	    ptres->bribe_cost=unit_bribe_cost (ptres);
+	    if ( ptres->bribe_cost <
+		 (pplayer->economic.gold-pplayer->ai.est_upkeep)) {
+	      dact.diplomat_id=pdiplomat->id;
+	      dact.target_id=ptres->id;
+	      dact.action_type=DIPLOMAT_BRIBE;
+	      handle_diplomat_action(pplayer, &dact);
+	      return;
+	    }
+	  }
+	}
+      }
+    }
+    /*
+     *  We're wandering in the desert, or there is more than one diplomat
+     *  here.  Go elsewhere.
+     *  First, we look for an embassy, to steal a tech, or for a city to
+     *  subvert.  Then we look for a city of our own without a defending
+     *  diplomat.  This may not prove to work so very well, but it's
+     *  possible otherwise that all diplomats we ever produce are home
+     *  guard, and that's kind of silly.  -AJS, 20000130
+     */
+    ctarget=0;
+    ptres=0;
+    dist=MAX(map.xsize, map.ysize);
+    continent=map_get_continent(pdiplomat->x, pdiplomat->y);
+    handicap = ai_handicap(pplayer, H_TARGETS);
+    for( i = 0; i < game.nplayers; i++) {
+      aplayer = &game.players[i];
+      if (aplayer == pplayer) continue;
+      /* sneaky way of avoiding foul diplomat capture  -AJS */
+      has_emb=player_has_embassy(pplayer, aplayer) || pdiplomat->foul;
+      city_list_iterate(aplayer->cities, acity)
+	if (handicap && !map_get_known(acity->x, acity->y, pplayer)) continue;
+	if (continent != map_get_continent(acity->x, acity->y)) continue;
+	city_incite_cost(acity);
+	/* figure our incite cost */
+	oic = acity->incite_revolt_cost;
+	if (pplayer->player_no == acity->original) oic = oic / 2;
+	rmd=real_map_distance(pdiplomat->x, pdiplomat->y, acity->x, acity->y);
+	if (!ctarget || (dist > rmd)) {
+	  if (!has_emb || !acity->steal || (oic < 
+               pplayer->economic.gold - pplayer->ai.est_upkeep)) {
+	    /* We have the closest enemy city so far on the same continent */
+	    ctarget = acity;
+	    dist = rmd;
+	  }
+	}
+      city_list_iterate_end;
+    }
+    if (!ctarget && aplayer) {
+      /* No enemy cities are useful.  Check our own. -AJS */
+      city_list_iterate(aplayer->cities, acy)
+	if (!count_diplomats_on_tile(acy->x, acy->y)) {
+	  ctarget=acy;
+	  dist=real_map_distance(pdiplomat->x, pdiplomat->y, acy->x, acy->y);
+	  /* keep dist's integrity, and we can use it later. -AJS */
+	}
+      city_list_iterate_end;
+    }
+    if (ctarget) {
+      /* Otherwise, we just kinda sit here.  -AJS */
+      did = -1;
+      if ((dist == 1) && (pplayer->player_no != ctarget->owner)) {
+	dact.diplomat_id=pdiplomat->id;
+	dact.target_id=ctarget->id;
+	if (!pdiplomat->foul && diplomat_can_do_action(pdiplomat,
+	     DIPLOMAT_EMBASSY, ctarget->x, ctarget->y)) {
+	    did=pdiplomat->id;
+	    dact.action_type=DIPLOMAT_EMBASSY;
+	    handle_diplomat_action(pplayer, &dact);
+	} else if (!ctarget->steal && diplomat_can_do_action(pdiplomat,
+		    DIPLOMAT_STEAL, ctarget->x, ctarget->y)) {
+	    did=pdiplomat->id;
+	    dact.action_type=DIPLOMAT_STEAL;
+	    handle_diplomat_action(pplayer, &dact);
+	} else if (diplomat_can_do_action(pdiplomat, DIPLOMAT_INCITE,
+		   ctarget->x, ctarget->y)) {
+	    did=pdiplomat->id;
+	    dact.action_type=DIPLOMAT_INCITE;
+	    handle_diplomat_action(pplayer, &dact);
+	}
+      }
+      if ((did < 0) || find_unit_by_id(did)) {
+	pdiplomat->goto_dest_x=ctarget->x;
+	pdiplomat->goto_dest_y=ctarget->y;
+	set_unit_activity(pdiplomat, ACTIVITY_GOTO);
+	do_unit_goto(pplayer, pdiplomat, GOTO_MOVE_ANY);
+      }
+    }
+  }
+  return;
+}
+
+/*************************************************************************
+Barbarian leader tries to stack with other barbarian units, and if it's
+not possible it runs away. When on coast, it may disappear with 33% chance.
+**************************************************************************/
+
+static void ai_manage_barbarian_leader(struct player *pplayer, struct unit *leader)
+{
+  int con = map_get_continent(leader->x, leader->y), i, x, y, dx, dy,
+    safest = 0, safest_x = leader->x, safest_y = leader->y;
+  struct unit *closest_unit = NULL;
+  int dist, mindist = 10000;
+
+  if (leader->moves_left == 0 || 
+      (map_get_terrain(leader->x, leader->y) != T_OCEAN &&
+       unit_list_size(&(map_get_tile(leader->x, leader->y)->units)) > 1) ) {
+      handle_unit_activity_request(pplayer, leader, ACTIVITY_SENTRY);
+      return;
+  }
+  /* the following takes much CPU time and could be avoided */
+  generate_warmap(map_get_city(leader->x, leader->y), leader);
+
+  /* duck under own units */
+  unit_list_iterate(pplayer->units, aunit) {
+    if (unit_has_role(aunit->type, L_BARBARIAN_LEADER)
+	|| !is_ground_unit(aunit)
+	|| map_get_continent(aunit->x, aunit->y) != con)
+      continue;
+
+    if (warmap.cost[aunit->x][aunit->y] < mindist) {
+      mindist = warmap.cost[aunit->x][aunit->y];
+      closest_unit = aunit;
+    }
+  } unit_list_iterate_end;
+
+  if (closest_unit != NULL
+      && !same_pos(closest_unit->x, closest_unit->y, leader->x, leader->y)
+      && map_same_continent(leader->x, leader->y, closest_unit->x, closest_unit->y)) {
+    auto_settler_do_goto(pplayer, leader, closest_unit->x, closest_unit->y);
+    handle_unit_activity_request(pplayer, leader, ACTIVITY_IDLE);
+    return; /* sticks better to own units with this -- jk */
+  }
+
+  freelog(LOG_DEBUG, "Barbarian leader needs to flee");
+  mindist = 1000000;
+  closest_unit = NULL;
+  for (i = 0; i < game.nplayers; i++) {
+    unit_list_iterate(game.players[i].units, aunit) {
+      if (is_military_unit(aunit)
+	  && is_ground_unit(aunit)
+	  && map_get_continent(aunit->x, aunit->y) == con) {
+	/* questionable assumption: aunit needs as many moves to reach us as we
+	   need to reach it */
+	dist = warmap.cost[aunit->x][aunit->y] - unit_move_rate(aunit);
+	if (dist < mindist) {
+	  freelog(LOG_DEBUG, "Barbarian leader: closest enemy is %s at %d, %d, dist %d",
+                  unit_name(aunit->type), aunit->x, aunit->y, dist);
+	  mindist = dist;
+	  closest_unit = aunit;
+	}
+      }
+    } unit_list_iterate_end;
+  }
+
+  /* Disappearance - 33% chance on coast, when older than barbarian life span */
+  if( is_at_coast(leader->x, leader->y) && !leader->fuel) {
+    if(myrand(3) == 0) {
+      freelog(LOG_DEBUG, "Barbarian leader disappeared at %d %d", leader->x, leader->y);
+      wipe_unit(pplayer,leader);
+      return;
+    }
+  }
+
+  if (closest_unit == NULL) {
+    handle_unit_activity_request(pplayer, leader, ACTIVITY_IDLE);
+    freelog(LOG_DEBUG, "Barbarian leader: No enemy.");
+    return;
+  }
+
+  generate_warmap(map_get_city(closest_unit->x, closest_unit->y), closest_unit);
+
+  do {
+    int last_x, last_y;
+    freelog(LOG_DEBUG, "Barbarian leader: moves left: %d\n", leader->moves_left);
+    for (dx = -1; dx <= 1; dx++) {
+      for (dy = -1; dy <= 1; dy++) {
+	x = map_adjust_x(leader->x + dx);
+	y = map_adjust_x(leader->y + dy);
+
+	if (warmap.cost[x][y] > safest
+	    && can_unit_move_to_tile(leader, x, y, FALSE)) {
+	  safest = warmap.cost[x][y];
+	  freelog(LOG_DEBUG, "Barbarian leader: safest is %d, %d, safeness %d",
+                  x, y, safest);
+	  safest_x = x;
+	  safest_y = y;
+	}
+      }
+    }
+
+    if (same_pos(leader->x, leader->y, safest_x, safest_y)) {
+      freelog(LOG_DEBUG, "Barbarian leader reached the safest position.");
+      handle_unit_activity_request(pplayer, leader, ACTIVITY_IDLE);
+      return;
+    }
+
+    freelog(LOG_DEBUG, "Fleeing to %d, %d.", safest_x, safest_y);
+    last_x = leader->x;
+    last_y = leader->y;
+    auto_settler_do_goto(pplayer, leader, safest_x, safest_y);
+    if (leader->x == last_x && leader->y == last_y) {
+      /* Deep inside the goto handling code, in 
+	 server/unithand.c::handle_unite_move_request(), the server
+	 may decide that a unit is better off not moving this turn,
+	 because the unit doesn't have quite enough movement points
+	 remaining.  Unfortunately for us, this favor that the server
+	 code does may lead to an endless loop here in the barbarian
+	 leader code:  the BL will try to flee to a new location, execute 
+	 the goto, find that it's still at its present (unsafe) location,
+	 and repeat.  To break this loop, we test for the condition
+	 where the goto doesn't do anything, and break if this is
+	 the case. */
+      break;
+    }
+  } while (leader->moves_left > 0);
+}
+
