@@ -20,7 +20,8 @@ struct ai_choice;
 
 enum ai_unit_task { AIUNIT_NONE, AIUNIT_AUTO_SETTLER, AIUNIT_BUILD_CITY,
                     AIUNIT_DEFEND_HOME, AIUNIT_ATTACK, AIUNIT_FORTIFY,
-                    AIUNIT_RUNAWAY, AIUNIT_ESCORT, AIUNIT_EXPLORE};
+                    AIUNIT_RUNAWAY, AIUNIT_ESCORT, AIUNIT_EXPLORE,
+                    AIUNIT_PILLAGE };
 
 void ai_manage_units(struct player *pplayer); 
 
@@ -33,7 +34,6 @@ void ai_military_findjob(struct player *pplayer,struct unit *punit);
 void ai_military_gohome(struct player *pplayer,struct unit *punit);
 void ai_military_attack(struct player *pplayer,struct unit *punit);
 
-void ai_manage_unit(struct player *pplayer, struct unit *punit);
 void ai_manage_caravan(struct player *pplayer, struct unit *punit);
 int find_something_to_kill(struct player *pplayer, struct unit *punit, 
                             int *x, int *y);
