@@ -21,6 +21,7 @@
 #include "fcintl.h"
 #include "mem.h"
 #include "packets.h"
+#include "support.h"
 
 #include "clinet.h"
 #include "gui_stuff.h"
@@ -40,8 +41,7 @@ void inputline_return(GtkWidget *w, gpointer data)
   theinput = gtk_entry_get_text(GTK_ENTRY(w));
   
   if(*theinput) {
-    strncpy(apacket.message, theinput, MAX_LEN_MSG-MAX_LEN_USERNAME);
-    apacket.message[MAX_LEN_MSG-MAX_LEN_USERNAME]='\0';
+    mystrlcpy(apacket.message, theinput, MAX_LEN_MSG-MAX_LEN_USERNAME+1);
     send_packet_generic_message(&aconnection, PACKET_CHAT_MSG, &apacket);
   }
 
