@@ -31,6 +31,7 @@
 #include "packets.h"
 #include "player.h"
 #include "shared.h"
+#include "support.h"
 
 #include "cityrep.h"
 #include "colors.h"
@@ -41,8 +42,10 @@
 #include "helpdlg.h"
 #include "inputdlg.h"
 #include "mapview.h"
+#include "options.h"
 #include "repodlgs.h"
 #include "tilespec.h"
+#include "wldlg.h"
 
 #include "citydlg.h"
 
@@ -52,7 +55,6 @@ extern GtkWidget *toplevel, *map_canvas;
 extern GdkWindow *root_window;
 extern struct connection aconnection;
 extern int map_view_x0, map_view_y0;
-extern int flags_are_transparent;
 extern GdkGC *fill_bg_gc;
 extern GdkGC *civ_gc;
 
@@ -78,7 +80,8 @@ struct city_dialog {
   GtkWidget *sell_command;
   GtkWidget *close_command, *rename_command, *trade_command, *activate_command;
   GtkWidget *show_units_command, *cityopt_command;
-  GtkWidget *building_label, *progress_label, *buy_command, *change_command;
+  GtkWidget *building_label, *progress_label, *buy_command, *change_command,
+    *worklist_command, *worklist_label;
   GtkWidget *improvement_viewport, *improvement_list;
   GtkWidget *support_unit_label;
   GtkWidget *support_unit_boxes		[NUM_UNITS_SHOWN];
@@ -88,6 +91,7 @@ struct city_dialog {
   GtkWidget *present_unit_pixmaps	[NUM_UNITS_SHOWN];
   GtkWidget *change_list;
   GtkWidget *rename_input;
+  GtkWidget *worklist_shell;
   
   enum improvement_type_id sell_id;
   
@@ -128,6 +132,9 @@ void city_dialog_update_pollution(struct city_dialog *pdialog);
 void sell_callback	(GtkWidget *w, gpointer data);
 void buy_callback	(GtkWidget *w, gpointer data);
 void change_callback	(GtkWidget *w, gpointer data);
+void worklist_callback	(GtkWidget *w, gpointer data);
+void commit_city_worklist(struct worklist *pwl, void *data);
+void cancel_city_worklist(void *data);
 void close_callback	(GtkWidget *w, gpointer data);
 void rename_callback	(GtkWidget *w, gpointer data);
 void trade_callback	(GtkWidget *w, gpointer data);
@@ -209,6 +216,7 @@ void refresh_city_dialog(struct city *pcity)
       /* Set the buttons we do not want live while a Diplomat investigates */
       gtk_widget_set_sensitive(pdialog->buy_command, FALSE);
       gtk_widget_set_sensitive(pdialog->change_command, FALSE);
+      gtk_widget_set_sensitive(pdialog->worklist_command, FALSE);
       gtk_widget_set_sensitive(pdialog->sell_command, FALSE);
       gtk_widget_set_sensitive(pdialog->rename_command, FALSE);
       gtk_widget_set_sensitive(pdialog->activate_command, FALSE);
@@ -303,7 +311,7 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 {
   int i;
   struct city_dialog *pdialog;
-  GtkWidget *box, *table, *frame, *vbox, *scrolled;
+  GtkWidget *box, *table, *frame, *vbox, *scrolled, *hbox;
   GtkAccelGroup *accel=gtk_accel_group_new();
 
   pdialog=fc_malloc(sizeof(struct city_dialog));
@@ -430,9 +438,20 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 	GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_box_pack_start(GTK_BOX(box),scrolled, TRUE, TRUE, 0);
 
+  hbox = gtk_hbox_new(TRUE, 0);
   pdialog->sell_command=gtk_accelbutton_new(_("_Sell"), accel);
-  gtk_box_pack_start(GTK_BOX(box), pdialog->sell_command, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(hbox), pdialog->sell_command, FALSE, TRUE, 0);
 
+  pdialog->worklist_command=gtk_accelbutton_new(_("_Worklist"), accel);
+  gtk_box_pack_start(GTK_BOX(hbox), pdialog->worklist_command, 
+			    FALSE, TRUE, 0);
+
+  pdialog->worklist_label=gtk_label_new("");
+  gtk_box_pack_start(GTK_BOX(hbox), pdialog->worklist_label, 
+			    FALSE, TRUE, 0);
+
+  gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, TRUE, 0);
+  pdialog->worklist_shell = NULL;
 
   /* "supported units" frame */
   pdialog->support_unit_label=gtk_frame_new(_("Supported units"));
@@ -535,6 +554,9 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
   gtk_signal_connect(GTK_OBJECT(pdialog->change_command), "clicked",
 	GTK_SIGNAL_FUNC(change_callback), pdialog);
 
+  gtk_signal_connect(GTK_OBJECT(pdialog->worklist_command), "clicked",
+	GTK_SIGNAL_FUNC(worklist_callback), pdialog);
+
   gtk_signal_connect(GTK_OBJECT(pdialog->close_command), "clicked",
 	GTK_SIGNAL_FUNC(close_callback), pdialog);
 
@@ -807,8 +829,8 @@ static void rename_city_callback(GtkWidget *w, gpointer data)
 
   if((pdialog=(struct city_dialog *)data)) {
     packet.city_id=pdialog->pcity->id;
-    strncpy(packet.name, input_dialog_get_input(w), MAX_LEN_NAME);
-    packet.name[MAX_LEN_NAME-1]='\0';
+    packet.worklist.name[0] = '\0';
+    sz_strlcpy(packet.name, input_dialog_get_input(w));
     send_packet_city_request(&aconnection, &packet, PACKET_CITY_RENAME);
   }
 
@@ -847,14 +869,16 @@ void trade_callback(GtkWidget *w, gpointer data)
 {
   int i;
   int x=0,total=0;
-  char buf[512],*bptr=buf;
+  char buf[512], *bptr=buf;
+  int nleft = sizeof(buf);
   struct city_dialog *pdialog;
 
   pdialog=(struct city_dialog *)data;
 
-  sprintf(buf, _("These trade routes have been established with %s:\n"),
-	  pdialog->pcity->name);
-  bptr += strlen(bptr);
+  my_snprintf(buf, sizeof(buf),
+	      _("These trade routes have been established with %s:\n"),
+	      pdialog->pcity->name);
+  bptr = end_of_strn(bptr, &nleft);
   
   for(i=0; i<4; i++)
     if(pdialog->pcity->trade[i]) {
@@ -862,21 +886,22 @@ void trade_callback(GtkWidget *w, gpointer data)
       x=1;
       total+=pdialog->pcity->trade_value[i];
       if((pcity=find_city_by_id(pdialog->pcity->trade[i]))) {
-	sprintf(bptr, _("%32s: %2d Trade/Year\n"),
+	my_snprintf(bptr, nleft, _("%32s: %2d Trade/Year\n"),
 		      pcity->name, pdialog->pcity->trade_value[i]);
-	bptr += strlen(bptr);
+	bptr = end_of_strn(bptr, &nleft);
       }
       else {
-	sprintf(bptr, _("%32s: %2d Trade/Year\n"), _("Unknown"),
+	my_snprintf(bptr, nleft, _("%32s: %2d Trade/Year\n"), _("Unknown"),
 		      pdialog->pcity->trade_value[i]);
-	bptr += strlen(bptr);
+	bptr = end_of_strn(bptr, &nleft);
       }
 
     }
-  if(!x)
-    sprintf(bptr, _("No trade routes exist.\n"));
-  else
-    sprintf(bptr, _("\nTotal trade %d Trade/Year\n"),total);
+  if(!x) {
+    mystrlcpy(bptr, _("No trade routes exist.\n"), nleft);
+  } else {
+    my_snprintf(bptr, nleft, _("\nTotal trade %d Trade/Year\n"), total);
+  }
 
   popup_message_dialog(pdialog->shell, 
 		       /*"citytradedialog"*/ _("Trade Routes"), 
@@ -896,7 +921,7 @@ void city_dialog_update_pollution(struct city_dialog *pdialog)
   GtkStyle *s;
   int col;
 
-  sprintf(buf, _("Pollution:   %3d"), pcity->pollution);
+  my_snprintf(buf, sizeof(buf), _("Pollution:   %3d"), pcity->pollution);
 
   gtk_set_label(pdialog->pollution_label, buf);
 
@@ -921,7 +946,7 @@ void city_dialog_update_storage(struct city_dialog *pdialog)
   char buf[512];
   struct city *pcity=pdialog->pcity;
   
-  sprintf(buf, _("Granary: %3d/%-3d"), pcity->food_stock,
+  my_snprintf(buf, sizeof(buf), _("Granary: %3d/%-3d"), pcity->food_stock,
 	  game.foodbox*(pcity->size+1));
 
   gtk_set_label(pdialog->storage_label, buf);
@@ -933,29 +958,47 @@ void city_dialog_update_storage(struct city_dialog *pdialog)
 void city_dialog_update_building(struct city_dialog *pdialog)
 {
   char buf[32], buf2[64];
+  int turns;
   struct city *pcity=pdialog->pcity;
   
   gtk_widget_set_sensitive(pdialog->buy_command, !pcity->did_buy);
   gtk_widget_set_sensitive(pdialog->sell_command, !pcity->did_sell);
 
   if(pcity->is_building_unit) {
-    sprintf(buf, "%3d/%3d", pcity->shield_stock, 
-	    get_unit_type(pcity->currently_building)->build_cost);
-    sprintf(buf2, "%s", get_unit_type(pcity->currently_building)->name);
+    turns = city_turns_to_build (pcity, pcity->currently_building, TRUE);
+    my_snprintf(buf, sizeof(buf),
+ 		concise_city_production ? "%3d/%3d:%3d" :
+		  turns == 1 ? _("%3d/%3d %3d turn") : _("%3d/%3d %3d turns"),
+ 		pcity->shield_stock,
+ 		get_unit_type(pcity->currently_building)->build_cost,
+ 		turns);
+    sz_strlcpy(buf2, get_unit_type(pcity->currently_building)->name);
   } else {
     if(pcity->currently_building==B_CAPITAL)  {
       /* Capitalization is special, you can't buy it or finish making it */
-      sprintf(buf,"%3d/XXX", pcity->shield_stock);
+      my_snprintf(buf, sizeof(buf),
+		  concise_city_production ? "%3d/XXX:XXX" :
+		    _("%3d/XXX XXX turns"),
+		  pcity->shield_stock);
       gtk_widget_set_sensitive(pdialog->buy_command, FALSE);
     } else {
-      sprintf(buf, "%3d/%3d", pcity->shield_stock, 
-	     get_improvement_type(pcity->currently_building)->build_cost);
+      turns = city_turns_to_build (pcity, pcity->currently_building, FALSE);
+      my_snprintf(buf, sizeof(buf),
+		  concise_city_production ? "%3d/%3d:%3d" :
+		    turns == 1 ? _("%3d/%3d %3d turn") : _("%3d/%3d %3d turns"),
+		  pcity->shield_stock, 
+		  get_improvement_type(pcity->currently_building)->build_cost,
+		  turns);
     }
-    sprintf(buf2, "%s", 
-	    get_imp_name_ex(pcity, pcity->currently_building));
+    sz_strlcpy(buf2, get_imp_name_ex(pcity, pcity->currently_building));
   }    
+    
   gtk_frame_set_label(GTK_FRAME(pdialog->building_label), buf2);
   gtk_set_label(pdialog->progress_label, buf);
+  gtk_set_label(pdialog->worklist_label,
+		worklist_is_empty(pcity->worklist) ?
+		_("(list empty)") :
+		_("(in progress)"));
 }
 
 
@@ -967,7 +1010,8 @@ void city_dialog_update_production(struct city_dialog *pdialog)
   char buf[512];
   struct city *pcity=pdialog->pcity;
   
-  sprintf(buf, _("Food:    %2d (%+2d)\nProd:    %2d (%+2d)\nTrade:   %2d (%+2d)"),
+  my_snprintf(buf, sizeof(buf),
+	  _("Food:    %2d (%+2d)\nProd:    %2d (%+2d)\nTrade:   %2d (%+2d)"),
 	  pcity->food_prod, pcity->food_surplus,
 	  pcity->shield_prod, pcity->shield_surplus,
 	  pcity->trade_prod+pcity->corruption, pcity->trade_prod);
@@ -982,7 +1026,8 @@ void city_dialog_update_output(struct city_dialog *pdialog)
   char buf[512];
   struct city *pcity=pdialog->pcity;
   
-  sprintf(buf, _("Gold:    %2d (%+2d)\nLuxury:  %2d\nScience: %2d"),
+  my_snprintf(buf, sizeof(buf),
+	  _("Gold:    %2d (%+2d)\nLuxury:  %2d\nScience: %2d"),
 	  pcity->tax_total, city_gold_surplus(pcity),
 	  pcity->luxury_total,
 	  pcity->science_total);
@@ -1266,7 +1311,7 @@ void city_dialog_update_title(struct city_dialog *pdialog)
   char buf[512];
   char *now;
   
-  sprintf(buf, _("%s - %s citizens"),
+  my_snprintf(buf, sizeof(buf), _("%s - %s citizens"),
 	  pdialog->pcity->name, int_to_text(city_population(pdialog->pcity)));
 
   now=GTK_FRAME(pdialog->cityname_label)->label;
@@ -1288,7 +1333,8 @@ void city_dialog_update_improvement_list(struct city_dialog *pdialog)
 	 strcmp(pdialog->improvlist_names_ptrs[n],
 		get_imp_name_ex(pdialog->pcity, i)))
 	flag=1;
-      strcpy(pdialog->improvlist_names[n], get_imp_name_ex(pdialog->pcity, i));
+      sz_strlcpy(pdialog->improvlist_names[n],
+		 get_imp_name_ex(pdialog->pcity, i));
       pdialog->improvlist_names_ptrs[n]=pdialog->improvlist_names[n];
       n++;
     }
@@ -1334,6 +1380,7 @@ gint button_down_citymap(GtkWidget *w, GdkEventButton *ev)
     packet.worker_x=xtile;
     packet.worker_y=ytile;
     packet.name[0]='\0';
+    packet.worklist.name[0] = '\0';
     
     if(pcity->city_map[xtile][ytile]==C_TILE_WORKER)
       send_packet_city_request(&aconnection, &packet, 
@@ -1358,6 +1405,7 @@ gint elvis_callback(GtkWidget *w, GdkEventButton *ev, gpointer data)
   packet.name[0]='\0';
   packet.specialist_from=SP_ELVIS;
   packet.specialist_to=SP_SCIENTIST;
+  packet.worklist.name[0] = '\0';
   
   send_packet_city_request(&aconnection, &packet, 
 			   PACKET_CITY_CHANGE_SPECIALIST);
@@ -1378,6 +1426,7 @@ gint scientist_callback(GtkWidget *w, GdkEventButton *ev, gpointer data)
   packet.name[0]='\0';
   packet.specialist_from=SP_SCIENTIST;
   packet.specialist_to=SP_TAXMAN;
+  packet.worklist.name[0] = '\0';
   
   send_packet_city_request(&aconnection, &packet, 
 			   PACKET_CITY_CHANGE_SPECIALIST);
@@ -1398,6 +1447,7 @@ gint taxman_callback(GtkWidget *w, GdkEventButton *ev, gpointer data)
   packet.name[0]='\0';
   packet.specialist_from=SP_TAXMAN;
   packet.specialist_to=SP_ELVIS;
+  packet.worklist.name[0] = '\0';
   
   send_packet_city_request(&aconnection, &packet, 
 			   PACKET_CITY_CHANGE_SPECIALIST);
@@ -1416,6 +1466,7 @@ static void buy_callback_yes(GtkWidget *w, gpointer data)
 
   packet.city_id=pdialog->pcity->id;
   packet.name[0]='\0';
+  packet.worklist.name[0] = '\0';
   send_packet_city_request(&aconnection, &packet, PACKET_CITY_BUY);
 
   destroy_message_dialog(w);
@@ -1452,7 +1503,8 @@ void buy_callback(GtkWidget *w, gpointer data)
   value=city_buy_cost(pdialog->pcity);
 
   if(game.player_ptr->economic.gold>=value) {
-    sprintf(buf, _("Buy %s for %d gold?\nTreasury contains %d gold."), 
+    my_snprintf(buf, sizeof(buf),
+	    _("Buy %s for %d gold?\nTreasury contains %d gold."), 
 	    name, value, game.player_ptr->economic.gold);
 
     popup_message_dialog(pdialog->shell, /*"buydialog"*/ _("Buy It!"), buf,
@@ -1460,7 +1512,8 @@ void buy_callback(GtkWidget *w, gpointer data)
 			 _("No"), buy_callback_no, 0, 0);
   }
   else {
-    sprintf(buf, _("%s costs %d gold.\nTreasury contains %d gold."), 
+    my_snprintf(buf, sizeof(buf),
+	    _("%s costs %d gold.\nTreasury contains %d gold."), 
 	    name, value, game.player_ptr->economic.gold);
 
     popup_message_dialog(pdialog->shell, /*"buynodialog"*/ _("Buy It!"), buf,
@@ -1510,7 +1563,8 @@ void upgrade_callback(GtkWidget *w, gpointer data)
 
     if (ut2==-1) {
       /* this shouldn't generally happen, but it is conceivable */
-      sprintf(buf, _("Sorry: cannot upgrade %s."), unit_types[ut1].name);
+      my_snprintf(buf, sizeof(buf),
+		  _("Sorry: cannot upgrade %s."), unit_types[ut1].name);
       popup_message_dialog(toplevel, /*"upgradenodialog"*/_("Upgrade Unit!"), buf,
 			   _("Darn"), unitupgrade_callback_no, 0,
 			   NULL);
@@ -1518,7 +1572,7 @@ void upgrade_callback(GtkWidget *w, gpointer data)
       value=unit_upgrade_price(game.player_ptr, ut1, ut2);
 
       if(game.player_ptr->economic.gold>=value) {
-	sprintf(buf, _("Upgrade %s to %s for %d gold?\n"
+	my_snprintf(buf, sizeof(buf), _("Upgrade %s to %s for %d gold?\n"
 	       "Treasury contains %d gold."),
 	       unit_types[ut1].name, unit_types[ut2].name,
 	       value, game.player_ptr->economic.gold);
@@ -1530,7 +1584,7 @@ void upgrade_callback(GtkWidget *w, gpointer data)
 			       unitupgrade_callback_no, 0,
 			     NULL);
       } else {
-	sprintf(buf, _("Upgrading %s to %s costs %d gold.\n"
+	my_snprintf(buf, sizeof(buf), _("Upgrading %s to %s costs %d gold.\n"
 	       "Treasury contains %d gold."),
 	       unit_types[ut1].name, unit_types[ut2].name,
 	       value, game.player_ptr->economic.gold);
@@ -1561,6 +1615,7 @@ static void change_to_callback(GtkWidget *w, gpointer data)
 
     packet.city_id=pdialog->pcity->id;
     packet.name[0]='\0';
+    packet.worklist.name[0] = '\0';
     packet.build_id=pdialog->change_list_ids[row];
     packet.is_build_id_unit_id=
       (row >= pdialog->change_list_num_improvements);
@@ -1627,6 +1682,16 @@ static gint change_deleted_callback(GtkWidget *w, GdkEvent *ev, gpointer data)
   return FALSE;
 }
 
+/****************************************************************
+...
+*****************************************************************/
+static void change_list_callback(GtkWidget *w, gint row, gint col, GdkEvent *ev, gpointer data)
+     /* Allows new production options to be selected via a double-click */
+{
+  if(ev && ev->type==GDK_2BUTTON_PRESS)
+    change_to_callback(w, data);
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -1634,7 +1699,7 @@ void change_callback(GtkWidget *w, gpointer data)
 {
   GtkWidget *cshell, *button, *scrolled;
   struct city_dialog *pdialog;
-  int i, n;
+  int i, n, turns;
   static gchar *title_[1] = { N_("Select new production") };
   static gchar **title = NULL;
   GtkAccelGroup *accel=gtk_accel_group_new();
@@ -1656,9 +1721,13 @@ void change_callback(GtkWidget *w, gpointer data)
   scrolled = gtk_scrolled_window_new(NULL, NULL);
   gtk_container_add(GTK_CONTAINER(scrolled), pdialog->change_list);
 
+  /* Set up the doubleclick-on-list-item handler */
+  gtk_signal_connect(GTK_OBJECT(pdialog->change_list), "select_row",
+		     GTK_SIGNAL_FUNC(change_list_callback), pdialog);
+
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
 	GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_widget_set_usize(scrolled, 220, 350);
+  gtk_widget_set_usize(scrolled, 280, 350);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cshell)->vbox), scrolled,
 	TRUE, TRUE, 0);
   
@@ -1690,9 +1759,20 @@ void change_callback(GtkWidget *w, gpointer data)
 
   for(i=0, n=0; i<B_LAST; i++)
     if(can_build_improvement(pdialog->pcity, i)) {
-      sprintf(pdialog->change_list_names[n], "%s (%d)",
-	get_imp_name_ex(pdialog->pcity, i),get_improvement_type(i)->build_cost);
-      
+      if (i==B_CAPITAL) { /* Total and turns left meaningless for captialization */
+	my_snprintf(pdialog->change_list_names[n],
+		    sizeof(pdialog->change_list_names[n]),
+		    "%s (XX)",
+		    get_imp_name_ex(pdialog->pcity, i));
+      } else {
+	turns = city_turns_to_build (pdialog->pcity, i, FALSE);
+	my_snprintf(pdialog->change_list_names[n],
+		    sizeof(pdialog->change_list_names[n]),
+		    turns == 1 ? _("%s (%d) %3d turn") : _("%s (%d) %3d turns"),
+		    get_imp_name_ex(pdialog->pcity, i),
+		    get_improvement_type(i)->build_cost,
+		    turns);
+      }
       pdialog->change_list_names_ptrs[n]=pdialog->change_list_names[n];
       pdialog->change_list_ids[n++]=i;
     }
@@ -1702,9 +1782,13 @@ void change_callback(GtkWidget *w, gpointer data)
 
   for(i=0; i<game.num_unit_types; i++)
     if(can_build_unit(pdialog->pcity, i)) {
-      sprintf(pdialog->change_list_names[n],"%s (%d)",
-	get_unit_name(i), get_unit_type(i)->build_cost);
-
+      turns = city_turns_to_build (pdialog->pcity, i, TRUE);
+      my_snprintf(pdialog->change_list_names[n],
+		  sizeof(pdialog->change_list_names[n]),
+		  turns == 1 ? _("%s (%d) %3d turn") : _("%s (%d) %3d turns"),
+		  get_unit_name(i),
+		  get_unit_type(i)->build_cost,
+		  turns);
       pdialog->change_list_names_ptrs[n]=pdialog->change_list_names[n];
       pdialog->change_list_ids[n++]=i;
     }
@@ -1727,6 +1811,90 @@ void change_callback(GtkWidget *w, gpointer data)
 }
 
 
+/****************************************************************
+  Display the city's worklist.
+*****************************************************************/
+void worklist_callback(GtkWidget *w, gpointer data)
+{
+  struct city_dialog *pdialog;
+  
+  pdialog = (struct city_dialog *)data;
+
+  if (pdialog->worklist_shell) {
+    gtk_set_relative_position(pdialog->shell, pdialog->worklist_shell,
+			      20, 20);
+    gtk_widget_show(pdialog->worklist_shell);
+  } else {
+    pdialog->worklist_shell = 
+      popup_worklist(pdialog->pcity->worklist, pdialog->pcity, pdialog->shell, 
+		     (void *)pdialog, commit_city_worklist, 
+		     cancel_city_worklist);
+  }
+}
+
+/****************************************************************
+  Commit the changes to the worklist for the city.
+*****************************************************************/
+void commit_city_worklist(struct worklist *pwl, void *data)
+{
+  struct packet_city_request packet;
+  struct city_dialog *pdialog = (struct city_dialog *)data;
+  int i, k, id, is_unit;
+
+  /* Update the worklist.  Remember, though -- the current build 
+     target really isn't in the worklist; don't send it to the 
+     server as part of the worklist.  Of course, we have to
+     search through the current worklist to find the first
+     _now_available_ build target (to cope with players who try
+     mean things like adding a Battleship to a city worklist when
+     the player doesn't even yet have the Map Making tech).  */
+
+  for (k = 0; k < MAX_LEN_WORKLIST; k++) {
+    if (! worklist_peek_ith(pwl, &id, &is_unit, k))
+      break;
+
+    /* If it can be built... */
+    if (( is_unit && can_build_unit(pdialog->pcity, id)) ||
+	(!is_unit && can_build_improvement(pdialog->pcity, id))) {
+      /* ...but we're not yet building it, then switch. */
+      if (id != pdialog->pcity->currently_building || 
+	  is_unit != pdialog->pcity->is_building_unit) {
+
+	/* Change the current target */
+	packet.city_id=pdialog->pcity->id;
+	packet.name[0] = '\0';
+	packet.worklist.name[0] = '\0';
+	packet.build_id = id;
+	packet.is_build_id_unit_id = is_unit;
+	send_packet_city_request(&aconnection, &packet, PACKET_CITY_CHANGE);
+      }
+
+      /* This item is now (and may have always been) the current
+	 build target.  Drop it out of the worklist. */
+      worklist_remove(pwl, k);
+      break;
+    }
+  }
+
+  /* Send the rest of the worklist on its way. */
+  packet.city_id=pdialog->pcity->id;
+  packet.name[0] = '\0';
+  packet.worklist.name[0] = '\0';
+  packet.worklist.is_valid = 1;
+  for (i = 0; i < MAX_LEN_WORKLIST; i++) {
+    packet.worklist.ids[i] = pwl->ids[i];
+  }
+    
+  send_packet_city_request(&aconnection, &packet, PACKET_CITY_WORKLIST);
+
+  pdialog->worklist_shell = NULL;
+}
+
+void cancel_city_worklist(void *data) {
+  struct city_dialog *pdialog = (struct city_dialog *)data;
+  pdialog->worklist_shell = NULL;
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -1740,6 +1908,7 @@ static void sell_callback_yes(GtkWidget *w, gpointer data)
   packet.city_id=pdialog->pcity->id;
   packet.build_id=pdialog->sell_id;
   packet.name[0]='\0';
+  packet.worklist.name[0] = '\0';
   send_packet_city_request(&aconnection, &packet, PACKET_CITY_SELL);
 
   destroy_message_dialog(w);
@@ -1777,7 +1946,7 @@ void sell_callback(GtkWidget *w, gpointer data)
 	    return;
 	  
 	  pdialog->sell_id=i;
-	  sprintf(buf, _("Sell %s for %d gold?"), 
+	  my_snprintf(buf, sizeof(buf), _("Sell %s for %d gold?"), 
 		  get_imp_name_ex(pdialog->pcity, i),
 		  improvement_value(i));
 
@@ -1802,6 +1971,9 @@ void close_city_dialog(struct city_dialog *pdialog)
   if(pdialog->is_modal)
     gtk_widget_set_sensitive(toplevel, TRUE);
 
+  if (pdialog->worklist_shell)
+    gtk_widget_destroy(pdialog->worklist_shell);
+
   gtk_widget_destroy(pdialog->shell);
   free(pdialog);
 }
