@@ -28,10 +28,12 @@
 #include "game.h"
 #include "packets.h"
 #include "shared.h"
+#include "support.h"
 #include "unit.h"
 
 #include "chatline.h"
 #include "citydlg.h"
+#include "cityrepdata.h"
 #include "gui_stuff.h"
 #include "mapview.h"
 #include "optiondlg.h"
@@ -42,227 +44,11 @@
 #include "cityrep.h"
 
 extern GtkWidget *toplevel;
-extern GdkWindow *root_window;
 
 extern struct connection aconnection;
 extern int delay_report_update;
 
-/* abbreviate long city names to this length in the city report: */
-#define REPORT_CITYNAME_ABBREV 15 
-
-/************************************************************************
- cr_entry = return an entry (one column for one city) for the city report
- These return ptrs to filled in static strings.
- Note the returned string may not be exactly the right length; that
- is handled later.
-*************************************************************************/
-
-static char *cr_entry_cityname(struct city *pcity)
-{
-  static char buf[REPORT_CITYNAME_ABBREV+1];
-  if (strlen(pcity->name) <= REPORT_CITYNAME_ABBREV) {
-    return pcity->name;
-  } else {
-    strncpy(buf, pcity->name, REPORT_CITYNAME_ABBREV-1);
-    buf[REPORT_CITYNAME_ABBREV-1] = '.';
-    buf[REPORT_CITYNAME_ABBREV] = '\0';
-    return buf;
-  }
-}
-
-static char *cr_entry_size(struct city *pcity)
-{
-  static char buf[8];
-  sprintf(buf, "%2d", pcity->size);
-  return buf;
-}
-
-static char *cr_entry_hstate_concise(struct city *pcity)
-{
-  static char buf[4];
-  sprintf(buf, "%s", (city_celebrating(pcity) ? "*" :
-		     (city_unhappy(pcity) ? "X" : " ")));
-  return buf;
-}
-
-static char *cr_entry_hstate_verbose(struct city *pcity)
-{
-  static char buf[16];
-  sprintf(buf, "%s", (city_celebrating(pcity) ? _("Rapture") :
-		     (city_unhappy(pcity) ? _("Disorder") : _("Peace"))));
-  return buf;
-}
-
-static char *cr_entry_workers(struct city *pcity)
-{
-  static char buf[32];
-  sprintf(buf, "%d/%d/%d",
-	 pcity->ppl_happy[4],
-	 pcity->ppl_content[4],
-	 pcity->ppl_unhappy[4]);
-  return buf;
-}
-
-static char *cr_entry_specialists(struct city *pcity)
-{
-  static char buf[32];
-  sprintf(buf, "%d/%d/%d",
-	 pcity->ppl_elvis,
-	  pcity->ppl_scientist,
-	  pcity->ppl_taxman);
-  return buf;
-}
-
-static char *cr_entry_resources(struct city *pcity)
-{
-  static char buf[32];
-  sprintf(buf, "%d/%d/%d",
-	 pcity->food_surplus, 
-	 pcity->shield_surplus, 
-	 pcity->trade_prod);
-  return buf;
-}
-
-static char *cr_entry_output(struct city *pcity)
-{
-  static char buf[32];
-  int goldie;
-
-  goldie = city_gold_surplus(pcity);
-  sprintf(buf, "%s%d/%d/%d",
-	  (goldie < 0) ? "-" : (goldie > 0) ? "+" : "",
-	  (goldie < 0) ? (-goldie) : goldie,
-	 pcity->luxury_total,
-	 pcity->science_total);
-  return buf;
-}
-
-static char *cr_entry_food(struct city *pcity)
-{
-  static char buf[32];
-  sprintf(buf,"%d/%d",
-	 pcity->food_stock,
-	 (pcity->size+1) * game.foodbox);
-  return buf;
-}
-
-static char *cr_entry_pollution(struct city *pcity)
-{
-  static char buf[8];
-  sprintf(buf,"%3d", pcity->pollution);
-  return buf;
-}
-
-static char *cr_entry_num_trade(struct city *pcity)
-{
-  static char buf[8];
-  sprintf(buf,"%d", city_num_trade_routes(pcity));
-  return buf;
-}
-
-static char *cr_entry_building(struct city *pcity)
-{
-  static char buf[64];
-  if(pcity->is_building_unit)
-    sprintf(buf, "%s(%d/%d/%d)", 
-	    get_unit_type(pcity->currently_building)->name,
-	   pcity->shield_stock,
-	   get_unit_type(pcity->currently_building)->build_cost,
-	   city_buy_cost(pcity));
-  else
-    sprintf(buf, "%s(%d/%d/%d)", 
-	   get_imp_name_ex(pcity, pcity->currently_building),
-	   pcity->shield_stock,
-	   get_improvement_type(pcity->currently_building)->build_cost,
-	   city_buy_cost(pcity));
-  return buf;
-}
-
-static char *cr_entry_corruption(struct city *pcity)
-{
-  static char buf[8];
-  sprintf(buf,"%3d", pcity->corruption);
-  return buf;
-}
-
-/* City report options (which columns get shown)
- * To add a new entry, you should just have to:
- * - add a function like those above
- * - add an entry in the city_report_specs[] table
- */
-
-struct city_report_spec {
-  int show;		       /* modify this to customize */
-  int width;		       /* 0 means variable; rightmost only */
-  int space;		       /* number of leading spaces (see below) */
-  char *title1;
-  char *title2;
-  char *explanation;
-  char *(*func)(struct city*);
-  char *tagname;	       /* for save_options */
-};
-
-/* This generates the function name and the tagname: */
-#define FUNC_TAG(var)  cr_entry_##var, #var 
-
-/* Use tagname rather than index for load/save, because later
-   additions won't necessarily be at the end.
-*/
-
-/* Note on space: you can do spacing and alignment in various ways;
-   you can avoid explicit space between columns if they are bracketted,
-   but the problem is that with a configurable report you don't know
-   what's going to be next to what.
-*/
-
-static struct city_report_spec city_report_specs[] = {
-  { 1,-15, 0, "",  N_("Name"),	      N_("City Name"),
-				      FUNC_TAG(cityname) },
-  { 0, -2, 1, "",  N_("Sz"),	      N_("Size"),
-				      FUNC_TAG(size) },
-  { 1, -8, 1, "",  N_("State"),	      N_("Rapture/Peace/Disorder"),
-				      FUNC_TAG(hstate_verbose) },
-  { 0, -1, 1, "",  "",  	      N_("Concise *=Rapture, X=Disorder"),
-				      FUNC_TAG(hstate_concise) },
-  { 1, -8, 1, N_("Workers"), N_("H/C/U"),     N_("Workers: Happy, Content, Unhappy"),
-				      FUNC_TAG(workers) },
-  { 0, -7, 1, N_("Special"), N_("E/S/T"),     N_("Entertainers, Scientists, Taxmen"),
-				      FUNC_TAG(specialists) },
-  { 1,-10, 1, N_("Surplus"), N_("F/P/T"),     N_("Surplus: Food, Production, Trade"),
-				      FUNC_TAG(resources) },
-  { 1,-10, 1, N_("Economy"), N_("G/L/S"),     N_("Economy: Gold, Luxuries, Science"),
-				      FUNC_TAG(output) },
-  { 0, -1, 1, "n", "T", 	      N_("Number of Trade Routes"),
-				      FUNC_TAG(num_trade) },
-  { 1, -7, 1, N_("Food"), N_("Stock"),        N_("Food Stock"),
-				      FUNC_TAG(food) },
-  { 0, -3, 1, "", N_("Pol"),	      N_("Pollution"),
-				      FUNC_TAG(pollution) },
-  { 0, -3, 1, "", N_("Cor"),              N_("Corruption"),
-                                      FUNC_TAG(corruption) },
-  { 1,  0, 1, N_("Currently Building"),   N_("(Stock,Target,Buy Cost)"),
-				      N_("Currently Building"),
-				      FUNC_TAG(building) }
-};
-
-#define NUM_CREPORT_COLS \
-	 sizeof(city_report_specs)/sizeof(city_report_specs[0])
-     
-/******************************************************************
-Some simple wrappers:
-******************************************************************/
-int num_city_report_spec(void)
-{
-  return NUM_CREPORT_COLS;
-}
-int *city_report_spec_show_ptr(int i)
-{
-  return &(city_report_specs[i].show);
-}
-char *city_report_spec_tagname(int i)
-{
-  return city_report_specs[i].tagname;
-}
+#define NEG_VAL(x)  ((x)<0 ? (x) : (-x))
 
 /******************************************************************/
 GtkWidget *config_shell;
@@ -281,23 +67,25 @@ void city_center_callback(GtkWidget *widget, gpointer data);
 void city_popup_callback(GtkWidget *widget, gpointer data);
 void city_buy_callback(GtkWidget *widget, gpointer data);
 void city_refresh_callback(GtkWidget *widget, gpointer data);
-void city_change_callback(GtkWidget *widget, gpointer data);
 void city_change_all_dialog_callback(GtkWidget *w, gpointer data);
 void city_change_all_callback(GtkWidget *w, gpointer data);
 void city_list_callback(GtkWidget *w, gint row, gint column);
-void city_list_ucallback(GtkWidget *w, gint row, gint column);
 void city_config_callback(GtkWidget *widget, gpointer data);
+static gboolean city_change_callback(GtkWidget *w, GdkEvent *event, gpointer data);
+static gboolean city_select_callback(GtkWidget *w, GdkEvent *event, gpointer data);
+static void city_report_list_callback(GtkWidget *w, gint row, gint col, GdkEvent *ev, gpointer data);
 
 GtkWidget *city_dialog_shell=NULL;
 GtkWidget *city_label;
 GtkWidget *city_list;
 GtkWidget *city_center_command, *city_popup_command, *city_buy_command,
 	  *city_refresh_command, *city_config_command;
-GtkWidget *city_change_command, *city_popupmenu;
+GtkWidget *city_change_command;
 GtkWidget *city_change_all_command;
 GtkWidget *city_change_all_dialog_shell;
 GtkWidget *city_change_all_from_list;
 GtkWidget *city_change_all_to_list;
+GtkWidget *city_select_command;
 
 int city_dialog_shell_is_modal;
 
@@ -326,7 +114,7 @@ static void sort_cities_callback( GtkButton *button, gpointer *data )
 /****************************************************************
  Create the text for a line in the city report
 *****************************************************************/
-static void get_city_text(struct city *pcity, char *buf[])
+static void get_city_text(struct city *pcity, char *buf[], int n)
 {
   struct city_report_spec *spec;
   int i;
@@ -335,21 +123,22 @@ static void get_city_text(struct city *pcity, char *buf[])
     buf[i][0]='\0';
     if(!spec->show) continue;
 
-    sprintf(buf[i], "%*s", spec->width, (spec->func)(pcity));
+    my_snprintf(buf[i], n, "%*s", NEG_VAL(spec->width), (spec->func)(pcity));
   }
 }
 
 /****************************************************************
  Return text line for the column headers for the city report
 *****************************************************************/
-static void get_city_table_header(char *text[])
+static void get_city_table_header(char *text[], int n)
 {
   struct city_report_spec *spec;
   int i;
 
   for(i=0, spec=city_report_specs; i<NUM_CREPORT_COLS; i++, spec++) {
-    sprintf(text[i], "%*s\n%*s",
-            spec->width, _(spec->title1), spec->width, _(spec->title2));
+    my_snprintf(text[i], n, "%*s\n%*s",
+            NEG_VAL(spec->width), _(spec->title1),
+	    NEG_VAL(spec->width), _(spec->title2));
   }
 }
 
@@ -381,14 +170,239 @@ void popup_city_report_dialog(int make_modal)
 /****************************************************************
 ...
 *****************************************************************/
-static gint city_change_mbutton_callback(GtkWidget *w, GdkEvent *event)
+static struct city* 
+city_from_glist(GList* list)
 {
+  struct city* retval;
+
+  g_assert (list);
+  
+  retval = gtk_clist_get_row_data(GTK_CLIST(city_list),
+				  GPOINTER_TO_INT(list->data));
+
+  g_assert (retval);
+
+  return retval;
+}
+
+typedef gboolean TestCityFunc(struct city *, gint);
+
+static gboolean city_can_build_imp_or_unit(struct city *pcity, gint number)
+{
+  if(number >= B_LAST)
+    return can_build_unit(pcity, number - B_LAST);
+  else
+    return can_build_improvement(pcity, number);
+}
+
+static void
+append_imp_or_unit_to_menu_sub(GtkWidget *menu,
+			       gchar *nothing_appended_text,
+			       gboolean append_units,
+			       gboolean append_wonders,
+			       gboolean change_prod,
+			       TestCityFunc test_func,
+			       GtkSignalFunc callback)
+{  
+  gint i;
+  gint first = append_units ? B_LAST : 0;
+  gint last = append_units ? game.num_unit_types + B_LAST : B_LAST;
+  gboolean something_appended = FALSE;
+  GtkWidget *item; 
+
+  for(i=first; i<last; i++)
+    {
+      gboolean append = FALSE;
+
+      /* Those many ! are to ensure TRUE is 1 and FALSE is 0 */
+      if( !append_units && ( !append_wonders != !is_wonder(i) ) )
+	continue;
+
+      if(!change_prod)
+	{  
+	  city_list_iterate(game.player_ptr->cities, pcity) 
+	    {
+	      append |= test_func(pcity, i);    
+	    } city_list_iterate_end;
+	}
+      else
+	{
+	  GList *selection;
+
+	  g_assert (GTK_CLIST(city_list)->selection);
+	  selection = GTK_CLIST(city_list)->selection;
+	  for(; selection; selection = g_list_next(selection))
+	    append |= test_func(city_from_glist(selection), i);
+	}
+    
+      if(append) 
+	{
+	  gint cost;
+	  gchar* name;
+	  
+	  if(append_units)
+	    {
+	      cost = get_unit_type(i-B_LAST)->build_cost;
+	      name = get_unit_name(i-B_LAST);
+	    }
+	  else
+	    {
+	      cost = (i==B_CAPITAL) ? -1 : get_improvement_type(i)->build_cost;
+	      if(append_wonders)
+		{
+		  /* We need a city to get the right name for wonders */
+		  struct city *pcity = GTK_CLIST(city_list)->row_list->data;
+		  name = get_imp_name_ex(pcity, i);
+		}
+	      else
+		name = get_improvement_name(i);
+	    }
+	  something_appended = TRUE;
+
+	  if (change_prod)
+	    {
+	      gchar *label;
+	      if (cost < 0) {
+		label = g_strdup_printf("%s (XX)",name);
+	      } else {
+		label = g_strdup_printf("%s (%d)",name, cost);
+	      }
+	      item=gtk_menu_item_new_with_label( label );
+	      g_free (label);
+	    }
+	  else
+	    item=gtk_menu_item_new_with_label(name);
+	  
+	  gtk_menu_append(GTK_MENU(menu),item);
+	  
+	  gtk_signal_connect(GTK_OBJECT(item),"activate", callback, 
+			     GINT_TO_POINTER(i));
+	}
+    }
+
+  if(!something_appended)
+    {
+      item=gtk_menu_item_new_with_label( nothing_appended_text );
+      gtk_widget_set_sensitive(item, FALSE);
+      gtk_menu_append(GTK_MENU(menu),item);  
+    }
+}
+
+static
+void select_imp_or_unit_callback(GtkWidget *w, gpointer data)
+{
+  gint number = GPOINTER_TO_INT(data);
+  gint i;
+  GtkObject *parent = GTK_OBJECT(w->parent);
+  TestCityFunc *test_func = gtk_object_get_data(parent, "freeciv_test_func");
+  gboolean change_prod = 
+    GPOINTER_TO_INT(gtk_object_get_data(parent, "freeciv_change_prod"));
+
+  /* If this is not the change production button */
+  if(!change_prod)
+    {
+      gtk_clist_unselect_all( GTK_CLIST(city_list));
+
+      for(i = 0; i < GTK_CLIST(city_list)->rows; i++)
+	{
+	  struct city* pcity = gtk_clist_get_row_data(GTK_CLIST(city_list),i);
+	  if (test_func(pcity, number))
+	    gtk_clist_select_row(GTK_CLIST(city_list),i,0);
+	}
+    }
+  else
+    {
+      gboolean is_unit = number >= B_LAST;
+      GList* selection = GTK_CLIST(city_list)->selection;
+
+      g_assert(selection);
+  
+      for(; selection; selection = g_list_next(selection))
+	{
+	  struct packet_city_request packet;
+
+	  if (is_unit)
+	    number -= B_LAST;
+	  
+	  packet.city_id=city_from_glist(selection)->id;
+	  packet.name[0]='\0';
+	  packet.worklist.name[0] = '\0';
+	  packet.build_id=number;
+	  packet.is_build_id_unit_id=is_unit;
+	  send_packet_city_request(&aconnection, &packet, PACKET_CITY_CHANGE);
+	}
+    }
+}
+
+static void
+append_imp_or_unit_to_menu(GtkWidget *menu,
+			   gboolean change_prod,
+			   gboolean append_improvements,
+			   gboolean append_units,
+			   TestCityFunc test_func)
+{
+  if(append_improvements)
+    {
+      /* Add all buildings */
+      append_imp_or_unit_to_menu_sub(menu, _("No Buildings Available"),
+				     FALSE, FALSE, change_prod,
+				     (gboolean (*)(struct city*,gint))
+				     test_func,
+				     select_imp_or_unit_callback);
+      /* Add a separator */
+      gtk_menu_append(GTK_MENU(menu),gtk_menu_item_new ());  
+  
+      /* Add all wonders */
+      append_imp_or_unit_to_menu_sub(menu, _("No Wonders Available"),
+				     FALSE, TRUE, change_prod,
+				     (gboolean (*)(struct city*,gint))
+				     test_func,
+				     select_imp_or_unit_callback);
+      /* Add a separator */
+      if(append_units)
+	gtk_menu_append(GTK_MENU(menu),gtk_menu_item_new ());   
+    }
+  
+  if(append_units)
+    {
+      /* Add all units */
+      append_imp_or_unit_to_menu_sub(menu, _("No Units Available"),
+				     TRUE, FALSE, change_prod,
+				     test_func,
+				     select_imp_or_unit_callback);
+    }
+  
+  gtk_object_set_data(GTK_OBJECT(menu), "freeciv_test_func", test_func);
+  gtk_object_set_data(GTK_OBJECT(menu), "freeciv_change_prod", 
+		      GINT_TO_POINTER(change_prod));
+}
+
+
+static gint 
+city_change_callback(GtkWidget *w, GdkEvent *event, gpointer data)
+{
+  static GtkWidget* menu = NULL;
   GdkEventButton *bevent = (GdkEventButton *)event;
   
   if ( event->type != GDK_BUTTON_PRESS )
     return FALSE;
 
-  gtk_menu_popup(GTK_MENU(city_popupmenu),
+  /* This migth happen, whenever a selection is still in progress, while 
+     "Changed" is pressed (e.g. when holding the shift key) */
+  if(!GTK_CLIST(city_list)->selection)
+    return FALSE;
+
+  if (menu)
+    gtk_widget_destroy(menu);
+  
+  menu=gtk_menu_new();
+  
+  append_imp_or_unit_to_menu(menu, TRUE, TRUE, TRUE, 
+			     city_can_build_imp_or_unit);
+
+  gtk_widget_show_all(menu);
+  
+  gtk_menu_popup(GTK_MENU(menu),
 	NULL,NULL,NULL,NULL,bevent->button,bevent->time);
 
   return TRUE;
@@ -424,7 +438,7 @@ void create_city_report_dialog(int make_modal)
   for (i=0;i<NUM_CREPORT_COLS;i++)
     titles[i]=buf[i];
 
-  get_city_table_header(titles);
+  get_city_table_header(titles, sizeof(buf[0]));
 
   city_list = gtk_clist_new_with_titles(NUM_CREPORT_COLS,titles);
   gtk_clist_column_titles_active(GTK_CLIST(city_list));
@@ -436,6 +450,9 @@ void create_city_report_dialog(int make_modal)
   			  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
   gtk_widget_set_usize(city_list, 620, 250);
 
+  gtk_signal_connect(GTK_OBJECT(city_list), "select_row",
+		     GTK_SIGNAL_FUNC(city_report_list_callback), NULL);
+
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG(city_dialog_shell)->vbox ),
         scrolled, TRUE, TRUE, 0 );
 
@@ -468,10 +485,8 @@ void create_city_report_dialog(int make_modal)
         city_change_command, TRUE, TRUE, 0 );
   GTK_WIDGET_SET_FLAGS( city_change_command, GTK_CAN_DEFAULT );
 
-  city_popupmenu=gtk_menu_new();
-
   gtk_signal_connect( GTK_OBJECT( city_change_command ), "event",
-	GTK_SIGNAL_FUNC( city_change_mbutton_callback ), NULL );
+	GTK_SIGNAL_FUNC( city_change_callback ), NULL );
 
   city_change_all_command = gtk_accelbutton_new(_("Change _All"), accel);
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG(city_dialog_shell)->action_area ),
@@ -483,6 +498,14 @@ void create_city_report_dialog(int make_modal)
         city_refresh_command, TRUE, TRUE, 0 );
   GTK_WIDGET_SET_FLAGS( city_refresh_command, GTK_CAN_DEFAULT );
   
+  city_select_command	= gtk_accelbutton_new(_("_Select"), accel);
+  gtk_box_pack_start( GTK_BOX( GTK_DIALOG(city_dialog_shell)->action_area ),
+        city_select_command, TRUE, TRUE, 0 );
+  GTK_WIDGET_SET_FLAGS( city_select_command, GTK_CAN_DEFAULT );
+
+  gtk_signal_connect(GTK_OBJECT(city_select_command), "event",
+	GTK_SIGNAL_FUNC(city_select_callback), NULL);
+
   city_config_command	= gtk_accelbutton_new(_("Con_figure"), accel);
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG(city_dialog_shell)->action_area ),
         city_config_command, TRUE, TRUE, 0 );
@@ -505,7 +528,7 @@ void create_city_report_dialog(int make_modal)
   gtk_signal_connect(GTK_OBJECT(city_list), "select_row",
 	GTK_SIGNAL_FUNC(city_list_callback), NULL);
   gtk_signal_connect(GTK_OBJECT(city_list), "unselect_row",
-	GTK_SIGNAL_FUNC(city_list_ucallback), NULL);
+	GTK_SIGNAL_FUNC(city_list_callback), NULL);
 
   for ( i = 0;i < GTK_CLIST(city_list)->columns; i++ )
     gtk_signal_connect(GTK_OBJECT(GTK_CLIST(city_list)->column[i].button),
@@ -525,59 +548,12 @@ void create_city_report_dialog(int make_modal)
 *****************************************************************/
 void city_list_callback(GtkWidget *w, gint row, gint column)
 {
-  struct city *pcity;
-
-  if(row!=-1 && (pcity=gtk_clist_get_row_data(GTK_CLIST(city_list),row)))
+  if (GTK_CLIST(city_list)->selection)
   {
-    int flag;
-    size_t i;
-    char buf[512];
-
     gtk_widget_set_sensitive(city_change_command, TRUE);
     gtk_widget_set_sensitive(city_center_command, TRUE);
     gtk_widget_set_sensitive(city_popup_command, TRUE);
     gtk_widget_set_sensitive(city_buy_command, TRUE);
-    if (city_popupmenu)
-      gtk_widget_destroy(city_popupmenu);
-
-    city_popupmenu=gtk_menu_new();
-
-    flag = 0;
-
-    for(i=0; i<B_LAST; i++)
-      if(can_build_improvement(pcity, i)) 
-      {
-        GtkWidget *item;
-
-        sprintf(buf,"%s (%d)",get_imp_name_ex(pcity, i),get_improvement_type(i)->build_cost);
-        item=gtk_menu_item_new_with_label( buf );
-        gtk_menu_append(GTK_MENU(city_popupmenu),item);
-
-        gtk_signal_connect(GTK_OBJECT(item),"activate",
-            GTK_SIGNAL_FUNC(city_change_callback),
-            (gpointer)i);
-        flag=1;
-      }
-
-    for(i=0; i<game.num_unit_types; i++)
-      if(can_build_unit(pcity, i)) {
-        GtkWidget *item;
-
-        sprintf(buf,"%s (%d)", 
-                get_unit_name(i),get_unit_type(i)->build_cost);
-        item=gtk_menu_item_new_with_label( buf );
-        gtk_menu_append(GTK_MENU(city_popupmenu),item);
-
-        gtk_signal_connect(GTK_OBJECT(item),"activate",
-            GTK_SIGNAL_FUNC(city_change_callback),
-            (gpointer)(i+B_LAST));
-        flag = 1;
-      }
-
-    gtk_widget_show_all(city_popupmenu);
-
-    if(!flag)
-      gtk_widget_set_sensitive(city_change_command, FALSE);
   }
   else
   {
@@ -588,15 +564,198 @@ void city_list_callback(GtkWidget *w, gint row, gint column)
   }
 }
 
-void city_list_ucallback(GtkWidget *w, gint row, gint column)
+static gboolean
+city_select_all_callback(GtkWidget *w, gpointer data)
 {
-  if (!GTK_CLIST( city_list )->selection)
+  gtk_clist_select_all( GTK_CLIST(city_list));
+  return TRUE;
+}
+
+static gboolean
+city_unselect_all_callback(GtkWidget *w, gpointer data)
+{
+  gtk_clist_unselect_all( GTK_CLIST(city_list));
+  return TRUE;
+}
+
+static gboolean
+city_invert_selection_callback(GtkWidget *w, gpointer data)
+{
+  gint i;
+  GList* row_list = GTK_CLIST(city_list)->row_list;
+  for(i = 0; i < GTK_CLIST(city_list)->rows; i++)
   {
-      gtk_widget_set_sensitive(city_change_command, FALSE);
-      gtk_widget_set_sensitive(city_center_command, FALSE);
-      gtk_widget_set_sensitive(city_popup_command, FALSE);
-      gtk_widget_set_sensitive(city_buy_command, FALSE);
+      GtkCListRow* row = GTK_CLIST_ROW(g_list_nth(row_list,i));
+      if(row->state == GTK_STATE_SELECTED)
+	gtk_clist_unselect_row(GTK_CLIST(city_list),i,0);
+      else
+	gtk_clist_select_row(GTK_CLIST(city_list),i,0);     
   }
+  return TRUE;
+}
+
+static gboolean
+city_select_coastal_callback(GtkWidget *w, gpointer data)
+{
+  gint i;
+
+  gtk_clist_unselect_all( GTK_CLIST(city_list));
+
+  for(i = 0; i < GTK_CLIST(city_list)->rows; i++)
+  {
+      struct city* pcity = gtk_clist_get_row_data(GTK_CLIST(city_list),i);
+
+      if (is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN))
+	gtk_clist_select_row(GTK_CLIST(city_list),i,0);
+  }
+
+  return TRUE;
+}
+
+static gboolean
+city_select_same_island_callback(GtkWidget *w, gpointer data)
+{
+  gint i;
+  GList *selection = GTK_CLIST(city_list)->selection;
+  GList *copy = NULL;
+
+  for(; selection; selection = g_list_next(selection))
+    copy = g_list_append (copy, city_from_glist(selection));
+
+  for(i = 0; i < GTK_CLIST(city_list)->rows; i++)
+    {
+      struct city* pcity = gtk_clist_get_row_data(GTK_CLIST(city_list),i);
+      GList *current = copy;
+      
+      for(; current; current = g_list_next(current))
+	{
+	  struct city* selectedcity = current->data;
+	  if (map_same_continent(pcity->x,pcity->y,
+				 selectedcity->x,selectedcity->y))
+	    {
+	      gtk_clist_select_row(GTK_CLIST(city_list),i,0);
+	      break;
+	    }
+	}
+  }
+
+  g_list_free(copy);
+  return TRUE;
+}
+
+static gboolean
+city_unit_supported(struct city *pcity, gint unit)
+{
+  struct unit_list *punit_list = &pcity->units_supported;
+
+  unit_list_iterate((*punit_list), punit) 
+  {
+      if(punit->type == unit - B_LAST)
+	return TRUE;
+    } unit_list_iterate_end;
+  return FALSE;
+}
+
+static gboolean
+city_unit_present(struct city *pcity, gint unit)
+{
+  struct unit_list *punit_list = &map_get_tile(pcity->x,pcity->y)->units;
+
+  unit_list_iterate((*punit_list), punit) 
+  {
+      if(punit->type == unit - B_LAST)
+	return TRUE;
+    } unit_list_iterate_end;
+  return FALSE;
+}
+      
+static gboolean
+city_select_callback(GtkWidget *w, GdkEvent *event, gpointer data)
+{
+  static GtkWidget* menu = NULL;
+  GtkWidget* submenu = NULL;
+
+  GtkWidget* item;
+  GdkEventButton *bevent = (GdkEventButton *)event;
+
+  if ( event->type != GDK_BUTTON_PRESS )
+    return FALSE;
+
+  if (menu)
+    gtk_widget_destroy(menu);
+
+  menu=gtk_menu_new();
+  
+  item=gtk_menu_item_new_with_label( _("All Cities") );
+  gtk_menu_append(GTK_MENU(menu),item);  
+  gtk_signal_connect(GTK_OBJECT(item),"activate",
+		     GTK_SIGNAL_FUNC(city_select_all_callback),
+		     NULL);
+
+  item=gtk_menu_item_new_with_label( _("No Cities") );
+  gtk_menu_append(GTK_MENU(menu),item);  
+  gtk_signal_connect(GTK_OBJECT(item),"activate",
+		     GTK_SIGNAL_FUNC(city_unselect_all_callback),
+		     NULL);
+
+  item=gtk_menu_item_new_with_label( _("Invert Selection") );
+  gtk_menu_append(GTK_MENU(menu),item);  
+  gtk_signal_connect(GTK_OBJECT(item),"activate",
+		     GTK_SIGNAL_FUNC(city_invert_selection_callback),
+		     NULL);
+
+  /* Add a separator */
+  gtk_menu_append(GTK_MENU(menu),gtk_menu_item_new ());  
+  
+  item=gtk_menu_item_new_with_label( _("Coastal Cities") );
+  gtk_menu_append(GTK_MENU(menu),item);  
+  gtk_signal_connect(GTK_OBJECT(item),"activate",
+		     GTK_SIGNAL_FUNC(city_select_coastal_callback),
+		     NULL);
+
+  item=gtk_menu_item_new_with_label( _("Same Island") );
+  gtk_menu_append(GTK_MENU(menu),item);  
+  gtk_signal_connect(GTK_OBJECT(item),"activate",
+		     GTK_SIGNAL_FUNC(city_select_same_island_callback),
+		     NULL);
+  if(!GTK_CLIST(city_list)->selection)
+    gtk_widget_set_sensitive(item,FALSE);
+
+  /* Add a separator */
+  gtk_menu_append(GTK_MENU(menu),gtk_menu_item_new ());  
+
+  item=gtk_menu_item_new_with_label( _("Supported Units") );
+  gtk_menu_append(GTK_MENU(menu),item);  
+  submenu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
+  append_imp_or_unit_to_menu(submenu, FALSE, FALSE, TRUE, city_unit_supported);
+
+  item=gtk_menu_item_new_with_label( _("Units Present") );
+  gtk_menu_append(GTK_MENU(menu),item);  
+  submenu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
+  append_imp_or_unit_to_menu(submenu, FALSE, FALSE, TRUE, city_unit_present);
+
+  item=gtk_menu_item_new_with_label( _("Available To Build") );
+  gtk_menu_append(GTK_MENU(menu),item);  
+  submenu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
+  append_imp_or_unit_to_menu(submenu, FALSE, TRUE, TRUE,
+			     city_can_build_imp_or_unit);
+
+  item=gtk_menu_item_new_with_label( _("Improvements in City") );
+  gtk_menu_append(GTK_MENU(menu),item);  
+  submenu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
+  append_imp_or_unit_to_menu(submenu, FALSE, TRUE, FALSE,
+			     (TestCityFunc*)city_got_building);
+
+  gtk_widget_show_all(menu);
+  
+  gtk_menu_popup(GTK_MENU(menu), NULL,NULL,NULL,NULL,
+		 bevent->button,bevent->time);
+
+  return TRUE;
 }
 
 /****************************************************************
@@ -605,64 +764,20 @@ void city_list_ucallback(GtkWidget *w, gint row, gint column)
 void city_refresh_callback(GtkWidget *w, gpointer data)
 
 { /* added by Syela - I find this very useful */
-  GList *selection;
-  struct city *pcity;
+  GList *selection = GTK_CLIST(city_list)->selection;
   struct packet_generic_integer packet;
 
-  if ( !( selection = GTK_CLIST( city_list )->selection ) )
+  if ( !selection )
   {
     packet.value = 0;
     send_packet_generic_integer(&aconnection, PACKET_CITY_REFRESH, &packet);
-  }
-  else
-  {
-    gint row = (gint)selection->data;
-
-    if(!(pcity = gtk_clist_get_row_data(GTK_CLIST(city_list),row)))
-      return;
-
-    packet.value = pcity->id;
-    send_packet_generic_integer(&aconnection, PACKET_CITY_REFRESH, &packet);
-  }
-}
-
-/****************************************************************
-...
-*****************************************************************/
-void city_change_callback(GtkWidget *w, gpointer data)
-{
-  GList              *selection;
-  gint                row;
-  struct city *pcity;
-
-  while((selection = GTK_CLIST(city_list)->selection) != NULL)
-  {
-  row = (gint)selection->data;
-
-  if((pcity=gtk_clist_get_row_data(GTK_CLIST(city_list),row)))
-  {
-      struct packet_city_request packet;
-      int build_nr;
-      int unit;
-      
-      build_nr = (size_t) data;
-
-      if (build_nr >= B_LAST)
-	{
-	  build_nr -= B_LAST;
-	  unit = 1;
-	}
-      else
-	unit = 0;
-
-      packet.city_id=pcity->id;
-      packet.name[0]='\0';
-      packet.build_id=build_nr;
-      packet.is_build_id_unit_id=unit;
-      send_packet_city_request(&aconnection, &packet, PACKET_CITY_CHANGE);
-
     }
-    gtk_clist_unselect_row(GTK_CLIST(city_list),row,0);
+  else
+    for(; selection; selection = g_list_next(selection))
+      {
+	packet.value = city_from_glist(selection)->id;
+	send_packet_generic_integer(&aconnection, PACKET_CITY_REFRESH, 
+				    &packet);
   }
 }
 
@@ -679,8 +794,8 @@ void city_change_all_dialog_callback(GtkWidget *w, gpointer data)
 
     if (cmd != NULL) {
 	if ( !( selection = GTK_CLIST( city_change_all_from_list )->selection ) ) {
-	    sprintf(buf,_("Game: Select a unit or improvement to change production from."));
-	    append_output_window(buf);
+	    append_output_window(_("Game: Select a unit or improvement"
+				   " to change production from."));
 	    return;
 	}
 	else {
@@ -689,8 +804,8 @@ void city_change_all_dialog_callback(GtkWidget *w, gpointer data)
 					       row);
 	}
 	if ( !( selection = GTK_CLIST( city_change_all_to_list )->selection ) ) {
-	    sprintf(buf,_("Game: Select a unit or improvement to change production to."));
-	    append_output_window(buf);
+	    append_output_window(_("Game: Select a unit or improvement"
+				   " to change production to."));
 	    return;
 	}
 	else {
@@ -699,11 +814,11 @@ void city_change_all_dialog_callback(GtkWidget *w, gpointer data)
 					       row);
 	}
 	if (from==to) {
-	    sprintf(buf,_("Game: That's the same thing!"));
-	    append_output_window(buf);
+	    append_output_window(_("Game: That's the same thing!"));
 	    return;
 	}
-	sprintf(buf,_("Game: Changing production of every %s into %s."),
+	my_snprintf(buf, sizeof(buf),
+		    _("Game: Changing production of every %s into %s."),
 		(from >= B_LAST) ?
   		  get_unit_type(from-B_LAST)->name : get_improvement_name(from),
 		(to >= B_LAST) ?
@@ -721,6 +836,7 @@ void city_change_all_dialog_callback(GtkWidget *w, gpointer data)
 Change all cities building one type of thing to another thing.
 This is a callback for the "change all" button.
 *****************************************************************/
+#define MAX_LEN_BUF 256
 void city_change_all_callback(GtkWidget *w, gpointer data)
 {
     GList              *selection;
@@ -768,8 +884,8 @@ void city_change_all_callback(GtkWidget *w, gpointer data)
 					GTK_POLICY_AUTOMATIC );
 	gtk_widget_set_usize(city_change_all_from_list, 200, 200);
 	
-	buf[0] = (char *)g_malloc(256 * sizeof(char));
-	is_building = (int *)g_malloc0((B_LAST+U_LAST) * sizeof(int));
+	buf[0] = g_malloc(MAX_LEN_BUF);
+	is_building = g_malloc0((B_LAST+U_LAST) * sizeof(int));
 
 	city_list_iterate(game.player_ptr->cities, pcity) {
 	    if (pcity->is_building_unit)
@@ -792,7 +908,7 @@ void city_change_all_callback(GtkWidget *w, gpointer data)
 
 	for(i=0; i<B_LAST; i++)
 	    if (is_building[i]) {
-		sprintf(buf[0],"%s",get_improvement_name(i));
+		mystrlcpy(buf[0], get_improvement_name(i), MAX_LEN_BUF);
 		j = gtk_clist_append(GTK_CLIST(city_change_all_from_list), buf);
 		gtk_clist_set_row_data(GTK_CLIST(city_change_all_from_list),
 				       j, (gpointer)i);
@@ -804,7 +920,7 @@ void city_change_all_callback(GtkWidget *w, gpointer data)
 
 	for(i=0; i<U_LAST; i++)
 	    if (is_building[B_LAST+i]) {
-		sprintf(buf[0],"%s",get_unit_name(i));
+		mystrlcpy(buf[0], get_unit_name(i), MAX_LEN_BUF);
 		j = gtk_clist_append(GTK_CLIST(city_change_all_from_list), buf);
 		gtk_clist_set_row_data(GTK_CLIST(city_change_all_from_list),
 				       j, (gpointer)(i+B_LAST));
@@ -837,7 +953,7 @@ void city_change_all_callback(GtkWidget *w, gpointer data)
 
 	for(i=0; i<B_LAST; i++)
 	    if(can_player_build_improvement(game.player_ptr, i)) {
-		sprintf(buf[0], "%s (%d)",
+		my_snprintf(buf[0], MAX_LEN_BUF, "%s (%d)",
 			get_improvement_name(i),
 			get_improvement_type(i)->build_cost);
 		j = gtk_clist_append(GTK_CLIST(city_change_all_to_list), buf);
@@ -847,7 +963,7 @@ void city_change_all_callback(GtkWidget *w, gpointer data)
 
 	for(i=0; i<U_LAST; i++)
 	    if(can_player_build_unit(game.player_ptr, i)) {
-		sprintf(buf[0], "%s (%d)",
+		my_snprintf(buf[0], MAX_LEN_BUF, "%s (%d)",
 			get_unit_name(i),
 			get_unit_type(i)->build_cost);
 		j = gtk_clist_append(GTK_CLIST(city_change_all_to_list), buf);
@@ -892,16 +1008,13 @@ void city_change_all_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 void city_buy_callback(GtkWidget *w, gpointer data)
 {
-  GList              *selection;
-  gint                row;
-  struct city *pcity;
+  GList *current = GTK_CLIST(city_list)->selection;
 
-  while ((selection = GTK_CLIST(city_list)->selection) != NULL)
-  {
-  row = (gint)selection->data;
+  g_assert(current);
 
-  if((pcity=gtk_clist_get_row_data(GTK_CLIST(city_list),row)))
+  for(; current; current = g_list_next(current))
   {
+      struct city *pcity = city_from_glist (current);
       int value;
       char *name;
       char buf[512];
@@ -917,17 +1030,17 @@ void city_buy_callback(GtkWidget *w, gpointer data)
 	  struct packet_city_request packet;
 	  packet.city_id=pcity->id;
 	  packet.name[0]='\0';
+	  packet.worklist.name[0] = '\0';
 	  send_packet_city_request(&aconnection, &packet, PACKET_CITY_BUY);
 	}
       else
 	{
-	  sprintf(buf, _("Game: %s costs %d gold and you only have %d gold."),
-		  name,value,game.player_ptr->economic.gold);
+	  my_snprintf(buf, sizeof(buf),
+		      _("Game: %s costs %d gold and you only have %d gold."),
+		      name,value,game.player_ptr->economic.gold);
 	  append_output_window(buf);
 	}
   }
-    gtk_clist_unselect_row(GTK_CLIST(city_list),row,0);
-  }
 }
 
 /****************************************************************
@@ -947,16 +1060,13 @@ void city_close_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 void city_center_callback(GtkWidget *w, gpointer data)
 {
-  GList              *selection;
-  gint                row;
+  GList *current = GTK_CLIST(city_list)->selection;
   struct city        *pcity;
 
-  if ( !( selection = GTK_CLIST( city_list )->selection ) )
+  if (!current)
       return;
 
-  row = (gint)selection->data;
-
-  if((pcity=gtk_clist_get_row_data(GTK_CLIST(city_list),row)))
+  pcity = city_from_glist (current);
     center_tile_mapcanvas(pcity->x, pcity->y);
 }
 
@@ -965,20 +1075,37 @@ void city_center_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 void city_popup_callback(GtkWidget *w, gpointer data)
 {
-  GList              *selection;
-  gint                row;
+  GList *current = GTK_CLIST(city_list)->selection;
+  GList *copy = NULL;
   struct city        *pcity;
 
-  while((selection = GTK_CLIST(city_list)->selection) != NULL)
-  {
-  row = (gint)selection->data;
+  if (!current)
+    return;
 
-  if((pcity=gtk_clist_get_row_data(GTK_CLIST(city_list),row))){
+  pcity = city_from_glist (current);
+  if (center_when_popup_city) {
     center_tile_mapcanvas(pcity->x, pcity->y);
-    popup_city_dialog(pcity, 0);
-  }
-    gtk_clist_unselect_row(GTK_CLIST(city_list),row,0);
   }
+
+  /* We have to copy the list as the popup_city_dialog destroys the data */
+  for(; current; current = g_list_next(current))
+    copy = g_list_append (copy, city_from_glist(current));
+  
+  for(; copy; copy = g_list_next(copy))
+    popup_city_dialog(copy->data, 0);
+
+  g_list_free(copy);
+}
+
+/****************************************************************
+...
+*****************************************************************/
+
+static void city_report_list_callback(GtkWidget *w, gint row, gint col, GdkEvent *ev, gpointer data)
+{
+  /* Pops up a city when it's double-clicked */
+  if(ev && ev->type==GDK_2BUTTON_PRESS)
+    city_popup_callback(w, data);
 }
 
 /****************************************************************
@@ -994,6 +1121,8 @@ void city_config_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 void city_report_dialog_update(void)
 {
+  GList *selection = NULL;
+  GList *copy = NULL;
   if(city_dialog_shell) {
     char *row	[NUM_CREPORT_COLS];
     char  buf	[NUM_CREPORT_COLS][64];
@@ -1008,33 +1137,33 @@ void city_report_dialog_update(void)
     gtk_set_label(city_label, report_title);
     free(report_title);
 
-    if (city_popupmenu)
-      {
-	gtk_widget_destroy(city_popupmenu);
-	city_popupmenu = 0;
-      }    
-
     for (i=0, spec=city_report_specs;i<NUM_CREPORT_COLS;i++, spec++)
     {
       row[i] = buf[i];
       gtk_clist_set_column_visibility(GTK_CLIST(city_list), i, spec->show);
     }
 
+    for(selection = GTK_CLIST(city_list)->selection; 
+	selection; selection = g_list_next(selection))
+      copy = g_list_append (copy, city_from_glist(selection));
+
     gtk_clist_freeze(GTK_CLIST(city_list));
     gtk_clist_clear(GTK_CLIST(city_list));
 
     city_list_iterate(game.player_ptr->cities, pcity) {
-      get_city_text(pcity,row);
+      get_city_text(pcity, row, sizeof(buf[0]));
       i=gtk_clist_append(GTK_CLIST(city_list), row);
       gtk_clist_set_row_data (GTK_CLIST(city_list), i, pcity);
+      if(g_list_find(copy,pcity))
+	gtk_clist_select_row(GTK_CLIST(city_list), i, -1);
     } city_list_iterate_end;
     gtk_clist_thaw(GTK_CLIST(city_list));
     gtk_widget_show_all(city_list);
 
-    gtk_widget_set_sensitive(city_change_command, FALSE);
-    gtk_widget_set_sensitive(city_center_command, FALSE);
-    gtk_widget_set_sensitive(city_popup_command, FALSE);
-    gtk_widget_set_sensitive(city_buy_command, FALSE);
+    g_list_free(copy);
+
+    /* Set sensitivity right. */
+    city_list_callback(NULL,0,0);
   }
 }
 
@@ -1058,18 +1187,43 @@ void city_report_dialog_update_city(struct city *pcity)
   }
 
   if((i=gtk_clist_find_row_from_data(GTK_CLIST(city_list), pcity))!=-1)  {
+#if 1
+    /* This method avoids removing and re-adding the entry, because
+       that seemed to cause problems in some cases, when the list
+       is sorted by one of the columns.  Doing "Popup" for the top
+       entry when sorted could cause core dumps or memory corruption,
+       and in other cases could change the ordering (confusing) when
+       multiple cities have the same string value in the column being
+       sorted by.  -- dwp
+    */
+    int j;
+    get_city_text(pcity, row, sizeof(buf[0]));
+    gtk_clist_freeze(GTK_CLIST(city_list));
+    for (j=0; j<NUM_CREPORT_COLS; j++) {
+      gtk_clist_set_text(GTK_CLIST(city_list), i, j, row[j]);
+    }
+    gtk_clist_thaw(GTK_CLIST(city_list));
+    gtk_clist_sort(GTK_CLIST(city_list));
+    gtk_widget_show_all(city_list);
+#else
+    /* Old method, see above. */
     char *text;
+    gboolean selected = (gboolean) g_list_find(GTK_CLIST(city_list)->selection,
+					       (gpointer)i);
 
     gtk_clist_get_text(GTK_CLIST(city_list),i,0,&text);
 
-    get_city_text(pcity,row);
+    get_city_text(pcity, row, sizeof(buf[0]));
 
     gtk_clist_freeze(GTK_CLIST(city_list));
     gtk_clist_remove(GTK_CLIST(city_list),i);
     i=gtk_clist_append(GTK_CLIST(city_list),row);
     gtk_clist_set_row_data (GTK_CLIST(city_list), i, pcity);
+    if (selected)
+      gtk_clist_select_row(GTK_CLIST(city_list), i, -1);
     gtk_clist_thaw(GTK_CLIST(city_list));
     gtk_widget_show_all(city_list);
+#endif
   }
   else
     city_report_dialog_update();
