@@ -32,6 +32,7 @@
 #include "mem.h"
 #include "packets.h"
 #include "player.h"
+#include "support.h"
 
 #include "chatline.h"
 #include "civclient.h"
@@ -47,7 +48,6 @@
 extern GtkWidget *toplevel;
 extern GdkWindow *root_window;
 extern struct connection aconnection;
-extern int flags_are_transparent;
 extern int ai_popup_windows;
 extern GdkGC *fill_bg_gc;
 
@@ -74,7 +74,7 @@ GtkWidget *spy_advances_list, *spy_advances_list_label;
 GtkWidget *spy_steal_command;
 
 int spy_tech_shell_is_modal;
-int advance_type[A_LAST];
+int advance_type[A_LAST+1];
 int steal_advance = 0;
 
 /******************************************************************/
@@ -83,7 +83,7 @@ GtkWidget *spy_improvements_list;
 GtkWidget *spy_sabotage_command;
 
 int spy_sabotage_shell_is_modal;
-int improvement_type[B_LAST];
+int improvement_type[B_LAST+1];
 int sabotage_improvement = 0;
 
 /******************************************************************/
@@ -132,6 +132,10 @@ static GtkWidget *caravan_dialog;
 
 extern GtkStyle *notify_dialog_style;
 
+static int is_showing_unit_connect_dialog = FALSE;
+static int unit_to_use_to_connect;
+static int connect_unit_x;
+static int connect_unit_y;
 
 /****************************************************************
 ...
@@ -392,14 +396,17 @@ void popup_bribe_dialog(struct unit *punit)
   char buf[128];
   
   if(game.player_ptr->economic.gold>=punit->bribe_cost) {
-    sprintf(buf, _("Bribe unit for %d gold?\nTreasury contains %d gold."), 
-	   punit->bribe_cost, game.player_ptr->economic.gold);
+    my_snprintf(buf, sizeof(buf),
+		_("Bribe unit for %d gold?\nTreasury contains %d gold."), 
+		punit->bribe_cost, game.player_ptr->economic.gold);
     popup_message_dialog(toplevel, /*"diplomatbribedialog"*/_("Bribe Enemy Unit"), buf,
 			_("Yes"), diplomat_bribe_yes_callback, 0,
 			_("No"), diplomat_bribe_no_callback, 0, 0);
   } else {
-    sprintf(buf, _("Bribing the unit costs %d gold.\nTreasury contains %d gold."), 
-	   punit->bribe_cost, game.player_ptr->economic.gold);
+    my_snprintf(buf, sizeof(buf),
+		_("Bribing the unit costs %d gold.\n"
+		  "Treasury contains %d gold."), 
+		punit->bribe_cost, game.player_ptr->economic.gold);
     popup_message_dialog(toplevel, /*"diplomatnogolddialog"*/_("Traitors Demand Too Much!"), buf,
 			_("Darn"), diplomat_bribe_no_callback, 0, 
 			0);
@@ -701,9 +708,9 @@ static int create_advances_list(struct player *pplayer,
   advance_type[j] = -1;
 
   if (pvictim) { /* you don't want to know what lag can do -- Syela */
-    for(i=A_FIRST; i<game.num_tech_types; i++) {
-      gchar *row[1];
+    gchar *row[1];
 
+    for(i=A_FIRST; i<game.num_tech_types; i++) {
       if(get_invention(pvictim, i)==TECH_KNOWN && 
 	 (get_invention(pplayer, i)==TECH_UNKNOWN || 
 	  get_invention(pplayer, i)==TECH_REACHABLE)) {
@@ -713,6 +720,12 @@ static int create_advances_list(struct player *pplayer,
         advance_type[j++] = i;
       }
     }
+
+    if(j > 0) {
+      row[0] = _("At Spy's Discretion");
+      gtk_clist_append(GTK_CLIST(spy_improvements_list), row);
+      advance_type[j++] = game.num_tech_types;
+    }
   }
 
   if(j == 0) {
@@ -741,6 +754,7 @@ static int create_improvements_list(struct player *pplayer,
 {  
   GtkWidget *close_command, *scrolled;
   int i, j;
+  gchar *row[1];
   static gchar *title_[1] = { N_("Select Improvement to Sabotage") };
   static gchar **title;
   GtkAccelGroup *accel=gtk_accel_group_new();
@@ -789,18 +803,26 @@ static int create_improvements_list(struct player *pplayer,
   gtk_clist_freeze(GTK_CLIST(spy_improvements_list));
 
   j = 0;
+  row[0] = _("City Production");
+  gtk_clist_append(GTK_CLIST(spy_improvements_list), row);
   improvement_type[j++] = -1;
 
   for(i=0; i<B_LAST; i++) {
-    gchar *row[1];
-
     if(i != B_PALACE && pcity->improvements[i] && !is_wonder(i)) {
       row[0] = get_imp_name_ex(pcity, i);
-
       gtk_clist_append(GTK_CLIST(spy_improvements_list), row);
       improvement_type[j++] = i;
     }  
   }
+
+  if(j > 1) {
+    row[0] = _("At Spy's Discretion");
+    gtk_clist_append(GTK_CLIST(spy_improvements_list), row);
+    improvement_type[j++] = B_LAST;
+  } else {
+    improvement_type[0] = B_LAST; /* fake "discretion", since must be production */
+  }
+
   gtk_clist_thaw(GTK_CLIST(spy_improvements_list));
 
   gtk_widget_set_sensitive(spy_sabotage_command, FALSE);
@@ -908,15 +930,18 @@ void popup_incite_dialog(struct city *pcity)
   char buf[128];
 
   if(game.player_ptr->economic.gold>=pcity->incite_revolt_cost) {
-   sprintf(buf, _("Incite a revolt for %d gold?\nTreasury contains %d gold."), 
-	  pcity->incite_revolt_cost, game.player_ptr->economic.gold);
+    my_snprintf(buf, sizeof(buf),
+		_("Incite a revolt for %d gold?\nTreasury contains %d gold."), 
+		pcity->incite_revolt_cost, game.player_ptr->economic.gold);
    diplomat_target_id = pcity->id;
    popup_message_dialog(toplevel, /*"diplomatrevoltdialog"*/_("Incite a Revolt!"), buf,
 		       _("Yes"), diplomat_incite_yes_callback, 0,
 		       _("No"), diplomat_incite_no_callback, 0, 0);
   } else {
-   sprintf(buf, _("Inciting a revolt costs %d gold.\nTreasury contains %d gold."), 
-	     pcity->incite_revolt_cost, game.player_ptr->economic.gold);
+    my_snprintf(buf, sizeof(buf),
+		_("Inciting a revolt costs %d gold.\n"
+		  "Treasury contains %d gold."), 
+		pcity->incite_revolt_cost, game.player_ptr->economic.gold);
    popup_message_dialog(toplevel, /*"diplomatnogolddialog"*/_("Traitors Demand Too Much!"), buf,
 		       _("Darn"), diplomat_incite_no_callback, 0, 
 		       0);
@@ -1089,8 +1114,9 @@ void popup_caravan_dialog(struct unit *punit,
   GtkWidget *b;
   char buf[128];
   
-  sprintf(buf, _("Your caravan from %s reaches the city of %s.\nWhat now?"),
-	  phomecity->name, pdestcity->name);
+  my_snprintf(buf, sizeof(buf),
+	      _("Your caravan from %s reaches the city of %s.\nWhat now?"),
+	      phomecity->name, pdestcity->name);
   
   caravan_city_id=pdestcity->id; /* callbacks need these */
   caravan_unit_id=punit->id;
@@ -1310,6 +1336,103 @@ void popup_pillage_dialog(struct unit *punit, int may_pillage)
   }
 }
 
+/****************************************************************
+handle buttons in unit connect dialog
+*****************************************************************/
+static void unit_connect_callback (GtkWidget *w, gpointer data)
+{
+  struct unit *punit;
+  int activity = (int)data;
+
+  if (!is_showing_unit_connect_dialog) {
+    destroy_message_dialog(w);
+    return;
+  }
+
+  punit = find_unit_by_id(unit_to_use_to_connect);
+
+  if (punit) {
+    if (activity != ACTIVITY_IDLE) {
+      struct packet_unit_connect req;
+      req.activity_type = activity;
+      req.unit_id = punit->id;
+      req.dest_x = connect_unit_x;
+      req.dest_y = connect_unit_y;
+      send_packet_unit_connect(&aconnection, &req);
+    }
+    else {
+      update_unit_info_label(punit);
+    }
+  }
+
+  destroy_message_dialog(w);
+  is_showing_unit_connect_dialog = FALSE;
+}
+
+/****************************************************************
+popup dialog which prompts for activity type (unit connect)
+*****************************************************************/
+void popup_unit_connect_dialog(struct unit *punit, int dest_x, int dest_y)
+{
+  GtkWidget *dshell, *button, *dlabel, *vbox;
+  int activity;
+
+  if (is_showing_unit_connect_dialog) 
+    return;
+
+  is_showing_unit_connect_dialog = TRUE;
+  unit_to_use_to_connect = punit->id;
+  connect_unit_x = dest_x;
+  connect_unit_y = dest_y;
+
+  gtk_widget_set_sensitive (toplevel, FALSE);
+
+  dshell = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_position (GTK_WINDOW(dshell), GTK_WIN_POS_MOUSE);
+
+  gtk_signal_connect (
+    GTK_OBJECT (dshell),
+    "delete_event",
+    GTK_SIGNAL_FUNC (popup_mes_del_callback),
+    (gpointer)toplevel
+    );
+
+  gtk_window_set_title (GTK_WINDOW(dshell), _("Connect"));
+
+  vbox = gtk_vbox_new (0,TRUE);
+  gtk_container_add (GTK_CONTAINER (dshell), vbox);
+
+  gtk_container_border_width (GTK_CONTAINER (vbox), 5);
+
+  dlabel = gtk_label_new (_("Choose unit activity:"));
+  gtk_box_pack_start (GTK_BOX (vbox), dlabel, TRUE, FALSE, 0);
+
+  gtk_object_set_data (GTK_OBJECT (vbox), "parent", (gpointer)toplevel);
+
+  for (activity = ACTIVITY_IDLE + 1; activity < ACTIVITY_LAST; activity++) {
+    if (! can_unit_do_connect (punit, activity)) continue;
+
+    button = gtk_button_new_with_label (get_activity_text (activity));
+    gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, FALSE, 0);
+    gtk_signal_connect (
+      GTK_OBJECT (button),
+      "clicked",
+      GTK_SIGNAL_FUNC (unit_connect_callback),
+      (gpointer)activity
+      );
+  }
+  button = gtk_button_new_with_label (_("Cancel"));
+  gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, FALSE, 0);
+  gtk_signal_connect (
+    GTK_OBJECT (button),
+    "clicked",
+    GTK_SIGNAL_FUNC (unit_connect_callback),
+    (gpointer)ACTIVITY_IDLE
+    );
+
+  gtk_widget_show_all (vbox);
+  gtk_widget_show (dshell);
+}
 
 /****************************************************************
 ...
@@ -1372,7 +1495,7 @@ GtkWidget *popup_message_dialog(GtkWidget *parent, char *dialogname,
   while((name=va_arg(args, char *))) {
     fcb=va_arg(args, void *);
     data=va_arg(args, gpointer);
-    sprintf(button_name, "button%d", i++);
+    my_snprintf(button_name, sizeof(button_name), "button%d", i++);
     
     button=gtk_button_new_with_label(name);
     gtk_box_pack_start( GTK_BOX( vbox ), button, TRUE, FALSE, 0 );
@@ -1506,7 +1629,7 @@ void popup_unit_select_dialog(struct tile *ptile)
 
     pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity);
 
-    sprintf(buffer, "%s(%s)\n%s",
+    my_snprintf(buffer, sizeof(buffer), "%s(%s)\n%s",
 	    punittemp->name, 
 	    pcity ? pcity->name : "",
 	    unit_activity_text(punit));
@@ -1562,8 +1685,6 @@ void popup_unit_select_dialog(struct tile *ptile)
 }
 
 
-
-
 /****************************************************************
 popup the dialog 10% inside the main-window 
 *****************************************************************/
@@ -1588,12 +1709,12 @@ void popdown_races_dialog(void)
   } /* else there is no dialog shell to destroy */
 }
 
-
 /****************************************************************
 ...
 *****************************************************************/
 void create_races_dialog(void)
 {
+  int       per_row = 4;
   int       i, leader_num;
   GSList    *group = NULL;
   GSList    *sgroup = NULL;
@@ -1614,18 +1735,19 @@ void create_races_dialog(void)
 
   /* ------- nation name toggles ------- */
 
-  races_toggles_form = gtk_table_new( 3, (game.nation_count+2)/3, FALSE );
+  races_toggles_form =
+    gtk_table_new( per_row, ((game.playable_nation_count-1)/per_row)+1, FALSE );
   gtk_container_add( GTK_CONTAINER( f ), races_toggles_form );
 
   free(races_toggles);
-  races_toggles = fc_calloc( game.nation_count, sizeof(GtkWidget*) );
+  races_toggles = fc_calloc( game.playable_nation_count, sizeof(GtkWidget*) );
 
-  for(i=0; i<game.nation_count; i++) {
+  for(i=0; i<game.playable_nation_count; i++) {
     races_toggles[i]= gtk_radio_button_new_with_label( group, get_nation_name(i) );
     gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON( races_toggles[i] ), FALSE );
     group = gtk_radio_button_group( GTK_RADIO_BUTTON( races_toggles[i] ) );
     gtk_table_attach_defaults( GTK_TABLE(races_toggles_form), races_toggles[i],
-			i%3,i%3+1,i/3,i/3+1 );
+			i%per_row,i%per_row+1,i/per_row,i/per_row+1 );
   }
 
   /* ------- nation leader combo ------- */
@@ -1685,7 +1807,8 @@ void create_races_dialog(void)
 
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG( races_dialog_shell )->vbox ),
                       fa, FALSE, FALSE, 0 );
-  city_style_toggles_form = gtk_table_new( 3, (b_s_num+2)/3, FALSE );
+  city_style_toggles_form =
+    gtk_table_new( per_row, ((b_s_num-1)/per_row)+1, FALSE );
   gtk_container_add( GTK_CONTAINER( fa ), city_style_toggles_form ); 
 
   for(i=0; i<b_s_num; i++) {
@@ -1693,8 +1816,9 @@ void create_races_dialog(void)
                                            cgroup, city_styles[city_style_idx[i]].name);
       gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON( city_style_toggles[i] ), FALSE );
       cgroup = gtk_radio_button_group( GTK_RADIO_BUTTON( city_style_toggles[i] ) );
-      gtk_table_attach_defaults( GTK_TABLE(city_style_toggles_form), city_style_toggles[i],
-                                 i%3, i%3+1, i/3, i/3+1 );
+      gtk_table_attach_defaults( GTK_TABLE(city_style_toggles_form),
+				 city_style_toggles[i],
+                                 i%per_row, i%per_row+1, i/per_row, i/per_row+1 );
   }
 
   /* ------- OK/Disc/Quit buttons ------- */
@@ -1718,7 +1842,7 @@ void create_races_dialog(void)
 
   /* ------- connect callback functions ------- */
 
-  for(i=0; i<game.nation_count; i++)
+  for(i=0; i<game.playable_nation_count; i++)
 	gtk_signal_connect( GTK_OBJECT( races_toggles[i] ), "toggled",
 	    GTK_SIGNAL_FUNC( races_toggles_callback ), NULL );
 
@@ -1781,7 +1905,7 @@ void races_toggles_set_sensitive(int bits1, int bits2)
 
   mybits=bits1;
 
-  for(i=0; i<game.nation_count && i<32; i++) {
+  for(i=0; i<game.playable_nation_count && i<32; i++) {
     if(mybits&1)
       gtk_widget_set_sensitive( races_toggles[i], FALSE );
     else
@@ -1791,7 +1915,7 @@ void races_toggles_set_sensitive(int bits1, int bits2)
 
   mybits=bits2;
 
-  for(i=32; i<game.nation_count; i++) {
+  for(i=32; i<game.playable_nation_count; i++) {
     if(mybits&1)
       gtk_widget_set_sensitive( races_toggles[i], FALSE );
     else
@@ -1829,7 +1953,7 @@ void races_toggles_callback( GtkWidget *w, gpointer data )
   g_list_free(leader_strings);
   leader_strings = 0;
 
-  for(i=0; i<game.nation_count; i++)
+  for(i=0; i<game.playable_nation_count; i++)
     if(w==races_toggles[i]) {
       leaders = get_nation_leader_names( i, &leader_count);
       for( j=0; j<leader_count; j++) {
@@ -1931,8 +2055,7 @@ void races_buttons_callback( GtkWidget *w, gpointer data )
   packet.nation_no=selected;
   packet.is_male = selected_sex;
   packet.city_style = city_style_idx[selected_style];
-  strncpy(packet.name, (char*)s, MAX_LEN_NAME);
-  packet.name[MAX_LEN_NAME-1]='\0';
+  sz_strlcpy(packet.name, (char*)s);
   
   if(!get_sane_name(packet.name)) {
     append_output_window(_("You must type a legal name."));
