@@ -32,6 +32,7 @@
 #include "map.h"
 #include "mem.h"
 #include "shared.h"
+#include "support.h"
 #include "version.h"
 
 #include "chatline.h"
@@ -68,12 +69,6 @@ extern SPRITE *		intro_gfx_sprite;
 extern SPRITE *		radar_gfx_sprite;
 
 
-extern char     metaserver      [];
-extern char	server_host	[];
-extern char	name		[];
-extern int	server_port;
-static char *	tile_set_name;
-
 extern enum client_states	client_state;
 
 extern int			seconds_to_turndone;
@@ -156,26 +151,13 @@ static gint timer_callback(gpointer data);
 
 
 /**************************************************************************
-  Print the usage information, including one line help on each option,
+  Print extra usage information, including one line help on each option,
   to stderr.
 **************************************************************************/
 static void print_usage(const char *argv0)
 {
-  fprintf(stderr, _("Usage: %s [option ...]\nValid options are:\n"), argv0);
-  fprintf(stderr, _("  -h, --help\t\tPrint a summary of the options.\n"));
-  fprintf(stderr, _("  -l, --log=FILE\tUse FILE as logfile.\n"));
-  fprintf(stderr, _("  -N, --Name=NAME\tUse NAME.\n"));
-  fprintf(stderr, _("  -p, --port=PORT\tConnect to PORT.\n"));
-  fprintf(stderr, _("  -s, --server=SERVER\tConnect to the server SERVER.\n"));
-  fprintf(stderr, _("  -m, --metaserver=METASERVER\tSet metaserver address to METASERVER.\n"));
-#ifdef DEBUG
-  fprintf(stderr, _("  -d, --debug=LEVEL\tSet debug log LEVEL (0,1,2,3,"
-	                                          "or 3:file1,min,max:...)\n"));
-#else
-  fprintf(stderr, _("  -d, --debug=LEVEL\tSet debug log LEVEL (0,1,2).\n"));
-#endif
-  fprintf(stderr, _("  -t, --tiles=DIR\tLook in directory DIR for the tiles.\n"));
-  fprintf(stderr, _("  -v, --version\t\tPrint the version number.\n"));
+  /* add client-specific usage information here */
+  fprintf(stderr, _("Report bugs to <%s>.\n"), BUG_EMAIL_ADDRESS);
 }
 
 /**************************************************************************
@@ -183,59 +165,18 @@ static void print_usage(const char *argv0)
 **************************************************************************/
 static void parse_options(int argc, char **argv)
 {
-  char *logfile = NULL;
-  char *option=NULL; /* really use as a pointer */
-  int loglevel;
   int i;
 
-  /* set default argument values */
-  loglevel=LOG_NORMAL;
-  server_port=DEFAULT_SOCK_PORT;
-  strcpy(server_host, "localhost");
-  strcpy(metaserver, METALIST_ADDR);
-  name[0] = '\0';
-
   i = 1;
   while (i < argc)
   {
     if (is_option("--help", argv[i]))
-      {
+    {
       print_usage(argv[0]);
       exit(0);
-      }
-    else if (is_option("--version",argv[i]))
-      {
-      fprintf(stderr, "%s\n", FREECIV_NAME_VERSION);
-      exit(0);
-      }
-    else if ((option = get_option("--log",argv,&i,argc)) != NULL)
-      logfile = mystrdup(option); /* never free()d */
-    else if ((option = get_option("--name",argv,&i,argc)) != NULL)
-      strcpy(name, option);
-    else if ((option = get_option("--metaserver",argv,&i,argc)) != NULL)
-      strcpy(metaserver,option);
-    else if ((option = get_option("--port",argv,&i,argc)) != NULL)
-      server_port=atoi(option);
-    else if ((option = get_option("--server",argv,&i,argc)) != NULL)
-      strcpy(server_host, option);
-    else if ((option = get_option("--debug",argv,&i,argc)) != NULL)
-    {
-      loglevel=log_parse_level_str(option);
-      if (loglevel==-1) {
-        exit(1);
-      }
     }
-    else if ((option = get_option("--tiles",argv,&i,argc)) != NULL)
-      tile_set_name=option;
     i += 1;
   }
-
-  log_init(logfile, loglevel, NULL);
-
-  /* after log_init: */
-  if (name[0] == '\0') {
-    strcpy(name, user_username());
-  }
 }
 
 
@@ -249,35 +190,39 @@ static gint keyboard_handler(GtkWidget *widget, GdkEventKey *event)
 
     switch (event->keyval)
     {
+    case GDK_Up:
     case GDK_8:
     case GDK_KP_8:
-    case GDK_KP_Up:		key_unit_north();		break;
+    case GDK_KP_Up:		key_move_north();		break;
     case GDK_9:
     case GDK_KP_9:
-    case GDK_KP_Page_Up:	key_unit_north_east();		break;
+    case GDK_KP_Page_Up:	key_move_north_east();		break;
+    case GDK_Right:
     case GDK_6:
     case GDK_KP_6:
-    case GDK_KP_Right:		key_unit_east();		break;
+    case GDK_KP_Right:		key_move_east();		break;
     case GDK_3:
     case GDK_KP_3:
-    case GDK_KP_Page_Down:	key_unit_south_east();		break;
+    case GDK_KP_Page_Down:	key_move_south_east();		break;
+    case GDK_Down:
     case GDK_2:
     case GDK_KP_2:
-    case GDK_KP_Down:		key_unit_south();		break;
+    case GDK_KP_Down:		key_move_south();		break;
     case GDK_1:
     case GDK_KP_1:
-    case GDK_KP_End:		key_unit_south_west();		break;
+    case GDK_KP_End:		key_move_south_west();		break;
+    case GDK_Left:
     case GDK_4:
     case GDK_KP_4:
-    case GDK_KP_Left:		key_unit_west();		break;
+    case GDK_KP_Left:		key_move_west();		break;
     case GDK_7:
     case GDK_KP_7:
-    case GDK_KP_Home:		key_unit_north_west();		break;
+    case GDK_KP_Home:		key_move_north_west();		break;
     case GDK_KP_Begin:
     case GDK_5:
     case GDK_KP_5:		focus_to_next_unit();		break;
 
-    case GDK_Return:		key_end_turn();			break;
+    case GDK_Return:
     case GDK_KP_Enter:		key_end_turn();			break;
 
     case GDK_Escape:		key_cancel_action();		break;
@@ -589,16 +534,12 @@ void ui_main(int argc, char **argv)
 {
   GdkBitmap	      *icon_bitmap;
 
-
   parse_options(argc, argv);
+
   /* GTK withdraw gtk options */
   /* Process GTK arguments */
   gtk_init(&argc, &argv);
 
-  boot_help_texts();	       /* after log_init */
-  
-  tilespec_read_toplevel(tile_set_name); /* get tile sizes etc */
-  
   display_color_type=get_visual();
 
   toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -761,7 +702,9 @@ static gint show_info_popup(GtkWidget *w, GdkEventButton *ev)
     GtkWidget *p;
     char buf[512];
     
-    sprintf(buf, _("%s People\nYear: %s\nGold: %d\nNet Income: %d\nTax:%d Lux:%d Sci:%d\nResearching %s: %d/%d"),
+    my_snprintf(buf, sizeof(buf),
+	    _("%s People\nYear: %s\nGold: %d\nNet Income: %d\n"
+	      "Tax:%d Lux:%d Sci:%d\nResearching %s: %d/%d"),
 	    int_to_text(civ_population(game.player_ptr)),
 	    textyear(game.year),
 	    game.player_ptr->economic.gold,
