@@ -29,7 +29,9 @@
 #include "government.h"		/* government_graphic() */
 #include "map.h"
 #include "player.h"
-#include "shared.h"		/* myusleep() */
+#include "rand.h"
+#include "support.h"
+#include "timing.h"
 
 #include "civclient.h"
 #include "colors.h"
@@ -37,6 +39,7 @@
 #include "graphics.h"
 #include "gui_stuff.h"
 #include "mapctrl.h"
+#include "options.h"
 #include "tilespec.h"
 
 #include "mapview.h"
@@ -52,11 +55,6 @@ extern int nuke_state;
 
 extern GtkWidget *	main_frame_civ_name;
 extern GtkWidget *	main_label_info;
-extern int		use_solid_color_behind_units;
-extern int		flags_are_transparent;
-extern int		ai_manual_turn_done;
-extern int		draw_diagonal_roads;
-extern int		draw_map_grid;
 extern GtkWidget *	econ_label			[10];
 extern GtkWidget *	bulb_label;
 extern GtkWidget *	sun_label;
@@ -131,33 +129,71 @@ extern GdkCursor *nuke_cursor;
 GtkObject *		map_hadj, *map_vadj;
 
 
-
 /**************************************************************************
 ...
 **************************************************************************/
 void decrease_unit_hp_smooth(struct unit *punit0, int hp0, 
 			     struct unit *punit1, int hp1)
 {
+  static struct timer *anim_timer = NULL; 
+  struct unit *losing_unit = (hp0 == 0 ? punit0 : punit1);
+  int i;
+
   set_unit_focus_no_center(punit0);
   set_unit_focus_no_center(punit1);
   
-  do {
+  if (!do_combat_animation) {
+    punit0->hp = hp0;
+    punit1->hp = hp1;
     refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
     refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
-    myusleep(100);
 
-    if(punit0->hp>hp0)
+    return;
+  }
+
+  do {
+    anim_timer = renew_timer_start(anim_timer, TIMER_USER, TIMER_ACTIVE);
+
+    if (punit0->hp > hp0
+	&& myrand((punit0->hp - hp0) + (punit1->hp - hp1)) < punit0->hp - hp0)
       punit0->hp--;
-    if(punit1->hp>hp1)
+    else if (punit1->hp > hp1)
       punit1->hp--;
-  } while(punit0->hp>hp0 || punit1->hp>hp1);
-  
+    else
+      punit0->hp--;
+
+    refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
+    refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
+
+    gdk_flush();
+    usleep_since_timer_start(anim_timer, 10000);
+
+  } while (punit0->hp > hp0 || punit1->hp > hp1);
+
+  for (i = 0; i < num_tiles_explode_unit; i++) {
+    anim_timer = renew_timer_start(anim_timer, TIMER_USER, TIMER_ACTIVE);
+
+    pixmap_put_tile(single_tile_pixmap, 0, 0,
+		    losing_unit->x, losing_unit->y, 0);
+    put_unit_pixmap(losing_unit, single_tile_pixmap, 0, 0);
+    pixmap_put_overlay_tile(single_tile_pixmap, 0, 0, sprites.explode.unit[i]);
+
+    gdk_draw_pixmap(map_canvas->window, civ_gc, single_tile_pixmap,
+		    0, 0,
+		    map_canvas_adjust_x(losing_unit->x) * NORMAL_TILE_WIDTH, 
+		    map_canvas_adjust_y(losing_unit->y) * NORMAL_TILE_HEIGHT,
+		    NORMAL_TILE_WIDTH,
+		    NORMAL_TILE_HEIGHT);
+
+    gdk_flush();
+    usleep_since_timer_start(anim_timer, 20000);
+  }
+
   refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
   refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
 }
 
 
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -240,7 +276,7 @@ void update_timeout_label(void)
 {
   char buffer[512];
 
-  sprintf(buffer, "%d", seconds_to_turndone);
+  my_snprintf(buffer, sizeof(buffer), "%d", seconds_to_turndone);
   gtk_set_label(timeout_label, buffer);
 }
 
@@ -254,7 +290,8 @@ void update_info_label( void )
 
   gtk_frame_set_label( GTK_FRAME( main_frame_civ_name ), get_nation_name(game.player_ptr->nation) );
 
-  sprintf( buffer, _("Population: %s\nYear: %s\nGold %d\nTax: %d Lux: %d Sci: %d"),
+  my_snprintf(buffer, sizeof(buffer),
+	  _("Population: %s\nYear: %s\nGold %d\nTax: %d Lux: %d Sci: %d"),
   	  int_to_text( civ_population( game.player_ptr ) ),
   	  textyear( game.year ),
   	  game.player_ptr->economic.gold,
@@ -297,12 +334,12 @@ void update_unit_info_label(struct unit *punit)
     struct city *pcity;
     pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity);
 
-    sprintf(buffer, "%s %s", 
+    my_snprintf(buffer, sizeof(buffer), "%s %s", 
             get_unit_type(punit->type)->name,
             (punit->veteran) ? _("(veteran)") : "" );
     gtk_frame_set_label( GTK_FRAME(unit_info_frame), buffer);
 
-    sprintf(buffer, "%s\n%s\n%s", 
+    my_snprintf(buffer, sizeof(buffer), "%s\n%s\n%s", 
             (goto_state==punit->id) ? 
             _("Select destination") : unit_activity_text(punit), 
             map_get_tile_info_text(punit->x, punit->y),
@@ -418,7 +455,7 @@ void update_unit_pix_label(struct unit *punit)
 **************************************************************************/
 GdkPixmap *get_thumb_pixmap(int onoff)
 {
-  return sprites.treaty_thumb[!!onoff]->pixmap;
+  return sprites.treaty_thumb[BOOL_VAL(onoff)]->pixmap;
 }
 
 /**************************************************************************
@@ -1297,20 +1334,27 @@ void put_city_workers(struct city *pcity, int color)
   city_map_iterate(i, j)  {
     enum city_tile_type t=get_worker_city(pcity, i, j);
     enum city_tile_type last_t=-1;
-    if(i==2 && j==2) continue;
-    if(t==C_TILE_EMPTY) {
-      if(last_t!=t) gdk_gc_set_stipple(fill_tile_gc,gray25);
-    } else if(t==C_TILE_WORKER) {
-      if(last_t!=t) gdk_gc_set_stipple(fill_tile_gc,gray50);
-    } else continue;
-    last_t=t;
-    gdk_draw_pixmap(map_canvas->window, civ_gc, map_canvas_store,
-	     (x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT, 
-	     (x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT,
-	     NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
-    gdk_draw_rectangle(map_canvas->window, fill_tile_gc, TRUE,
-		  (x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT,
-		  NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+    if(!(i==2 && j==2)) {
+      if(t==C_TILE_EMPTY) {
+	if(last_t!=t) gdk_gc_set_stipple(fill_tile_gc,gray25);
+      } else if(t==C_TILE_WORKER) {
+	if(last_t!=t) gdk_gc_set_stipple(fill_tile_gc,gray50);
+      } else continue;
+      last_t=t;
+      gdk_draw_pixmap(map_canvas->window, civ_gc, map_canvas_store,
+		      (x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT, 
+		      (x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT,
+		      NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+      gdk_draw_rectangle(map_canvas->window, fill_tile_gc, TRUE,
+			 (x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT,
+			 NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+    }
+    if(t==C_TILE_WORKER) {
+      put_city_tile_output(map_canvas->window, x+i-2, y+j-2, 
+			   get_food_tile(i, j, pcity),
+			   get_shields_tile(i, j, pcity), 
+			   get_trade_tile(i, j, pcity) );
+    }
   }
 
   last_pcity=pcity;
@@ -1322,11 +1366,17 @@ void put_city_workers(struct city *pcity, int color)
 **************************************************************************/
 void scrollbar_jump_callback(GtkAdjustment *adj, gpointer hscrollbar)
 {
+  int last_map_view_x0;
+  int last_map_view_y0;
+
   gfloat percent=adj->value;
 
   if(get_client_state()!=CLIENT_GAME_RUNNING_STATE)
      return;
 
+  last_map_view_x0=map_view_x0;
+  last_map_view_y0=map_view_y0;
+
   if(hscrollbar)
     map_view_x0=percent;
   else {
@@ -1336,6 +1386,8 @@ void scrollbar_jump_callback(GtkAdjustment *adj, gpointer hscrollbar)
        map.ysize-map_canvas_store_theight : map_view_y0;
   }
 
-  update_map_canvas(0, 0, map_canvas_store_twidth, map_canvas_store_theight, 1);
-  refresh_overview_viewrect();
+  if(last_map_view_x0!=map_view_x0 || last_map_view_y0!=map_view_y0) {
+    update_map_canvas(0, 0,map_canvas_store_twidth, map_canvas_store_theight,1);
+    refresh_overview_viewrect();
+  }
 }
