@@ -25,7 +25,8 @@
 #include "fcintl.h"
 #include "log.h"
 #include "map.h"
-#include "mem.c"
+#include "mem.h"
+#include "support.h"
 #include "unit.h"
 
 #include "chatline.h"
@@ -47,6 +48,7 @@
 #include "ratesdlg.h"
 #include "repodlgs.h"
 #include "spaceshipdlg.h"
+#include "wldlg.h"
 
 #include "menu.h"
 
@@ -75,6 +77,7 @@ enum MenuID {
   
   MENU_KINGDOM_TAX_RATE,
   MENU_KINGDOM_FIND_CITY,
+  MENU_KINGDOM_WORKLISTS,
   MENU_KINGDOM_REVOLUTION,
 
   MENU_VIEW_SHOW_MAP_GRID,
@@ -87,29 +90,30 @@ enum MenuID {
   MENU_ORDER_TRANSFORM,
   MENU_ORDER_FORTRESS,
   MENU_ORDER_AIRBASE,
-  MENU_ORDER_CITY,
+  MENU_ORDER_BUILD_CITY,
   MENU_ORDER_ROAD,
+  MENU_ORDER_CONNECT,
   MENU_ORDER_POLLUTION,
   MENU_ORDER_FORTIFY,
   MENU_ORDER_SENTRY,
   MENU_ORDER_PILLAGE,
-  MENU_ORDER_EXPLORE,
+  MENU_ORDER_AUTO_EXPLORE,
   MENU_ORDER_HOMECITY,
   MENU_ORDER_WAIT,
   MENU_ORDER_UNLOAD,
-  MENU_ORDER_WAKEUP,
+  MENU_ORDER_WAKEUP_OTHERS,
   MENU_ORDER_GOTO,
   MENU_ORDER_GOTO_CITY,
   MENU_ORDER_DISBAND,
   MENU_ORDER_BUILD_WONDER,
-  MENU_ORDER_TRADE_ROUTE,
+  MENU_ORDER_TRADEROUTE,
   MENU_ORDER_DONE,
   MENU_ORDER_NUKE,
 
   MENU_REPORT_CITY,
   MENU_REPORT_SCIENCE,
   MENU_REPORT_TRADE,
-  MENU_REPORT_ACTIVE_UNITS,
+  MENU_REPORT_MILITARY,
   MENU_REPORT_DEMOGRAPHIC,
   MENU_REPORT_TOP_CITIES,
   MENU_REPORT_WOW,
@@ -130,7 +134,8 @@ enum MenuID {
   MENU_HELP_COPYING,
   MENU_HELP_ABOUT,
   MENU_HELP_CONNECTING,
-  MENU_HELP_CHATLINE
+  MENU_HELP_CHATLINE,
+  MENU_HELP_LANGUAGES
 };
 
 
@@ -191,6 +196,9 @@ static void kingdom_menu_callback(gpointer callback_data,
   case MENU_KINGDOM_FIND_CITY:
     popup_find_dialog();
     break;
+  case MENU_KINGDOM_WORKLISTS:
+    popup_worklists_dialog(game.player_ptr);
+    break;
   case MENU_KINGDOM_REVOLUTION:
     popup_revolution_dialog();
     break;
@@ -206,7 +214,7 @@ static void view_menu_callback(gpointer callback_data, guint callback_action,
 {
   switch(callback_action) {
   case MENU_VIEW_SHOW_MAP_GRID:
-    key_map_grid ();
+    key_map_grid_toggle ();
     break;
   case MENU_VIEW_CENTER_VIEW:
     center_on_unit();
@@ -227,7 +235,7 @@ static void orders_menu_callback(gpointer callback_data,
     if(get_unit_in_focus())
       request_unit_auto(get_unit_in_focus());
     break;
-   case MENU_ORDER_CITY:
+   case MENU_ORDER_BUILD_CITY:
     if(get_unit_in_focus())
       request_unit_build_city(get_unit_in_focus());
     break;
@@ -254,8 +262,15 @@ static void orders_menu_callback(gpointer callback_data,
     if(get_unit_in_focus())
       request_new_unit_activity(get_unit_in_focus(), road_activity);
     break;
+   case MENU_ORDER_CONNECT:
+    if(get_unit_in_focus())
+      request_unit_connect();
+    break;
    case MENU_ORDER_POLLUTION:
-    key_unit_clean_pollution();
+    if(can_unit_paradropped(get_unit_in_focus()))
+      key_unit_paradrop();
+    else
+      key_unit_pollution();
     break;
    case MENU_ORDER_HOMECITY:
     if(get_unit_in_focus())
@@ -277,7 +292,7 @@ static void orders_menu_callback(gpointer callback_data,
     if(get_unit_in_focus())
       request_unit_unload(get_unit_in_focus());
     break;
-   case MENU_ORDER_WAKEUP:
+   case MENU_ORDER_WAKEUP_OTHERS:
     if(get_unit_in_focus())
       request_unit_wakeup(get_unit_in_focus());
     break;
@@ -297,7 +312,7 @@ static void orders_menu_callback(gpointer callback_data,
     if(get_unit_in_focus())
       request_unit_pillage(get_unit_in_focus());
      break;
-   case MENU_ORDER_EXPLORE:
+   case MENU_ORDER_AUTO_EXPLORE:
     if(get_unit_in_focus())
       request_new_unit_activity(get_unit_in_focus(), ACTIVITY_EXPLORE);
      break;
@@ -306,7 +321,7 @@ static void orders_menu_callback(gpointer callback_data,
       request_unit_caravan_action(get_unit_in_focus(),
 				 PACKET_UNIT_HELP_BUILD_WONDER);
      break;
-   case MENU_ORDER_TRADE_ROUTE:
+   case MENU_ORDER_TRADEROUTE:
     if(get_unit_in_focus())
       request_unit_caravan_action(get_unit_in_focus(),
 				 PACKET_UNIT_ESTABLISH_TRADE);
@@ -340,7 +355,7 @@ static void reports_menu_callback(gpointer callback_data,
    case MENU_REPORT_TRADE:
     popup_trade_report_dialog(0);
     break;
-   case MENU_REPORT_ACTIVE_UNITS:
+   case MENU_REPORT_MILITARY:
     popup_activeunits_report_dialog(0);
     break;
   case MENU_REPORT_DEMOGRAPHIC:
@@ -366,6 +381,9 @@ static void help_menu_callback(gpointer callback_data,
 			       guint callback_action, GtkWidget *widget)
 {
   switch(callback_action) {
+  case MENU_HELP_LANGUAGES:
+    popup_help_dialog_string(HELP_LANGUAGES_ITEM);
+    break;
   case MENU_HELP_CONNECTING:
     popup_help_dialog_string(HELP_CONNECTING_ITEM);
     break;
@@ -486,8 +504,10 @@ static GtkItemFactoryEntry menu_items[]	=
     MENU_KINGDOM_TAX_RATE						      },
   { "/" N_("Kingdom") "/sep1",		NULL,		NULL,
     0,					"<Separator>"			      },
-  { "/" N_("Kingdom") "/" N_("Find _City"),	"<shift>c",	kingdom_menu_callback,
+  { "/" N_("Kingdom") "/" N_("_Find City"),	"<control>f",	kingdom_menu_callback,
     MENU_KINGDOM_FIND_CITY							      },
+  { "/" N_("Kingdom") "/" N_("Work_lists"),	"<shift>l",	kingdom_menu_callback,
+    MENU_KINGDOM_WORKLISTS							      },
   { "/" N_("Kingdom") "/sep2",		NULL,		NULL,
     0,					"<Separator>"			      },
   { "/" N_("Kingdom") "/" N_("Revolution"),	NULL,	kingdom_menu_callback,
@@ -507,7 +527,7 @@ static GtkItemFactoryEntry menu_items[]	=
   { "/" N_("Orders") "/tearoff1",		NULL,		NULL,
     0,					"<Tearoff>"			      },
   { "/" N_("Orders") "/" N_("Build City"),	"b",		orders_menu_callback,
-    MENU_ORDER_CITY							      },
+    MENU_ORDER_BUILD_CITY						      },
   { "/" N_("Orders") "/" N_("Build Road"),	"r",		orders_menu_callback,
     MENU_ORDER_ROAD							      },
   { "/" N_("Orders") "/" N_("Build Irrigation"), "i",		orders_menu_callback,
@@ -537,7 +557,7 @@ static GtkItemFactoryEntry menu_items[]	=
   { "/" N_("Orders") "/" N_("Unload"),	"u",		orders_menu_callback,
     MENU_ORDER_UNLOAD							      },
   { "/" N_("Orders") "/" N_("Wake up others"), "<shift>w",	orders_menu_callback,
-    MENU_ORDER_WAKEUP							      },
+    MENU_ORDER_WAKEUP_OTHERS						      },
   { "/" N_("Orders") "/sep3",		NULL,		NULL,
     0,					"<Separator>"			      },
   { "/" N_("Orders") "/" N_("Auto Settler"),	"a",		orders_menu_callback,
@@ -545,7 +565,9 @@ static GtkItemFactoryEntry menu_items[]	=
   { "/" N_("Orders") "/" N_("Auto Attack"),	"<shift>a",	orders_menu_callback,
     MENU_ORDER_AUTO_ATTACK						      },
   { "/" N_("Orders") "/" N_("Auto Explore"),	"x",		orders_menu_callback,
-    MENU_ORDER_EXPLORE							      },
+    MENU_ORDER_AUTO_EXPLORE						      },
+  { "/" N_("Orders") "/" N_("Connect"),	        "<shift>c",	orders_menu_callback,
+    MENU_ORDER_CONNECT							      },
   { "/" N_("Orders") "/" N_("Go to"),		"g",		orders_menu_callback,
     MENU_ORDER_GOTO							      },
   { "/" N_("Orders") "/" N_("Go|Airlift to City"), "l",	orders_menu_callback,
@@ -557,7 +579,7 @@ static GtkItemFactoryEntry menu_items[]	=
   { "/" N_("Orders") "/" N_("Help Build Wonder"), "<shift>b",	orders_menu_callback,
     MENU_ORDER_BUILD_WONDER						      },
   { "/" N_("Orders") "/" N_("Make Trade Route"), "<shift>r",	orders_menu_callback,
-    MENU_ORDER_TRADE_ROUTE						      },
+    MENU_ORDER_TRADEROUTE						      },
   { "/" N_("Orders") "/" N_("Explode Nuclear"), "<shift>n",	orders_menu_callback,
     MENU_ORDER_NUKE							      },
   { "/" N_("Orders") "/sep5",		NULL,		NULL,
@@ -574,7 +596,7 @@ static GtkItemFactoryEntry menu_items[]	=
   { "/" N_("Reports") "/" N_("City Report"),	"F1",		reports_menu_callback,
     MENU_REPORT_CITY							      },
   { "/" N_("Reports") "/" N_("Military Report"), "F2",	reports_menu_callback,
-    MENU_REPORT_ACTIVE_UNITS						      },
+    MENU_REPORT_MILITARY						      },
   { "/" N_("Reports") "/" N_("Trade Report"),	  "F5",		reports_menu_callback,
     MENU_REPORT_TRADE							      },
   { "/" N_("Reports") "/" N_("Science Report"), "F6",		reports_menu_callback,
@@ -594,6 +616,8 @@ static GtkItemFactoryEntry menu_items[]	=
     0,					"<LastBranch>"			      },
   { "/" N_("Help") "/tearoff1",		NULL,		NULL,
     0,					"<Tearoff>"			      },
+  { "/" N_("Help") "/" N_("Help Languages"),	NULL,		help_menu_callback,
+    MENU_HELP_LANGUAGES							      },
   { "/" N_("Help") "/" N_("Help Connecting"),	NULL,		help_menu_callback,
     MENU_HELP_CONNECTING						      },
   { "/" N_("Help") "/" N_("Help Controls"),	NULL,		help_menu_callback,
@@ -644,7 +668,8 @@ static const char *translate_menu_path(const char *path)
   return path;
 #else
   static struct astring in, out, tmp;   /* these are never free'd */
-  char *tok, *s, *t;
+  char *tok, *s, *trn, *t;
+  int len;
 
   /* copy to in so can modify with strtok: */
   astr_minsize(&in, strlen(path)+1);
@@ -655,13 +680,15 @@ static const char *translate_menu_path(const char *path)
 
   s = in.str;
   while ((tok=strtok(s, "/")) != NULL) {
-    int len = strlen(tok);
+    len = strlen(tok);
     freelog(LOG_DEBUG, "tok \"%s\", len %d", tok, len);
     if (len && tok[0] == '<' && tok[len-1] == '>') {
       t = tok;
     } else {
-      astr_minsize(&tmp, len+2);
-      sprintf(tmp.str, "/%s", _(tok));
+      trn = _(tok);
+      len = strlen(trn) + 1;	/* string plus leading '/' */
+      astr_minsize(&tmp, len+1);
+      sprintf(tmp.str, "/%s", trn);
       t = tmp.str;
       len = strlen(t);
     }
@@ -818,9 +845,9 @@ void update_menus(void)
       enum tile_terrain_type  ttype;
       struct tile_type *      tinfo;
 
-      strcpy(irrtext, _("Build Irrigation"));
-      strcpy(mintext, _("Build Mine"));
-      strcpy(transtext, _("Transform Terrain"));
+      sz_strlcpy(irrtext, _("Build Irrigation"));
+      sz_strlcpy(mintext, _("Build Mine"));
+      sz_strlcpy(transtext, _("Transform Terrain"));
       
       menus_set_sensitive("<main>/Orders/Auto Settler",
 			  (can_unit_do_auto(punit)
@@ -840,6 +867,8 @@ void update_menus(void)
       menus_set_sensitive("<main>/Orders/Build Road",
 			   can_unit_do_activity(punit, ACTIVITY_ROAD) ||
 			   can_unit_do_activity(punit, ACTIVITY_RAILROAD));
+      menus_set_sensitive("<main>/Orders/Connect",
+			  can_unit_do_connect(punit, ACTIVITY_IDLE));
       menus_set_sensitive("<main>/Orders/Clean Pollution",
 			   can_unit_do_activity(punit, ACTIVITY_POLLUTION) ||
 			   can_unit_paradropped(punit));
@@ -871,7 +900,7 @@ void update_menus(void)
       menus_set_sensitive("<main>/Orders/Make Trade Route",
 			   unit_can_est_traderoute_here(punit));
 
-      if (unit_flag(punit->type, F_SETTLERS)
+      if (unit_flag(punit->type, F_CITIES)
 	 && map_get_city(punit->x, punit->y)) {
        menus_rename("<main>/Orders/Build City", _("Add to City"));
       } else {
@@ -882,22 +911,22 @@ void update_menus(void)
       tinfo = get_tile_type(ttype);
       if ((tinfo->irrigation_result != T_LAST) && (tinfo->irrigation_result != ttype))
 	{
-	  sprintf (irrtext, chgfmt,
+	  my_snprintf (irrtext, sizeof(irrtext), chgfmt,
 		   (get_tile_type(tinfo->irrigation_result))->terrain_name);
 	}
       else if ((map_get_tile(punit->x,punit->y)->special&S_IRRIGATION) &&
-	       improvement_exists(B_SUPERMARKET))
+	       player_knows_techs_with_flag(game.player_ptr, TF_FARMLAND))
 	{
-	  strcpy (irrtext, _("Build Farmland"));
+	  sz_strlcpy (irrtext, _("Build Farmland"));
 	}
       if ((tinfo->mining_result != T_LAST) && (tinfo->mining_result != ttype))
 	{
-	  sprintf (mintext, chgfmt,
+	  my_snprintf (mintext, sizeof(mintext), chgfmt,
 		   (get_tile_type(tinfo->mining_result))->terrain_name);
 	}
       if ((tinfo->transform_result != T_LAST) && (tinfo->transform_result != ttype))
 	{
-	  sprintf (transtext, transfmt,
+	  my_snprintf (transtext, sizeof(transtext), transfmt,
 		   (get_tile_type(tinfo->transform_result))->terrain_name);
 	}
 
