@@ -23,7 +23,9 @@
 #include <X11/Xaw/Form.h>
 #include <X11/Xaw/Label.h>
 #include <X11/Xaw/Command.h>
+#include <X11/Xaw/MenuButton.h>
 #include <X11/Xaw/SimpleMenu.h>
+#include <X11/Xaw/SmeBSB.h>
 #include <X11/Xaw/Command.h>
 #include <X11/Xaw/List.h>
 #include <X11/Xaw/Viewport.h>
@@ -43,6 +45,7 @@
 #include "packets.h"
 #include "player.h"
 #include "shared.h"
+#include "support.h"
 
 #include "cityrep.h"
 #include "colors.h"
@@ -55,8 +58,10 @@
 #include "mapctrl.h"
 #include "mapview.h"
 #include "optiondlg.h"		/* for toggle_callback */
+#include "options.h"
 #include "repodlgs.h"
 #include "tilespec.h"
+#include "wldlg.h"
 
 #include "citydlg.h"
 
@@ -67,7 +72,6 @@ extern Widget toplevel, main_form, map_canvas;
 extern int display_depth;
 extern struct connection aconnection;
 extern int map_view_x0, map_view_y0;
-extern int flags_are_transparent;
 extern GC fill_bg_gc;
 
 
@@ -98,7 +102,8 @@ struct city_dialog {
   Widget sell_command;
   Widget close_command, rename_command, trade_command, activate_command;
   Widget show_units_command, cityopt_command;
-  Widget building_label, progress_label, buy_command, change_command;
+  Widget building_label, progress_label, buy_command, change_command,
+    worklist_command, worklist_label;
   Widget improvement_viewport, improvement_list;
   Widget support_unit_label;
   Widget *support_unit_pixcomms;
@@ -110,6 +115,7 @@ struct city_dialog {
   Widget present_unit_prev_command;
   Widget change_list;
   Widget rename_input;
+  Widget worklist_shell;
   
   enum improvement_type_id sell_id;
   
@@ -150,6 +156,9 @@ static void city_dialog_update_pollution(struct city_dialog *pdialog);
 static void sell_callback(Widget w, XtPointer client_data, XtPointer call_data);
 static void buy_callback(Widget w, XtPointer client_data, XtPointer call_data);
 static void change_callback(Widget w, XtPointer client_data, XtPointer call_data);
+static void worklist_callback(Widget w, XtPointer client_data, XtPointer call_data);
+void commit_city_worklist(struct worklist *pwl, void *data);
+void cancel_city_worklist(void *data);
 static void close_callback(Widget w, XtPointer client_data, XtPointer call_data);
 static void rename_callback(Widget w, XtPointer client_data, XtPointer call_data);
 static void trade_callback(Widget w, XtPointer client_data, XtPointer call_data);
@@ -181,7 +190,7 @@ extern Atom wm_delete_window;
 ...
 *****************************************************************/
 static void get_contents_of_pollution(struct city_dialog *pdialog,
-				      char *retbuf)
+				      char *retbuf, int n)
 {
   struct city *pcity;
   int pollution=0;
@@ -191,16 +200,14 @@ static void get_contents_of_pollution(struct city_dialog *pdialog,
     pollution=pcity->pollution;
   }
 
-  sprintf(retbuf,
-	  _("Pollution:    %3d"),
-	  pollution);
+  my_snprintf(retbuf, n, _("Pollution:    %3d"), pollution);
 }
 
 /****************************************************************
 ...
 *****************************************************************/
 static void get_contents_of_storage(struct city_dialog *pdialog,
-				    char *retbuf)
+				    char *retbuf, int n)
 {
   struct city *pcity;
   char granary='?';
@@ -214,18 +221,15 @@ static void get_contents_of_storage(struct city_dialog *pdialog,
     foodbox=game.foodbox*(pcity->size+1);
   }
 
-  sprintf(retbuf,
-	  _("Granary: %c%3d/%-3d"),
-	  granary,
-	  foodstock,
-	  foodbox);
+  my_snprintf(retbuf, n, _("Granary: %c%3d/%-3d"),
+	      granary, foodstock, foodbox);
 }
 
 /****************************************************************
 ...
 *****************************************************************/
 static void get_contents_of_production(struct city_dialog *pdialog,
-				       char *retbuf)
+				       char *retbuf, int n)
 {
   struct city *pcity;
   int foodprod=0;
@@ -245,7 +249,7 @@ static void get_contents_of_production(struct city_dialog *pdialog,
     tradesurplus=pcity->trade_prod;
   }
 
-  sprintf(retbuf,
+  my_snprintf(retbuf, n,
 	  _("Food:  %3d (%+-4d)\n"
 	    "Prod:  %3d (%+-4d)\n"
 	    "Trade: %3d (%+-4d)"),
@@ -258,7 +262,7 @@ static void get_contents_of_production(struct city_dialog *pdialog,
 ...
 *****************************************************************/
 static void get_contents_of_output(struct city_dialog *pdialog,
-				   char *retbuf)
+				   char *retbuf, int n)
 {
   struct city *pcity;
   int goldtotal=0;
@@ -274,7 +278,7 @@ static void get_contents_of_output(struct city_dialog *pdialog,
     scitotal=pcity->science_total;
   }
 
-  sprintf(retbuf,
+  my_snprintf(retbuf, n, 
 	  _("Gold:  %3d (%+-4d)\n"
 	    "Lux:   %3d\n"
 	    "Sci:   %3d"),
@@ -283,6 +287,58 @@ static void get_contents_of_output(struct city_dialog *pdialog,
 	  scitotal);
 }
 
+/****************************************************************
+...
+*****************************************************************/
+static void get_contents_of_progress(struct city_dialog *pdialog,
+				     char *retbuf, int n)
+{
+  struct city *pcity = pdialog ? pdialog->pcity : NULL;
+  int stock=0;
+  int cost=0;
+  int turns=0;
+
+  if (pcity) {
+    stock = pcity->shield_stock;
+    if(pcity->is_building_unit) {
+      cost = get_unit_type(pcity->currently_building)->build_cost;
+      turns = city_turns_to_build(pcity, pcity->currently_building, TRUE);
+    } else {
+      cost = get_improvement_type(pcity->currently_building)->build_cost;
+      turns = city_turns_to_build(pcity, pcity->currently_building, FALSE);
+    }
+  }
+
+  if (pcity && (pcity->currently_building==B_CAPITAL)) {
+    /* Capitalization is special, you can't buy it or finish making it */
+    my_snprintf(retbuf, n,
+		concise_city_production ? " %3d/XXX:XXX " :
+		  _(" %3d/XXX XXX turns "),
+		stock);
+  } else {
+    my_snprintf(retbuf, n,
+		concise_city_production ? " %3d/%3d:%3d " :
+		  turns == 1 ? _(" %3d/%3d %3d turn ") :
+		    _(" %3d/%3d %3d turns "),
+		stock, cost, turns);
+  }
+}
+
+/****************************************************************
+...
+*****************************************************************/
+static void get_contents_of_worklist(struct city_dialog *pdialog,
+				     char *retbuf, int n)
+{
+  struct city *pcity = pdialog ? pdialog->pcity : NULL;
+
+  if (pcity && worklist_is_empty(pcity->worklist)) {
+    mystrlcpy(retbuf, _("(is empty)"), n);
+  } else {
+    mystrlcpy(retbuf, _("(in prog.)"), n);
+  }
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -342,6 +398,7 @@ void refresh_city_dialog(struct city *pcity)
       /* Set the buttons we do not want live while a Diplomat investigates */
       XtSetSensitive(pdialog->buy_command, FALSE);
       XtSetSensitive(pdialog->change_command, FALSE);
+      XtSetSensitive(pdialog->worklist_command, FALSE);
       XtSetSensitive(pdialog->sell_command, FALSE);
       XtSetSensitive(pdialog->rename_command, FALSE);
       XtSetSensitive(pdialog->activate_command, FALSE);
@@ -430,8 +487,8 @@ static void city_map_canvas_expose(Widget w, XEvent *event, Region exposed,
 
 struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 {
-  static char *dummy_improvement_list[]={ 
-    "XXXXXXXXXXXXXXXXXXXXXXXXXXX",
+  char *dummy_improvement_list[]={ 
+    "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",
     "2",
     "3",
     "4",
@@ -459,12 +516,15 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 
   if (NORMAL_TILE_HEIGHT<45) dummy_improvement_list[5]=0;
 
+  if (concise_city_production) {
+    dummy_improvement_list[0] = "XXXXXXXXXXXXXXXXXXXXXXXXXXXX";
+  }
 
   pdialog=fc_malloc(sizeof(struct city_dialog));
   pdialog->pcity=pcity;
   pdialog->support_unit_base=0;
   pdialog->present_unit_base=0;
-
+  pdialog->worklist_shell = NULL;
 
   if(!icon_pixmap)
     icon_pixmap=
@@ -520,7 +580,7 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 			    pdialog->sub_form, 
 			    NULL);
 
-  get_contents_of_production(NULL, lblbuf);
+  get_contents_of_production(NULL, lblbuf, sizeof(lblbuf));
   pdialog->production_label=
     XtVaCreateManagedWidget("cityprodlabel", 
 			    labelWidgetClass,
@@ -528,7 +588,7 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 			    XtNlabel, lblbuf,
 			    NULL);
 
-  get_contents_of_output(NULL, lblbuf);
+  get_contents_of_output(NULL, lblbuf, sizeof(lblbuf));
   pdialog->output_label=
     XtVaCreateManagedWidget("cityoutputlabel", 
 			    labelWidgetClass,
@@ -538,7 +598,7 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 			    (XtArgVal)pdialog->production_label,
 			    NULL);
 
-  get_contents_of_storage(NULL, lblbuf);
+  get_contents_of_storage(NULL, lblbuf, sizeof(lblbuf));
   pdialog->storage_label=
     XtVaCreateManagedWidget("citystoragelabel", 
 			    labelWidgetClass,
@@ -548,7 +608,7 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 			    (XtArgVal)pdialog->output_label,
 			    NULL);
 
-  get_contents_of_pollution(NULL, lblbuf);
+  get_contents_of_pollution(NULL, lblbuf, sizeof(lblbuf));
   pdialog->pollution_label=
     XtVaCreateManagedWidget("citypollutionlabel", 
 			    labelWidgetClass,
@@ -570,15 +630,20 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 			    XtNheight, NORMAL_TILE_HEIGHT*5,
 			    NULL);
 
-  
+
   pdialog->building_label=
     XtVaCreateManagedWidget("citybuildinglabel",
 			    labelWidgetClass,
 			    pdialog->sub_form,
 			    XtNfromHoriz, 
 			    (XtArgVal)pdialog->map_canvas,
+			    XtNlabel,
+			    concise_city_production
+				? "XXXXXXXXXXXXXXXXXXXXXXXXXXXX"
+				: "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",
 			    NULL);
-  
+
+  get_contents_of_progress(NULL, lblbuf, sizeof(lblbuf));
   pdialog->progress_label=
     XtVaCreateManagedWidget("cityprogresslabel",
 			    labelWidgetClass,
@@ -587,17 +652,18 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 			    (XtArgVal)pdialog->map_canvas,
 			    XtNfromVert, 
 			    pdialog->building_label,
+			    XtNlabel, lblbuf,
 			    NULL);
 
   pdialog->buy_command=
     I_L(XtVaCreateManagedWidget("citybuycommand", 
-			    commandWidgetClass,
-			    pdialog->sub_form,
-			    XtNfromVert, 
-			    pdialog->building_label,
-			    XtNfromHoriz, 
-			    (XtArgVal)pdialog->progress_label,
-			    NULL));
+				commandWidgetClass,
+				pdialog->sub_form,
+				XtNfromVert, 
+				pdialog->building_label,
+				XtNfromHoriz, 
+				pdialog->progress_label,
+				NULL));
 
   pdialog->change_command=
     I_L(XtVaCreateManagedWidget("citychangecommand", 
@@ -606,7 +672,7 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 			    XtNfromVert, 
 			    pdialog->building_label,
 			    XtNfromHoriz, 
-			    (XtArgVal)pdialog->buy_command,
+			    pdialog->buy_command,
 			    NULL));
  
   pdialog->improvement_viewport=
@@ -616,7 +682,7 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 			    XtNfromHoriz, 
 			    (XtArgVal)pdialog->map_canvas,
 			    XtNfromVert, 
-			    pdialog->progress_label,
+			    pdialog->change_command,
 			    NULL);
 
   pdialog->improvement_list=
@@ -640,6 +706,28 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 			    (XtArgVal)pdialog->map_canvas,
 			    NULL));
 
+  pdialog->worklist_command=
+    I_L(XtVaCreateManagedWidget("cityworklistcommand", 
+			    commandWidgetClass,
+			    pdialog->sub_form,
+			    XtNfromVert, 
+			    pdialog->improvement_viewport,
+			    XtNfromHoriz, 
+			    pdialog->sell_command,
+			    NULL));
+
+  get_contents_of_worklist(NULL, lblbuf, sizeof(lblbuf));
+  pdialog->worklist_label=
+    XtVaCreateManagedWidget("cityworklistlabel",
+			    labelWidgetClass,
+			    pdialog->sub_form,
+			    XtNfromVert,
+			    pdialog->improvement_viewport,
+			    XtNfromHoriz,
+			    pdialog->worklist_command,
+			    XtNlabel, lblbuf,
+			    NULL);
+
 
   pdialog->support_unit_label=
     I_L(XtVaCreateManagedWidget("supportunitlabel",
@@ -942,6 +1030,9 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
   XtAddCallback(pdialog->change_command, XtNcallback, change_callback,
 		(XtPointer)pdialog);
 
+  XtAddCallback(pdialog->worklist_command, XtNcallback, worklist_callback,
+		(XtPointer)pdialog);
+
   XtAddCallback(pdialog->close_command, XtNcallback, close_callback,
 		(XtPointer)pdialog);
 
@@ -989,7 +1080,7 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 
   XSetWMProtocols(display, XtWindow(pdialog->shell), &wm_delete_window, 1);
   XtOverrideTranslations(pdialog->shell, 
-    XtParseTranslationTable ("<Message>WM_PROTOCOLS: close-citydialog()"));
+    XtParseTranslationTable ("<Message>WM_PROTOCOLS: msg-close-city()"));
 
   XtSetKeyboardFocus(pdialog->shell, pdialog->close_command);
 
@@ -1255,8 +1346,8 @@ static void rename_city_callback(Widget w, XtPointer client_data,
 
   if((pdialog=(struct city_dialog *)client_data)) {
     packet.city_id=pdialog->pcity->id;
-    strncpy(packet.name, input_dialog_get_input(w), MAX_LEN_NAME);
-    packet.name[MAX_LEN_NAME-1]='\0';
+    packet.worklist.name[0] = '\0';
+    sz_strlcpy(packet.name, input_dialog_get_input(w));
     send_packet_city_request(&aconnection, &packet, PACKET_CITY_RENAME);
   }
   input_dialog_destroy(w);
@@ -1299,14 +1390,16 @@ void trade_callback(Widget w, XtPointer client_data, XtPointer call_data)
 {
   int i;
   int x=0,total=0;
-  char buf[512],*bptr=buf;
+  char buf[512], *bptr=buf;
+  int nleft = sizeof(buf);
   struct city_dialog *pdialog;
 
   pdialog=(struct city_dialog *)client_data;
 
-  sprintf(buf, _("These trade routes have been established with %s:\n"),
-	  pdialog->pcity->name);
-  bptr += strlen(bptr);
+  my_snprintf(buf, sizeof(buf),
+	      _("These trade routes have been established with %s:\n"),
+	      pdialog->pcity->name);
+  bptr = end_of_strn(bptr, &nleft);
   
   for(i=0; i<4; i++)
     if(pdialog->pcity->trade[i]) {
@@ -1314,19 +1407,20 @@ void trade_callback(Widget w, XtPointer client_data, XtPointer call_data)
       x=1;
       total+=pdialog->pcity->trade_value[i];
       if((pcity=find_city_by_id(pdialog->pcity->trade[i]))) {
-       sprintf(bptr, _("%32s: %2d Trade/Year\n"),
-               pcity->name, pdialog->pcity->trade_value[i]);
-       bptr += strlen(bptr);
+	my_snprintf(bptr, nleft, _("%32s: %2d Trade/Year\n"),
+		    pcity->name, pdialog->pcity->trade_value[i]);
+	bptr = end_of_strn(bptr, &nleft);
       } else {	
-       sprintf(bptr, _("%32s: %2d Trade/Year\n"), _("Unknown"),
-               pdialog->pcity->trade_value[i]);
-       bptr += strlen(bptr);
+	my_snprintf(bptr, nleft, _("%32s: %2d Trade/Year\n"), _("Unknown"),
+		    pdialog->pcity->trade_value[i]);
+	bptr = end_of_strn(bptr, &nleft);
       }
     }
-  if (!x)
-    sprintf(bptr, _("No trade routes exist.\n"));
-  else
-    sprintf(bptr, _("\nTotal trade %d Trade/Year\n"), total);
+  if (!x) {
+    mystrlcpy(bptr, _("No trade routes exist.\n"), nleft);
+  } else {
+    my_snprintf(bptr, nleft, _("\nTotal trade %d Trade/Year\n"), total);
+  }
   
   popup_message_dialog(pdialog->shell, 
 		       "citytradedialog", 
@@ -1343,7 +1437,7 @@ void city_dialog_update_pollution(struct city_dialog *pdialog)
 {
   char buf[512];
 
-  get_contents_of_pollution(pdialog, buf);
+  get_contents_of_pollution(pdialog, buf, sizeof(buf));
   xaw_set_label(pdialog->pollution_label, buf);
 }
 
@@ -1354,7 +1448,7 @@ void city_dialog_update_storage(struct city_dialog *pdialog)
 {
   char buf[512];
 
-  get_contents_of_storage(pdialog, buf);
+  get_contents_of_storage(pdialog, buf, sizeof(buf));
   xaw_set_label(pdialog->storage_label, buf);
 }
 
@@ -1363,31 +1457,25 @@ void city_dialog_update_storage(struct city_dialog *pdialog)
 *****************************************************************/
 void city_dialog_update_building(struct city_dialog *pdialog)
 {
-  char buf[32], buf2[64];
+  char buf[32];
   struct city *pcity=pdialog->pcity;
-  
-  XtSetSensitive(pdialog->buy_command, !pcity->did_buy);
+
+  if (pcity->currently_building==B_CAPITAL)
+    XtSetSensitive(pdialog->buy_command, False);
+  else
+    XtSetSensitive(pdialog->buy_command, !pcity->did_buy);
   XtSetSensitive(pdialog->sell_command, !pcity->did_sell);
 
-  if(pcity->is_building_unit) {
-    sprintf(buf, "%3d/%3d", pcity->shield_stock, 
-	    get_unit_type(pcity->currently_building)->build_cost);
-    strcpy(buf2, get_unit_type(pcity->currently_building)->name);
-  }
-  else {
-    if(pcity->currently_building==B_CAPITAL)  {
-      /* Capitalization is special, you can't buy it or finish making it */
-      sprintf(buf,"%3d/XXX", pcity->shield_stock);
-      XtSetSensitive(pdialog->buy_command, False);
-    } else {
-      sprintf(buf, "%3d/%3d", pcity->shield_stock, 
-	      get_improvement_type(pcity->currently_building)->build_cost);
-    }
-    strcpy(buf2, get_imp_name_ex(pcity, pcity->currently_building));
-  }
-    
-  xaw_set_label(pdialog->building_label, buf2);
+  xaw_set_label(pdialog->building_label,
+		pcity->is_building_unit ?
+		  get_unit_type(pcity->currently_building)->name :
+		  get_imp_name_ex(pcity, pcity->currently_building));
+
+  get_contents_of_progress(pdialog, buf, sizeof(buf));
   xaw_set_label(pdialog->progress_label, buf);
+
+  get_contents_of_worklist(pdialog, buf, sizeof(buf));
+  xaw_set_label(pdialog->worklist_label, buf);
 }
 
 /****************************************************************
@@ -1397,7 +1485,7 @@ void city_dialog_update_production(struct city_dialog *pdialog)
 {
   char buf[512];
 
-  get_contents_of_production(pdialog, buf);
+  get_contents_of_production(pdialog, buf, sizeof(buf));
   xaw_set_label(pdialog->production_label, buf);
 }
 
@@ -1408,7 +1496,7 @@ void city_dialog_update_output(struct city_dialog *pdialog)
 {
   char buf[512];
 
-  get_contents_of_output(pdialog, buf);
+  get_contents_of_output(pdialog, buf, sizeof(buf));
   xaw_set_label(pdialog->output_label, buf);
 }
 
@@ -1720,7 +1808,7 @@ void city_dialog_update_title(struct city_dialog *pdialog)
   char buf[512];
   String now;
   
-  sprintf(buf, _("%s - %s citizens"),
+  my_snprintf(buf, sizeof(buf), _("%s - %s citizens"),
 	  pdialog->pcity->name, int_to_text(city_population(pdialog->pcity)));
 
   XtVaGetValues(pdialog->cityname_label, XtNlabel, &now, NULL);
@@ -1742,7 +1830,8 @@ void city_dialog_update_improvement_list(struct city_dialog *pdialog)
       if(!pdialog->improvlist_names_ptrs[n] ||
 	 strcmp(pdialog->improvlist_names_ptrs[n], get_imp_name_ex(pdialog->pcity, i)))
 	flag=1;
-      strcpy(pdialog->improvlist_names[n], get_imp_name_ex(pdialog->pcity, i));
+      sz_strlcpy(pdialog->improvlist_names[n],
+		 get_imp_name_ex(pdialog->pcity, i));
       pdialog->improvlist_names_ptrs[n]=pdialog->improvlist_names[n];
       n++;
     }
@@ -1766,7 +1855,7 @@ void city_dialog_update_improvement_list(struct city_dialog *pdialog)
 /**************************************************************************
 ...
 **************************************************************************/
-void button_down_citymap(Widget w, XEvent *event, String *argv, Cardinal *argc)
+void citydlg_btn_select_citymap(Widget w, XEvent *event)
 {
   XButtonEvent *ev=&event->xbutton;
   struct genlist_iterator myiter;
@@ -1788,6 +1877,7 @@ void button_down_citymap(Widget w, XEvent *event, String *argv, Cardinal *argc)
     packet.worker_x=xtile;
     packet.worker_y=ytile;
     packet.name[0]='\0';
+    packet.worklist.name[0] = '\0';
     
     if(pcity->city_map[xtile][ytile]==C_TILE_WORKER)
       send_packet_city_request(&aconnection, &packet, 
@@ -1809,6 +1899,7 @@ void elvis_callback(Widget w, XtPointer client_data, XtPointer call_data)
 
   packet.city_id=pdialog->pcity->id;
   packet.name[0]='\0';
+  packet.worklist.name[0] = '\0';
   packet.specialist_from=SP_ELVIS;
   packet.specialist_to=SP_SCIENTIST;
   
@@ -1828,6 +1919,7 @@ void scientist_callback(Widget w, XtPointer client_data, XtPointer call_data)
 
   packet.city_id=pdialog->pcity->id;
   packet.name[0]='\0';
+  packet.worklist.name[0] = '\0';
   packet.specialist_from=SP_SCIENTIST;
   packet.specialist_to=SP_TAXMAN;
   
@@ -1847,6 +1939,7 @@ void taxman_callback(Widget w, XtPointer client_data, XtPointer call_data)
 
   packet.city_id=pdialog->pcity->id;
   packet.name[0]='\0';
+  packet.worklist.name[0] = '\0';
   packet.specialist_from=SP_TAXMAN;
   packet.specialist_to=SP_ELVIS;
   
@@ -1867,6 +1960,7 @@ static void buy_callback_yes(Widget w, XtPointer client_data,
 
   packet.city_id=pdialog->pcity->id;
   packet.name[0]='\0';
+  packet.worklist.name[0] = '\0';
   send_packet_city_request(&aconnection, &packet, PACKET_CITY_BUY);
 
   destroy_message_dialog(w);
@@ -1904,16 +1998,18 @@ void buy_callback(Widget w, XtPointer client_data, XtPointer call_data)
   value=city_buy_cost(pdialog->pcity);
  
   if(game.player_ptr->economic.gold>=value) {
-    sprintf(buf, _("Buy %s for %d gold?\nTreasury contains %d gold."), 
-	    name, value, game.player_ptr->economic.gold);
+    my_snprintf(buf, sizeof(buf),
+		_("Buy %s for %d gold?\nTreasury contains %d gold."), 
+		name, value, game.player_ptr->economic.gold);
     popup_message_dialog(pdialog->shell, "buydialog", buf,
 			 buy_callback_yes, pdialog, 0,
 			 buy_callback_no, 0, 0,
 			 NULL);
   }
   else {
-    sprintf(buf, _("%s costs %d gold.\nTreasury contains %d gold."), 
-	    name, value, game.player_ptr->economic.gold);
+    my_snprintf(buf, sizeof(buf),
+		_("%s costs %d gold.\nTreasury contains %d gold."), 
+		name, value, game.player_ptr->economic.gold);
     popup_message_dialog(pdialog->shell, "buynodialog", buf,
 			 buy_callback_no, 0, 0,
 			 NULL);
@@ -1963,7 +2059,8 @@ void upgrade_callback(Widget w, XtPointer client_data, XtPointer call_data)
 
     if ( ut2 == -1 ) {
       /* this shouldn't generally happen, but it is conceivable */
-      sprintf(buf, _("Sorry: cannot upgrade %s."), unit_types[ut1].name);
+      my_snprintf(buf, sizeof(buf),
+		  _("Sorry: cannot upgrade %s."), unit_types[ut1].name);
       popup_message_dialog(toplevel, "upgradenodialog", buf,
                            unitupgrade_callback_no, 0, 0,
                            NULL);
@@ -1971,19 +2068,21 @@ void upgrade_callback(Widget w, XtPointer client_data, XtPointer call_data)
       value=unit_upgrade_price(game.player_ptr, ut1, ut2);
 
       if(game.player_ptr->economic.gold>=value) {
-        sprintf(buf, _("Upgrade %s to %s for %d gold?\n"
-		       "Treasury contains %d gold."),
-		unit_types[ut1].name, unit_types[ut2].name,
-		value, game.player_ptr->economic.gold);
+        my_snprintf(buf, sizeof(buf),
+		    _("Upgrade %s to %s for %d gold?\n"
+		      "Treasury contains %d gold."),
+		    unit_types[ut1].name, unit_types[ut2].name,
+		    value, game.player_ptr->economic.gold);
         popup_message_dialog(toplevel, "upgradedialog", buf,
                              unitupgrade_callback_yes, (XtPointer)(punit->id), 0,
                              unitupgrade_callback_no, 0, 0,
                              NULL);
       } else {
-        sprintf(buf, _("Upgrading %s to %s costs %d gold.\n"
-		       "Treasury contains %d gold."),
-		unit_types[ut1].name, unit_types[ut2].name,
-		value, game.player_ptr->economic.gold);
+        my_snprintf(buf, sizeof(buf),
+		    _("Upgrading %s to %s costs %d gold.\n"
+		      "Treasury contains %d gold."),
+		    unit_types[ut1].name, unit_types[ut2].name,
+		    value, game.player_ptr->economic.gold);
         popup_message_dialog(toplevel, "upgradenodialog", buf,
                              unitupgrade_callback_no, 0, 0,
                              NULL);
@@ -2012,6 +2111,7 @@ static void change_to_callback(Widget w, XtPointer client_data,
   
     packet.city_id=pdialog->pcity->id;
     packet.name[0]='\0';
+    packet.worklist.name[0] = '\0';
     packet.build_id=pdialog->change_list_ids[ret->list_index];
     packet.is_build_id_unit_id=
       (ret->list_index >= pdialog->change_list_num_improvements);
@@ -2072,7 +2172,7 @@ static void change_help_callback(Widget w, XtPointer client_data,
 void change_callback(Widget w, XtPointer client_data, XtPointer call_data)
 {
   static char *dummy_change_list[]={ 
-    "XXXXXXXXXXXXXXXXXXXXXXXXXX  888 turns",
+    "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX  888 turns",
     "2",
     "3",
     "4",
@@ -2094,7 +2194,7 @@ void change_callback(Widget w, XtPointer client_data, XtPointer call_data)
   Position x, y;
   Dimension width, height;
   struct city_dialog *pdialog;
-  int i, n;
+  int i, n, turns;
   
   pdialog=(struct city_dialog *)client_data;
   
@@ -2167,8 +2267,20 @@ void change_callback(Widget w, XtPointer client_data, XtPointer call_data)
 
   for(i=0, n=0; i<B_LAST; i++)
     if(can_build_improvement(pdialog->pcity, i)) {
-      sprintf(pdialog->change_list_names[n], "%s (%d)", get_imp_name_ex(pdialog->pcity, i),get_improvement_type(i)->build_cost);
-      
+      if (i==B_CAPITAL) {
+	my_snprintf(pdialog->change_list_names[n],
+		    sizeof(pdialog->change_list_names[n]),
+		    "%s (XX)",
+		    get_imp_name_ex(pdialog->pcity, i));
+      } else {
+	turns = city_turns_to_build (pdialog->pcity, i, FALSE);
+	my_snprintf(pdialog->change_list_names[n],
+		    sizeof(pdialog->change_list_names[n]),
+		    turns == 1 ? _("%s (%d) %d turn") : _("%s (%d) %d turns"),
+		    get_imp_name_ex(pdialog->pcity, i),
+		    get_improvement_type(i)->build_cost,
+		    turns);
+      }
       pdialog->change_list_names_ptrs[n]=pdialog->change_list_names[n];
       pdialog->change_list_ids[n++]=i;
     }
@@ -2178,7 +2290,13 @@ void change_callback(Widget w, XtPointer client_data, XtPointer call_data)
 
   for(i=0; i<game.num_unit_types; i++)
     if(can_build_unit(pdialog->pcity, i)) {
-      sprintf(pdialog->change_list_names[n],"%s (%d)", get_unit_name(i), get_unit_type(i)->build_cost);
+      turns = city_turns_to_build (pdialog->pcity, i, TRUE);
+      my_snprintf(pdialog->change_list_names[n],
+		  sizeof(pdialog->change_list_names[n]),
+		  turns == 1 ? _("%s (%d) %d turn") : _("%s (%d) %d turns"),
+		  get_unit_name(i),
+		  get_unit_type(i)->build_cost,
+		  turns);
       pdialog->change_list_names_ptrs[n]=pdialog->change_list_names[n];
       pdialog->change_list_ids[n++]=i;
     }
@@ -2193,6 +2311,89 @@ void change_callback(Widget w, XtPointer client_data, XtPointer call_data)
 }
 
 
+/****************************************************************
+  Display the city's worklist.
+*****************************************************************/
+void worklist_callback(Widget w, XtPointer client_data, XtPointer call_data)
+{
+  struct city_dialog *pdialog;
+  
+  pdialog = (struct city_dialog *)client_data;
+
+  if (pdialog->worklist_shell) {
+    XtPopup(pdialog->worklist_shell, XtGrabNone);
+  } else {
+    pdialog->worklist_shell = 
+      popup_worklist(pdialog->pcity->worklist, pdialog->pcity, pdialog->shell, 
+		     (void *)pdialog, commit_city_worklist,
+		     cancel_city_worklist);
+  }
+}
+
+/****************************************************************
+  Commit the changes to the worklist for the city.
+*****************************************************************/
+void commit_city_worklist(struct worklist *pwl, void *data)
+{
+  struct packet_city_request packet;
+  struct city_dialog *pdialog = (struct city_dialog *)data;
+  int i, k, id, is_unit;
+
+  /* Update the worklist.  Remember, though -- the current build 
+     target really isn't in the worklist; don't send it to the 
+     server as part of the worklist.  Of course, we have to
+     search through the current worklist to find the first
+     _now_available_ build target (to cope with players who try
+     mean things like adding a Battleship to a city worklist when
+     the player doesn't even yet have the Map Making tech).  */
+
+  for (k = 0; k < MAX_LEN_WORKLIST; k++) {
+    if (! worklist_peek_ith(pwl, &id, &is_unit, k))
+      break;
+
+    /* If it can be built... */
+    if (( is_unit && can_build_unit(pdialog->pcity, id)) ||
+	(!is_unit && can_build_improvement(pdialog->pcity, id))) {
+      /* ...but we're not yet building it, then switch. */
+      if (id != pdialog->pcity->currently_building || 
+	  is_unit != pdialog->pcity->is_building_unit) {
+
+	/* Change the current target */
+	packet.city_id=pdialog->pcity->id;
+	packet.name[0] = '\0';
+	packet.worklist.name[0] = '\0';
+	packet.build_id = id;
+	packet.is_build_id_unit_id = is_unit;
+	send_packet_city_request(&aconnection, &packet, PACKET_CITY_CHANGE);
+      }
+
+      /* This item is now (and may have always been) the current
+	 build target.  Drop it out of the worklist. */
+      worklist_remove(pwl, k);
+      break;
+    }
+  }
+
+  /* Send the rest of the worklist on its way. */
+  packet.city_id=pdialog->pcity->id;
+  packet.name[0] = '\0';
+  packet.worklist.name[0] = '\0';
+  packet.worklist.is_valid = 1;
+  for (i = 0; i < MAX_LEN_WORKLIST; i++) {
+    packet.worklist.ids[i] = pwl->ids[i];
+  }
+    
+  send_packet_city_request(&aconnection, &packet, PACKET_CITY_WORKLIST);
+
+  pdialog->worklist_shell = NULL;
+}
+
+void cancel_city_worklist(void *data) {
+  struct city_dialog *pdialog = (struct city_dialog *)data;
+  pdialog->worklist_shell = NULL;
+}
+
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -2207,6 +2408,7 @@ static void sell_callback_yes(Widget w, XtPointer client_data,
   packet.city_id=pdialog->pcity->id;
   packet.build_id=pdialog->sell_id;
   packet.name[0]='\0';
+  packet.worklist.name[0] = '\0';
   send_packet_city_request(&aconnection, &packet, PACKET_CITY_SELL);
 
   destroy_message_dialog(w);
@@ -2246,9 +2448,9 @@ void sell_callback(Widget w, XtPointer client_data, XtPointer call_data)
 	    return;
 	  
 	  pdialog->sell_id=i;
-	  sprintf(buf, _("Sell %s for %d gold?"), 
-		  get_imp_name_ex(pdialog->pcity, i),
-		  improvement_value(i));
+	  my_snprintf(buf, sizeof(buf), _("Sell %s for %d gold?"), 
+		      get_imp_name_ex(pdialog->pcity, i),
+		      improvement_value(i));
 
 	  popup_message_dialog(pdialog->shell, "selldialog", buf,
 			       sell_callback_yes, pdialog, 0,
@@ -2267,6 +2469,9 @@ void sell_callback(Widget w, XtPointer client_data, XtPointer call_data)
 *****************************************************************/
 void close_city_dialog(struct city_dialog *pdialog)
 {
+  if (pdialog->worklist_shell)
+    XtDestroyWidget(pdialog->worklist_shell);
+
   XtDestroyWidget(pdialog->shell);
   genlist_unlink(&dialog_list, pdialog);
 
@@ -2284,15 +2489,15 @@ void close_city_dialog(struct city_dialog *pdialog)
 /****************************************************************
 ...
 *****************************************************************/
-void city_dialog_returnkey(Widget w, XEvent *event, String *params, Cardinal *num_params)
+void citydlg_key_return(Widget w)
 {
-  close_city_dialog_action(XtParent(XtParent(w)), 0, 0, 0);
+  citydlg_msg_close(XtParent(XtParent(w)));
 }
 
 /****************************************************************
 ...
 *****************************************************************/
-void close_city_dialog_action(Widget w, XEvent *event, String *argv, Cardinal *argc)
+void citydlg_msg_close(Widget w)
 {
   struct genlist_iterator myiter;
 
@@ -2304,7 +2509,6 @@ void close_city_dialog_action(Widget w, XEvent *event, String *argv, Cardinal *a
     }
 }
 
-
 /****************************************************************
 ...
 *****************************************************************/
