@@ -39,12 +39,14 @@
 #include "mem.h"
 #include "packets.h"
 #include "shared.h"
+#include "support.h"
 #include "unit.h"
 
 #include "climisc.h"
 
 #include "chatline.h"
 #include "citydlg.h"
+#include "cityrepdata.h"
 #include "gui_stuff.h"
 #include "mapview.h"
 #include "optiondlg.h"
@@ -60,234 +62,6 @@ extern Atom wm_delete_window;
 extern struct connection aconnection;
 extern int delay_report_update;
 
-/* abbreviate long city names to this length in the city report: */
-#define REPORT_CITYNAME_ABBREV 15 
-
-/************************************************************************
- cr_entry = return an entry (one column for one city) for the city report
- These return ptrs to filled in static strings.
- Note the returned string may not be exactly the right length; that
- is handled later.
-*************************************************************************/
-
-static char *cr_entry_cityname(struct city *pcity)
-{
-  static char buf[REPORT_CITYNAME_ABBREV+1];
-  if (strlen(pcity->name) <= REPORT_CITYNAME_ABBREV) {
-    return pcity->name;
-  } else {
-    strncpy(buf, pcity->name, REPORT_CITYNAME_ABBREV-1);
-    buf[REPORT_CITYNAME_ABBREV-1] = '.';
-    buf[REPORT_CITYNAME_ABBREV] = '\0';
-    return buf;
-  }
-}
-
-static char *cr_entry_size(struct city *pcity)
-{
-  static char buf[8];
-  sprintf(buf, "%2d", pcity->size);
-  return buf;
-}
-
-static char *cr_entry_hstate_concise(struct city *pcity)
-{
-  static char buf[4];
-  sprintf(buf, "%s", (city_celebrating(pcity) ? "*" :
-		      (city_unhappy(pcity) ? "X" : " ")));
-  return buf;
-}
-
-static char *cr_entry_hstate_verbose(struct city *pcity)
-{
-  static char buf[16];
-  sprintf(buf, "%s", (city_celebrating(pcity) ? _("Rapture") :
-		      (city_unhappy(pcity) ? _("Disorder") : _("Peace"))));
-  return buf;
-}
-
-static char *cr_entry_workers(struct city *pcity)
-{
-  static char buf[32];
-  sprintf(buf, "%d/%d/%d",
-	  pcity->ppl_happy[4],
-	  pcity->ppl_content[4],
-	  pcity->ppl_unhappy[4]);
-  return buf;
-}
-
-static char *cr_entry_specialists(struct city *pcity)
-{
-  static char buf[32];
-  sprintf(buf, "%d/%d/%d",
-  	  pcity->ppl_elvis,
-          pcity->ppl_scientist,
-          pcity->ppl_taxman);
-  return buf;
-}
-
-static char *cr_entry_resources(struct city *pcity)
-{
-  static char buf[32];
-  sprintf(buf, "%d/%d/%d",
-	  pcity->food_surplus, 
-	  pcity->shield_surplus, 
-	  pcity->trade_prod);
-  return buf;
-}
-
-static char *cr_entry_output(struct city *pcity)
-{
-  static char buf[32];
-  int goldie;
-
-  goldie = city_gold_surplus(pcity);
-  sprintf(buf, "%s%d/%d/%d",
-	  (goldie < 0) ? "-" : (goldie > 0) ? "+" : "",
-	  (goldie < 0) ? (-goldie) : goldie,
-	  pcity->luxury_total,
-	  pcity->science_total);
-  return buf;
-}
-
-static char *cr_entry_food(struct city *pcity)
-{
-  static char buf[32];
-  sprintf(buf,"%d/%d",
-	  pcity->food_stock,
-	  (pcity->size+1) * game.foodbox);
-  return buf;
-}
-
-static char *cr_entry_pollution(struct city *pcity)
-{
-  static char buf[8];
-  sprintf(buf,"%3d", pcity->pollution);
-  return buf;
-}
-
-static char *cr_entry_num_trade(struct city *pcity)
-{
-  static char buf[8];
-  sprintf(buf,"%d", city_num_trade_routes(pcity));
-  return buf;
-}
-
-static char *cr_entry_building(struct city *pcity)
-{
-  static char buf[64];
-  if(pcity->is_building_unit)  {
-    sprintf(buf, "%s(%d/%d/%d)", 
-            get_unit_type(pcity->currently_building)->name,
-	    pcity->shield_stock,
-	    get_unit_type(pcity->currently_building)->build_cost,
-	    city_buy_cost(pcity));
-  } else {
-    if(pcity->currently_building==B_CAPITAL)  {
-      sprintf(buf, "%s(%d/X/X)",
-              get_imp_name_ex(pcity, pcity->currently_building),
-	      pcity->shield_stock);
-    } else {
-      sprintf(buf, "%s(%d/%d/%d)", 
-	      get_imp_name_ex(pcity, pcity->currently_building),
-	      pcity->shield_stock,
-	      get_improvement_type(pcity->currently_building)->build_cost,
-	      city_buy_cost(pcity));
-    }
-  }
-  return buf;
-}
-
-static char *cr_entry_corruption(struct city *pcity)
-{
-  static char buf[8];
-  sprintf(buf,"%3d", pcity->corruption);
-  return buf;
-}
-
-/* City report options (which columns get shown)
- * To add a new entry, you should just have to:
- * - add a function like those above
- * - add an entry in the city_report_specs[] table
- */
-
-struct city_report_spec {
-  int show;			/* modify this to customize */
-  int width;			/* 0 means variable; rightmost only */
-  int space;			/* number of leading spaces (see below) */
-  char *title1;
-  char *title2;
-  char *explanation;
-  char *(*func)(struct city*);
-  char *tagname;		/* for save_options */
-};
-
-/* This generates the function name and the tagname: */
-#define FUNC_TAG(var)  cr_entry_##var, #var 
-
-/* Use tagname rather than index for load/save, because later
-   additions won't necessarily be at the end.
-*/
-
-/* Note on space: you can do spacing and alignment in various ways;
-   you can avoid explicit space between columns if they are bracketted,
-   but the problem is that with a configurable report you don't know
-   what's going to be next to what.
-*/
-
-static struct city_report_spec city_report_specs[] = {
-  { 1,-15, 0, "",  N_("Name"),        N_("City Name"),
-                                      FUNC_TAG(cityname) },
-  { 0,  2, 1, "",  N_("Sz"),          N_("Size"),
-                                      FUNC_TAG(size) },
-  { 1, -8, 1, "",  N_("State"),       N_("Rapture/Peace/Disorder"),
-                                      FUNC_TAG(hstate_verbose) },
-  { 0,  1, 1, "",  "",                N_("Concise *=Rapture, X=Disorder"),
-                                      FUNC_TAG(hstate_concise) },
-  { 1,  8, 1, N_("Workers"), N_("H/C/U"),
-                                      N_("Workers: Happy, Content, Unhappy"),
-                                      FUNC_TAG(workers) },
-  { 0,  7, 1, N_("Special"), N_("E/S/T"),
-                                      N_("Entertainers, Scientists, Taxmen"),
-                                      FUNC_TAG(specialists) },
-  { 1, 10, 1, N_("Surplus"), N_("F/P/T"),
-                                      N_("Surplus: Food, Production, Trade"),
-                                      FUNC_TAG(resources) },
-  { 1, 10, 1, N_("Economy"), N_("G/L/S"),
-                                      N_("Economy: Gold, Luxuries, Science"),
-                                      FUNC_TAG(output) },
-  { 0,  1, 1, "n", "T",               N_("Number of Trade Routes"),
-                                      FUNC_TAG(num_trade) },
-  { 1,  7, 1, N_("Food"), N_("Stock"), N_("Food Stock"),
-                                      FUNC_TAG(food) },
-  { 0,  3, 1, "", N_("Pol"),          N_("Pollution"),
-                                      FUNC_TAG(pollution) },
-  { 0,  3, 1, "", N_("Cor"),          N_("Corruption"),
-                                      FUNC_TAG(corruption) },
-  { 1,  0, 1, N_("Currently Building"), N_("(Stock,Target,Buy Cost)"),
-                                      N_("Currently Building"),
-                                      FUNC_TAG(building) }
-};
-
-#define NUM_CREPORT_COLS \
-         sizeof(city_report_specs)/sizeof(city_report_specs[0])
-     
-/******************************************************************
-Some simple wrappers:
-******************************************************************/
-int num_city_report_spec(void)
-{
-  return NUM_CREPORT_COLS;
-}
-int *city_report_spec_show_ptr(int i)
-{
-  return &(city_report_specs[i].show);
-}
-char *city_report_spec_tagname(int i)
-{
-  return city_report_specs[i].tagname;
-}
-
 /******************************************************************/
 static void popup_chgall_dialog (Widget parent);
 
@@ -342,22 +116,21 @@ static char *dummy_city_list[]={
 
 
 /****************************************************************
- Create the text for a line in the city report
+ Create the text for a line in the city report; n is size of buffer
 *****************************************************************/
-static void get_city_text(struct city *pcity, char *text)
+static void get_city_text(struct city *pcity, char *text, int n)
 {
   struct city_report_spec *spec;
   int i;
 
-  text[0] = '\0';		/* init for strlen */
+  text[0] = '\0';		/* init for strcat */
   for(i=0, spec=city_report_specs; i<NUM_CREPORT_COLS; i++, spec++) {
     if(!spec->show) continue;
 
     if(spec->space>0)
-      sprintf(text+strlen(text), "%*s", spec->space, " ");
+      cat_snprintf(text, n, "%*s", spec->space, " ");
 
-    sprintf(text+strlen(text), "%*s",
-	    spec->width, (spec->func)(pcity));
+    cat_snprintf(text, n, "%*s", spec->width, (spec->func)(pcity));
   }
 }
 
@@ -370,18 +143,18 @@ static char *get_city_table_header(void)
   struct city_report_spec *spec;
   int i, j;
 
-  text[0] = '\0';		/* init for strlen */
+  text[0] = '\0';		/* init for strcat */
   for(j=0; j<=1; j++) {
     for(i=0, spec=city_report_specs; i<NUM_CREPORT_COLS; i++, spec++) {
       if(!spec->show) continue;
 
       if(spec->space>0)
-	sprintf(text+strlen(text), "%*s", spec->space, " ");
+	cat_snprintf(text, sizeof(text), "%*s", spec->space, " ");
 
-      sprintf(text+strlen(text), "%*s", spec->width,
-	      (j?_(spec->title2):_(spec->title1)));
+      cat_snprintf(text, sizeof(text), "%*s", spec->width,
+		   (j ? _(spec->title2) : _(spec->title1)));
     }
-    if (j==0) strcat(text, "\n");
+    if (j==0) sz_strlcat(text, "\n");
   }
   return text;
 }
@@ -521,7 +294,7 @@ void create_city_report_dialog(int make_modal)
     XSetWMProtocols(display, XtWindow(city_dialog_shell), 
 		    &wm_delete_window, 1);
     XtOverrideTranslations(city_dialog_shell,
-	 XtParseTranslationTable("<Message>WM_PROTOCOLS: close-cityreport()"));
+	 XtParseTranslationTable("<Message>WM_PROTOCOLS: msg-close-city-report()"));
   }
 
   city_report_dialog_update();
@@ -535,6 +308,7 @@ void city_list_callback(Widget w, XtPointer client_data,
 {
   XawListReturnStruct *ret=XawListShowCurrent(city_list);
   struct city *pcity;
+  int turns;
 
   if(ret->list_index!=XAW_LIST_NONE && 
      (pcity=cities_in_list[ret->list_index])) {
@@ -558,7 +332,18 @@ void city_list_callback(Widget w, XtPointer client_data,
     for(i=0; i<B_LAST; i++)
       if(can_build_improvement(pcity, i)) {
 	Widget entry;
-	sprintf(buf,"%s (%d)", get_imp_name_ex(pcity, i),get_improvement_type(i)->build_cost);
+	if (i==B_CAPITAL) {
+	  my_snprintf(buf, sizeof(buf),
+		      "%s (XX)",
+		      get_imp_name_ex(pcity, i));
+	} else {
+	  turns = city_turns_to_build (pcity, i, FALSE);
+	  my_snprintf(buf, sizeof(buf),
+		      turns == 1 ? _("%s (%d) %d turn") : _("%s (%d) %d turns"),
+		      get_imp_name_ex(pcity, i),
+		      get_improvement_type(i)->build_cost,
+		      turns);
+	}
 	entry = XtVaCreateManagedWidget(buf, smeBSBObjectClass, city_popupmenu, NULL);
 	XtAddCallback(entry, XtNcallback, city_change_callback, (XtPointer) i);
 	flag=1;
@@ -567,8 +352,12 @@ void city_list_callback(Widget w, XtPointer client_data,
     for(i=0; i<game.num_unit_types; i++)
       if(can_build_unit(pcity, i)) {
 	Widget entry;
-	sprintf(buf,"%s (%d)", 
-		get_unit_name(i),get_unit_type(i)->build_cost);
+	turns = city_turns_to_build (pcity, i, TRUE);
+	my_snprintf(buf, sizeof(buf),
+		    turns == 1 ? _("%s (%d) %d turn") : _("%s (%d) %d turns"),
+		    get_unit_name(i),
+		    get_unit_type(i)->build_cost,
+		    turns);
 	entry = XtVaCreateManagedWidget(buf, smeBSBObjectClass, 
 					city_popupmenu, NULL);
 	XtAddCallback(entry, XtNcallback, city_change_callback, 
@@ -614,6 +403,7 @@ void city_change_callback(Widget w, XtPointer client_data,
 
     packet.city_id=pcity->id;
     packet.name[0]='\0';
+    packet.worklist.name[0] = '\0';
     packet.build_id=build_nr;
     packet.is_build_id_unit_id=unit;
     send_packet_city_request(&aconnection, &packet, PACKET_CITY_CHANGE);
@@ -646,12 +436,14 @@ void city_buy_callback(Widget w, XtPointer client_data,
 	  struct packet_city_request packet;
 	  packet.city_id=pcity->id;
 	  packet.name[0]='\0';
+	  packet.worklist.name[0] = '\0';
 	  send_packet_city_request(&aconnection, &packet, PACKET_CITY_BUY);
 	}
       else
 	{
-	  sprintf(buf, _("Game: %s costs %d gold and you only have %d gold."),
-		  name,value,game.player_ptr->economic.gold);
+	  my_snprintf(buf, sizeof(buf),
+		      _("Game: %s costs %d gold and you only have %d gold."),
+		      name,value,game.player_ptr->economic.gold);
 	  append_output_window(buf);
 	}
     }
@@ -702,8 +494,7 @@ void city_close_callback(Widget w, XtPointer client_data,
 /****************************************************************
 ...
 *****************************************************************/
-void close_city_report_action(Widget w, XEvent *event, 
-				     String *argv, Cardinal *argc)
+void cityrep_msg_close(Widget w)
 {
   city_close_callback(w, NULL, NULL);
 }
@@ -751,6 +542,7 @@ void city_config_callback(Widget w, XtPointer client_data,
   popup_city_report_config_dialog();
 }
 
+#define MAX_LEN_CITY_TEXT 200
 /****************************************************************
 ...
 *****************************************************************/
@@ -775,7 +567,9 @@ void city_report_dialog_update(void)
       cities_in_list = fc_realloc(cities_in_list,
 				  n_alloc*sizeof(*cities_in_list));
       city_list_text = fc_realloc(city_list_text, n_alloc*sizeof(char*));
-      for(j=n_prev; j<n_alloc; j++)  city_list_text[j] = malloc(128);
+      for(j=n_prev; j<n_alloc; j++) {
+	city_list_text[j] = malloc(MAX_LEN_CITY_TEXT);
+      }
     }
        
     report_title=get_report_title(_("City Advisor"));
@@ -800,12 +594,13 @@ void city_report_dialog_update(void)
     assert(i==n);
     qsort(cities_in_list, n, sizeof(struct city*), city_name_compare);
     for(i=0; i<n; i++) {
-      get_city_text(cities_in_list[i], city_list_text[i]);
+      get_city_text(cities_in_list[i], city_list_text[i], MAX_LEN_CITY_TEXT);
     }
     i = n;
     if(!n) {
-      strcpy(city_list_text[0], 
-	     "                                                             ");
+      mystrlcpy(city_list_text[0], 
+		"                                                             ",
+		MAX_LEN_CITY_TEXT);
       i=1;
       cities_in_list[0]=NULL;
     }
@@ -844,16 +639,16 @@ void city_report_dialog_update_city(struct city *pcity)
       int n;
       String *list;
       Dimension w;
-      char new_city_line[200];
+      char new_city_line[MAX_LEN_CITY_TEXT];
 
       XtVaGetValues(city_list, XtNnumberStrings, &n, XtNlist, &list, NULL);
       if(strncmp(pcity->name,list[i],
 		 MIN(strlen(pcity->name),REPORT_CITYNAME_ABBREV-1))) {
 	 break;
       }
-      get_city_text(pcity,new_city_line);
+      get_city_text(pcity, new_city_line, sizeof(new_city_line));
       if(strcmp(new_city_line, list[i])==0) return; /* no change */
-      strcpy(list[i], new_city_line);
+      mystrlcpy(list[i], new_city_line, MAX_LEN_CITY_TEXT);
 
       /* It seems really inefficient to regenerate the whole list just to
          change one line.  It's also annoying to have to set the size
@@ -867,7 +662,7 @@ void city_report_dialog_update_city(struct city *pcity)
       XtVaSetValues(city_label, XtNwidth, w+15, NULL);
       XawFormDoLayout(city_form, True);
       return;
-    };
+    }
   }
   city_report_dialog_update();
 }
@@ -925,7 +720,7 @@ void create_city_report_config_dialog(void)
 					     config_form, NULL));
 
   for(i=1, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) {
-    sprintf(buf, "%-32s", _(spec->explanation));
+    my_snprintf(buf, sizeof(buf), "%-32s", _(spec->explanation));
     above = (i==1)?config_label:config_optlabel;
 
     config_optlabel = XtVaCreateManagedWidget("cityconfiglabel", 
@@ -1018,7 +813,7 @@ struct chgall_data
 
 static struct chgall_data *chgall_state = NULL;
 
-static void chgall_change_action (Widget w, XEvent *event,
+static void chgall_accept_action (Widget w, XEvent *event,
 				  String *params, Cardinal *num_params);
 static void chgall_cancel_action (Widget w, XEvent *event,
 				  String *params, Cardinal *num_params);
@@ -1059,8 +854,8 @@ static void popup_chgall_dialog (Widget parent)
     {
       static XtActionsRec actions[] =
       {
-	{ "chgall-change", chgall_change_action },
-	{ "chgall-cancel", chgall_cancel_action }
+	{ "key-dialog-chgall-accept", chgall_accept_action },
+	{ "key-dialog-chgall-cancel", chgall_cancel_action }
       };
 
       initialized = TRUE;
@@ -1211,7 +1006,7 @@ static void popup_chgall_dialog (Widget parent)
 ...
 **************************************************************************/
 
-static void chgall_change_action (Widget w, XEvent *event,
+static void chgall_accept_action (Widget w, XEvent *event,
 				  String *params, Cardinal *num_params)
 {
   Widget target = XtNameToWidget (w, "chgallchangecommand");
@@ -1286,7 +1081,7 @@ static void chgall_change_command_callback (Widget w, XtPointer client_data,
       return;
     }
 
-  sprintf (msgbuf,
+  my_snprintf (msgbuf, sizeof(msgbuf),
 	   _("Game: Changing production of every %s into %s."),
 	   state->fr_list[state->fr_index],
 	   state->to_list[state->to_index]);
