@@ -40,6 +40,7 @@
 #include "mem.h"
 #include "packets.h"
 #include "player.h"
+#include "support.h"
 
 #include "chatline.h"
 #include "civclient.h"
@@ -58,7 +59,6 @@ extern Widget toplevel, main_form, map_canvas;
 extern struct connection aconnection;
 extern Display	*display;
 extern int display_depth;
-extern int flags_are_transparent;
 extern int ai_popup_windows;
 extern GC fill_bg_gc;
 
@@ -80,7 +80,7 @@ Widget spy_advances_list, spy_advances_list_label;
 Widget spy_steal_command;
 
 int spy_tech_shell_is_modal;
-int advance_type[A_LAST];
+int advance_type[A_LAST+1];
 int steal_advance = 0;
 
 /******************************************************************/
@@ -89,7 +89,7 @@ Widget spy_improvements_list, spy_improvements_list_label;
 Widget spy_sabotage_command;
 
 int spy_sabotage_shell_is_modal;
-int improvement_type[B_LAST];
+int improvement_type[B_LAST+1];
 int sabotage_improvement = 0;
 
 /******************************************************************/
@@ -172,6 +172,10 @@ struct unit *punit_caravan;
 
 static Widget caravan_dialog;
 
+static int is_showing_unit_connect_dialog = FALSE;
+static int unit_to_use_to_connect;
+static int connect_unit_x;
+static int connect_unit_y;
 
 /****************************************************************
 ...
@@ -213,10 +217,11 @@ void popup_notify_dialog(char *caption, char *headline, char *lines)
 			  XtNlabel, lines,
 			  NULL);   
 
-  notify_command = XtVaCreateManagedWidget("notifycommand", 
-					   commandWidgetClass,
-					   notify_form,
-					   NULL);
+  notify_command =
+    I_L(XtVaCreateManagedWidget("notifycommand", 
+				commandWidgetClass,
+				notify_form,
+				NULL));
 
   XtVaGetValues(notify_label, XtNwidth, &width, NULL);
   XtVaGetValues(notify_headline, XtNwidth, &width2, NULL);
@@ -338,15 +343,17 @@ void popup_notify_goto_dialog(char *headline, char *lines,int x, int y)
 			  XtNlabel, lines,
 			  NULL);   
 
-  notify_command = XtVaCreateManagedWidget("notifycommand", 
-					   commandWidgetClass,
-					   notify_form,
-					   NULL);
+  notify_command =
+    I_L(XtVaCreateManagedWidget("notifycommand", 
+				commandWidgetClass,
+				notify_form,
+				NULL));
 
-  notify_goto_command = XtVaCreateManagedWidget("notifygotocommand", 
-					   commandWidgetClass,
-					   notify_form,
-					   NULL);
+  notify_goto_command =
+    I_L(XtVaCreateManagedWidget("notifygotocommand", 
+				commandWidgetClass,
+				notify_form,
+				NULL));
   
   XtVaGetValues(notify_label, XtNwidth, &width, NULL);
   XtVaGetValues(notify_headline, XtNwidth, &width2, NULL);
@@ -417,17 +424,19 @@ void popup_bribe_dialog(struct unit *punit)
   char buf[128];
   
   if(game.player_ptr->economic.gold>=punit->bribe_cost) {
-    sprintf(buf, _("Bribe unit for %d gold?\n"
-		   "Treasury contains %d gold."), 
-	    punit->bribe_cost, game.player_ptr->economic.gold);
+    my_snprintf(buf, sizeof(buf),
+		_("Bribe unit for %d gold?\n"
+		  "Treasury contains %d gold."), 
+		punit->bribe_cost, game.player_ptr->economic.gold);
     popup_message_dialog(toplevel, "diplomatbribedialog", buf,
 			 diplomat_bribe_yes_callback, 0, 0,
 			 diplomat_bribe_no_callback, 0, 0,
 			 NULL);
   } else {
-    sprintf(buf, _("Bribing the unit costs %d gold.\n"
-		   "Treasury contains %d gold."), 
-	    punit->bribe_cost, game.player_ptr->economic.gold);
+    my_snprintf(buf, sizeof(buf),
+		_("Bribing the unit costs %d gold.\n"
+		  "Treasury contains %d gold."), 
+		punit->bribe_cost, game.player_ptr->economic.gold);
     popup_message_dialog(toplevel, "diplomatnogolddialog", buf,
 			 diplomat_bribe_no_callback, 0, 0,
 			 NULL);
@@ -685,7 +694,7 @@ static int create_advances_list(struct player *pplayer,
   Dimension width1, width2; 
   int i, j;
 
-  static char *advances_can_steal[A_LAST]; 
+  static char *advances_can_steal[A_LAST+1]; 
 
   spy_tech_shell =
     I_T(XtVaCreatePopupShell("spystealtechpopup", 
@@ -739,6 +748,10 @@ static int create_advances_list(struct player *pplayer,
         advance_type[j++] = i;
       }
     }
+    if(j > 0) {
+      advances_can_steal[j] = _("At Spy's Discretion");
+      advance_type[j++] = game.num_tech_types;
+    }
   }
 
   if(j == 0) j++;
@@ -766,7 +779,7 @@ static int create_improvements_list(struct player *pplayer,
   Dimension width1, width2; 
   int i, j;
 
-  static char *improvements_can_sabotage[B_LAST]; 
+  static char *improvements_can_sabotage[B_LAST+1]; 
   
   spy_sabotage_shell =
     I_T(XtVaCreatePopupShell("spysabotageimprovementspopup", 
@@ -820,7 +833,14 @@ static int create_improvements_list(struct player *pplayer,
       improvements_can_sabotage[j] = get_imp_name_ex(pcity, i);
       improvement_type[j++] = i;
     }  
-  
+
+  if(j > 1) {
+    improvements_can_sabotage[j] = _("At Spy's Discretion");
+    improvement_type[j++] = B_LAST;
+  } else {
+    improvement_type[0] = B_LAST; /* fake "discretion", since must be production */
+  }
+
   improvements_can_sabotage[j] = NULL;
   
   XtSetSensitive(spy_sabotage_command, FALSE);
@@ -949,18 +969,20 @@ void popup_incite_dialog(struct city *pcity)
   char buf[128];
 
   if(game.player_ptr->economic.gold>=pcity->incite_revolt_cost) {
-   sprintf(buf, _("Incite a revolt for %d gold?\n"
+    my_snprintf(buf, sizeof(buf),
+		_("Incite a revolt for %d gold?\n"
 		  "Treasury contains %d gold."), 
-	   pcity->incite_revolt_cost, game.player_ptr->economic.gold);
+		pcity->incite_revolt_cost, game.player_ptr->economic.gold);
    diplomat_target_id = pcity->id;
    popup_message_dialog(toplevel, "diplomatrevoltdialog", buf,
 			diplomat_incite_yes_callback, 0, 0,
 			diplomat_incite_no_callback, 0, 0,
 			NULL);
   } else {
-   sprintf(buf, _("Inciting a revolt costs %d gold.\n"
-		  "Treasury contains %d gold."), 
-	   pcity->incite_revolt_cost, game.player_ptr->economic.gold);
+   my_snprintf(buf, sizeof(buf),
+	       _("Inciting a revolt costs %d gold.\n"
+		 "Treasury contains %d gold."), 
+	       pcity->incite_revolt_cost, game.player_ptr->economic.gold);
    popup_message_dialog(toplevel, "diplomatnogolddialog", buf,
 			diplomat_incite_no_callback, 0, 0,
 			NULL);
@@ -1137,8 +1159,9 @@ void popup_caravan_dialog(struct unit *punit,
 {
   char buf[128];
   
-  sprintf(buf, _("Your caravan from %s reaches the city of %s.\nWhat now?"),
-	  phomecity->name, pdestcity->name);
+  my_snprintf(buf, sizeof(buf),
+	      _("Your caravan from %s reaches the city of %s.\nWhat now?"),
+	      phomecity->name, pdestcity->name);
   
   caravan_city_id=pdestcity->id; /* callbacks need these */
   caravan_unit_id=punit->id;
@@ -1317,6 +1340,86 @@ void popup_pillage_dialog(struct unit *punit, int may_pillage)
   XtPopup (shell, XtGrabNone);
 }
 
+/****************************************************************
+common code for all buttons in unit connect dialog
+*****************************************************************/
+static void unit_connect_callback (Widget w, XtPointer client_data, 
+				   XtPointer call_data)
+{
+  struct unit *punit;
+  int activity = (int)client_data;
+
+  if (!is_showing_unit_connect_dialog) {
+    destroy_message_dialog(w);
+    return;
+  }
+
+  punit = find_unit_by_id(unit_to_use_to_connect);
+
+  if (punit) {
+    if (activity != ACTIVITY_IDLE) {
+      struct packet_unit_connect req;
+      req.activity_type = activity;
+      req.unit_id = punit->id;
+      req.dest_x = connect_unit_x;
+      req.dest_y = connect_unit_y;
+      send_packet_unit_connect(&aconnection, &req);
+    }
+    else {
+      update_unit_info_label(punit);
+    }
+  }
+
+  destroy_message_dialog(w);
+  is_showing_unit_connect_dialog = FALSE;
+}
+
+/****************************************************************
+popup dialog which prompts for activity type (unit connect)
+*****************************************************************/
+void popup_unit_connect_dialog(struct unit *punit, int dest_x, int dest_y)
+{
+  Widget shell, form, dlabel, button, prev;
+  int activity;
+
+  if (is_showing_unit_connect_dialog)
+    return;
+
+  is_showing_unit_connect_dialog = TRUE;
+  unit_to_use_to_connect = punit->id;
+  connect_unit_x = dest_x;
+  connect_unit_y = dest_y;
+
+  XtSetSensitive (toplevel, FALSE);
+
+  shell = I_T(XtCreatePopupShell("unitconnectdialog", transientShellWidgetClass,
+				 toplevel, NULL, 0));
+  form = XtVaCreateManagedWidget ("form", formWidgetClass, shell, NULL);
+  dlabel = I_L(XtVaCreateManagedWidget("dlabel", labelWidgetClass, form, NULL));
+
+  prev = dlabel;
+  for (activity = ACTIVITY_IDLE + 1; activity < ACTIVITY_LAST; activity++) {
+    if (! can_unit_do_connect (punit, activity)) continue;
+
+    button =
+      XtVaCreateManagedWidget ("button", commandWidgetClass, form,
+			       XtNfromVert, prev,
+			       XtNlabel,
+			         (XtArgVal)(get_activity_text (activity)),
+			       NULL);
+    XtAddCallback (button, XtNcallback, unit_connect_callback, 
+		   (XtPointer)activity);
+    prev = button;
+  }
+  button =
+    I_L(XtVaCreateManagedWidget("closebutton", commandWidgetClass, form,
+				XtNfromVert, prev,
+				NULL));
+  XtAddCallback (button, XtNcallback, unit_connect_callback, ACTIVITY_IDLE);
+
+  xaw_set_relative_position (toplevel, shell, 10, 0);
+  XtPopup (shell, XtGrabNone);
+}
 
 /****************************************************************
   Parameters after named parameters should be in triplets:
@@ -1351,7 +1454,7 @@ Widget popup_message_dialog(Widget parent, char *dialogname, char *text, ...)
   while((fcb=va_arg(args, void *))) {
     client_data=va_arg(args, XtPointer);
     fixed_width=va_arg(args, int);
-    sprintf(button_name, "button%d", i++);
+    my_snprintf(button_name, sizeof(button_name), "button%d", i++);
     
     button=XtVaCreateManagedWidget(button_name, commandWidgetClass, 
 				   dform, NULL);
@@ -1457,7 +1560,7 @@ void popup_unit_select_dialog(struct tile *ptile)
 
     pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity);
     
-    sprintf(buffer, "%s(%s)\n%s", 
+    my_snprintf(buffer, sizeof(buffer), "%s(%s)\n%s", 
 	    punittemp->name, 
 	    pcity ? pcity->name : "",
 	    unit_activity_text(punit));
@@ -1570,11 +1673,8 @@ void unit_select_callback(Widget w, XtPointer client_data,
 }
 
 
-
-
-
 /****************************************************************
-popup the dialog 10% inside the main-window 
+popup the dialog 5% inside the main-window 
 *****************************************************************/
 void popup_races_dialog(void)
 {
@@ -1587,7 +1687,7 @@ void popup_races_dialog(void)
 
   XtVaGetValues(toplevel, XtNwidth, &width, XtNheight, &height, NULL);
 
-  XtTranslateCoords(toplevel, (Position) width/10, (Position) height/10,
+  XtTranslateCoords(toplevel, (Position) width/20, (Position) height/20,
 		    &x, &y);
   XtVaSetValues(races_dialog_shell, XtNx, x, XtNy, y, NULL);
 
@@ -1606,25 +1706,26 @@ void popdown_races_dialog(void)
   } /* else there is no dialog shell to destroy */
 }
 
-
 /****************************************************************
 ...
 *****************************************************************/
 void create_races_dialog(void)
 {
-  int i, j, len, maxracelen;
+  int per_row = 4;
+  int i, j, len, maxracelen, index;
   char maxracename[MAX_LEN_NAME];
+  char namebuf[64];
   int space;
   XtWidgetGeometry geom;
   XtTranslations textfieldtranslations;
 
   maxracelen = 0;
-  for(i=0; i<game.nation_count; i++) {
+  for(i=0; i<game.playable_nation_count; i++) {
     len = strlen(get_nation_name(i));
     maxracelen = MAX(maxracelen, len);
   }
   maxracelen = MIN(maxracelen, MAX_LEN_NAME-1);
-  sprintf(maxracename, "%*s", maxracelen+2, "W");
+  my_snprintf(maxracename, sizeof(maxracename), "%*s", maxracelen+2, "W");
 
   races_dialog_shell = I_T(XtCreatePopupShell("racespopup", 
 					  transientShellWidgetClass,
@@ -1645,60 +1746,60 @@ void create_races_dialog(void)
 					       NULL);   
 
   free(races_toggles);
-  races_toggles = fc_calloc(game.nation_count,sizeof(Widget));
+  races_toggles = fc_calloc(game.playable_nation_count,sizeof(Widget));
 
-  races_toggles[0]=XtVaCreateManagedWidget("racestoggle0", 
-					   toggleWidgetClass, 
-					   races_toggles_form,
-					   XtNlabel, maxracename,
-					   NULL);
-  if( game.nation_count > 1 )
-    races_toggles[1]=XtVaCreateManagedWidget("racestoggle1", 
-					     toggleWidgetClass, 
-					     races_toggles_form,
-					     XtNradioGroup, 
-					     races_toggles[0],
-					     XtNfromHoriz,
-					     races_toggles[0],
-					     XtNlabel, maxracename,
-					     NULL);
-  if( game.nation_count > 2 )
-    races_toggles[2]=XtVaCreateManagedWidget("racestoggle2", 
-					     toggleWidgetClass, 
-					     races_toggles_form,
-					     XtNradioGroup, 
-					     races_toggles[1],
-					     XtNfromHoriz,
-					     races_toggles[1],
-					     XtNlabel, maxracename,
-					     NULL);
+  for( i = 0; i < ((game.playable_nation_count-1)/per_row)+1; i++) {
+    index = i * per_row;
+    my_snprintf(namebuf, sizeof(namebuf), "racestoggle%d", index);
+    if( i == 0 ) {
+      races_toggles[index] =
+	XtVaCreateManagedWidget(namebuf,
+				toggleWidgetClass,
+				races_toggles_form,
+				XtNlabel, maxracename,
+				NULL);
+    } else {
+      races_toggles[index] =
+	XtVaCreateManagedWidget(namebuf,
+				toggleWidgetClass,
+				races_toggles_form,
+				XtNradioGroup,
+				races_toggles[index-1],
+				XtNfromVert,
+				races_toggles[index-per_row],
+				XtNlabel, maxracename,
+				NULL);
+    }
 
-  for( i = 1; i < (game.nation_count+2)/3; i++) {
-    int idx = i*3;
-    char buf[64];
-    sprintf(buf, "racestoggle%d", idx);
-    races_toggles[idx]=XtVaCreateManagedWidget(buf, 
-					       toggleWidgetClass, 
-					       races_toggles_form,
-					       XtNradioGroup, 
-					       races_toggles[idx-1],
-					       XtNfromVert,
-					       races_toggles[idx-3],
-					       XtNlabel, maxracename,
-					       NULL);
-    for( j=0,idx=i*3+1; (j<2) && (idx<game.nation_count); idx++,j++) {
-      sprintf(buf, "racestoggle%d", idx);
-      races_toggles[idx]=XtVaCreateManagedWidget(buf,
-						 toggleWidgetClass, 
-						 races_toggles_form,
-						 XtNradioGroup, 
-						 races_toggles[idx-1],
-						 XtNfromVert,
-						 races_toggles[idx-3],
-						 XtNfromHoriz,
-						 races_toggles[idx-1],
-						 XtNlabel, maxracename,
-					         NULL);
+    for( j = 1; j < per_row; j++) {
+      index = i * per_row + j;
+      if( index >= game.playable_nation_count ) break;
+      my_snprintf(namebuf, sizeof(namebuf), "racestoggle%d", index);
+      if( i == 0 ) {
+	races_toggles[index] =
+	  XtVaCreateManagedWidget(namebuf,
+				  toggleWidgetClass,
+				  races_toggles_form,
+				  XtNradioGroup,
+				  races_toggles[index-1],
+				  XtNfromHoriz,
+				  races_toggles[index-1],
+				  XtNlabel, maxracename,
+				  NULL);
+      } else {
+	races_toggles[index] =
+	  XtVaCreateManagedWidget(namebuf,
+				  toggleWidgetClass,
+				  races_toggles_form,
+				  XtNradioGroup,
+				  races_toggles[index-1],
+				  XtNfromVert,
+				  races_toggles[index-per_row],
+				  XtNfromHoriz,
+				  races_toggles[index-1],
+				  XtNlabel, maxracename,
+				  NULL);
+      }
     }
   }
 
@@ -1780,62 +1881,61 @@ void create_races_dialog(void)
   free(races_style_toggles);
   races_style_toggles = fc_calloc(b_s_num,sizeof(Widget));
 
-  races_style_toggles[0]=XtVaCreateManagedWidget("racesstyle0", 
-					   toggleWidgetClass, 
-					   races_style_form,
-					   XtNlabel, maxracename,
-					   NULL);
-  if( b_s_num > 1 )
-    races_style_toggles[1]=XtVaCreateManagedWidget("racesstyle1", 
-					     toggleWidgetClass, 
-					     races_style_form,
-					     XtNradioGroup, 
-					     races_style_toggles[0],
-					     XtNfromHoriz,
-					     races_style_toggles[0],
-					     XtNlabel, maxracename,
-					     NULL);
-  if( b_s_num > 2 )
-    races_style_toggles[2]=XtVaCreateManagedWidget("racesstyle2", 
-					     toggleWidgetClass, 
-					     races_style_form,
-					     XtNradioGroup, 
-					     races_style_toggles[1],
-					     XtNfromHoriz,
-					     races_style_toggles[1],
-					     XtNlabel, maxracename,
-					     NULL);
+  for( i = 0; i < ((b_s_num-1)/per_row)+1; i++) {
+    index = i * per_row;
+    my_snprintf(namebuf, sizeof(namebuf), "racesstyle%d", index);
+    if( i == 0 ) {
+      races_style_toggles[index] =
+	XtVaCreateManagedWidget(namebuf,
+				toggleWidgetClass,
+				races_style_form,
+				XtNlabel, maxracename,
+				NULL);
+    } else {
+      races_style_toggles[index] =
+	XtVaCreateManagedWidget(namebuf,
+				toggleWidgetClass,
+				races_style_form,
+				XtNradioGroup,
+				races_style_toggles[index-1],
+				XtNfromVert,
+				races_style_toggles[index-per_row],
+				XtNlabel, maxracename,
+				NULL);
+    }
 
-  for( i = 1; i < (b_s_num+2)/3; i++) {
-    int idx = i*3;
-    char buf[64];
-    sprintf(buf, "racesstyle%d", idx);
-    races_style_toggles[idx]=XtVaCreateManagedWidget(buf, 
-					       toggleWidgetClass, 
-					       races_style_form,
-					       XtNradioGroup, 
-					       races_style_toggles[idx-1],
-					       XtNfromVert,
-					       races_style_toggles[idx-3],
-					       XtNlabel, maxracename,
-					       NULL);
-    for( j=0,idx=i*3+1; (j<2) && (idx<b_s_num); idx++,j++) {
-      sprintf(buf, "racesstyle%d", idx);
-      races_style_toggles[idx]=XtVaCreateManagedWidget(buf,
-						 toggleWidgetClass, 
-						 races_style_form,
-						 XtNradioGroup, 
-						 races_style_toggles[idx-1],
-						 XtNfromVert,
-						 races_style_toggles[idx-3],
-						 XtNfromHoriz,
-						 races_style_toggles[idx-1],
-						 XtNlabel, maxracename,
-					         NULL);
+    for( j = 1; j < per_row; j++) {
+      index = i * per_row + j;
+      if( index >= b_s_num ) break;
+      my_snprintf(namebuf, sizeof(namebuf), "racesstyle%d", index);
+      if( i == 0 ) {
+	races_style_toggles[index] =
+	  XtVaCreateManagedWidget(namebuf,
+				  toggleWidgetClass,
+				  races_style_form,
+				  XtNradioGroup,
+				  races_style_toggles[index-1],
+				  XtNfromHoriz,
+				  races_style_toggles[index-1],
+				  XtNlabel, maxracename,
+				  NULL);
+      } else {
+	races_style_toggles[index] =
+	  XtVaCreateManagedWidget(namebuf,
+				  toggleWidgetClass,
+				  races_style_form,
+				  XtNradioGroup,
+				  races_style_toggles[index-1],
+				  XtNfromVert,
+				  races_style_toggles[index-per_row],
+				  XtNfromHoriz,
+				  races_style_toggles[index-1],
+				  XtNlabel, maxracename,
+				  NULL);
+      }
     }
   }
 
-
   races_action_form = XtVaCreateManagedWidget("racesactionform",
 					      formWidgetClass,
 					      races_form,
@@ -1871,7 +1971,7 @@ void create_races_dialog(void)
   }
 
 
-  for(i=0; i<game.nation_count; i++) {
+  for(i=0; i<game.playable_nation_count; i++) {
     XtAddCallback(races_toggles[i], XtNcallback, 
 		  races_toggles_callback, (XtPointer) 0);
   }
@@ -1881,7 +1981,7 @@ void create_races_dialog(void)
 		races_ok_command_callback, NULL);
 
   textfieldtranslations = 
-    XtParseTranslationTable("<Key>Return: races-dialog-returnkey()");
+    XtParseTranslationTable("<Key>Return: key-dialog-races-return()");
   XtOverrideTranslations(races_leader, textfieldtranslations);
 
 
@@ -1889,7 +1989,7 @@ void create_races_dialog(void)
 
   XtRealizeWidget(races_dialog_shell);
 
-  for(i=0; i<game.nation_count; i++) {
+  for(i=0; i<game.playable_nation_count; i++) {
     XtVaSetValues(races_toggles[i], XtNlabel, (XtArgVal)get_nation_name(i), NULL);
   }
 
@@ -1904,8 +2004,7 @@ void create_races_dialog(void)
 /****************************************************************
 ...
 *****************************************************************/
-void races_dialog_returnkey(Widget w, XEvent *event, String *params,
-			    Cardinal *num_params)
+void racesdlg_key_return(Widget w)
 {
   Widget ok = XtNameToWidget(XtParent(XtParent(w)), "*racesokcommand");
   if (ok)
@@ -1922,7 +2021,7 @@ void races_toggles_set_sensitive(int bits1, int bits2)
 
   mybits=bits1;
 
-  for(i=0; i<game.nation_count && i<32; i++) {
+  for(i=0; i<game.playable_nation_count && i<32; i++) {
     if(mybits&1)
       XtSetSensitive(races_toggles[i], FALSE);
     else
@@ -1932,7 +2031,7 @@ void races_toggles_set_sensitive(int bits1, int bits2)
 
   mybits=bits2;
 
-  for(i=32; i<game.nation_count; i++) {
+  for(i=32; i<game.playable_nation_count; i++) {
     if(mybits&1)
       XtSetSensitive(races_toggles[i], FALSE);
     else
@@ -1960,7 +2059,7 @@ void races_toggles_callback(Widget w, XtPointer client_data,
   char **leaders;
   Widget entry;
 
-  for(i=0; i<game.nation_count; i++) {
+  for(i=0; i<game.playable_nation_count; i++) {
     if(w==races_toggles[i]) {
       leaders = get_nation_leader_names(i, &leader_count);
 
@@ -2036,13 +2135,13 @@ int races_buttons_get_current(void)
   int i;
   XtPointer dp, yadp;
 
-  if(game.nation_count==1)
+  if((game.playable_nation_count)==1)
     return 0;
 
   if(!(dp=XawToggleGetCurrent(races_toggles[0])))
     return -1;
 
-  for(i=0; i<game.nation_count; i++) {
+  for(i=0; i<game.playable_nation_count; i++) {
     XtVaGetValues(races_toggles[i], XtNradioData, &yadp, NULL);
     if(dp==yadp)
       return i;
@@ -2139,8 +2238,7 @@ void races_ok_command_callback(Widget w, XtPointer client_data,
   packet.city_style = get_nation_city_style(packet.nation_no);
   packet.is_male = selected_sex? 0: 1;     /* first button is male */
   packet.city_style = city_style_idx[selected_style];
-  strncpy(packet.name, (char*)dp, MAX_LEN_NAME);
-  packet.name[MAX_LEN_NAME-1]='\0';
+  sz_strlcpy(packet.name, (char*)dp);
   
   if(!get_sane_name(packet.name)) {
     append_output_window(_("You must type a legal name."));
