@@ -23,8 +23,7 @@ Widget popup_message_dialog(Widget parent, char *shellname, char *text, ...);
 void destroy_message_dialog(Widget button);
 
 void popup_about_dialog(void);
-void races_dialog_returnkey(Widget w, XEvent *event, String *params,
-			    Cardinal *num_params);
+void racesdlg_key_return(Widget w);
 
 enum help_type_dialog {HELP_COPYING_DIALOG, HELP_KEYS_DIALOG};
 
