@@ -17,7 +17,8 @@
 
 #include "diplodlg_g.h"
 
-void diplo_dialog_returnkey(Widget w, XEvent *event, String *params,
-			    Cardinal *num_params);
+
+void diplodlg_key_return(Widget w);
+
 
 #endif  /* FC__DIPLODLG_H */
