@@ -39,36 +39,23 @@
 #include "game.h"
 #include "log.h"
 #include "map.h"
+#include "support.h"
 #include "version.h"
 
-#include "chatline.h"
-#include "citydlg.h"
-#include "cityrep.h"
+#include "actions.h"
 #include "civclient.h"
 #include "climisc.h"
 #include "clinet.h"
 #include "colors.h"
-#include "connectdlg.h"
 #include "dialogs.h"
-#include "diplodlg.h"
-#include "finddlg.h"
-#include "gotodlg.h"
 #include "graphics.h"
 #include "gui_stuff.h"		/* I_SW() */
 #include "helpdata.h"		/* boot_help_texts() */
-#include "helpdlg.h"
-#include "inputdlg.h"
-#include "mapctrl.h"
 #include "mapview.h"
 #include "menu.h"
-#include "messagewin.h"
 #include "optiondlg.h"
 #include "options.h"
-#include "plrdlg.h"
-#include "ratesdlg.h"
-#include "repodlgs.h"
 #include "resources.h"
-#include "spaceshipdlg.h"
 #include "tilespec.h"
 
 #include "gui_main.h"
@@ -78,19 +65,22 @@
 AppResources appResources;
 
 extern String fallback_resources[];
-extern char name[];
-extern char server_host[];
-extern char metaserver[];
-extern int  server_port;
 
 extern int num_units_below;
 
+
+static void setup_widgets(void);
+
+
 /**************************************************************************
 ...
 **************************************************************************/
 XtResource resources[] = {
     { "GotAppDefFile", "gotAppDefFile", XtRBoolean, sizeof(Boolean),
       XtOffset(AppResources *,gotAppDefFile), XtRImmediate, (XtPointer)False},
+    { "version", "Version", XtRString, sizeof(String),
+      XtOffset(AppResources *,version), XtRImmediate, (XtPointer)False},
+#ifdef UNUSED
     { "log", "Log", XtRString, sizeof(String),
       XtOffset(AppResources *,logfile), XtRImmediate, (XtPointer)False},
     { "name", "name", XtRString, sizeof(String),
@@ -103,21 +93,19 @@ XtResource resources[] = {
       XtOffset(AppResources *,metaserver), XtRImmediate, (XtPointer)False},
     { "logLevel", "LogLevel", XtRString, sizeof(String),
       XtOffset(AppResources *,loglevel_str), XtRImmediate, (XtPointer)False},
-    { "version", "Version", XtRString, sizeof(String),
-      XtOffset(AppResources *,version), XtRImmediate, (XtPointer)False},
-/*
     { "showHelp", "ShowHelp", XtRBoolean, sizeof(Boolean),
       XtOffset(AppResources *,showHelp), XtRImmediate, (XtPointer)False},
     { "showVersion", "ShowVersion", XtRBoolean, sizeof(Boolean),
       XtOffset(AppResources *,showVersion), XtRImmediate, (XtPointer)False},
-*/
     { "tileset", "TileSet", XtRString, sizeof(String),
       XtOffset(AppResources *,tileset), XtRImmediate, (XtPointer)False},
+#endif
 };
 
 /**************************************************************************
 ...
 **************************************************************************/
+#ifdef UNUSED
 static XrmOptionDescRec cmd_options[] = {
 /* { "-help",    ".showHelp",    XrmoptionNoArg,  (XPointer)"True" },*/
  { "-log",     ".log",         XrmoptionSepArg, (XPointer)"True" },
@@ -138,38 +126,14 @@ static XrmOptionDescRec cmd_options[] = {
  { "--tiles",   ".tileset",     XrmoptionSepArg, (XPointer)"True" }
 /* { "--version", ".showVersion", XrmoptionNoArg,  (XPointer)"True" }*/
 };
+#endif
+
+/**************************************************************************/
+
+static void end_turn_callback(Widget w, XtPointer client_data,
+			      XtPointer call_data);
 
 static void timer_callback(caddr_t client_data, XtIntervalId *id);
-static void show_info_popup(Widget w, XEvent *event, String *argv,
-			    Cardinal *argc);
-static void key_open_players(Widget w, XEvent *event, String *argv,
-			     Cardinal *argc);
-static void key_open_messages(Widget w, XEvent *event, String *argv,
-			      Cardinal *argc);
-static void key_open_rates(Widget w, XEvent *event, String *argv,
-			   Cardinal *argc);
-static void key_open_find_city(Widget w, XEvent *event, String *argv,
-			       Cardinal *argc);
-static void key_ask_revolution(Widget w, XEvent *event, String *argv,
-			       Cardinal *argc);
-static void key_open_city_report(Widget w, XEvent *event, String *argv,
-				 Cardinal *argc);
-static void key_open_military_report(Widget w, XEvent *event, String *argv,
-				     Cardinal *argc);
-static void key_open_trade_report(Widget w, XEvent *event, String *argv,
-				  Cardinal *argc);
-static void key_open_science_report(Widget w, XEvent *event, String *argv,
-				    Cardinal *argc);
-static void key_open_wonders(Widget w, XEvent *event, String *argv,
-			     Cardinal *argc);
-static void key_open_top_five(Widget w, XEvent *event, String *argv,
-			      Cardinal *argc);
-static void key_open_demographics(Widget w, XEvent *event, String *argv,
-				  Cardinal *argc);
-static void key_open_spaceship(Widget w, XEvent *event, String *argv,
-			       Cardinal *argc);
-static void quit_freeciv(Widget w, XEvent *event, String *argv,
-			 Cardinal *argc);
 
 /**************************************************************************/
 
@@ -240,79 +204,6 @@ XtInputId x_input_id;
 XtIntervalId x_interval_id;
 Atom wm_delete_window;
 
-XtActionsRec Actions[] = {
-  { "show-info-popup",  show_info_popup},
-  { "select-mapcanvas", butt_down_mapcanvas},
-  { "select-overviewcanvas", butt_down_overviewcanvas},
-  { "key-open-players", key_open_players},
-  { "key-open-messages", key_open_messages},
-  { "key-open-rates", key_open_rates},
-  { "key-open-find-city", key_open_find_city},
-  { "key-ask-revolution", key_ask_revolution},
-  { "key-open-city-report", key_open_city_report},
-  { "key-open-military-report", key_open_military_report},
-  { "key-open-trade-report", key_open_trade_report},
-  { "key-open-science-report", key_open_science_report},
-  { "key-open-wonders", key_open_wonders},
-  { "key-open-top-five", key_open_top_five},
-  { "key-open-demographics", key_open_demographics},
-  { "key-open-spaceship", key_open_spaceship},
-  { "focus-to-next-unit", focus_to_next_unit },
-  { "center-on-unit", center_on_unit },
-  { "inputline-return", inputline_return },
-  { "input-dialog-returnkey", input_dialog_returnkey},
-  { "races-dialog-returnkey", races_dialog_returnkey},
-  { "diplo-dialog-returnkey", diplo_dialog_returnkey},
-  { "city-dialog-returnkey",  city_dialog_returnkey},
-  { "connect-dialog-returnkey",  connect_dialog_returnkey},
-  { "key-unit-irrigate", xaw_key_unit_irrigate },
-  { "key-unit-road", xaw_key_unit_road },
-  { "key-unit-mine", xaw_key_unit_mine },
-  { "key-unit-transform", xaw_key_unit_transform },
-  { "key-unit-homecity", xaw_key_unit_homecity },
-  { "key-unit-clean-pollution", xaw_key_unit_clean_pollution },
-  { "key-unit-auto", xaw_key_unit_auto },
-  { "key-unit-pillage", xaw_key_unit_pillage },
-  { "key-unit-explore", xaw_key_unit_explore },
-  { "key-unit-disband", xaw_key_unit_disband },
-  { "key-unit-fortify", xaw_key_unit_fortify },
-  { "key-unit-airbase", xaw_key_unit_airbase },
-  { "key-unit-goto", xaw_key_unit_goto },
-  { "key-unit-sentry", xaw_key_unit_sentry },
-  { "key-unit-wait", xaw_key_unit_wait },
-  { "key-unit-done", xaw_key_unit_done },
-  { "key-unit-north", xaw_key_unit_north },
-  { "key-unit-north-east", xaw_key_unit_north_east },
-  { "key-unit-east", xaw_key_unit_east },
-  { "key-unit-south-east", xaw_key_unit_south_east },
-  { "key-unit-south", xaw_key_unit_south },
-  { "key-unit-south-west", xaw_key_unit_south_west },
-  { "key-unit-west", xaw_key_unit_west },
-  { "key-unit-north-west", xaw_key_unit_north_west },
-  { "key-unit-build-city", xaw_key_unit_build_city },
-  { "key-unit-unload", xaw_key_unit_unload },
-  { "key-unit-nuke", xaw_key_unit_nuke },
-  { "key-unit-wakeup", xaw_key_unit_wakeup },
-  { "wakeup", butt_down_wakeup },
-  { "key-end-turn", xaw_key_end_turn },
-  { "key-cancel-action", xaw_key_cancel_action },
-  { "select-citymap", button_down_citymap},
-  { "quit-freeciv", quit_freeciv},
-  { "close-citydialog", close_city_dialog_action},
-  { "close-sciencedialog", close_science_dialog_action },
-  { "close-tradedialog", close_trade_dialog_action },
-  { "close-activeunitsdialog", close_activeunits_dialog_action },
-  { "key-goto-dialog", popup_goto_dialog_action },
-  { "key-city-workers", key_city_workers },
-  { "adjust-workers", adjust_workers },
-  { "close-spaceshipdialog", close_spaceship_dialog_action },
-  { "spaceship-dialog-returnkey", spaceship_dialog_returnkey },
-  { "key-map-grid", xaw_key_map_grid },
-  { "close-helpdialog", close_help_dialog_action },
-  { "close-meswindialog", close_meswin_dialog_action },
-  { "close-playersdialog", close_players_dialog_action },
-  { "close-cityreport", close_city_report_action }
-};
 
 #ifdef UNUSED
 /**************************************************************************
@@ -326,6 +217,35 @@ static int myerr(Display *p, XErrorEvent *e)
 }
 #endif
 
+/**************************************************************************
+  Print extra usage information, including one line help on each option,
+  to stderr.
+**************************************************************************/
+static void print_usage(const char *argv0)
+{
+  /* add client-specific usage information here */
+  fprintf(stderr, _("Report bugs to <%s>.\n"), BUG_EMAIL_ADDRESS);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+static void parse_options(int argc, char **argv)
+{
+  int i;
+
+  i = 1;
+  while (i < argc)
+  {
+    if (is_option("--help", argv[i]))
+    {
+      print_usage(argv[0]);
+      exit(0);
+    }
+    i += 1;
+  }
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -343,19 +263,24 @@ static Boolean toplevel_work_proc(XtPointer client_data)
 **************************************************************************/
 void ui_main(int argc, char *argv[])
 {
-  int i, loglevel;
+  int i;
   Pixmap icon_pixmap; 
   XtTranslations TextFieldTranslations;
   Dimension w,h;
-  char *tile_set_name = NULL;
-  
+
+  parse_options(argc, argv);
+
   /* include later - pain to see the warning at every run */
   /* XtSetLanguageProc(NULL, (XtLanguageProc)NULL, NULL); */
   
   toplevel = XtVaAppInitialize(
 	       &app_context,               /* Application context */
 	       "Freeciv",                  /* application class name */
+#ifdef UNUSED
 	       cmd_options, XtNumber(cmd_options),
+#else
+	       NULL, 0,
+#endif
 	                                   /* command line option list */
 	       &argc, argv,                /* command line args */
 	       &fallback_resources[1],     /* for missing app-defaults file */
@@ -365,12 +290,6 @@ void ui_main(int argc, char *argv[])
   XtGetApplicationResources(toplevel, &appResources, resources,
                             XtNumber(resources), NULL, 0);
 
-  loglevel = log_parse_level_str(appResources.loglevel_str);
-  if (loglevel==-1) {
-    exit(1);
-  }
-  log_init(appResources.logfile, loglevel, NULL);
-
 /*  XSynchronize(display, 1); 
   XSetErrorHandler(myerr);*/
 
@@ -396,41 +315,12 @@ void ui_main(int argc, char *argv[])
     freelog(LOG_NORMAL, _("Using fallback resources - which is OK"));
   }
 
-  if(appResources.name) {
-    strcpy(name, appResources.name);
-  } else {
-    strcpy(name, user_username());
-  }
-
-  if(appResources.server)
-    strcpy(server_host, appResources.server);
-  else {
-    strcpy(server_host, "localhost");
-  }
-
-  if(appResources.metaserver)
-    strcpy(metaserver, appResources.metaserver);
-  else {
-    strcpy(metaserver, METALIST_ADDR);
-  }
-    
-  if(appResources.port)
-    server_port=appResources.port;
-  else
-    server_port=DEFAULT_SOCK_PORT;
-
-  if(appResources.tileset)
-    tile_set_name=appResources.tileset; 
-  
-  boot_help_texts();		/* after log_init */
-
   display = XtDisplay(toplevel);
   screen_number=XScreenNumberOfScreen(XtScreen(toplevel));
   display_depth=DefaultDepth(display, screen_number);
   root_window=DefaultRootWindow(display);
 
   display_color_type=get_visual(); 
-
   
   if(display_color_type!=COLOR_DISPLAY) {
     freelog(LOG_FATAL, _("Only color displays are supported for now..."));
@@ -445,11 +335,6 @@ void ui_main(int argc, char *argv[])
 
   init_color_system();
 
-
-  /* get tile sizes etc, required for setup_widgets: */
-  /* also, get cityname font name, required to load the font: */
-  tilespec_read_toplevel(tile_set_name);
-
   {
     XGCValues values;
 
@@ -504,10 +389,10 @@ void ui_main(int argc, char *argv[])
   XtSetKeyboardFocus(below_menu_form, map_canvas);
   
   TextFieldTranslations = XtParseTranslationTable  /*BLAH!*/
-		("<Key>Return: inputline-return()");
+		("<Key>Return: key-chatline-return()");
   XtOverrideTranslations(inputline_text, TextFieldTranslations);
 
-  XtAppAddActions(app_context, Actions, XtNumber(Actions));
+  InitializeActions(app_context);
 
   /* Do this outside setup_widgets() so after tiles are loaded */
   for(i=0;i<10;i++)  {
@@ -565,7 +450,8 @@ void ui_main(int argc, char *argv[])
 
   wm_delete_window = XInternAtom(XtDisplay(toplevel), "WM_DELETE_WINDOW", 0);
   XSetWMProtocols(display, XtWindow(toplevel), &wm_delete_window, 1);
-  XtOverrideTranslations(toplevel, XtParseTranslationTable ("<Message>WM_PROTOCOLS: quit-freeciv()"));
+  XtOverrideTranslations(toplevel,
+    XtParseTranslationTable ("<Message>WM_PROTOCOLS: msg-quit-freeciv()"));
 
   load_options();
 
@@ -653,10 +539,10 @@ void setup_widgets(void)
 					    NULL);  
   }
 
-  turn_done_button = XtVaCreateManagedWidget("turndonebutton", 
-					     commandWidgetClass,
-					     left_column_form,
-					     NULL);
+  turn_done_button = I_L(XtVaCreateManagedWidget("turndonebutton", 
+						 commandWidgetClass,
+						 left_column_form,
+						 NULL));
   
   bulb_label = XtVaCreateManagedWidget("bulblabel", 
 				       labelWidgetClass,
@@ -693,8 +579,9 @@ void setup_widgets(void)
 					   NULL);
 
   for(i=0; i<num_units_below; i++) {
-    char unit_below_name[32] = "\0";
-    sprintf(unit_below_name, "unitbelowcanvas%ld", i);
+    char unit_below_name[32];
+    my_snprintf(unit_below_name, sizeof(unit_below_name),
+		"unitbelowcanvas%ld", i);
     unit_below_canvas[i] = XtVaCreateManagedWidget(unit_below_name,
 						   pixcommWidgetClass,
 						   left_column_form, 
@@ -748,11 +635,78 @@ void setup_widgets(void)
 /**************************************************************************
 ...
 **************************************************************************/
-void quit_civ(Widget w, XtPointer client_data, XtPointer call_data)
+void main_quit_freeciv(void)
 { 
   exit(0);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void main_show_info_popup(XEvent *event)
+{
+  XButtonEvent *ev=(XButtonEvent *)event;
+  if(ev->button==Button1) {
+    Widget  p;
+    Position x, y;
+    Dimension w, h;
+    char buf[512];
+
+    my_snprintf(buf, sizeof(buf),
+		 _("%s People\n"
+		   "Year: %s\n"
+		   "Gold: %d\n"
+		   "Net Income: %d\n"
+		   "Tax:%d Lux:%d Sci:%d\n"
+		   "Researching %s: %d/%d"),
+	    int_to_text(civ_population(game.player_ptr)),
+	    textyear(game.year),
+	    game.player_ptr->economic.gold,
+	    turn_gold_difference,
+	    game.player_ptr->economic.tax,
+	    game.player_ptr->economic.luxury,
+	    game.player_ptr->economic.science,
+	    advances[game.player_ptr->research.researching].name,
+	    game.player_ptr->research.researched,
+	    research_time(game.player_ptr));
+
+    p=XtCreatePopupShell("popupinfo", 
+			 overrideShellWidgetClass, 
+			 info_command, NULL, 0);
+
+    XtAddCallback(p,XtNpopdownCallback,destroy_me_callback,NULL);
+
+    XtVaCreateManagedWidget("fullinfopopuplabel", 
+			    labelWidgetClass,
+			    p,
+			    XtNlabel, buf,
+			    NULL);
+
+    XtRealizeWidget(p);
+
+    XtVaGetValues(p, XtNwidth, &w, XtNheight, &h,  NULL);
+    XtTranslateCoords(info_command, ev->x, ev->y, &x, &y);
+    XtVaSetValues(p, XtNx, x-w/2, XtNy, y-h/2, NULL);
+    XtPopupSpringLoaded(p);
+  }
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void enable_turn_done_button(void)
+{
+  if(game.player_ptr->ai.control && !ai_manual_turn_done)
+    user_ended_turn();
+
+  update_turn_done_button(1);
+  XtSetSensitive(turn_done_button, 
+                 !game.player_ptr->ai.control||ai_manual_turn_done);
+
+  if(sound_bell_at_new_turn)
+    XBell(display, 100);
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -781,243 +735,6 @@ void remove_net_input(void)
   XtRemoveInput(x_input_id);
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-static void quit_freeciv(Widget w, XEvent *event, String *argv,
-			 Cardinal *argc)
-{
-  exit(0);
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void show_info_popup(Widget w, XEvent *event, String *argv,
-			    Cardinal *argc)
-{
-  XButtonEvent *ev=&event->xbutton;
-
-  if(ev->button==Button1) {
-    Widget  p;
-    Position x, y;
-    Dimension w, h;
-    char buf[512];
-    
-    sprintf(buf, _("%s People\n"
-		   "Year: %s\n"
-		   "Gold: %d\n"
-		   "Net Income: %d\n"
-		   "Tax:%d Lux:%d Sci:%d\n"
-		   "Researching %s: %d/%d"),
-	    int_to_text(civ_population(game.player_ptr)),
-	    textyear(game.year),
-	    game.player_ptr->economic.gold,
-	    turn_gold_difference,
-	    game.player_ptr->economic.tax,
-	    game.player_ptr->economic.luxury,
-	    game.player_ptr->economic.science,
-	    
-	    advances[game.player_ptr->research.researching].name,
-	    game.player_ptr->research.researched,
-	    research_time(game.player_ptr));
-    
-    p=XtCreatePopupShell("popupinfo", 
-			 overrideShellWidgetClass, 
-			 info_command, NULL, 0);
-
-    XtAddCallback(p,XtNpopdownCallback,destroy_me_callback,NULL);
-
-    XtVaCreateManagedWidget("fullinfopopuplabel", 
-			      labelWidgetClass,
-			      p,
-			      XtNlabel, buf,
-			      NULL);
-    
-    
-    
-    XtRealizeWidget(p);
-    
-    XtVaGetValues(p, XtNwidth, &w, XtNheight, &h,  NULL);
-    XtTranslateCoords(info_command, ev->x, ev->y, &x, &y);
-    XtVaSetValues(p, XtNx, x-w/2, XtNy, y-h/2, NULL);
-    XtPopupSpringLoaded(p);
-  }
-  
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void key_open_players(Widget w, XEvent *event, String *argv,
-			     Cardinal *argc)
-{
-  if (get_client_state()==CLIENT_GAME_RUNNING_STATE)
-    {
-      popup_players_dialog();
-    }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void key_open_messages(Widget w, XEvent *event, String *argv,
-			     Cardinal *argc)
-{
-  if (get_client_state()==CLIENT_GAME_RUNNING_STATE)
-    {
-      popup_meswin_dialog();
-    }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void key_open_rates(Widget w, XEvent *event, String *argv,
-			   Cardinal *argc)
-{
-  if (get_client_state()==CLIENT_GAME_RUNNING_STATE)
-    {
-      popup_rates_dialog();
-    }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void key_open_find_city(Widget w, XEvent *event, String *argv,
-			       Cardinal *argc)
-{
-  if (get_client_state()==CLIENT_GAME_RUNNING_STATE)
-    {
-      popup_find_dialog();
-    }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void key_ask_revolution(Widget w, XEvent *event, String *argv,
-			       Cardinal *argc)
-{
-  if (get_client_state()==CLIENT_GAME_RUNNING_STATE)
-    {
-      popup_revolution_dialog();
-    }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void key_open_city_report(Widget w, XEvent *event, String *argv,
-				 Cardinal *argc)
-{
-  if (get_client_state()==CLIENT_GAME_RUNNING_STATE)
-    {
-      popup_city_report_dialog(0);
-    }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void key_open_military_report(Widget w, XEvent *event, String *argv,
-				     Cardinal *argc)
-{
-  if (get_client_state()==CLIENT_GAME_RUNNING_STATE)
-    {
-      popup_activeunits_report_dialog(0);
-    }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void key_open_trade_report(Widget w, XEvent *event, String *argv,
-				  Cardinal *argc)
-{
-  if (get_client_state()==CLIENT_GAME_RUNNING_STATE)
-    {
-      popup_trade_report_dialog(0);
-    }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void key_open_science_report(Widget w, XEvent *event, String *argv,
-				    Cardinal *argc)
-{
-  if (get_client_state()==CLIENT_GAME_RUNNING_STATE)
-    {
-      popup_science_dialog(0);
-    }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void key_open_wonders(Widget w, XEvent *event, String *argv,
-			     Cardinal *argc)
-{
-  if (get_client_state()==CLIENT_GAME_RUNNING_STATE)
-    {
-      send_report_request(REPORT_WONDERS_OF_THE_WORLD);
-    }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void key_open_top_five(Widget w, XEvent *event, String *argv,
-			      Cardinal *argc)
-{
-  if (get_client_state()==CLIENT_GAME_RUNNING_STATE)
-    {
-      send_report_request(REPORT_TOP_5_CITIES);
-    }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void key_open_demographics(Widget w, XEvent *event, String *argv,
-				  Cardinal *argc)
-{
-  if (get_client_state()==CLIENT_GAME_RUNNING_STATE)
-    {
-      send_report_request(REPORT_DEMOGRAPHIC);
-    }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void key_open_spaceship(Widget w, XEvent *event, String *argv,
-			       Cardinal *argc)
-{
-  if ((get_client_state()==CLIENT_GAME_RUNNING_STATE) &&
-      (game.player_ptr->spaceship.state!=SSHIP_NONE))
-    {
-      popup_spaceship_dialog(game.player_ptr);
-    }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-void enable_turn_done_button(void)
-{
-  if(game.player_ptr->ai.control && !ai_manual_turn_done)
-    user_ended_turn();
-  XtSetSensitive(turn_done_button, 
-                 !game.player_ptr->ai.control||ai_manual_turn_done);
-
-  if(sound_bell_at_new_turn)
-    XBell(display, 100);
-}
-
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -1028,12 +745,10 @@ void end_turn_callback(Widget w, XtPointer client_data, XtPointer call_data)
   user_ended_turn();
 }
 
-
-
 /**************************************************************************
 ...
 **************************************************************************/
-static void timer_callback(caddr_t client_data, XtIntervalId *id)
+void timer_callback(caddr_t client_data, XtIntervalId *id)
 {
   static int flip;
   
@@ -1069,4 +784,3 @@ static void timer_callback(caddr_t client_data, XtIntervalId *id)
     flip=!flip;
   }
 }
-
