@@ -38,6 +38,7 @@
 #include "tech.h"
 #include "unit.h"
 #include "map.h"
+#include "support.h"
 #include "version.h"
 
 #include "climisc.h"
@@ -239,7 +240,7 @@ static void create_help_dialog(void)
   XSetWMProtocols(display, XtWindow(help_dialog_shell), 
 		  &wm_delete_window, 1);
   XtOverrideTranslations(help_dialog_shell,
-	 XtParseTranslationTable("<Message>WM_PROTOCOLS: close-helpdialog()"));
+	 XtParseTranslationTable("<Message>WM_PROTOCOLS: msg-close-help()"));
 
   /* create_help_page(HELP_IMPROVEMENT); */
   create_help_page(HELP_TEXT);
@@ -271,7 +272,7 @@ static void create_tech_tree(Widget tree, Widget parent, int tech, int levels)
   
   if(tech==A_LAST ||
      (advances[tech].req[0]==A_LAST && advances[tech].req[1]==A_LAST))  {
-    strcpy(label, _("Removed"));
+    sz_strlcpy(label, _("Removed"));
     bg=TREE_NODE_REMOVED_TECH_BG;
     l=XtVaCreateManagedWidget("treenode", commandWidgetClass, 
 			      tree,
@@ -282,9 +283,9 @@ static void create_tech_tree(Widget tree, Widget parent, int tech, int levels)
      return;
   }
   
-  sprintf(label,"%s:%d",advances[tech].name,
-                        tech_goal_turns(game.player_ptr,tech));
-  
+  my_snprintf(label, sizeof(label),
+	      "%s:%d",advances[tech].name,
+	      tech_goal_turns(game.player_ptr,tech));
 
   if(parent) {
     l=XtVaCreateManagedWidget("treenode", 
@@ -1221,8 +1222,7 @@ static void help_close_command_callback(Widget w, XtPointer client_data,
 /****************************************************************
 ...
 *****************************************************************/
-void close_help_dialog_action(Widget w, XEvent *event, 
-			      String *argv, Cardinal *argc)
+void helpdlg_msg_close(Widget w)
 {
   help_close_command_callback(w, NULL, NULL);
 }
