@@ -29,15 +29,19 @@
 #include "government.h"		/* government_graphic() */
 #include "map.h"
 #include "player.h"
-#include "shared.h"		/* myusleep() */
+#include "rand.h"
+#include "support.h"
+#include "timing.h"
 #include "unit.h"
 
+
 #include "civclient.h"
 #include "colors.h"
 #include "control.h" /* set_unit_focus_no_center and get_unit_in_focus */
 #include "graphics.h"
 #include "gui_stuff.h"
 #include "mapctrl.h"
+#include "options.h"
 #include "tilespec.h"
 
 #include "mapview.h"
@@ -66,12 +70,6 @@ extern int city_workers_color;
 
 extern int seconds_to_turndone;
 
-extern int use_solid_color_behind_units;
-extern int flags_are_transparent;
-extern int ai_manual_turn_done;
-extern int draw_diagonal_roads;
-extern int draw_map_grid;
-
 extern struct Sprite *intro_gfx_sprite;
 extern struct Sprite *radar_gfx_sprite;
 extern Cursor goto_cursor;
@@ -105,32 +103,71 @@ static void show_city_names(void);
 Pixmap scaled_intro_pixmap;
 int scaled_intro_pixmap_width, scaled_intro_pixmap_height;
 
+
 /**************************************************************************
 ...
 **************************************************************************/
 void decrease_unit_hp_smooth(struct unit *punit0, int hp0, 
 			     struct unit *punit1, int hp1)
 {
+  static struct timer *anim_timer = NULL; 
+  struct unit *losing_unit = (hp0 == 0 ? punit0 : punit1);
+  int i;
+
   set_unit_focus_no_center(punit0);
   set_unit_focus_no_center(punit1);
   
-  do {
+  if (!do_combat_animation) {
+    punit0->hp = hp0;
+    punit1->hp = hp1;
     refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
     refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
-    myusleep(100);
 
-    if(punit0->hp>hp0)
+    return;
+  }
+
+  do {
+    anim_timer = renew_timer_start(anim_timer, TIMER_USER, TIMER_ACTIVE);
+
+    if (punit0->hp > hp0
+	&& myrand((punit0->hp - hp0) + (punit1->hp - hp1)) < punit0->hp - hp0)
       punit0->hp--;
-    if(punit1->hp>hp1)
+    else if (punit1->hp > hp1)
       punit1->hp--;
-  } while(punit0->hp>hp0 || punit1->hp>hp1);
-  
+    else
+      punit0->hp--;
+
+    refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
+    refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
+
+    XSync(display, 0);
+    usleep_since_timer_start(anim_timer, 10000);
+
+  } while (punit0->hp > hp0 || punit1->hp > hp1);
+
+  for (i = 0; i < num_tiles_explode_unit; i++) {
+    anim_timer = renew_timer_start(anim_timer, TIMER_USER, TIMER_ACTIVE);
+
+    pixmap_put_tile(single_tile_pixmap, 0, 0,
+		    losing_unit->x, losing_unit->y, 0);
+    put_unit_pixmap(losing_unit, single_tile_pixmap, 0, 0);
+    pixmap_put_overlay_tile(single_tile_pixmap, 0, 0, sprites.explode.unit[i]);
+
+    XCopyArea(display, single_tile_pixmap, XtWindow(map_canvas), civ_gc,
+	      0, 0,
+	      NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
+	      map_canvas_adjust_x(losing_unit->x) * NORMAL_TILE_WIDTH,
+	      map_canvas_adjust_y(losing_unit->y) * NORMAL_TILE_HEIGHT );
+
+    XSync(display, 0);
+    usleep_since_timer_start(anim_timer, 20000);
+  }
+
   refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
   refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
 }
 
 
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -224,7 +261,7 @@ void update_timeout_label(void)
 {
   char buffer[512];
 
-  sprintf(buffer, "%d", seconds_to_turndone);
+  my_snprintf(buffer, sizeof(buffer), "%d", seconds_to_turndone);
   xaw_set_label(timeout_label, buffer);
 }
 
@@ -236,10 +273,11 @@ void update_info_label(void)
 {
   char buffer[512]; int d;
   
-  sprintf(buffer, _("%s People\n"
-		    "Year: %s\n"
-		    "Gold: %d\n"
-		    "Tax:%d Lux:%d Sci:%d"),
+  my_snprintf(buffer, sizeof(buffer),
+	      _("%s People\n"
+		"Year: %s\n"
+		"Gold: %d\n"
+		"Tax:%d Lux:%d Sci:%d"),
 	  int_to_text(civ_population(game.player_ptr)),
 	  textyear(game.year),
 	  game.player_ptr->economic.gold,
@@ -277,7 +315,7 @@ void update_unit_info_label(struct unit *punit)
     char buffer[512];
     struct city *pcity;
     pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity);
-    sprintf(buffer, "%s %s\n%s\n%s\n%s", 
+    my_snprintf(buffer, sizeof(buffer), "%s %s\n%s\n%s\n%s", 
 	    get_unit_type(punit->type)->name,
 	    (punit->veteran) ? _("(veteran)") : "",
 	    (goto_state==punit->id) ? 
@@ -396,7 +434,7 @@ void update_unit_pix_label(struct unit *punit)
 **************************************************************************/
 Pixmap get_thumb_pixmap(int onoff)
 {
-  return sprites.treaty_thumb[!!onoff]->pixmap;
+  return sprites.treaty_thumb[BOOL_VAL(onoff)]->pixmap;
 }
 
 /**************************************************************************
@@ -746,7 +784,7 @@ void map_canvas_expose(Widget w, XEvent *event, Region exposed,
   if(scaled_intro_pixmap) {
     XFreePixmap(display, scaled_intro_pixmap);
     scaled_intro_pixmap=0; scaled_intro_pixmap_height=0;
-  };
+  }
   
   if(map.xsize) { /* do we have a map at all */
     int tile_width=(width+NORMAL_TILE_WIDTH-1)/NORMAL_TILE_WIDTH;
@@ -1208,20 +1246,27 @@ void put_city_workers(struct city *pcity, int color)
   city_map_iterate(i, j)  {
     enum city_tile_type t=get_worker_city(pcity, i, j);
     enum city_tile_type last_t=-1;
-    if(i==2 && j==2) continue;
-    if(t==C_TILE_EMPTY) {
-      if(last_t!=t) XSetStipple(display,fill_tile_gc,gray25);
-    } else if(t==C_TILE_WORKER) {
-      if(last_t!=t) XSetStipple(display,fill_tile_gc,gray50);
-    } else continue;
-    last_t=t;
-    XCopyArea(display, map_canvas_store, XtWindow(map_canvas), civ_gc, 
-	      (x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT, 
-	      NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
-	      (x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT);
-    XFillRectangle(display, XtWindow(map_canvas), fill_tile_gc,
-		   (x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT,
-		   NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+    if(!(i==2 && j==2)) {
+      if(t==C_TILE_EMPTY) {
+	if(last_t!=t) XSetStipple(display,fill_tile_gc,gray25);
+      } else if(t==C_TILE_WORKER) {
+	if(last_t!=t) XSetStipple(display,fill_tile_gc,gray50);
+      } else continue;
+      last_t=t;
+      XCopyArea(display, map_canvas_store, XtWindow(map_canvas), civ_gc, 
+		(x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT, 
+		NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
+		(x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT);
+      XFillRectangle(display, XtWindow(map_canvas), fill_tile_gc,
+		     (x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT,
+		     NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+    }
+    if(t==C_TILE_WORKER) {
+      put_city_tile_output(XtWindow(map_canvas), x+i-2, y+j-2, 
+			   get_food_tile(i, j, pcity),
+			   get_shields_tile(i, j, pcity), 
+			   get_trade_tile(i, j, pcity) );
+    }
   }
 
   last_pcity=pcity;
