@@ -33,6 +33,7 @@
 #include "map.h"
 #include "mem.h"
 #include "registry.h"
+#include "support.h"
 #include "unit.h"
 
 #include "helpdata.h"
@@ -98,6 +99,7 @@ static void free_help_nodes(void)
 
 /****************************************************************
 ...
+ FIXME: check buffer length
 *****************************************************************/
 static void insert_generated_table(const char* name, char* outbuf)
 {
@@ -154,13 +156,24 @@ static struct help_item *new_help_item(int type)
 
 /****************************************************************
  for genlist_sort(); sort by topic via strcmp
+ (sort topics with more leading spaces after those with fewer)
 *****************************************************************/
 static int help_item_compar(const void *a, const void *b)
 {
   const struct help_item *ha, *hb;
+  char *ta, *tb;
   ha = (const struct help_item*) *(const void**)a;
   hb = (const struct help_item*) *(const void**)b;
-  return strcmp(ha->topic, hb->topic);
+  for (ta = ha->topic, tb = hb->topic; *ta && *tb; ta++, tb++) {
+    if (*ta != ' ') {
+      if (*tb == ' ') return -1;
+      break;
+    } else if (*tb != ' ') {
+      if (*ta == ' ') return 1;
+      break;
+    }
+  }
+  return strcmp(ta, tb);
 }
 
 /****************************************************************
@@ -241,7 +254,7 @@ void boot_help_texts(void)
 	  for(i=0; i<game.num_unit_types; i++) {
 	    if(unit_type_exists(i)) {
 	      pitem = new_help_item(current_type);
-	      sprintf(name, " %s", unit_name(i));
+	      my_snprintf(name, sizeof(name), " %s", unit_name(i));
 	      pitem->topic = mystrdup(name);
 	      pitem->text = mystrdup("");
 	      genlist_insert(&category_nodes, pitem, -1);
@@ -251,7 +264,7 @@ void boot_help_texts(void)
 	  for(i=A_FIRST; i<game.num_tech_types; i++) {  /* skip A_NONE */
 	    if(tech_exists(i)) {
 	      pitem = new_help_item(current_type);
-	      sprintf(name, " %s", advances[i].name);
+	      my_snprintf(name, sizeof(name), " %s", advances[i].name);
 	      pitem->topic = mystrdup(name);
 	      pitem->text = mystrdup("");
 	      genlist_insert(&category_nodes, pitem, -1);
@@ -261,16 +274,25 @@ void boot_help_texts(void)
 	  for(i=T_FIRST; i<T_COUNT; i++) {
 	    if(*(tile_types[i].terrain_name)) {
 	      pitem = new_help_item(current_type);
-	      sprintf(name, " %s", tile_types[i].terrain_name);
+	      my_snprintf(name, sizeof(name), " %s", tile_types[i].terrain_name);
 	      pitem->topic = mystrdup(name);
 	      pitem->text = mystrdup("");
 	      genlist_insert(&category_nodes, pitem, -1);
 	    }
 	  }
+	  /* Add special Civ2-style river help text if it's supplied. */
+	  if (terrain_control.river_help_text) {
+	    pitem = new_help_item(HELP_TEXT);
+	    pitem->topic = mystrdup(_("  Rivers"));
+	    strcpy(long_buffer, _(terrain_control.river_help_text));
+	    wordwrap_string(long_buffer, 68);
+	    pitem->text = mystrdup(long_buffer);
+	    genlist_insert(&category_nodes, pitem, -1);
+	  }
 	} else if(current_type==HELP_GOVERNMENT) {
 	  for(i=0; i<game.government_count; i++) {
 	      pitem = new_help_item(current_type);
-	      sprintf(name, " %s", get_government(i)->name);
+	      my_snprintf(name, sizeof(name), " %s", get_government(i)->name);
 	      pitem->topic = mystrdup(name);
 	      pitem->text = mystrdup("");
 	      genlist_insert(&category_nodes, pitem, -1);
@@ -279,7 +301,7 @@ void boot_help_texts(void)
 	  for(i=0; i<B_LAST; i++) {
 	    if(improvement_exists(i) && !is_wonder(i)) {
 	      pitem = new_help_item(current_type);
-	      sprintf(name, " %s", improvement_types[i].name);
+	      my_snprintf(name, sizeof(name), " %s", improvement_types[i].name);
 	      pitem->topic = mystrdup(name);
 	      pitem->text = mystrdup("");
 	      genlist_insert(&category_nodes, pitem, -1);
@@ -289,7 +311,7 @@ void boot_help_texts(void)
 	  for(i=0; i<B_LAST; i++) {
 	    if(improvement_exists(i) && is_wonder(i)) {
 	      pitem = new_help_item(current_type);
-	      sprintf(name, " %s", improvement_types[i].name);
+	      my_snprintf(name, sizeof(name), " %s", improvement_types[i].name);
 	      pitem->topic = mystrdup(name);
 	      pitem->text = mystrdup("");
 	      genlist_insert(&category_nodes, pitem, -1);
@@ -408,15 +430,17 @@ get_help_item_spec(const char *name, enum help_page_type htype, int *pos)
   if(!pitem) {
     idx = -1;
     vitem.topic = vtopic;
-    strncpy(vtopic, name, sizeof(vtopic)-1);
+    sz_strlcpy(vtopic, name);
     vitem.text = vtext;
     if(htype==HELP_ANY || htype==HELP_TEXT) {
-      sprintf(vtext, _("Sorry, no help topic for %s.\n"), vitem.topic);
+      my_snprintf(vtext, sizeof(vtext),
+		  _("Sorry, no help topic for %s.\n"), vitem.topic);
       vitem.type = HELP_TEXT;
     } else {
-      sprintf(vtext, _("Sorry, no help topic for %s.\n"
-	      "This page was auto-generated.\n\n"),
-	      vitem.topic);
+      my_snprintf(vtext, sizeof(vtext),
+		  _("Sorry, no help topic for %s.\n"
+		    "This page was auto-generated.\n\n"),
+		  vitem.topic);
       vitem.type = htype;
     }
     pitem = &vitem;
@@ -453,6 +477,7 @@ const struct help_item *help_iter_next(void)
 
 
 /****************************************************************
+  FIXME:
   All these helptext_* functions have a pretty crappy interface:
   we just write to buf and hope that its long enough.
   But I'm not going to fix it right now --dwp.
@@ -584,7 +609,8 @@ void helptext_wonder(char *buf, int which,
     sprintf(buf+strlen(buf), "%s  ", _(imp->helptext));
   }
   if(which==B_MICHELANGELO) {
-    helptext_cathedral_techs(buf+strlen(buf));
+    if (improvement_variant(B_MICHELANGELO)==0)
+      helptext_cathedral_techs(buf+strlen(buf));
   }
   if (strcmp(user_text, "")!=0) {
     sprintf(buf+strlen(buf), "\n\n%s", user_text);
@@ -624,6 +650,9 @@ void helptext_unit(char *buf, int i, const char *user_text)
     sprintf(buf+strlen(buf),
 	    _("* Can establish trade routes and help build wonders.\n"));
   }
+  if (unit_flag(i, F_CITIES)) {
+    sprintf(buf+strlen(buf), _("* Can build new cities.\n"));
+  }
   if (unit_flag(i, F_SETTLERS)) {
     sprintf(buf+strlen(buf), _("* Can perform settler actions.\n"));
   }
@@ -777,6 +806,12 @@ void helptext_tech(char *buf, int i, const char *user_text)
     sprintf(buf+strlen(buf), _("Allows %s to upgrade roads to railroads.\n"),units_str);
     free(units_str);
   }
+
+  if(tech_flag(i,TF_FARMLAND)) {
+    char *units_str = get_units_with_flag_string(F_SETTLERS);
+    sprintf(buf+strlen(buf), _("Allows %s to upgrade irrigation to farmland.\n"),units_str);
+    free(units_str);
+  }
 }
 
 /****************************************************************
@@ -850,10 +885,11 @@ char *helptext_unit_upkeep_str(int i)
   return buf;
 }
 
+#if 0
 /****************************************************************
   Similar to above, with eg: "1s/1h"
 *****************************************************************/
-char *helptext_unit_upkeep_str_short(int i)
+static char *helptext_unit_upkeep_str_short(int i)
 {
   static char buf[32];
   struct unit_type *utype = get_unit_type(i);
@@ -883,3 +919,4 @@ char *helptext_unit_upkeep_str_short(int i)
   }
   return buf;
 }
+#endif
