@@ -14,6 +14,9 @@
 #define FC__MAPCTRL_G_H
 
 struct unit;
+struct city;
+
+extern struct city *city_workers_display;
 
 void popup_newcity_dialog(struct unit *punit, char *suggestname);
 
