@@ -16,13 +16,14 @@
 extern int use_solid_color_behind_units;
 extern int sound_bell_at_new_turn;
 extern int smooth_move_units;
-extern int flags_are_transparent;
+extern int do_combat_animation;
 extern int ai_popup_windows;
 extern int ai_manual_turn_done;
 extern int auto_center_on_unit;
 extern int wakeup_focus;
 extern int draw_diagonal_roads;
 extern int center_when_popup_city;
+extern int concise_city_production;
 extern int draw_map_grid;
 
 typedef struct {
@@ -35,9 +36,6 @@ typedef struct {
 } client_option;
 extern client_option options[];
 
-#define GEN_OPTION(name, description) { #name, description, &name, NULL }
-#define NULL_OPTION { NULL, NULL, NULL, NULL }
-
 
 /** Message Options: **/
 
