@@ -31,7 +31,9 @@
 #include "nation.h"
 #include "packets.h"
 #include "spaceship.h"
+#include "support.h"
 #include "unit.h"
+#include "worklist.h"
 
 #include "chatline_g.h"
 #include "citydlg_g.h"
@@ -53,6 +55,7 @@
 #include "repodlgs_g.h"
 #include "spaceshipdlg_g.h"
 #include "tilespec.h"
+#include "wldlg_g.h"
 
 #include "packhand.h"
 
@@ -60,7 +63,6 @@ extern int seconds_to_turndone;
 extern int last_turn_gold_amount;
 extern int did_advance_tech_this_turn;
 extern char name[512];
-extern struct city *city_workers_display;
 
 /**************************************************************************
 ...
@@ -70,19 +72,22 @@ void handle_join_game_reply(struct packet_join_game_reply *packet)
   char msg[MAX_LEN_MSG];
   char *s_capability = aconnection.capability;
 
-  strcpy(s_capability, packet->capability);
+  mystrlcpy(s_capability, packet->capability, sizeof(aconnection.capability));
 
   if (packet->you_can_join) {
     freelog(LOG_VERBOSE, "join game accept:%s", packet->message);
   } else {
-    sprintf(msg, _("You were rejected from the game: %s"), packet->message);
+    my_snprintf(msg, sizeof(msg),
+		_("You were rejected from the game: %s"), packet->message);
     append_output_window(msg);
   }
   if (strcmp(s_capability, our_capability)==0)
     return;
-  sprintf(msg, _("Client capability string: %s"), our_capability);
+  my_snprintf(msg, sizeof(msg),
+	      _("Client capability string: %s"), our_capability);
   append_output_window(msg);
-  sprintf(msg, _("Server capability string: %s"), s_capability);
+  my_snprintf(msg, sizeof(msg),
+	      _("Server capability string: %s"), s_capability);
   append_output_window(msg);
 }
 
@@ -184,7 +189,7 @@ void handle_city_info(struct packet_city_info *packet)
   pcity->owner=packet->owner;
   pcity->x=packet->x;
   pcity->y=packet->y;
-  strcpy(pcity->name, packet->name);
+  sz_strlcpy(pcity->name, packet->name);
   
   pcity->size=packet->size;
   pcity->ppl_happy[4]=packet->ppl_happy;
@@ -218,15 +223,24 @@ void handle_city_info(struct packet_city_info *packet)
     
   pcity->is_building_unit=packet->is_building_unit;
   pcity->currently_building=packet->currently_building;
+  if (city_is_new)
+    pcity->worklist = create_worklist();
+  copy_worklist(pcity->worklist, &packet->worklist);
   pcity->did_buy=packet->did_buy;
   pcity->did_sell=packet->did_sell;
   pcity->was_happy=packet->was_happy;
   pcity->airlift=packet->airlift;
   
   i=0;
-  for(y=0; y<CITY_MAP_SIZE; y++)
-    for(x=0; x<CITY_MAP_SIZE; x++)
+  for(y=0; y<CITY_MAP_SIZE; y++) {
+    for(x=0; x<CITY_MAP_SIZE; x++) {
+      if (city_is_new) {
+	/* Need to pre-initialize before set_worker_city()  -- dwp */
+	pcity->city_map[x][y] = C_TILE_EMPTY;
+      }
       set_worker_city(pcity,x,y,packet->city_map[i++]-'0');
+    }
+  }
     
   for(i=0; i<B_LAST; i++)
     pcity->improvements[i]=(packet->improvements[i]=='1') ? 1 : 0;
@@ -532,6 +546,7 @@ void handle_unit_info(struct packet_unit_info *packet)
     punit->goto_dest_x=packet->goto_dest_x;
     punit->goto_dest_y=packet->goto_dest_y;
     punit->paradropped=packet->paradropped;
+    punit->connecting=packet->connecting;
   }
   
   else {      /* create new unit */
@@ -557,6 +572,7 @@ void handle_unit_info(struct packet_unit_info *packet)
     punit->activity_target=packet->activity_target;
     punit->ai.control=packet->ai;
     punit->paradropped=packet->paradropped;
+    punit->connecting=packet->connecting;
     
     punit->activity_count=0;	/* never used in client/ or common/  --dwp */
     punit->bribe_cost=0;	/* done by handle_incite_cost() */
@@ -666,7 +682,7 @@ void handle_player_info(struct packet_player_info *pinfo)
   char msg[MAX_LEN_MSG];
   struct player *pplayer=&game.players[pinfo->playerno];
 
-  strcpy(pplayer->name, pinfo->name);
+  sz_strlcpy(pplayer->name, pinfo->name);
   pplayer->nation=pinfo->nation;
   pplayer->is_male=pinfo->is_male;
 
@@ -678,6 +694,9 @@ void handle_player_info(struct packet_player_info *pinfo)
   pplayer->embassy=pinfo->embassy;
   pplayer->city_style=pinfo->city_style;
 
+  for (i = 0; i < MAX_NUM_WORKLISTS; i++)
+    copy_worklist(&pplayer->worklists[i], &pinfo->worklists[i]);
+
   for(i=0; i<game.num_tech_types; i++)
     pplayer->research.inventions[i]=pinfo->inventions[i]-'0';
   update_research(pplayer);
@@ -698,10 +717,10 @@ void handle_player_info(struct packet_player_info *pinfo)
     pplayer->conn->player = NULL;
   }
 
-  strcpy(pplayer->conn->capability, pinfo->capability);
+  sz_strlcpy(pplayer->conn->capability, pinfo->capability);
   
   if(get_client_state()==CLIENT_GAME_RUNNING_STATE && pplayer==game.player_ptr) {
-    strcpy(name, pplayer->name);
+    sz_strlcpy(name, pplayer->name);
     if(poptechup) {
       if(!game.player_ptr->ai.control || ai_popup_windows)
 	popup_science_dialog(1);
@@ -715,13 +734,15 @@ void handle_player_info(struct packet_player_info *pinfo)
   pplayer->is_alive=pinfo->is_alive;
   
   pplayer->is_connected=pinfo->is_connected;
-  strcpy(pplayer->addr, pinfo->addr);
+  sz_strlcpy(pplayer->addr, pinfo->addr);
 
+  pplayer->ai.is_barbarian=pinfo->is_barbarian;
   pplayer->revolution=pinfo->revolution;
   if(pplayer->ai.control!=pinfo->ai)  {
     pplayer->ai.control=pinfo->ai;
     if(pplayer==game.player_ptr)  {
-      sprintf(msg,_("AI Mode is now %s."),game.player_ptr->ai.control?_("ON"):_("OFF"));
+      my_snprintf(msg, sizeof(msg), _("AI Mode is now %s."),
+		  game.player_ptr->ai.control?_("ON"):_("OFF"));
       append_output_window(msg);
     }
   }
@@ -734,11 +755,14 @@ void handle_player_info(struct packet_player_info *pinfo)
     popup_government_dialog();
   
   update_players_dialog();
+  update_worklist_report_dialog();
 
   if(pplayer==game.player_ptr) {
     if(get_client_state()==CLIENT_GAME_RUNNING_STATE) {
       if (!game.player_ptr->turn_done) {
 	enable_turn_done_button();
+      } else {
+	update_turn_done_button(1);
       }
       update_info_label();
     }
@@ -1034,6 +1058,7 @@ void handle_ruleset_control(struct packet_ruleset_control *packet)
       
   game.aqueduct_size = packet->aqueduct_size;
   game.sewer_size = packet->sewer_size;
+  game.add_to_size_limit = packet->add_to_size_limit;
   
   game.rtech.get_bonus_tech = packet->rtech.get_bonus_tech;
   game.rtech.cathedral_plus = packet->rtech.cathedral_plus;
@@ -1061,6 +1086,7 @@ void handle_ruleset_control(struct packet_ruleset_control *packet)
   free_nations(game.nation_count);
   game.nation_count = packet->nation_count;
   alloc_nations(game.nation_count);
+  game.playable_nation_count = packet->playable_nation_count;
 
   tilespec_free_city_tiles(game.styles_count);
   free(city_styles);
@@ -1083,9 +1109,9 @@ void handle_ruleset_unit(struct packet_ruleset_unit *p)
   }
   u = get_unit_type(p->id);
 
-  strcpy(u->name, p->name);
-  strcpy(u->graphic_str, p->graphic_str);
-  strcpy(u->graphic_alt, p->graphic_alt);
+  sz_strlcpy(u->name, p->name);
+  sz_strlcpy(u->graphic_str, p->graphic_str);
+  sz_strlcpy(u->graphic_alt, p->graphic_alt);
   u->move_type          = p->move_type;
   u->build_cost         = p->build_cost;
   u->attack_strength    = p->attack_strength;
@@ -1128,7 +1154,7 @@ void handle_ruleset_tech(struct packet_ruleset_tech *p)
   }
   a = &advances[p->id];
 
-  strcpy(a->name, p->name);
+  sz_strlcpy(a->name, p->name);
   a->req[0] = p->req[0];
   a->req[1] = p->req[1];
   a->flags = p->flags;
@@ -1152,7 +1178,7 @@ void handle_ruleset_building(struct packet_ruleset_building *p)
   }
   b = &improvement_types[p->id];
 
-  strcpy(b->name, p->name);
+  sz_strlcpy(b->name, p->name);
   b->is_wonder        = p->is_wonder;
   b->tech_requirement = p->tech_requirement;
   b->build_cost       = p->build_cost;      
@@ -1187,6 +1213,7 @@ void handle_ruleset_government(struct packet_ruleset_government *p)
   gov->martial_law_max   = p->martial_law_max;
   gov->martial_law_per   = p->martial_law_per;
   gov->empire_size_mod   = p->empire_size_mod;
+  gov->empire_size_inc   = p->empire_size_inc;
   gov->rapture_size      = p->rapture_size;
   
   gov->unit_happy_cost_factor  = p->unit_happy_cost_factor;
@@ -1225,9 +1252,9 @@ void handle_ruleset_government(struct packet_ruleset_government *p)
   gov->hints               = p->hints;
   gov->num_ruler_titles    = p->num_ruler_titles;
     
-  strcpy(gov->name, p->name);
-  strcpy(gov->graphic_str, p->graphic_str);
-  strcpy(gov->graphic_alt, p->graphic_alt);
+  sz_strlcpy(gov->name, p->name);
+  sz_strlcpy(gov->graphic_str, p->graphic_str);
+  sz_strlcpy(gov->graphic_alt, p->graphic_alt);
 
   gov->ruler_titles = fc_calloc(gov->num_ruler_titles,
 				sizeof(struct ruler_title));
@@ -1253,8 +1280,8 @@ void handle_ruleset_government_ruler_title
     return;
   }
   gov->ruler_titles[p->id].nation = p->nation;
-  strcpy(gov->ruler_titles[p->id].male_title, p->male_title);
-  strcpy(gov->ruler_titles[p->id].female_title, p->female_title);
+  sz_strlcpy(gov->ruler_titles[p->id].male_title, p->male_title);
+  sz_strlcpy(gov->ruler_titles[p->id].female_title, p->female_title);
 }
 
 /**************************************************************************
@@ -1273,26 +1300,26 @@ void handle_ruleset_terrain(struct packet_ruleset_terrain *p)
   }
   t = &(tile_types[p->id]);
 
-  strcpy(t->terrain_name, p->terrain_name);
-  strcpy(t->graphic_str, p->graphic_str);
-  strcpy(t->graphic_alt, p->graphic_alt);
+  sz_strlcpy(t->terrain_name, p->terrain_name);
+  sz_strlcpy(t->graphic_str, p->graphic_str);
+  sz_strlcpy(t->graphic_alt, p->graphic_alt);
   t->movement_cost = p->movement_cost;
   t->defense_bonus = p->defense_bonus;
   t->food = p->food;
   t->shield = p->shield;
   t->trade = p->trade;
-  strcpy(t->special_1_name, p->special_1_name);
+  sz_strlcpy(t->special_1_name, p->special_1_name);
   t->food_special_1 = p->food_special_1;
   t->shield_special_1 = p->shield_special_1;
   t->trade_special_1 = p->trade_special_1;
-  strcpy(t->special_2_name, p->special_2_name);
+  sz_strlcpy(t->special_2_name, p->special_2_name);
   t->food_special_2 = p->food_special_2;
   t->shield_special_2 = p->shield_special_2;
   t->trade_special_2 = p->trade_special_2;
 
   for(j=0; j<2; j++) {
-    strcpy(t->special[j].graphic_str, p->special[j].graphic_str);
-    strcpy(t->special[j].graphic_alt, p->special[j].graphic_alt);
+    sz_strlcpy(t->special[j].graphic_str, p->special[j].graphic_str);
+    sz_strlcpy(t->special[j].graphic_alt, p->special[j].graphic_alt);
   }
 
   t->road_time = p->road_time;
@@ -1325,6 +1352,8 @@ void handle_ruleset_terrain_control(struct terrain_misc *p)
   terrain_control.river_move_mode = p->river_move_mode;
   terrain_control.river_defense_bonus = p->river_defense_bonus;
   terrain_control.river_trade_incr = p->river_trade_incr;
+  terrain_control.river_help_text =
+    p->river_help_text ? mystrdup(p->river_help_text) : NULL;
   terrain_control.fortress_defense_bonus = p->fortress_defense_bonus;
   terrain_control.road_superhighway_trade_bonus = p->road_superhighway_trade_bonus;
   terrain_control.rail_food_bonus = p->rail_food_bonus;
@@ -1351,14 +1380,13 @@ void handle_ruleset_nation(struct packet_ruleset_nation *p)
   }
   pl = get_nation_by_idx(p->id);
 
-  strcpy(pl->name, p->name);
-  strcpy(pl->name_plural, p->name_plural);
-  strcpy(pl->flag_graphic_str, p->graphic_str);
-  strcpy(pl->flag_graphic_alt, p->graphic_alt);
+  sz_strlcpy(pl->name, p->name);
+  sz_strlcpy(pl->name_plural, p->name_plural);
+  sz_strlcpy(pl->flag_graphic_str, p->graphic_str);
+  sz_strlcpy(pl->flag_graphic_alt, p->graphic_alt);
   pl->leader_count = p->leader_count;
   for( i=0; i<pl->leader_count; i++) {
-    pl->leader_name[i] = fc_calloc(strlen(p->leader_name[i])+1, sizeof(char));
-    strcpy(pl->leader_name[i], p->leader_name[i]);
+    pl->leader_name[i] = mystrdup(p->leader_name[i]);
     pl->leader_is_male[i] = p->leader_sex[i];
   }
   pl->city_style = p->city_style;
@@ -1372,14 +1400,22 @@ void handle_ruleset_nation(struct packet_ruleset_nation *p)
 void handle_ruleset_city(struct packet_ruleset_city *packet)
 {
   int id;
+  struct citystyle *cs;
 
   id = packet->style_id;
-  city_styles[id].techreq = packet->techreq;
-  city_styles[id].replaced_by = packet->replaced_by;
+  if (id < 0 || id >= game.styles_count) {
+    freelog(LOG_NORMAL, "Received bad citystyle id %d in handle_ruleset_city()",
+	    id);
+    return;
+  }
+  cs = &city_styles[id];
+  
+  cs->techreq = packet->techreq;
+  cs->replaced_by = packet->replaced_by;
 
-  strcpy(city_styles[id].name, packet->name);
-  strcpy(city_styles[id].graphic, packet->graphic);
-  strcpy(city_styles[id].graphic_alt, packet->graphic_alt);
+  sz_strlcpy(cs->name, packet->name);
+  sz_strlcpy(cs->graphic, packet->graphic);
+  sz_strlcpy(cs->graphic_alt, packet->graphic_alt);
 
   tilespec_setup_city_tiles(id);
 }
