@@ -37,6 +37,7 @@
 #include "player.h"
 #include "registry.h"
 #include "shared.h"
+#include "support.h"
 #include "unit.h"
 
 #include "colors_g.h"
@@ -58,6 +59,10 @@ int SMALL_TILE_HEIGHT;
 
 char *city_names_font;
 
+int flags_are_transparent=1;
+
+int num_tiles_explode_unit=0;
+
 
 static int num_spec_files = 0;
 static char **spec_filenames;	/* full pathnames */
@@ -84,8 +89,25 @@ static struct athing atile_sprites;   /* .ptr is tile_sprites */
 */
 static struct section_file tag_sf;
 
+
+#define TILESPEC_CAPSTR "+tilespec2 duplicates_ok"
+/*
+   Tilespec capabilities acceptable to this program:
+   +tilespec2     -  basic format, required
+   duplicates_ok  -  we can handle existence of duplicate tags
+                     (lattermost tag which appears is used; tilesets which
+		     have duplicates should specify "+duplicates_ok")
+*/
+
+#define SPEC_CAPSTR "+spec2"
+/*
+   Individual spec file capabilities acceptable to this program:
+   +spec2          -  basic format, required
+*/
+
+
 /***************************************************************************
-  Crop sprite for source and store in gloabl tile_sprites.
+  Crop sprite for source and store in global tile_sprites.
   Reallocates tile_sprites as required, using an athing to
   keep track.  (Note this is not as bad as it sounds: we
   are only reallocing the pointers to sprites, not the sprites
@@ -195,7 +217,7 @@ static void check_tilespec_capabilities(struct section_file *file,
   Returns the correct name of the gfx file (with path and extension)
   Must be free'd when no longer used
 ***********************************************************************/
-char *tilespec_gfx_filename(const char *gfx_filename)
+static char *tilespec_gfx_filename(const char *gfx_filename)
 {
   char **gfx_fileexts;
   char *full_name,*real_full_name,*gfx_current_fileext;
@@ -204,7 +226,7 @@ char *tilespec_gfx_filename(const char *gfx_filename)
 
   while((gfx_current_fileext = *gfx_fileexts++))
   {
-    full_name = (char*)fc_malloc(strlen(gfx_filename)+strlen(gfx_current_fileext)+2);
+    full_name = fc_malloc(strlen(gfx_filename)+strlen(gfx_current_fileext)+2);
     sprintf(full_name,"%s.%s",gfx_filename,gfx_current_fileext);
 
     real_full_name = datafilename(full_name);
@@ -235,7 +257,7 @@ void tilespec_read_toplevel(const char *tileset_name)
     freelog(LOG_FATAL, _("Could not open \"%s\"."), fname);
     exit(1);
   }
-  check_tilespec_capabilities(file, "tilespec", "+tilespec2", fname);
+  check_tilespec_capabilities(file, "tilespec", TILESPEC_CAPSTR, fname);
 
   section_file_lookup(file, "tilespec.name"); /* currently unused */
 
@@ -250,8 +272,8 @@ void tilespec_read_toplevel(const char *tileset_name)
   c = secfile_lookup_str_default(file, "10x20", "tilespec.city_names_font");
   city_names_font = mystrdup(c);
 
-  /* TODO: use this */
-  section_file_lookup(file, "tilespec.flags_are_transparent"); 
+  flags_are_transparent =
+    secfile_lookup_int(file, "tilespec.flags_are_transparent"); 
 
   c = secfile_lookup_str(file, "tilespec.main_intro_file");
   main_intro_filename = tilespec_gfx_filename(c);
@@ -303,7 +325,7 @@ static void tilespec_load_one(const char *spec_filename)
     freelog(LOG_FATAL, _("Could not open \"%s\"."), spec_filename);
     exit(1);
   }
-  check_tilespec_capabilities(file, "spec", "+spec2", spec_filename);
+  check_tilespec_capabilities(file, "spec", SPEC_CAPSTR, spec_filename);
 
   section_file_lookup(file, "info.artists"); /* currently unused */
 
@@ -313,7 +335,7 @@ static void tilespec_load_one(const char *spec_filename)
   while((!big_sprite) && (gfx_current_fileext = *gfx_fileexts++))
   {
     char *full_name,*real_full_name;
-    full_name = (char*)fc_malloc(strlen(gfx_filename)+strlen(gfx_current_fileext)+2);
+    full_name = fc_malloc(strlen(gfx_filename)+strlen(gfx_current_fileext)+2);
     sprintf(full_name,"%s.%s",gfx_filename,gfx_current_fileext);
 
     if((real_full_name = datafilename(full_name)))
@@ -382,8 +404,8 @@ static char *nsew_str(int idx)
 {
   static char c[] = "n0s0e0w0";
 
-  sprintf(c, "n%ds%de%dw%d", !!(idx&BIT_NORTH), !!(idx&BIT_SOUTH),
-     	                     !!(idx&BIT_EAST), !!(idx&BIT_WEST));
+  sprintf(c, "n%ds%de%dw%d", BOOL_VAL(idx&BIT_NORTH), BOOL_VAL(idx&BIT_SOUTH),
+     	                     BOOL_VAL(idx&BIT_EAST),  BOOL_VAL(idx&BIT_WEST));
   return c;
 }
      
@@ -407,9 +429,9 @@ static void tilespec_lookup_sprite_tags(void)
   SET_SPRITE(treaty_thumb[1], "treaty.agree_thumb_up");
 
   for(i=0; i<NUM_TILES_PROGRESS; i++) {
-    sprintf(buffer, "s.science_bulb_%d", i);
+    my_snprintf(buffer, sizeof(buffer), "s.science_bulb_%d", i);
     SET_SPRITE(bulb[i], buffer);
-    sprintf(buffer, "s.warming_sun_%d", i);
+    my_snprintf(buffer, sizeof(buffer), "s.warming_sun_%d", i);
     SET_SPRITE(warming[i], buffer);
   }
 
@@ -437,26 +459,42 @@ static void tilespec_lookup_sprite_tags(void)
   SET_SPRITE(rail.isolated, "r.rail_isolated");
   
   for(i=1; i<NUM_DIRECTION_NSEW; i++) {
-    sprintf(buffer, "r.c_road_%s", nsew_str(i));
+    my_snprintf(buffer, sizeof(buffer), "r.c_road_%s", nsew_str(i));
     SET_SPRITE(road.cardinal[i], buffer);
     
-    sprintf(buffer, "r.d_road_%s", nsew_str(i));
+    my_snprintf(buffer, sizeof(buffer), "r.d_road_%s", nsew_str(i));
     SET_SPRITE(road.diagonal[i], buffer);
     
-    sprintf(buffer, "r.c_rail_%s", nsew_str(i));
+    my_snprintf(buffer, sizeof(buffer), "r.c_rail_%s", nsew_str(i));
     SET_SPRITE(rail.cardinal[i], buffer);
     
-    sprintf(buffer, "r.d_rail_%s", nsew_str(i));
+    my_snprintf(buffer, sizeof(buffer), "r.d_rail_%s", nsew_str(i));
     SET_SPRITE(rail.diagonal[i], buffer);
   }
 
   for(i=0; i<3; i++) {
     for(j=0; j<3; j++) {
-      sprintf(buffer, "explode.nuke_%d%d", i, j);
+      my_snprintf(buffer, sizeof(buffer), "explode.nuke_%d%d", i, j);
       SET_SPRITE(explode.nuke[i][j], buffer);
     }
   }
 
+  num_tiles_explode_unit = 0;
+  while (section_file_lookup(&tag_sf, "explode.unit_%d", num_tiles_explode_unit))
+    num_tiles_explode_unit++;
+
+  if (num_tiles_explode_unit==0) {
+    sprites.explode.unit = NULL;
+  } else {
+    sprites.explode.unit = fc_calloc(num_tiles_explode_unit,
+				     sizeof(struct Sprite *));
+
+    for (i = 0; i < num_tiles_explode_unit; i++) {
+      my_snprintf(buffer, sizeof(buffer), "explode.unit_%d", i);
+      SET_SPRITE(explode.unit[i], buffer);
+    }
+  }
+
   SET_SPRITE(unit.auto_attack,  "unit.auto_attack");
   SET_SPRITE(unit.auto_settler, "unit.auto_settler");
   SET_SPRITE(unit.auto_explore, "unit.auto_explore");
@@ -472,26 +510,27 @@ static void tilespec_lookup_sprite_tags(void)
   SET_SPRITE(unit.sentry,	"unit.sentry");      
   SET_SPRITE(unit.stack,	"unit.stack");
   SET_SPRITE(unit.transform,    "unit.transform");   
+  SET_SPRITE(unit.connect,      "unit.connect");   
 
   for(i=0; i<NUM_TILES_HP_BAR; i++) {
-    sprintf(buffer, "unit.hp_%d", i*10);
+    my_snprintf(buffer, sizeof(buffer), "unit.hp_%d", i*10);
     SET_SPRITE(unit.hp_bar[i], buffer);
   }
 
   SET_SPRITE(city.disorder, "city.disorder");
 
   for(i=0; i<NUM_TILES_DIGITS; i++) {
-    sprintf(buffer, "city.size_%d", i);
+    my_snprintf(buffer, sizeof(buffer), "city.size_%d", i);
     SET_SPRITE(city.size[i], buffer);
     if(i!=0) {
-      sprintf(buffer, "city.size_%d", i*10);
+      my_snprintf(buffer, sizeof(buffer), "city.size_%d", i*10);
       SET_SPRITE(city.size_tens[i], buffer);
     }
-    sprintf(buffer, "city.t_food_%d", i);
+    my_snprintf(buffer, sizeof(buffer), "city.t_food_%d", i);
     SET_SPRITE(city.tile_foodnum[i], buffer);
-    sprintf(buffer, "city.t_shields_%d", i);
+    my_snprintf(buffer, sizeof(buffer), "city.t_shields_%d", i);
     SET_SPRITE(city.tile_shieldnum[i], buffer);
-    sprintf(buffer, "city.t_trade_%d", i);
+    my_snprintf(buffer, sizeof(buffer), "city.t_trade_%d", i);
     SET_SPRITE(city.tile_tradenum[i], buffer);
   }
 
@@ -513,22 +552,22 @@ static void tilespec_lookup_sprite_tags(void)
   SET_SPRITE(tx.airbase,    "tx.airbase");
 
   for(i=0; i<NUM_DIRECTION_NSEW; i++) {
-    sprintf(buffer, "tx.s_river_%s", nsew_str(i));
+    my_snprintf(buffer, sizeof(buffer), "tx.s_river_%s", nsew_str(i));
     SET_SPRITE(tx.spec_river[i], buffer);
   }
   for(i=1; i<NUM_DIRECTION_NSEW; i++) {
-    sprintf(buffer, "tx.darkness_%s", nsew_str(i));
+    my_snprintf(buffer, sizeof(buffer), "tx.darkness_%s", nsew_str(i));
     SET_SPRITE(tx.darkness[i], buffer);
-    sprintf(buffer, "tx.coast_cape_%s", nsew_str(i));
+    my_snprintf(buffer, sizeof(buffer), "tx.coast_cape_%s", nsew_str(i));
     SET_SPRITE(tx.coast_cape[i], buffer);
   }
   for(i=0; i<4; i++) {
-    sprintf(buffer, "tx.river_outlet_%c", dir_char[i]);
+    my_snprintf(buffer, sizeof(buffer), "tx.river_outlet_%c", dir_char[i]);
     SET_SPRITE(tx.river_outlet[i], buffer);
   }
   for(i=0; i<2; i++) {
     for(j=0; j<3; j++) {
-      sprintf(buffer, "tx.denmark_%d%d", i, j);
+      my_snprintf(buffer, sizeof(buffer), "tx.denmark_%d%d", i, j);
       SET_SPRITE(tx.denmark[i][j], buffer);
     }
   }
@@ -554,7 +593,7 @@ void tilespec_load_tiles(void)
   }
   final_realloc_tile_sprites();
 
-  secfilehash_build(&tag_sf);
+  secfilehash_build(&tag_sf, 1);
   /* section_file_save(&tag_sf, "_debug_tilespec"); */
 
   tilespec_lookup_sprite_tags();
@@ -635,8 +674,8 @@ void tilespec_setup_tile_type(int id)
   }
   for(i=0; i<NUM_DIRECTION_NSEW; i++) {
     nsew = nsew_str(i);
-    sprintf(buffer1, "%s_%s", tt->graphic_str, nsew);
-    sprintf(buffer2, "%s_%s", tt->graphic_alt, nsew);
+    my_snprintf(buffer1, sizeof(buffer1), "%s_%s", tt->graphic_str, nsew);
+    my_snprintf(buffer2, sizeof(buffer1), "%s_%s", tt->graphic_alt, nsew);
     
     tt->sprite[i] = lookup_sprite_tag_alt(buffer1, buffer2, 1, "tile_type",
 					  tt->terrain_name);
@@ -827,6 +866,7 @@ int fill_unit_sprite_array(struct Sprite **sprs, struct unit *punit)
       break;
     case ACTIVITY_TRANSFORM:
       s = sprites.unit.transform;
+      break;
     default:
       break;
     }
@@ -842,6 +882,10 @@ int fill_unit_sprite_array(struct Sprite **sprs, struct unit *punit)
     }
   }
 
+  if (punit->connecting) {
+    *sprs++ = sprites.unit.connect;
+  }
+
   ihp = ((NUM_TILES_HP_BAR-1)*punit->hp) / get_unit_type(punit->type)->hp;
   ihp = CLIP(0, ihp, NUM_TILES_HP_BAR-1);
   *sprs++ = sprites.unit.hp_bar[ihp];
@@ -991,31 +1035,31 @@ int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int cit
   if((tspecial & S_ROAD) || (tspecial & S_RAILROAD)) {
     int n, s, e, w;
     
-    n = !!(tspecial_north&S_RAILROAD);
-    s = !!(tspecial_south&S_RAILROAD);
-    e = !!(tspecial_east&S_RAILROAD);
-    w = !!(tspecial_west&S_RAILROAD);
+    n = BOOL_VAL(tspecial_north&S_RAILROAD);
+    s = BOOL_VAL(tspecial_south&S_RAILROAD);
+    e = BOOL_VAL(tspecial_east&S_RAILROAD);
+    w = BOOL_VAL(tspecial_west&S_RAILROAD);
     rail_card_count = n + s + e + w;
     rail_card_tileno = INDEX_NSEW(n,s,e,w);
     
-    n = !!(tspecial_north&S_ROAD);
-    s = !!(tspecial_south&S_ROAD);
-    e = !!(tspecial_east&S_ROAD);
-    w = !!(tspecial_west&S_ROAD);
+    n = BOOL_VAL(tspecial_north&S_ROAD);
+    s = BOOL_VAL(tspecial_south&S_ROAD);
+    e = BOOL_VAL(tspecial_east&S_ROAD);
+    w = BOOL_VAL(tspecial_west&S_ROAD);
     road_card_count = n + s + e + w;
     road_card_tileno = INDEX_NSEW(n,s,e,w);
     
-    n = !!(tspecial_north_east&S_RAILROAD);
-    s = !!(tspecial_south_west&S_RAILROAD);
-    e = !!(tspecial_south_east&S_RAILROAD);
-    w = !!(tspecial_north_west&S_RAILROAD);
+    n = BOOL_VAL(tspecial_north_east&S_RAILROAD);
+    s = BOOL_VAL(tspecial_south_west&S_RAILROAD);
+    e = BOOL_VAL(tspecial_south_east&S_RAILROAD);
+    w = BOOL_VAL(tspecial_north_west&S_RAILROAD);
     rail_semi_count = n + s + e + w;
     rail_semi_tileno = INDEX_NSEW(n,s,e,w);
     
-    n = !!(tspecial_north_east&S_ROAD);
-    s = !!(tspecial_south_west&S_ROAD);
-    e = !!(tspecial_south_east&S_ROAD);
-    w = !!(tspecial_north_west&S_ROAD);
+    n = BOOL_VAL(tspecial_north_east&S_ROAD);
+    s = BOOL_VAL(tspecial_south_west&S_ROAD);
+    e = BOOL_VAL(tspecial_south_east&S_ROAD);
+    w = BOOL_VAL(tspecial_north_west&S_ROAD);
     road_semi_count = n + s + e + w;
     road_semi_tileno = INDEX_NSEW(n,s,e,w);
 
@@ -1224,8 +1268,10 @@ void tilespec_free_city_tiles(int count)
   since still limited to less than 14.  Could possibly improve
   to allow players to choose their preferred color etc.
   Return is really "enum color_std".
+  A hack added to avoid returning more that COLOR_STD_RACE13.
+  But really there should be more colors available -- jk.
 ***********************************************************************/
 int player_color(struct player *pplayer)
 {
-  return COLOR_STD_RACE0 + pplayer->player_no;
+  return COLOR_STD_RACE0 + (pplayer->player_no % MAX_NUM_PLAYERS);
 }
