@@ -15,6 +15,8 @@
 #include <stdlib.h>		/* getenv() */
 
 #include "packets.h"		/* MAX_LEN_CAPSTR */
+#include "support.h"
+
 #include "capstr.h"
 
 static char our_capability_internal[MAX_LEN_CAPSTR];
@@ -68,9 +70,9 @@ const char * const our_capability = our_capability_internal;
  * are not directly related to the capability strings discussed here.)
  */
 
-#define CAPABILITY "+1.9"
+#define CAPABILITY "+1.10"
 
-/* "+1.9" is protocol for 1.9.0 stable release
+/* "+1.10" is protocol for 1.10.0 stable release
 */
 
 void init_our_capability(void)
@@ -81,5 +83,5 @@ void init_our_capability(void)
   if (s == NULL) {
     s = CAPABILITY;
   }
-  strncpy(our_capability_internal, s, MAX_LEN_CAPSTR-1);
+  sz_strlcpy(our_capability_internal, s);
 }
