@@ -10,16 +10,22 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "fcintl.h"
 #include "game.h"
 #include "government.h"
 #include "log.h"
 #include "map.h"
 #include "mem.h"
 #include "shared.h"
+#include "support.h"
 #include "tech.h"
 
 #include "city.h"
@@ -134,7 +140,8 @@ int wonder_replacement(struct city *pcity, enum improvement_type_id id)
       return 1;
     break;
   case B_CATHEDRAL:
-    if (city_affected_by_wonder(pcity, B_MICHELANGELO))
+    if (improvement_variant(B_MICHELANGELO)==0
+	&& city_affected_by_wonder(pcity, B_MICHELANGELO))
       return 1;
     break;
   case B_CITY:  
@@ -161,20 +168,25 @@ int wonder_replacement(struct city *pcity, enum improvement_type_id id)
   return 0;
 }
 
+/****************************************************************
+...
+*****************************************************************/
 char *get_imp_name_ex(struct city *pcity, enum improvement_type_id id)
 {
   static char buffer[256];
-  char state ='w';
+  char *state = Q_("?wonder:w");
   if (wonder_replacement(pcity, id)) {
-    sprintf(buffer, "%s(*)", get_improvement_type(id)->name);
+    my_snprintf(buffer, sizeof(buffer), "%s(*)",
+		get_improvement_type(id)->name);
     return buffer;
   }
   if (!is_wonder(id)) 
     return get_improvement_name(id);
 
-  if (game.global_wonders[id]) state='B';
-  if (wonder_obsolete(id)) state='O';
-  sprintf(buffer, "%s(%c)", get_improvement_type(id)->name, state); 
+  if (game.global_wonders[id]) state = Q_("?built:B");
+  if (wonder_obsolete(id)) state = Q_("?obsolete:O");
+  my_snprintf(buffer, sizeof(buffer), "%s(%s)",
+	      get_improvement_type(id)->name, state); 
   return buffer;
 }
 
@@ -380,10 +392,16 @@ int could_build_improvement(struct city *pcity, enum improvement_type_id id)
   Whether player could build this improvement, assuming they had
   the tech req, and assuming a city with the right pre-reqs etc.
 *****************************************************************/
-int could_player_build_improvement(struct player *p, enum improvement_type_id id)
+int could_player_eventually_build_improvement(struct player *p, enum improvement_type_id id)
 {
   if (!improvement_exists(id))
     return 0;
+
+  /* You can't build an improvement if it's tech requirement is
+     Never. */
+  if (improvement_types[id].tech_requirement == A_LAST)
+    return 0;
+  
   if (id == B_SSTRUCTURAL || id == B_SCOMP || id == B_SMODULE) {
     if (!game.global_wonders[B_APOLLO]) {
       return 0;
@@ -406,6 +424,19 @@ int could_player_build_improvement(struct player *p, enum improvement_type_id id
   return 1;
 }
 
+int could_player_build_improvement(struct player *p, enum improvement_type_id id)
+{
+  if (!could_player_eventually_build_improvement(p, id))
+    return 0;
+
+  /* Make sure we have the tech /now/.*/
+  if (get_invention(p, improvement_types[id].tech_requirement) == TECH_KNOWN)
+    return 1;
+  return 0;
+}
+  
+
+
 /****************************************************************
 Can this improvement get built in this city, by the player
 who owns the city?
@@ -420,6 +451,56 @@ int can_build_improvement(struct city *pcity, enum improvement_type_id id)
   return(could_build_improvement(pcity, id));
 }
 
+/****************************************************************
+Will this city ever be able to build this improvement?
+*****************************************************************/
+int can_eventually_build_improvement(struct city *pcity, enum improvement_type_id id)
+{
+  /* Can the _player_ ever build the improvement? */
+  if (!could_player_eventually_build_improvement(city_owner(pcity), id))
+    return 0;
+  
+  /* The player can build the improvement.  Check for other requirements
+     that are city-specific.  At this point, all we really care about is
+     whether there are any terrain restrictions -- we figure that improvement
+     preconditions (like building a marketplace before a bank) will 
+     be settled out on their own. */
+  
+  /* Does the city already have one of these improvements? */
+  if (city_got_building(pcity, id))
+    return 0;
+
+  /* At most one power plant per city. */
+  if ((city_got_building(pcity, B_HYDRO) || 
+       city_got_building(pcity, B_POWER) ||
+       city_got_building(pcity, B_NUCLEAR)) && 
+      (id==B_POWER || id==B_HYDRO || id==B_NUCLEAR))
+    return 0;
+
+  /* Costal improvements require a coast. */
+  if ((id==B_HARBOUR || id==B_COASTAL || id == B_OFFSHORE || id == B_PORT) &&
+      !is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN))
+    return 0;
+  
+  /* A hydro dam and the Hoover dam wonder have some fun terrain 
+     requirements. */
+  if ((id == B_HYDRO || id == B_HOOVER)
+      && !(map_get_terrain(pcity->x, pcity->y) == T_RIVER)
+      && !(map_get_special(pcity->x, pcity->y) & S_RIVER)
+      && !(map_get_terrain(pcity->x, pcity->y) == T_MOUNTAINS)
+      && !is_terrain_near_tile(pcity->x, pcity->y, T_MOUNTAINS)
+      && !is_terrain_near_tile(pcity->x, pcity->y, T_RIVER)
+      && !is_special_near_tile(pcity->x, pcity->y, S_RIVER)
+     )
+    return 0;
+
+  /* Don't allow improvements that are supplanted by wonders. */
+  if (wonder_replacement(pcity, id))
+    return 0;
+
+  return 1;
+}
+
 /****************************************************************
 Can a player build this improvement somewhere?  Ignores the
 fact that player may not have a city with appropriate prereqs.
@@ -480,6 +561,25 @@ int can_build_unit(struct city *pcity, Unit_Type_id id)
   return 1;
 }
 
+/****************************************************************
+Whether player can eventually build given unit in the city;
+returns 0 if unit can never possibily be built in this city.
+*****************************************************************/
+int can_eventually_build_unit(struct city *pcity, Unit_Type_id id)
+{
+  /* Can the _player_ ever build this unit? */
+  if (!can_player_eventually_build_unit(city_owner(pcity), id))
+    return 0;
+
+  /* Some units can be built only in certain cities -- for instance,
+     ships may be built only in cities adjacent to ocean. */
+  if (!is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN) && is_water_unit(id))
+    return 0;
+
+  return 1;
+}
+
+
 /****************************************************************
 Whether player can build given unit somewhere;
 returns 0 if unit is obsolete.
@@ -493,6 +593,20 @@ int can_player_build_unit(struct player *p, Unit_Type_id id)
   return 1;
 }
 
+/****************************************************************
+Whether player can _eventually_ build given unit somewhere -- ie,
+returns 1 if unit is available with current tech OR will be available
+with future tech.  returns 0 if unit is obsolete.
+*****************************************************************/
+int can_player_eventually_build_unit(struct player *p, Unit_Type_id id)
+{
+  if (!unit_type_exists(id))
+    return 0;
+  if (can_player_build_unit_direct(p, unit_types[id].obsoleted_by))
+    return 0;
+  return 1;
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -675,7 +789,9 @@ int get_food_tile(int x, int y, struct city *pcity)
 
   if ((spec_t & S_IRRIGATION) || city_auto_water) {
     f += type->irrigation_food_incr;
-    if (((spec_t & S_FARMLAND) || city_auto_water) &&
+    if (((spec_t & S_FARMLAND) ||
+	 (city_auto_water &&
+	  player_knows_techs_with_flag(city_owner(pcity), TF_FARMLAND))) &&
 	city_got_building(pcity, B_SUPERMARKET)) {
       f += (f * terrain_control.farmland_supermarket_food_bonus) / 100;
     }
@@ -687,6 +803,9 @@ int get_food_tile(int x, int y, struct city *pcity)
   if (spec_t & S_RAILROAD)
     f+=(f*terrain_control.rail_food_bonus)/100;
 
+  if (game.civstyle==2 && tile_t==T_MOUNTAINS && x==2 && y==2)
+    f++;
+
   if (f)
     f += (celeb ? g->celeb_food_bonus : g->food_bonus);
   if (before_penalty && f > before_penalty) 
@@ -836,6 +955,7 @@ int city_affected_by_wonder(struct city *pcity, enum improvement_type_id id) /*F
   case B_UNITED:
   case B_WOMENS:
   case B_DARWIN:
+  case B_LIGHTHOUSE:
   case B_MAGELLAN:
   case B_MICHELANGELO:
   case B_SETI:
@@ -878,11 +998,28 @@ int city_unhappy(struct city *pcity)
   return (pcity->ppl_happy[4]<pcity->ppl_unhappy[4]);
 }
 
+/**************************************************************************
+cities celebrate only after consecutive happy turns
+**************************************************************************/
 int city_celebrating(struct city *pcity)
 {
-  return (pcity->size>=5 && pcity->was_happy/* city_happy(pcity)*/);
+  struct government *g = get_gov_pcity(pcity);
+  return (pcity->size>=g->rapture_size && pcity->was_happy && city_happy(pcity));
 }
 
+/**************************************************************************
+.rapture is checked instead of city_celebrating() because this function is
+called after .was_happy was updated.
+**************************************************************************/
+int city_rapture_grow(struct city *pcity)
+{
+  struct government *g = get_gov_pcity(pcity);
+  return (pcity->rapture>0 && pcity->food_surplus>0 &&
+	  government_has_flag(g, G_RAPTURE_CITY_GROWTH));
+}
+
+/**************************************************************************/
+
 /* The find_city_by_id() code has returned from its trip to server land and
  * lives in common once again.  There are two ways find_city_by_id() works. 
  * If citycachesize!=0, it uses a fast (and potentially large, 256K) lookup
@@ -1112,3 +1249,39 @@ int get_style_by_name(char *style_name)
   else
     return -1;
 }
+
+/**************************************************************************
+ Calculates the turns which are needed to build the requested
+ type in the city. GUI Independent.
+**************************************************************************/
+int city_turns_to_build(struct city *pcity, int id, int id_is_unit)
+{
+  int city_shield_surplus = pcity->shield_surplus;
+
+  if (city_shield_surplus > 0)
+  {
+    int rounds, cost;
+    int shield_stock = pcity->shield_stock;
+
+    if (id_is_unit)
+    {
+      if (!pcity->is_building_unit)
+	shield_stock /= 2;
+      cost = get_unit_type(id)->build_cost;
+    }
+    else
+    {
+      if (pcity->is_building_unit ||
+	  (is_wonder(pcity->currently_building) != is_wonder(id)))
+	shield_stock /= 2;
+      cost = get_improvement_type(id)->build_cost;
+    }
+
+    rounds = (cost - shield_stock + city_shield_surplus - 1) /
+      city_shield_surplus;
+
+    return (rounds > 0) ? rounds : 1;
+  }
+
+  return 999;
+}
