@@ -16,6 +16,7 @@
 #include "genlist.h"
 #include "shared.h"		/* MAX_LEN_NAME */
 #include "unit.h"		/* struct unit_list */
+#include "worklist.h"
 
 struct player;
 struct government;
@@ -136,7 +137,8 @@ struct ai_city {
   signed short int detox[5][5], mine[5][5], irrigate[5][5], road[5][5], railroad[5][5], transform[5][5];
 /* caching these so that CPU usage is O(cities) instead of O(cities^2) -- Syela */
   signed short int tile_value[5][5]; /* caching these will help too. */
-  int settler_want; /* so we can contemplate with warmap fresh and decide later */
+/* so we can contemplate with warmap fresh and decide later */
+  int settler_want, founder_want; /* for builder (F_SETTLERS) and founder (F_CITIES) */
   int a, f, invasion; /* who's coming to kill us, for attack co-ordination */
 };
 
@@ -175,13 +177,18 @@ struct city {
   
   unsigned char improvements[B_LAST];
   
+  struct worklist *worklist;
+
   enum city_tile_type city_map[CITY_MAP_SIZE][CITY_MAP_SIZE];
 
   struct unit_list units_supported;
-  int steal;                  /* diplomats can only steal once */
+  int steal;		      /* diplomats steal once; for spies, gets harder */
   /* turn states */
   int did_buy, did_sell, is_updated;
-  int anarchy;                /* anarchy rounds count */ 
+  int turn_last_built;	      /* The last year in which something was built */
+  int turn_changed_target;    /* Suffer shield loss at most once per turn */
+  int anarchy;		      /* anarchy rounds count */ 
+  int rapture;                /* rapture rounds count */ 
   int was_happy;
   int airlift;
   int original;			/* original owner */
@@ -229,6 +236,7 @@ int city_buy_cost(struct city *pcity);
 int city_happy(struct city *pcity);  /* generally use celebrating instead */
 int city_unhappy(struct city *pcity);                /* anarchy??? */
 int city_celebrating(struct city *pcity);            /* love the king ??? */
+int city_rapture_grow(struct city *pcity);
 
 /* improvement functions */
 
@@ -243,10 +251,12 @@ int improvement_variant(enum improvement_type_id id);
 
 /* player related improvement and unit functions */
 
+int could_player_eventually_build_improvement(struct player *p, enum improvement_type_id id);
 int could_player_build_improvement(struct player *p, enum improvement_type_id id);
 int can_player_build_improvement(struct player *p, enum improvement_type_id id);
 int can_player_build_unit_direct(struct player *p, Unit_Type_id id);
 int can_player_build_unit(struct player *p, Unit_Type_id id);
+int can_player_eventually_build_unit(struct player *p, Unit_Type_id id);
 
 
 /* city related improvement and unit functions */
@@ -254,13 +264,16 @@ int can_player_build_unit(struct player *p, Unit_Type_id id);
 int improvement_upkeep(struct city *pcity, int i); 
 int could_build_improvement(struct city *pcity, enum improvement_type_id id);
 int can_build_improvement(struct city *pcity, enum improvement_type_id id);
+int can_eventually_build_improvement(struct city *pcity, enum improvement_type_id id);
 int can_build_unit(struct city *pcity, Unit_Type_id id);
 int can_build_unit_direct(struct city *pcity, Unit_Type_id id);
+int can_eventually_build_unit(struct city *pcity, Unit_Type_id id);
 int city_got_building(struct city *pcity,  enum improvement_type_id id); 
 int city_affected_by_wonder(struct city *pcity, enum improvement_type_id id);
 int city_got_effect(struct city *pcity, enum improvement_type_id id);
 int city_got_citywalls(struct city *pcity);
 int wonder_replacement(struct city *pcity, enum improvement_type_id id);
+int city_turns_to_build(struct city *pcity, int id, int id_is_unit);
 
 /* textual representation of buildings */
 
