@@ -49,6 +49,11 @@ enum {
   E_UNIT_WIN_ATT,
   E_CITY_GRAN_THROTTLE,
   E_SPACESHIP,
+  E_UPRISING,
+  E_WORKLIST,
+  /* Note:  If you add a new event, make sure you make a similar change to
+     message_text in client/options.c
+     */
   E_LAST
 };
 
