@@ -28,6 +28,7 @@
 #include "player.h"
 #include "shared.h"
 #include "spaceship.h"
+#include "support.h"
 #include "tech.h"
 #include "unit.h"
 
@@ -197,7 +198,7 @@ Allocates, fills and returns a land area claim map.
 Call free_landarea_map(&cmap) to free allocated memory.
 **************************************************************************/
 
-static int no_owner = MAX_NUM_PLAYERS;
+static int no_owner = MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS;
 
 /* allocate and clear claim map; determine city radii */
 
@@ -533,6 +534,14 @@ int civ_score(struct player *pplayer)
   pplayer->score.mfg=0;                         /* done */
   pplayer->score.literacy=0;
   pplayer->score.spaceship=0;
+
+  if (is_barbarian(pplayer)) {
+    if (pplayer->player_no == (game.nplayers - 1)) {
+      free_landarea_map(&cmap);
+    }
+    return 0;
+  }
+
   city_list_iterate(pplayer->cities, pcity) {
     pplayer->score.happy+=pcity->ppl_happy[4];
     pplayer->score.content+=pcity->ppl_content[4];
@@ -543,7 +552,10 @@ int civ_score(struct player *pplayer)
     pplayer->score.population+=city_population(pcity);
     pplayer->score.cities++;
     pplayer->score.pollution+=pcity->pollution;
-    pplayer->score.techout+=(1+pcity->science_total);
+    pplayer->score.techout+=pcity->science_total;
+#ifdef CITIES_PROVIDE_RESEARCH
+    (pplayer->score.techout)++;
+#endif
     pplayer->score.bnp+=pcity->trade_prod;
     pplayer->score.mfg+=pcity->shield_surplus;
     if (city_got_building(pcity, B_UNIVERSITY)) 
@@ -699,6 +711,7 @@ void game_remove_city(struct city *pcity)
 void game_init(void)
 {
   int i;
+  game.is_new_game = 1;
   game.globalwarming=0;
   game.warminglevel=8;
   game.gold        = GAME_DEFAULT_GOLD;
@@ -728,19 +741,21 @@ void game_init(void)
   game.civstyle    = GAME_DEFAULT_CIVSTYLE;
   game.razechance  = GAME_DEFAULT_RAZECHANCE;
   game.spacerace   = GAME_DEFAULT_SPACERACE;
+  game.barbarianrate  = GAME_DEFAULT_BARBARIANRATE;
+  game.onsetbarbarian = GAME_DEFAULT_ONSETBARBARIAN;
+  game.occupychance= GAME_DEFAULT_OCCUPYCHANCE;
   game.heating     = 0;
-  game.scenario    = 0;
-  strcpy(game.save_name, "civgame");
+  sz_strlcpy(game.save_name, "civgame");
   game.save_nturns=10;
   game.randseed=GAME_DEFAULT_RANDSEED;
 
-  strcpy(game.ruleset.techs, GAME_DEFAULT_RULESET);
-  strcpy(game.ruleset.units, GAME_DEFAULT_RULESET);
-  strcpy(game.ruleset.buildings, GAME_DEFAULT_RULESET);
-  strcpy(game.ruleset.terrain, GAME_DEFAULT_RULESET);
-  strcpy(game.ruleset.governments, GAME_DEFAULT_RULESET);
-  strcpy(game.ruleset.nations, GAME_DEFAULT_RULESET);
-  strcpy(game.ruleset.cities, GAME_DEFAULT_RULESET);
+  sz_strlcpy(game.ruleset.techs,       GAME_DEFAULT_RULESET);
+  sz_strlcpy(game.ruleset.units,       GAME_DEFAULT_RULESET);
+  sz_strlcpy(game.ruleset.buildings,   GAME_DEFAULT_RULESET);
+  sz_strlcpy(game.ruleset.terrain,     GAME_DEFAULT_RULESET);
+  sz_strlcpy(game.ruleset.governments, GAME_DEFAULT_RULESET);
+  sz_strlcpy(game.ruleset.nations,     GAME_DEFAULT_RULESET);
+  sz_strlcpy(game.ruleset.cities,      GAME_DEFAULT_RULESET);
   game.firepower_factor = 1;
   game.num_unit_types = 0;
   game.num_tech_types = 0;
@@ -750,11 +765,11 @@ void game_init(void)
   game.government_when_anarchy = G_MAGIC;   /* flag */
   game.ai_goal_government = G_MAGIC;        /* flag */
 
-  strcpy(game.demography, GAME_DEFAULT_DEMOGRAPHY);
+  sz_strlcpy(game.demography, GAME_DEFAULT_DEMOGRAPHY);
 
   map_init();
   
-  for(i=0; i<MAX_NUM_PLAYERS; i++)
+  for(i=0; i<MAX_NUM_PLAYERS+MAX_NUM_BARBARIANS; i++)
     player_init(&game.players[i]);
   for (i=0; i<A_LAST; i++)      /* game.num_tech_types = 0 here */
     game.global_advances[i]=0;
@@ -781,18 +796,18 @@ void initialize_globals(void)
 }
 
 /***************************************************************
-...
+  Returns the next year in the game.
 ***************************************************************/
-void game_next_year(void)
+int game_next_year(int year)
 {
   int spaceshipparts, i;
   int parts[] = { B_SCOMP, B_SMODULE, B_SSTRUCTURAL, B_LAST };
 
-  if (game.year == 1) /* hacked it to get rid of year 0 */
-    game.year = 0;
+  if (year == 1) /* hacked it to get rid of year 0 */
+    year = 0;
 
     /* !McFred: 
-       - want game.year += 1 for spaceship.
+       - want year += 1 for spaceship.
     */
 
   /* test game with 7 normal AI's, gen 4 map, foodbox 10, foodbase 0: 
@@ -813,25 +828,36 @@ void game_next_year(void)
     }
   }
 
-  if( game.year >= 1900 || ( spaceshipparts>=3 && game.year>0 ) )
-    game.year += 1;
-  else if( game.year >= 1750 || spaceshipparts>=2 )
-    game.year += 2;
-  else if( game.year >= 1500 || spaceshipparts>=1 )
-    game.year += 5;
-  else if( game.year >= 1000 )
-    game.year += 10;
-  else if( game.year >= 0 )
-    game.year += 20;
-  else if( game.year >= -1000 ) /* used this line for tuning (was -1250) */
-    game.year += 25;
+  if( year >= 1900 || ( spaceshipparts>=3 && year>0 ) )
+    year += 1;
+  else if( year >= 1750 || spaceshipparts>=2 )
+    year += 2;
+  else if( year >= 1500 || spaceshipparts>=1 )
+    year += 5;
+  else if( year >= 1000 )
+    year += 10;
+  else if( year >= 0 )
+    year += 20;
+  else if( year >= -1000 ) /* used this line for tuning (was -1250) */
+    year += 25;
   else
-    game.year += 50; 
+    year += 50; 
 
-  if (game.year == 0) 
-    game.year = 1;
+  if (year == 0) 
+    year = 1;
+
+  return year;
 }
 
+/***************************************************************
+  Advance the game year.
+***************************************************************/
+void game_advance_year(void)
+{
+  game.year = game_next_year(game.year);
+}
+
+
 /***************************************************************
 ...
 ***************************************************************/
@@ -917,50 +943,50 @@ void translate_data_names(void)
   
   for (i=0; i<game.num_tech_types; i++) {
     struct advance *this = &advances[i];
-    strcpy(this->name_orig, this->name);
-    strcpy(this->name, Q_(this->name_orig));
+    sz_strlcpy(this->name_orig, this->name);
+    sz_strlcpy(this->name, Q_(this->name_orig));
   }
   for (i=0; i<game.num_unit_types; i++) {
     struct unit_type *this = &unit_types[i];
-    strcpy(this->name_orig, this->name);
-    strcpy(this->name, Q_(this->name_orig));
+    sz_strlcpy(this->name_orig, this->name);
+    sz_strlcpy(this->name, Q_(this->name_orig));
   }
   for (i=0; i<B_LAST; i++) {
     struct improvement_type *this = &improvement_types[i];
-    strcpy(this->name_orig, this->name);
-    strcpy(this->name, Q_(this->name_orig));
+    sz_strlcpy(this->name_orig, this->name);
+    sz_strlcpy(this->name, Q_(this->name_orig));
   }
   for (i=T_FIRST; i<T_COUNT; i++) {
     struct tile_type *this = &tile_types[i];
-    strcpy(this->terrain_name_orig, this->terrain_name);
-    strcpy(this->terrain_name, Q_(this->terrain_name_orig));
-    strcpy(this->special_1_name_orig, this->special_1_name);
-    strcpy(this->special_1_name, Q_(this->special_1_name_orig));
-    strcpy(this->special_2_name_orig, this->special_2_name);
-    strcpy(this->special_2_name, Q_(this->special_2_name_orig));
+    sz_strlcpy(this->terrain_name_orig, this->terrain_name);
+    sz_strlcpy(this->terrain_name, Q_(this->terrain_name_orig));
+    sz_strlcpy(this->special_1_name_orig, this->special_1_name);
+    sz_strlcpy(this->special_1_name, Q_(this->special_1_name_orig));
+    sz_strlcpy(this->special_2_name_orig, this->special_2_name);
+    sz_strlcpy(this->special_2_name, Q_(this->special_2_name_orig));
   }
   for (i=0; i<game.government_count; i++) {
     struct government *this = &governments[i];
-    strcpy(this->name_orig, this->name);
-    strcpy(this->name, Q_(this->name_orig));
+    sz_strlcpy(this->name_orig, this->name);
+    sz_strlcpy(this->name, Q_(this->name_orig));
     for(j=0; j<this->num_ruler_titles; j++) {
       struct ruler_title *that = &this->ruler_titles[j];
-      strcpy(that->male_title_orig, that->male_title);
-      strcpy(that->male_title, Q_(that->male_title_orig));
-      strcpy(that->female_title_orig, that->female_title);
-      strcpy(that->female_title, Q_(that->female_title_orig));
+      sz_strlcpy(that->male_title_orig, that->male_title);
+      sz_strlcpy(that->male_title, Q_(that->male_title_orig));
+      sz_strlcpy(that->female_title_orig, that->female_title);
+      sz_strlcpy(that->female_title, Q_(that->female_title_orig));
     }
   }
   for (i=0; i<game.nation_count; i++) {
     struct nation_type *this = get_nation_by_idx(i);
-    strcpy(this->name_orig, this->name);
-    strcpy(this->name, Q_(this->name_orig));
-    strcpy(this->name_plural_orig, this->name_plural);
-    strcpy(this->name_plural, Q_(this->name_plural_orig));
+    sz_strlcpy(this->name_orig, this->name);
+    sz_strlcpy(this->name, Q_(this->name_orig));
+    sz_strlcpy(this->name_plural_orig, this->name_plural);
+    sz_strlcpy(this->name_plural, Q_(this->name_plural_orig));
   }
   for (i=0; i<game.styles_count; i++) {
     struct citystyle *this = &city_styles[i];
-    strcpy(this->name_orig, this->name);
-    strcpy(this->name, Q_(this->name_orig));
+    sz_strlcpy(this->name_orig, this->name);
+    sz_strlcpy(this->name, Q_(this->name_orig));
   }
 }
