@@ -17,6 +17,7 @@
 #include "player.h"
 
 #define MAX_LEN_DEMOGRAPHY  16
+#define MAX_NUM_BARBARIANS   2
 
 enum server_states { 
   PRE_GAME_STATE, 
@@ -37,6 +38,7 @@ struct unit;
 struct city;
 
 struct civ_game {
+  int is_new_game;		/* 1 for games never started */
   int version;
   int civstyle;
   int gold;
@@ -53,11 +55,14 @@ struct civ_game {
   int civilwarsize;
   int min_players, max_players, nplayers;
   int aifill;
+  int barbarianrate;
+  int onsetbarbarian;
+  int occupychance;
   int unhappysize;
   char *startmessage;
   int player_idx;
   struct player *player_ptr;
-  struct player players[MAX_NUM_PLAYERS];   
+  struct player players[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];   
   int global_advances[A_LAST];             /* a counter */
   int global_wonders[B_LAST];              /* contains city id's */
          /* global_wonders[] may also be (-1), or the id of a city
@@ -68,7 +73,6 @@ struct civ_game {
   int warminglevel;
   char save_name[MAX_LEN_NAME];
   int save_nturns;
-  int scenario;
   int foodbox;
   int aqueductloss;
   int techpenalty;
@@ -77,6 +81,7 @@ struct civ_game {
   int randseed;
   int aqueduct_size;
   int sewer_size;
+  int add_to_size_limit;
   int spacerace;
   int turnblock;
   
@@ -89,6 +94,7 @@ struct civ_game {
   int ai_goal_government;	/* kludge */
 
   int nation_count;
+  int playable_nation_count;
   int styles_count;
 
   struct {
@@ -127,7 +133,8 @@ struct lvldat {
 };
 
 void game_init(void);
-void game_next_year(void);
+int game_next_year(int);
+void game_advance_year(void);
 
 int civ_population(struct player *pplayer);
 struct unit *game_find_unit_by_id(int unit_id);
@@ -198,9 +205,9 @@ extern struct civ_game game;
 #define GAME_MIN_DIPLCOST            0
 #define GAME_MAX_DIPLCOST            100
 
-#define GAME_DEFAULT_DIPLCHANCE      3
+#define GAME_DEFAULT_DIPLCHANCE      80
 #define GAME_MIN_DIPLCHANCE          1
-#define GAME_MAX_DIPLCHANCE          10
+#define GAME_MAX_DIPLCHANCE          99
 
 #define GAME_DEFAULT_FREECOST        0
 #define GAME_MIN_FREECOST            0
@@ -250,12 +257,24 @@ extern struct civ_game game;
 #define GAME_MIN_TIMEOUT              0
 #define GAME_MAX_TIMEOUT            999
 
+#define GAME_DEFAULT_BARBARIANRATE   2
+#define GAME_MIN_BARBARIANRATE       0
+#define GAME_MAX_BARBARIANRATE       4
+
+#define GAME_DEFAULT_ONSETBARBARIAN  GAME_MIN_END_YEAR
+#define GAME_MIN_ONSETBARBARIAN      GAME_MIN_END_YEAR
+#define GAME_MAX_ONSETBARBARIAN      GAME_MAX_END_YEAR
+
+#define GAME_DEFAULT_OCCUPYCHANCE    0
+#define GAME_MIN_OCCUPYCHANCE        0
+#define GAME_MAX_OCCUPYCHANCE        100
+
 #define GAME_DEFAULT_RULESET         "default"
 
 #define GAME_DEFAULT_SKILL_LEVEL 3      /* easy */
 #define GAME_OLD_DEFAULT_SKILL_LEVEL 5  /* normal; for old save games */
 
-#define GAME_DEFAULT_DEMOGRAPHY      "NASRLPEMOqr"
+#define GAME_DEFAULT_DEMOGRAPHY      "NASRLPEMOqrb"
 
 #define GAME_START_YEAR -4000
 
