@@ -58,7 +58,7 @@ struct tile {
   unsigned short known;
   short assigned; /* these can save a lot of CPU usage -- Syela */
   struct city *worked;      /* city working tile, or NULL if none */
-  unsigned char continent;
+  signed short continent;
   signed char move_cost[8]; /* don't know if this helps! */
 };
 
@@ -79,6 +79,7 @@ struct terrain_misc
   enum special_river_move river_move_mode;
   int river_defense_bonus;             /* % added to defense if Civ2 river */
   int river_trade_incr;                /* added to trade if Civ2 river */
+  char *river_help_text;	       /* help for Civ2-style rivers */
   int fortress_defense_bonus;          /* % added to defense if fortress */
   int road_superhighway_trade_bonus;   /* % added to trade if road/s-highway */
   int rail_food_bonus;                 /* % added to food if railroad */
@@ -161,6 +162,9 @@ struct civ_map {
   int forestsize;
   int generator;
   int num_start_positions;
+  int fixed_start_positions;
+  int have_specials;
+  int num_continents;
   struct tile *tiles;
   struct map_position start_positions[MAX_NUM_NATIONS];
 };
@@ -178,7 +182,7 @@ int sq_map_distance(int x0, int y0, int x1, int y1);
 int same_pos(int x1, int y1, int x2, int y2);
 
 void map_set_continent(int x, int y, int val);
-char map_get_continent(int x, int y);
+signed short map_get_continent(int x, int y);
 int map_same_continent(int x1, int y1, int x2, int y2);
 
 void initialize_move_costs(void);
