@@ -13,10 +13,11 @@
 #ifndef FC__PACKETS_H
 #define FC__PACKETS_H
 
+#include "map.h"
 #include "player.h"
 #include "shared.h"		/* MAX_LEN_NAME, MAX_LEN_ADDR */
 #include "spaceship.h"
-#include "map.h"
+#include "worklist.h"
 
 #define MAX_LEN_PACKET    4096
 #define MAX_LEN_USERNAME    10	     /* see below */
@@ -48,6 +49,7 @@ enum packet_type {
   PACKET_CITY_SELL,
   PACKET_CITY_BUY,
   PACKET_CITY_CHANGE,
+  PACKET_CITY_WORKLIST,
   PACKET_CITY_MAKE_SPECIALIST,
   PACKET_CITY_MAKE_WORKER,
   PACKET_CITY_CHANGE_SPECIALIST,
@@ -56,6 +58,7 @@ enum packet_type {
   PACKET_PLAYER_REVOLUTION,
   PACKET_PLAYER_GOVERNMENT,
   PACKET_PLAYER_RESEARCH,
+  PACKET_PLAYER_WORKLIST,
   PACKET_UNIT_BUILD_CITY,
   PACKET_UNIT_DISBAND,
   PACKET_REMOVE_UNIT,
@@ -102,7 +105,8 @@ enum packet_type {
   PACKET_CITY_NAME_SUGGESTION,
   PACKET_RULESET_NATION,
   PACKET_UNIT_PARADROP_TO,
-  PACKET_RULESET_CITY
+  PACKET_RULESET_CITY,
+  PACKET_UNIT_CONNECT
 };
 
 enum report_type {
@@ -182,6 +186,17 @@ struct packet_unit_request
   char name[MAX_LEN_NAME];
 };
 
+/*********************************************************
+  unit connect
+*********************************************************/
+struct packet_unit_connect
+{
+  int activity_type;
+  int unit_id;
+  int dest_x;
+  int dest_y;
+};
+
 /*********************************************************
   unit request
 *********************************************************/
@@ -199,6 +214,8 @@ struct packet_player_request
   int tax, luxury, science;              /* rates */
   int government;                        /* government */
   int tech;                              /* research */
+  struct worklist worklist;              /* one worklist */
+  int wl_idx;                            /* which worklist */
 };
 
 /*********************************************************
@@ -212,6 +229,7 @@ struct packet_city_request
   int worker_x, worker_y;                /* make_worker, make_specialist */
   int specialist_from, specialist_to;    /* change_specialist */
   char name[MAX_LEN_NAME];            /* rename */
+  struct worklist worklist;              /* worklist */
 };
 
 
@@ -270,6 +288,7 @@ struct packet_unit_info {
   int goto_dest_x, goto_dest_y;
   int activity_target;
   int paradropped;
+  int connecting;
 };
 
 
@@ -300,6 +319,8 @@ struct packet_city_info {
   int is_building_unit;
   int currently_building;
 
+  struct worklist worklist;
+
   char improvements[B_LAST+1];
   char city_map[CITY_MAP_SIZE*CITY_MAP_SIZE+1];
 
@@ -395,7 +416,9 @@ struct packet_player_info {
   char addr[MAX_LEN_ADDR];
   int revolution;
   int ai;
+  int is_barbarian;
   char capability[MAX_LEN_CAPSTR];
+  struct worklist worklists[MAX_NUM_WORKLISTS];
 };
 
 /*********************************************************
@@ -450,6 +473,7 @@ struct packet_spaceship_action {
 struct packet_ruleset_control {
   int aqueduct_size;
   int sewer_size;
+  int add_to_size_limit;
   int num_unit_types;
   int num_tech_types;
   struct {
@@ -464,6 +488,7 @@ struct packet_ruleset_control {
   int default_government;
   int government_count;
   int nation_count;
+  int playable_nation_count;
   int style_count;
 };
 
@@ -582,6 +607,7 @@ struct packet_ruleset_government {
   int martial_law_max;
   int martial_law_per;
   int empire_size_mod;
+  int empire_size_inc;
   int rapture_size;
       
   int unit_happy_cost_factor;
@@ -776,6 +802,12 @@ struct packet_unit_combat *
 receive_packet_unit_combat(struct connection *pc);
 
 
+int send_packet_unit_connect(struct connection *pc, 
+			    struct packet_unit_connect *packet);
+struct packet_unit_connect *
+receive_packet_unit_connect(struct connection *pc);
+
+
 int send_packet_tile_info(struct connection *pc, 
 			 struct packet_tile_info *pinfo);
 struct packet_tile_info *receive_packet_tile_info(struct connection *pc);
