@@ -18,7 +18,9 @@
 #include "game.h"
 #include "government.h"
 #include "map.h"
+#include "rand.h"
 #include "shared.h"
+#include "support.h"
 #include "tech.h"
 #include "unit.h"
 
@@ -32,7 +34,8 @@ extern int is_server;
 int player_has_embassy(struct player *pplayer, struct player *pplayer2)
 {
   return (pplayer->embassy & (1<<pplayer2->player_no)) ||
-         (player_owns_active_wonder(pplayer, B_MARCO) && pplayer != pplayer2);
+         (player_owns_active_wonder(pplayer, B_MARCO) &&
+          pplayer != pplayer2 && !is_barbarian(pplayer2));
 }
 
 /****************************************************************
@@ -50,17 +53,19 @@ int player_owns_city(struct player *pplayer, struct city *pcity)
 ***************************************************************/
 void player_init(struct player *plr)
 {
+  int i;
+
   plr->player_no=plr-game.players;
 
-  strcpy(plr->name, "YourName");
-  strcpy(plr->username, "UserName");
+  sz_strlcpy(plr->name, "YourName");
+  sz_strlcpy(plr->username, "UserName");
   plr->is_male = 1;
   plr->government=game.default_government;
   plr->nation=MAX_NUM_NATIONS;
   plr->capital=0;
   unit_list_init(&plr->units);
   city_list_init(&plr->cities);
-  strcpy(plr->addr, "---.---.---.---");
+  sz_strlcpy(plr->addr, "---.---.---.---");
   plr->is_alive=1;
   plr->embassy=0;
   plr->city_style=0;            /* should be first basic style */
@@ -70,11 +75,17 @@ void player_init(struct player *plr)
   plr->ai.skill_level = 0;
   plr->ai.fuzzy = 0;
   plr->ai.expand = 100;
+  plr->ai.is_barbarian = 0;
   plr->future_tech=0;
   plr->economic.tax=PLAYER_DEFAULT_TAX_RATE;
   plr->economic.science=PLAYER_DEFAULT_SCIENCE_RATE;
   plr->economic.luxury=PLAYER_DEFAULT_LUXURY_RATE;
+  player_limit_to_government_rates(plr);
   spaceship_init(&plr->spaceship);
+  for (i = 0; i < MAX_NUM_WORKLISTS; i++) {
+    init_worklist(&plr->worklists[i]);
+    plr->worklists[i].is_valid = 0;
+  }
 }
 
 /***************************************************************
@@ -101,6 +112,30 @@ struct player *find_player_by_name(char *name)
   return 0;
 }
 
+/***************************************************************
+  Find player by name, allowing unambigous prefix (ie abbreviation).
+  Returns NULL if could not match, or if ambiguous or other
+  problem, and fills *result with characterisation of match/non-match
+  (see shared.[ch])
+***************************************************************/
+static const char *pname_accessor(int i) {
+  return game.players[i].name;
+}
+struct player *find_player_by_name_prefix(const char *name,
+					  enum m_pre_result *result)
+{
+  int ind;
+
+  *result = match_prefix(pname_accessor, game.nplayers, MAX_LEN_NAME-1,
+			 mystrncasecmp, name, &ind);
+
+  if (*result < M_PRE_AMBIGUOUS) {
+    return get_player(ind);
+  } else {
+    return NULL;
+  }
+}
+
 /***************************************************************
 Find player by its user name (not player/leader name)
 ***************************************************************/
@@ -237,6 +272,52 @@ int player_knows_techs_with_flag(struct player *pplayer, int flag)
   return count;
 }
 
+/**************************************************************************
+The following limits a player's rates to those that are acceptable for the
+present form of government.  If a rate exceeds maxrate for this government,
+it adjusts rates automatically adding the extra to the 2nd highest rate,
+preferring science to taxes and taxes to luxuries.
+(It assumes that for any government maxrate>=50)
+**************************************************************************/
+void player_limit_to_government_rates(struct player *pplayer)
+{
+  int maxrate, surplus;
+
+  /* ai players allowed to cheat */
+  if (pplayer->ai.control) {
+    return;
+  }
+
+  maxrate = get_government_max_rate(pplayer->government);
+
+  if (pplayer->economic.luxury > maxrate) {
+    surplus = pplayer->economic.luxury - maxrate;
+    pplayer->economic.luxury = maxrate;
+    if (pplayer->economic.science >= pplayer->economic.tax)
+      pplayer->economic.science += surplus;
+    else
+      pplayer->economic.tax += surplus;
+  }
+  if (pplayer->economic.tax > maxrate) {
+    surplus = pplayer->economic.tax - maxrate;
+    pplayer->economic.tax = maxrate;
+    if (pplayer->economic.science >= pplayer->economic.luxury)
+      pplayer->economic.science += surplus;
+    else
+      pplayer->economic.luxury += surplus;
+  }
+  if (pplayer->economic.science > maxrate) {
+    surplus = pplayer->economic.science - maxrate;
+    pplayer->economic.science = maxrate;
+    if (pplayer->economic.tax >= pplayer->economic.luxury)
+      pplayer->economic.tax += surplus;
+    else
+      pplayer->economic.luxury += surplus;
+  }
+
+  return;
+}
+
 /**************************************************************************
 Locate the city where the players palace is located, (NULL Otherwise) 
 **************************************************************************/
@@ -297,20 +378,20 @@ int ai_fuzzy(struct player *pplayer, int normal_decision)
 Command access levels for client-side use; at present, they are 
 only used to control access to server commands typed at the client chatline.
 **************************************************************************/
-static char *levelnames[] = {
+static const char *levelnames[] = {
   "none",
   "info",
   "ctrl",
   "hack"
 };
 
-char *cmdlevel_name(enum cmdlevel_id lvl)
+const char *cmdlevel_name(enum cmdlevel_id lvl)
 {
   assert (lvl >= 0 && lvl < ALLOW_NUM);
   return levelnames[lvl];
 }
 
-enum cmdlevel_id cmdlevel_named (char *token)
+enum cmdlevel_id cmdlevel_named(const char *token)
 {
   enum cmdlevel_id i;
   int len = strlen(token);
@@ -323,3 +404,11 @@ enum cmdlevel_id cmdlevel_named (char *token)
 
   return ALLOW_UNRECOGNIZED;
 }
+
+/**************************************************************************
+...
+**************************************************************************/
+int is_barbarian(struct player *pplayer)
+{
+  return (pplayer->ai.is_barbarian > 0);
+}
