@@ -15,14 +15,15 @@
 
 #include "city.h"
 #include "nation.h"
+#include "shared.h"
+#include "spaceship.h"
 #include "tech.h"
 #include "unit.h"
-#include "spaceship.h"
 
 struct tile;
 
-#define PLAYER_DEFAULT_TAX_RATE 50
-#define PLAYER_DEFAULT_SCIENCE_RATE 50
+#define PLAYER_DEFAULT_TAX_RATE 0
+#define PLAYER_DEFAULT_SCIENCE_RATE 100
 #define PLAYER_DEFAULT_LUXURY_RATE 0
 
 
@@ -85,6 +86,7 @@ struct player_ai {
   int tech_goal;
   int prev_gold;
   int maxbuycost;
+  int est_upkeep; /* estimated upkeep of buildings in cities */
   int tech_want[A_LAST+1];
   int tech_turns[A_LAST+1]; /* saves zillions of calculations! */
   int handicap;			/* sum of enum handicap_type */
@@ -92,6 +94,7 @@ struct player_ai {
   int fuzzy;			/* chance in 1000 to mis-decide */
   int expand;			/* percentage factor to value new cities */
   int warmth; /* threat of global warming */
+  int is_barbarian;
 };
 
 /**************************************************************************
@@ -142,10 +145,13 @@ struct player {
   int is_connected;
   struct connection *conn;
   char addr[MAX_LEN_ADDR];
+  struct worklist worklists[MAX_NUM_WORKLISTS];
 };
 
 void player_init(struct player *plr);
 struct player *find_player_by_name(char *name);
+struct player *find_player_by_name_prefix(const char *name,
+					  enum m_pre_result *result);
 struct player *find_player_by_user(char *name);
 void player_set_unit_focus_status(struct player *pplayer);
 int player_has_embassy(struct player *pplayer, struct player *pplayer2);
@@ -163,12 +169,16 @@ int player_knows_improvement_tech(struct player *pplayer,
 				   enum improvement_type_id id);
 int player_knows_techs_with_flag(struct player *pplayer, int flag);
 
+void player_limit_to_government_rates(struct player *pplayer);
+
 struct city *find_palace(struct player *pplayer);
 
 int ai_handicap(struct player *pplayer, enum handicap_type htype);
 int ai_fuzzy(struct player *pplayer, int normal_decision);
 
-char *cmdlevel_name(enum cmdlevel_id lvl);
-enum cmdlevel_id cmdlevel_named (char *token);
+const char *cmdlevel_name(enum cmdlevel_id lvl);
+enum cmdlevel_id cmdlevel_named(const char *token);
+
+int is_barbarian(struct player *pplayer);
 
 #endif  /* FC__PLAYER_H */
