@@ -154,9 +154,12 @@
 #include "mem.h"
 #include "sbuffer.h"
 #include "shared.h"
+#include "support.h"
 
 #include "registry.h"
 
+#define MAX_LEN_BUFFER 1024
+
 #define DO_HASH 1
 #define HASH_DEBUG 1		/* 0,1,2 */
 #define SAVE_TABLES 1		/* set to 0 for old-style savefiles */
@@ -215,6 +218,8 @@ struct hash_data {
   int num_buckets;
   int num_entries_hashbuild;
   int num_collisions;
+  int allow_duplicates;
+  int num_duplicates;
 };
 
 static int hashfunc(char *name, int num_buckets);
@@ -414,13 +419,13 @@ int section_file_load(struct section_file *sf, const char *filename)
 	tok = inf_token_required(inf, INF_TOK_VALUE);
 	if (i<columns_tab.n) {
 	  astr_minsize(&entry_name, base_name.n + 10 + columns[i].n);
-	  sprintf(entry_name.str, "%s%d.%s", base_name.str,
-		  table_lineno, columns[i].str);
+	  my_snprintf(entry_name.str, entry_name.n_alloc, "%s%d.%s",
+		      base_name.str, table_lineno, columns[i].str);
 	} else {
 	  astr_minsize(&entry_name, base_name.n + 20 + columns[i].n);
-	  sprintf(entry_name.str, "%s%d.%s,%d", base_name.str,
-		  table_lineno, columns[columns_tab.n-1].str,
-		  i-columns_tab.n+1);
+	  my_snprintf(entry_name.str, entry_name.n_alloc, "%s%d.%s,%d",
+		      base_name.str, table_lineno,
+		      columns[columns_tab.n-1].str, i-columns_tab.n+1);
 	}
 	pentry = new_entry(sb, entry_name.str, tok);
 	entry_list_insert_back(&psection->entries, pentry);
@@ -477,7 +482,8 @@ int section_file_load(struct section_file *sf, const char *filename)
 	pentry = new_entry(sb, base_name.str, tok);
       } else {
 	astr_minsize(&entry_name, base_name.n + 20);
-	sprintf(entry_name.str, "%s,%d", base_name.str, i);
+	my_snprintf(entry_name.str, entry_name.n_alloc,
+		    "%s,%d", base_name.str, i);
 	pentry = new_entry(sb, entry_name.str, tok);
       }
       entry_list_insert_back(&psection->entries, pentry);
@@ -501,7 +507,7 @@ int section_file_load(struct section_file *sf, const char *filename)
   ath_free(&columns_tab);
   
   if(DO_HASH) {
-    secfilehash_build(sf);
+    secfilehash_build(sf, 0);
   }
     
   return 1;
@@ -569,7 +575,7 @@ int section_file_save(struct section_file *my_section_file, const char *filename
 
 	offset = c - first;
 	first[offset-2] = '\0';
-	strcpy(base, first);
+	sz_strlcpy(base, first);
 	first[offset-2] = '0';
 	fprintf(fs, "%s={", base);
 
@@ -601,7 +607,8 @@ int section_file_save(struct section_file *my_section_file, const char *filename
 	  pentry = ITERATOR_PTR(ent_iter);
 	  col_pentry = ITERATOR_PTR(col_iter);
 
-	  sprintf(expect, "%s%d.%s", base, irow, col_pentry->name+offset);
+	  my_snprintf(expect, sizeof(expect), "%s%d.%s",
+		      base, irow, col_pentry->name+offset);
 
 	  /* break out of tabular if doesn't match: */
 	  if((!pentry) || (strcmp(pentry->name, expect) != 0)) {
@@ -658,11 +665,11 @@ int section_file_save(struct section_file *my_section_file, const char *filename
 char *secfile_lookup_str(struct section_file *my_section_file, char *path, ...)
 {
   struct entry *pentry;
-  char buf[512];
+  char buf[MAX_LEN_BUFFER];
   va_list ap;
 
   va_start(ap, path);
-  vsprintf(buf, path, ap);
+  my_vsnprintf(buf, sizeof(buf), path, ap);
   va_end(ap);
 
   if(!(pentry=section_file_lookup_internal(my_section_file, buf))) {
@@ -688,13 +695,13 @@ char *secfile_lookup_str_int(struct section_file *my_section_file,
 			     int *ival, char *path, ...)
 {
   struct entry *pentry;
-  char buf[512];
+  char buf[MAX_LEN_BUFFER];
   va_list ap;
 
   assert(ival);
   
   va_start(ap, path);
-  vsprintf(buf, path, ap);
+  my_vsnprintf(buf, sizeof(buf), path, ap);
   va_end(ap);
 
   if(!(pentry=section_file_lookup_internal(my_section_file, buf))) {
@@ -719,11 +726,11 @@ void secfile_insert_int(struct section_file *my_section_file,
 			int val, char *path, ...)
 {
   struct entry *pentry;
-  char buf[512];
+  char buf[MAX_LEN_BUFFER];
   va_list ap;
 
   va_start(ap, path);
-  vsprintf(buf, path, ap);
+  my_vsnprintf(buf, sizeof(buf), path, ap);
   va_end(ap);
 
   pentry=section_file_insert_internal(my_section_file, buf);
@@ -740,11 +747,11 @@ void secfile_insert_str(struct section_file *my_section_file,
 			char *sval, char *path, ...)
 {
   struct entry *pentry;
-  char buf[512];
+  char buf[MAX_LEN_BUFFER];
   va_list ap;
 
   va_start(ap, path);
-  vsprintf(buf, path, ap);
+  my_vsnprintf(buf, sizeof(buf), path, ap);
   va_end(ap);
 
   pentry = section_file_insert_internal(my_section_file, buf);
@@ -759,11 +766,11 @@ int secfile_lookup_int(struct section_file *my_section_file,
 		       char *path, ...)
 {
   struct entry *pentry;
-  char buf[512];
+  char buf[MAX_LEN_BUFFER];
   va_list ap;
 
   va_start(ap, path);
-  vsprintf(buf, path, ap);
+  my_vsnprintf(buf, sizeof(buf), path, ap);
   va_end(ap);
 
   if(!(pentry=section_file_lookup_internal(my_section_file, buf))) {
@@ -790,11 +797,11 @@ int secfile_lookup_int_default(struct section_file *my_section_file,
 			       int def, char *path, ...)
 {
   struct entry *pentry;
-  char buf[512];
+  char buf[MAX_LEN_BUFFER];
   va_list ap;
 
   va_start(ap, path);
-  vsprintf(buf, path, ap);
+  my_vsnprintf(buf, sizeof(buf), path, ap);
   va_end(ap);
 
   if(!(pentry=section_file_lookup_internal(my_section_file, buf))) {
@@ -816,11 +823,11 @@ char *secfile_lookup_str_default(struct section_file *my_section_file,
 				 char *def, char *path, ...)
 {
   struct entry *pentry;
-  char buf[512];
+  char buf[MAX_LEN_BUFFER];
   va_list ap;
 
   va_start(ap, path);
-  vsprintf(buf, path, ap);
+  my_vsnprintf(buf, sizeof(buf), path, ap);
   va_end(ap);
 
   if(!(pentry=section_file_lookup_internal(my_section_file, buf))) {
@@ -842,14 +849,14 @@ char *secfile_lookup_str_default(struct section_file *my_section_file,
 int section_file_lookup(struct section_file *my_section_file, 
 			char *path, ...)
 {
-  char buf[512];
+  char buf[MAX_LEN_BUFFER];
   va_list ap;
 
   va_start(ap, path);
-  vsprintf(buf, path, ap);
+  my_vsnprintf(buf, sizeof(buf), path, ap);
   va_end(ap);
 
-  return !!section_file_lookup_internal(my_section_file, buf);
+  return BOOL_VAL(section_file_lookup_internal(my_section_file, buf));
 }
 
 
@@ -861,9 +868,9 @@ section_file_lookup_internal(struct section_file *my_section_file,
 			     char *fullpath) 
 {
   char *pdelim;
-  char sec_name[512];
-  char ent_name[512];
-  char mod_fullpath[1024];
+  char sec_name[MAX_LEN_BUFFER];
+  char ent_name[MAX_LEN_BUFFER];
+  char mod_fullpath[2*MAX_LEN_BUFFER];
   int len;
   struct hash_entry *hentry;
   struct entry *result;
@@ -873,6 +880,7 @@ section_file_lookup_internal(struct section_file *my_section_file,
   /* treat "sec.foo,0" as "sec.foo": */
   len = strlen(fullpath);
   if(len>2 && fullpath[len-2]==',' && fullpath[len-1]=='0') {
+    assert(len<sizeof(mod_fullpath));
     strcpy(mod_fullpath, fullpath);
     fullpath = mod_fullpath;	/* reassign local pointer 'fullpath' */
     fullpath[len-2] = '\0';
@@ -892,9 +900,8 @@ section_file_lookup_internal(struct section_file *my_section_file,
   if(!(pdelim=strchr(fullpath, '.'))) /* i dont like strtok */
     return 0;
 
-  strncpy(sec_name, fullpath, pdelim-fullpath);
-  sec_name[pdelim-fullpath]='\0';
-  strcpy(ent_name, pdelim+1);
+  mystrlcpy(sec_name, fullpath, MIN(pdelim-fullpath+1, sizeof(sec_name)));
+  sz_strlcpy(ent_name, pdelim+1);
 
   section_list_iterate(*my_section_file->sections, psection) {
     if (strcmp(psection->name, sec_name) == 0 ) {
@@ -916,15 +923,16 @@ section_file_lookup_internal(struct section_file *my_section_file,
 
 /**************************************************************************
  The caller should ensure that "fullpath" should not refer to an entry
- which already exists in "my_section_file".
+ which already exists in "my_section_file".  (Actually, in some cases
+ now it is ok to have duplicate entries, but be careful...)
 **************************************************************************/
 static struct entry*
 section_file_insert_internal(struct section_file *my_section_file, 
 			     char *fullpath)
 {
   char *pdelim;
-  char sec_name[512];
-  char ent_name[512];
+  char sec_name[MAX_LEN_BUFFER];
+  char ent_name[MAX_LEN_BUFFER];
   struct section *psection;
   struct entry *pentry;
   struct sbuffer *sb = my_section_file->sb;
@@ -935,9 +943,8 @@ section_file_insert_internal(struct section_file *my_section_file,
 	    fullpath, secfile_filename(my_section_file));
     exit(1);
   }
-  strncpy(sec_name, fullpath, pdelim-fullpath);
-  sec_name[pdelim-fullpath]='\0';
-  strcpy(ent_name, pdelim+1);
+  mystrlcpy(sec_name, fullpath, MIN(pdelim-fullpath+1,sizeof(sec_name)));
+  sz_strlcpy(ent_name, pdelim+1);
   my_section_file->num_entries++;
   
   if(strlen(sec_name)==0 || strlen(ent_name)==0) {
@@ -1080,9 +1087,15 @@ static void secfilehash_insert(struct section_file *file,
 
   hentry = secfilehash_lookup(file, key, &hash_val);
   if (hentry->key_val != NULL) {
-    freelog(LOG_FATAL, "Tried to insert same value twice: %s (sectionfile %s)",
-	    key, secfile_filename(file));
-    exit(1);
+    if (file->hashd->allow_duplicates) {
+      file->hashd->num_duplicates++;
+      /* Fall-through, to replace entry; could 'return' here and
+	 then first entry would be used and following ones ignored. */
+    } else {
+      freelog(LOG_FATAL, "Tried to insert same value twice: %s (sectionfile %s)",
+	      key, secfile_filename(file));
+      exit(1);
+    }
   }
   hentry->data = data;
   hentry->key_val = sbuf_strdup(file->sb, key);
@@ -1092,8 +1105,11 @@ static void secfilehash_insert(struct section_file *file,
 /**************************************************************************
  Build a hash table for the file.  Note that the section_file should
  not be modified (except to free it) subsequently.
+ If allow_duplicates is true, then relax normal condition that
+ all entries must have unique names; in this case for duplicates
+ the hash ref will be to the _last_ entry.
 **************************************************************************/
-void secfilehash_build(struct section_file *file)
+void secfilehash_build(struct section_file *file, int allow_duplicates)
 {
   struct hash_data *hashd;
   char buf[256];
@@ -1115,6 +1131,8 @@ void secfilehash_build(struct section_file *file)
   hashd->num_buckets = pow2;
   hashd->num_entries_hashbuild = file->num_entries;
   hashd->num_collisions = 0;
+  hashd->allow_duplicates = allow_duplicates;
+  hashd->num_duplicates = 0;
   
   hashd->table = fc_malloc(hashd->num_buckets*sizeof(struct hash_entry));
 
@@ -1126,7 +1144,7 @@ void secfilehash_build(struct section_file *file)
 
   section_list_iterate(*file->sections, psection) {
     entry_list_iterate(psection->entries, pentry) {
-      sprintf(buf, "%s.%s", psection->name, pentry->name);
+      my_snprintf(buf, sizeof(buf), "%s.%s", psection->name, pentry->name);
       secfilehash_insert(file, buf, pentry);
     }
     entry_list_iterate_end;
@@ -1134,8 +1152,13 @@ void secfilehash_build(struct section_file *file)
   section_list_iterate_end;
   
   if (HASH_DEBUG>=1) {
-    freelog(LOG_DEBUG, "Hash collisions during build: %d (%d entries, %d buckets)",
-	hashd->num_collisions, file->num_entries, hashd->num_buckets );
+    freelog(LOG_DEBUG, "Hash collisions during build: "
+	    "%d (%d entries, %d buckets)",
+	    hashd->num_collisions, file->num_entries, hashd->num_buckets );
+    if (hashd->allow_duplicates) {
+      freelog(LOG_DEBUG, "Hash duplicates during build: %d",
+	      hashd->num_duplicates);
+    }
   }
 }
 
@@ -1159,12 +1182,12 @@ void secfilehash_free(struct section_file *file)
 int secfile_lookup_vec_dimen(struct section_file *my_section_file, 
 			     char *path, ...)
 {
-  char buf[512];
+  char buf[MAX_LEN_BUFFER];
   va_list ap;
   int j=0;
 
   va_start(ap, path);
-  vsprintf(buf, path, ap);
+  my_vsnprintf(buf, sizeof(buf), path, ap);
   va_end(ap);
 
   while(section_file_lookup(my_section_file, "%s,%d", buf, j)) {
@@ -1182,12 +1205,12 @@ int secfile_lookup_vec_dimen(struct section_file *my_section_file,
 int *secfile_lookup_int_vec(struct section_file *my_section_file,
 			    int *dimen, char *path, ...)
 {
-  char buf[512];
+  char buf[MAX_LEN_BUFFER];
   va_list ap;
   int j, *res;
 
   va_start(ap, path);
-  vsprintf(buf, path, ap);
+  my_vsnprintf(buf, sizeof(buf), path, ap);
   va_end(ap);
 
   *dimen = secfile_lookup_vec_dimen(my_section_file, buf);
@@ -1213,13 +1236,13 @@ int *secfile_lookup_int_vec(struct section_file *my_section_file,
 char **secfile_lookup_str_vec(struct section_file *my_section_file,
 			      int *dimen, char *path, ...)
 {
-  char buf[512];
+  char buf[MAX_LEN_BUFFER];
   va_list ap;
   int j;
   char **res;
 
   va_start(ap, path);
-  vsprintf(buf, path, ap);
+  my_vsnprintf(buf, sizeof(buf), path, ap);
   va_end(ap);
 
   *dimen = secfile_lookup_vec_dimen(my_section_file, buf);
