@@ -27,23 +27,11 @@
 #include <pwd.h>
 #endif
 
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_SELECT_H
-#include <sys/select.h>
-#endif
-
-#ifdef GENERATING_MAC
-#include <events.h> /* for WaitNextEvent() */
-#endif
-
+#include "astring.h"
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
+#include "support.h"
 
 #include "shared.h"
 
@@ -58,10 +46,6 @@
                           "~/.freeciv"
 #endif
 
-/* Random Number Generator variables */
-RANDOM_TYPE RandomState[56];
-int iRandJ, iRandK, iRandX; 
-
 /**************************************************************************
 ... 
 **************************************************************************/
@@ -131,27 +115,26 @@ char * get_option(const char *option_name,char **argv,int *i,int argc)
 {
   int len = strlen(option_name);
 
-  if (!strcmp(option_name,argv[*i]) || 
-      (!strncmp(option_name,argv[*i],len) &&
-       argv[*i][len]=='=') ||
-     !strncmp(option_name+1,argv[*i],2))
-     {
-     argv[*i] += (argv[*i][1] != '-' ? 0 : len); 
-     
-     if (argv[*i][0] == '=')
-	argv[*i]++;
-       else {
-	if (*i < argc - 1) 
-	  (*i)++;
-	else {
-          fprintf(stderr, _("Missing argument for \"%s\".\n"), option_name);
-          exit(1);
-	  }
-	}
-	
-     return argv[*i];
-     }
-  
+  if (!strcmp(option_name,argv[*i]) ||
+      (!strncmp(option_name,argv[*i],len) && argv[*i][len]=='=') ||
+      !strncmp(option_name+1,argv[*i],2)) {
+    char *opt = argv[*i] + (argv[*i][1] != '-' ? 0 : len);
+
+    if (*opt == '=') {
+      opt++;
+    } else {
+      if (*i < argc - 1) {
+	(*i)++;
+	opt = argv[*i];
+      }	else {
+	fprintf(stderr, _("Missing argument for \"%s\".\n"), option_name);
+	exit(1);
+      }
+    }
+
+    return opt;
+  }
+
   return NULL;
 }
 
@@ -175,7 +158,7 @@ char *int_to_text(int nr)
   static char buf[20];
   char *to=buf;
   char *from=tmpbuf;
-  sprintf(tmpbuf,"%d", nr);
+  my_snprintf(tmpbuf, sizeof(tmpbuf), "%d", nr);
   while (*from) {
     *to++=*from++;
     if (strlen(from)%3==0 && *from)  
@@ -201,8 +184,7 @@ char *get_sane_name(char *name)
   if(*cp)
     return 0; 
 
-  strncpy(str, name, MAX_LEN_NAME-1);
-  str[MAX_LEN_NAME-1]='\0';
+  sz_strlcpy(str, name);
   
   *str=toupper(*str);
   return str;
@@ -231,8 +213,7 @@ char *get_sane_name(char *name)
   if(*cp)
     return 0; 
 
-  strncpy(str, name, MAX_LEN_NAME-1);
-  str[MAX_LEN_NAME-1]='\0';
+  sz_strlcpy(str, name);
   
   return str;
 }
@@ -242,27 +223,14 @@ char *get_sane_name(char *name)
 ***************************************************************/
 char *textyear(int year)
 {
-  static char y[10];
+  static char y[32];
   if (year<0) 
-    sprintf(y, "%d BC", -year);
+    my_snprintf(y, sizeof(y), _("%d BC"), -year);
   else
-    sprintf(y, "%d AD", year);
+    my_snprintf(y, sizeof(y), _("%d AD"), year);
   return y;
 }
 
-
-/***************************************************************
-...
-***************************************************************/
-int mystrcasecmp(const char *str0, const char *str1)
-{
-  for(; tolower(*str0)==tolower(*str1); str0++, str1++)
-    if(*str0=='\0')
-      return 0;
-
-  return tolower(*str0)-tolower(*str1);
-}
-
 /**************************************************************************
 string_ptr_compare() - fiddles with pointers to do a simple string compare
                        when passed pointers to two strings -- called from
@@ -274,120 +242,6 @@ int string_ptr_compare(const void *first, const void *second)
    return mystrcasecmp(*((const char **)first), *((const char **)second));
 }
 
-/***************************************************************
-...
-***************************************************************/
-char *mystrerror(int errnum)
-{
-#if defined(HAVE_STRERROR) || !defined(HAVE_CONFIG_H)
-  /* if we have real strerror() or if we're not using configure */
-  return strerror(errnum);
-#else
-  static char buf[64];
-  sprintf(buf, _("error %d (compiled without strerror)"), errnum);
-  return buf;
-#endif
-}
-
-/***************************************************************
-...
-***************************************************************/
-void myusleep(unsigned long usec)
-{
-#ifdef HAVE_USLEEP
-  usleep(usec);
-#else
-#ifdef HAVE_SNOOZE		/* BeOS */
-  snooze(usec);
-#else
-#ifdef GENERATING_MAC
-  EventRecord the_event;  /* dummy var for timesharing */
-  WaitNextEvent(0, &the_event, GetCaretTime(), nil); /* this is suposed to
-     give other application procseor time for the mac*/
-#else
-  struct timeval tv;
-
-  tv.tv_sec=0;
-  tv.tv_usec=100;
-  select(0, NULL, NULL, NULL, &tv);
-#endif
-#endif
-#endif
-}
-
-
-/*************************************************************************
-   The following random number generator can be found in _The Art of 
-   Computer Programming Vol 2._ (2nd ed) by Donald E. Knuth. (C)  1998.
-   The algorithm is described in section 3.2.2 as Mitchell and Moore's
-   variant of a standard additive number generator.  Note that the
-   the constants 55 and 24 are not random.  Please become familiar with
-   this algorithm before you mess with it.
-
-   Since the additive number generator requires a table of numbers from
-   which to generate its random sequences, we must invent a way to 
-   populate that table from a single seed value.  I have chosen to do
-   this with a different PRNG, known as the "linear congruential method" 
-   (also found in Knuth, Vol2).  I must admit that my choices of constants
-   (3, 257, and MAX_UINT32) are probably not optimal, but they seem to
-   work well enough for our purposes.
-*************************************************************************/
-
-RANDOM_TYPE myrand(RANDOM_TYPE size) 
-{
-    RANDOM_TYPE newRand;
-
-    assert(size);
-
-    newRand = (RandomState[iRandJ] + RandomState[iRandK]) & MAX_UINT32;
-
-    iRandX = (iRandX +1) % 56;
-    iRandJ = (iRandJ +1) % 56;
-    iRandK = (iRandK +1) % 56;
-    RandomState[iRandX] = newRand;
-
-    return newRand % size;
-    
-} 
-
-void mysrand(RANDOM_TYPE seed) 
-{ 
-    int  i; 
-
-    RandomState[0]=(seed & MAX_UINT32);
-
-    for(i=1;i<56;i++) {
-       RandomState[i]= (3 * RandomState[i-1] + 257) & MAX_UINT32;
-    }
-
-    iRandJ=(55-55);
-    iRandK=(55-24);
-    iRandX=(55-0);
-} 
- 
- 
-/*************************************************************************
- save_restore_random() - Saves and restores the complete state of the 
-   random number generator to a temporary set of variables.  The first 
-   call to this function performs a save.  The second call performs 
-   a restore.  The third call performs a save again, etc.
-*************************************************************************/
-void save_restore_random(void)
-{
-   static RANDOM_TYPE State[56];
-   static int j, k, x;
-   static int mode=0;
-
-   if(mode) {
-      memcpy(RandomState,State,sizeof(RANDOM_TYPE)*56);
-      iRandJ=j; iRandK=k; iRandX=x;
-   } else {
-      memcpy(State,RandomState,sizeof(RANDOM_TYPE)*56);
-      j=iRandJ; k=iRandK; x=iRandX;
-   }
-   mode = (mode+1) % 2;
-}
-
 /***************************************************************************
   Returns 's' incremented to first non-space character.
 ***************************************************************************/
@@ -477,6 +331,58 @@ int wordwrap_string(char *s, int len)
   return num_lines;
 }
 
+/***************************************************************************
+  Returns pointer to '\0' at end of string 'str', and decrements
+  *nleft by the length of 'str'.  This is intended to be useful to
+  allow strcat-ing without traversing the whole string each time,
+  while still keeping track of the buffer length.
+  Eg:
+     char buf[128];
+     int n = sizeof(buf);
+     char *p = buf;
+
+     my_snprintf(p, n, "foo%p", p);
+     p = end_of_strn(p, &n);
+     mystrlcpy(p, "yyy", n);
+***************************************************************************/
+char *end_of_strn(char *str, int *nleft)
+{
+  int len = strlen(str);
+  *nleft -= len;
+  assert((*nleft)>0);		/* space for the terminating nul */
+  return str + len;
+}
+
+/********************************************************************** 
+ cat_snprintf is like a combination of my_snprintf and mystrlcat;
+ it does snprintf to the end of an existing string.
+ 
+ Like mystrlcat, n is the total length available for str, including
+ existing contents and trailing nul.  If there is no extra room
+ available in str, does not change the string. 
+
+ Also like mystrlcat, returns the final length that str would have
+ had without truncation.  Ie, if return is >= n, truncation occured.
+**********************************************************************/ 
+int cat_snprintf(char *str, size_t n, const char *format, ...)
+{
+  size_t len;
+  int ret;
+  va_list ap;
+
+  assert(format);
+  assert(str);
+  assert(n>0);
+  
+  len = strlen(str);
+  assert(len < n);
+  
+  va_start(ap, format);
+  ret = my_vsnprintf(str+len, n-len, format, ap);
+  va_end(ap);
+  return ret + len;
+}
+
 /***************************************************************************
   Returns string which gives users home dir, as specified by $HOME.
   Gets value once, and then caches result.
@@ -534,8 +440,8 @@ char *user_username(void)
     }
   }
 #endif
-  username = fc_malloc(30);
-  sprintf(username, "name%d", (int)getuid());
+  username = fc_malloc(MAX_LEN_NAME);
+  my_snprintf(username, MAX_LEN_NAME, "name%d", (int)getuid());
   freelog(LOG_VERBOSE, "fake username is %s", username);
   return username;
 }
@@ -563,8 +469,7 @@ char *datafilename(const char *filename)
   static int init = 0;
   static int num_dirs = 0;
   static char **dirs = NULL;
-  static char *realfile = NULL;
-  static int n_alloc_realfile = 0;
+  static struct astring realfile = ASTRING_INIT;
   int i;
 
   if (!init) {
@@ -598,8 +503,9 @@ char *datafilename(const char *filename)
 		    "No HOME, skipping data path component %s", tok);
 	    i = 0;
 	  } else {
-	    char *tmp = fc_malloc(strlen(home) + i);    /* +1 -1 */
-	    sprintf(tmp, "%s%s", home, tok+1);
+	    int len = strlen(home) + i;	   /* +1 -1 */
+	    char *tmp = fc_malloc(len);
+	    my_snprintf(tmp, len, "%s%s", home, tok+1);
 	    tok = tmp;
 	    i = -1;		/* flag to free tok below */
 	  }
@@ -626,36 +532,31 @@ char *datafilename(const char *filename)
 
   if (filename == NULL) {
     int len = 1;		/* in case num_dirs==0 */
+    int seplen = strlen(PATH_SEPARATOR);
     for(i=0; i<num_dirs; i++) {
-      len += strlen(dirs[i]) + 1; /* ':' or '\0' */
+      len += strlen(dirs[i]) + MAX(1,seplen);    /* separator or '\0' */
     }
-    if (len > n_alloc_realfile) {
-      realfile = fc_realloc(realfile, len);
-      n_alloc_realfile = len;
-    }
-    realfile[0] = '\0';
+    astr_minsize(&realfile, len);
+    realfile.str[0] = '\0';
     for(i=0; i<num_dirs; i++) {
-      strcat(realfile, dirs[i]);
+      mystrlcat(realfile.str, dirs[i], len);
       if(i != num_dirs-1) {
-	strcat(realfile, PATH_SEPARATOR);
+	mystrlcat(realfile.str, PATH_SEPARATOR, len);
       }
     }
-    return realfile;
+    return realfile.str;
   }
   
   for(i=0; i<num_dirs; i++) {
     FILE *fp;			/* see if we can open the file */
     int len = strlen(dirs[i]) + strlen(filename) + 2;
     
-    if (len > n_alloc_realfile) {
-      realfile = fc_realloc(realfile, len);
-      n_alloc_realfile = len;
-    } 
-    sprintf(realfile,"%s/%s", dirs[i], filename);
-    fp = fopen(realfile, "r");
+    astr_minsize(&realfile, len);
+    my_snprintf(realfile.str, len, "%s/%s", dirs[i], filename);
+    fp = fopen(realfile.str, "r");
     if (fp) {
       fclose(fp);
-      return realfile;
+      return realfile.str;
     }
   }
   return NULL;
@@ -721,3 +622,80 @@ void dont_run_as_root(const char *argv0, const char *fallback)
   }
 #endif
 }
+
+
+/***************************************************************************
+  Return a description string of the result.
+  In English, form of description is suitable to substitute in, eg:
+     prefix is <description>
+***************************************************************************/
+const char *m_pre_description(enum m_pre_result result)
+{
+  static const char * const descriptions[] = {
+    N_("exact match"),
+    N_("only match"),
+    N_("ambiguous"),
+    N_("empty"),
+    N_("too long"),
+    N_("non-match")
+  };
+  assert(result > 0 && result < sizeof(descriptions)/sizeof(descriptions[0]));
+  return _(descriptions[result]);
+}
+
+/***************************************************************************
+  Given n names, with maximum length max_len_name, accessed by
+  accessor_fn(0) to accessor_fn(n-1), look for matching prefix
+  according to given comparison function.
+  Returns type of match or fail, and for return <= M_PRE_AMBIGUOUS
+  sets *ind_result with matching index (or for ambiguous, first match).
+  If max_len_name==0, treat as no maximum.
+***************************************************************************/
+enum m_pre_result match_prefix(m_pre_accessor_fn_t accessor_fn,
+			       size_t n_names,
+			       size_t max_len_name,
+			       m_pre_strncmp_fn_t cmp_fn,
+			       const char *prefix,
+			       int *ind_result)
+{
+  int i, len, nmatches;
+
+  len = strlen(prefix);
+  if (len == 0) {
+    return M_PRE_EMPTY;
+  }
+  if (len > max_len_name && max_len_name > 0) {
+    return M_PRE_LONG;
+  }
+
+  nmatches = 0;
+  for(i=0; i<n_names; i++) {
+    const char *name = accessor_fn(i);
+    if (cmp_fn(name, prefix, len)==0) {
+      if (strlen(name) == len) {
+	*ind_result = i;
+	return M_PRE_EXACT;
+      }
+      if (nmatches==0) {
+	*ind_result = i;	/* first match */
+      }
+      nmatches++;
+    }
+  }
+
+  if (nmatches == 1) {
+    return M_PRE_ONLY;
+  } else if (nmatches > 1) {
+    return M_PRE_AMBIGUOUS;
+  } else {
+    return M_PRE_FAIL;
+  }
+}
+
+/***************************************************************************
+  Return the Freeciv motto.
+***************************************************************************/
+char *freeciv_motto(void)
+{
+  return _("'Cause civilization should be free!");
+}
