@@ -13,11 +13,15 @@
 #ifndef FC__SHARED_H
 #define FC__SHARED_H
 
+#include <stdlib.h>		/* size_t */
+
+#include "attribute.h"
+
 /* Note: the capability string is now in capstr.c --dwp */
 /* Version stuff is now in version.h --dwp */
 
-#define BUG_EMAIL_ADDRESS "bugs@cvs.freeciv.org"
-#define WEBSITE_URL "http://www.freeciv.org"
+#define BUG_EMAIL_ADDRESS "bugs@freeciv.freeciv.org"
+#define WEBSITE_URL "http://www.freeciv.org/"
 
 #define BROADCAST_EVENT -2
 
@@ -41,12 +45,14 @@
 #define CLIP(lower,this,upper) \
     ((this)<(lower)?(lower):(this)>(upper)?(upper):(this))
 
-#define WIPEBIT(val, no) ((~(-1<<no))&val)  |   ((  (-1<<(no+1)) &val) >>1)
+#define BOOL_VAL(x) ((x)!=0)
 
+#define WIPEBIT(val, no) ( ((~(-1<<(no)))&(val)) \
+                           | (( (-1<<((no)+1)) & (val)) >>1) )
+
+/* This is duplicated in rand.h to avoid extra includes: */
 #define MAX_UINT32 0xFFFFFFFF
 
-typedef unsigned int RANDOM_TYPE;
-
 char *n_if_vowel(char ch);
 char *create_centered_string(char *s);
 
@@ -55,21 +61,17 @@ int is_option(const char *option_name,char *option);
 char *int_to_text(int nr);
 char *get_sane_name(char *name);
 char *textyear(int year);
-int mystrcasecmp(const char *str0, const char *str1);
 int string_ptr_compare(const void *first, const void *second);
 
-char *mystrerror(int errnum);
-void myusleep(unsigned long usec);
-
-RANDOM_TYPE myrand(RANDOM_TYPE size);
-void mysrand(RANDOM_TYPE seed);
-void save_restore_random(void);
-
 char *skip_leading_spaces(char *s);
 void remove_trailing_spaces(char *s);
 void remove_trailing_char(char *s, char trailing);
 int wordwrap_string(char *s, int len);
 
+char *end_of_strn(char *str, int *nleft);
+int cat_snprintf(char *str, size_t n, const char *format, ...)
+     fc__attribute((format (printf, 3, 4)));
+
 char *user_home_dir(void);
 char *user_username(void);
 char *datafilename(const char *filename);
@@ -78,6 +80,35 @@ char *datafilename_required(const char *filename);
 void init_nls(void);
 void dont_run_as_root(const char *argv0, const char *fallback);
 
+/*** matching prefixes: ***/
+
+enum m_pre_result {
+  M_PRE_EXACT,		/* matches with exact length */
+  M_PRE_ONLY,		/* only matching prefix */
+  M_PRE_AMBIGUOUS,	/* first of multiple matching prefixes */
+  M_PRE_EMPTY,		/* prefix is empty string (no match) */
+  M_PRE_LONG,		/* prefix is too long (no match) */
+  M_PRE_FAIL,		/* no match at all */
+  M_PRE_LAST		/* flag value */
+};
+
+const char *m_pre_description(enum m_pre_result result);
+
+/* function type to access a name from an index: */
+typedef const char *(*m_pre_accessor_fn_t)(int);
+
+/* function type to compare prefix: */
+typedef int (*m_pre_strncmp_fn_t)(const char *, const char *, size_t n);
+
+enum m_pre_result match_prefix(m_pre_accessor_fn_t accessor_fn,
+			       size_t n_names,
+			       size_t max_len_name,
+			       m_pre_strncmp_fn_t cmp_fn,
+			       const char *prefix,
+			       int *ind_result);
+
+char *freeciv_motto(void);
+
 /*Mac constants-resource IDs*/
 enum DITL_ids{
   kChatDITL=133,
