@@ -17,6 +17,7 @@
 
 #include "game.h"
 #include "player.h"
+#include "support.h"
 
 #include "tech.h"
 
@@ -27,7 +28,7 @@ struct advance advances[A_LAST];
 
 static char *flag_names[] = {
   "Bonus_Tech","Boat_Fast","Bridge","Railroad","Fortress",
-  "Population_Pollution_Inc","Trade_Revenue_Reduce","Airbase"
+  "Population_Pollution_Inc","Trade_Revenue_Reduce","Airbase","Farmland"
 };
 
 
@@ -62,17 +63,17 @@ void set_invention(struct player *plr, int tech, int value)
 **************************************************************************/
 void update_research(struct player *plr) 
 {
-  int i;
+  int i, known;
   
   for (i=0;i<game.num_tech_types;i++) {
     if(!tech_exists(i)) {
       plr->research.inventions[i]=TECH_UNKNOWN;
       continue;
     }
-    if (get_invention(plr, i) == TECH_REACHABLE ||
-	get_invention(plr, i) == TECH_MARKED)
+    known=get_invention(plr, i);
+    if (known == TECH_REACHABLE || known == TECH_MARKED)
       plr->research.inventions[i]=TECH_UNKNOWN;
-    if(get_invention(plr, i) == TECH_UNKNOWN
+    if (get_invention(plr, i) == TECH_UNKNOWN
        && get_invention(plr, advances[i].req[0])==TECH_KNOWN   
        && get_invention(plr, advances[i].req[1])==TECH_KNOWN)
       plr->research.inventions[i]=TECH_REACHABLE;
@@ -177,7 +178,7 @@ Tech_Type_id find_tech_by_name(const char *s)
 int tech_flag(int tech, int flag)
 {
   assert(flag>=0 && flag<TF_LAST);
-  return (advances[tech].flags & (1<<flag))?TRUE:FALSE;
+  return BOOL_VAL(advances[tech].flags & (1<<flag));
 }
 
 /**************************************************************************
