@@ -27,6 +27,8 @@
 #include "map.h"
 #include "mem.h"
 #include "player.h"
+#include "shared.h"
+#include "support.h"
 #include "tech.h"
 
 #include "unit.h"
@@ -50,12 +52,14 @@ static char *flag_names[] = {
   "OneAttack", "Pikemen", "Horse", "IgWall", "FieldUnit", "AEGIS",
   "Fighter", "Marines", "Submarine", "Settlers", "Diplomat",
   "Trireme", "Nuclear", "Spy", "Transform", "Paratroopers",
-  "Airbase"
+  "Airbase", "Cities"
 };
 static char *role_names[] = {
   "FirstBuild", "Explorer", "Hut", "HutTech", "Partisan",
   "DefendOk", "DefendGood", "AttackFast", "AttackStrong",
-  "Ferryboat"
+  "Ferryboat", "Barbarian", "BarbarianTech", "BarbarianBoat",
+  "BarbarianBuild", "BarbarianBuildTech", "BarbarianLeader",
+  "BarbarianSea", "BarbarianSeaTech"
 };
 
 /***************************************************************
@@ -66,16 +70,22 @@ char *get_unit_name(Unit_Type_id id)
   struct unit_type *ptype;
   static char buffer[256];
   ptype =get_unit_type(id);
-  if (ptype->fuel)
-    sprintf(buffer,"%s [%d/%d/%d(%d)]", ptype->name, ptype->attack_strength,
-	    ptype->defense_strength,
-	    ptype->move_rate/3,(ptype->move_rate/3)*ptype->fuel);
-  else
-    sprintf(buffer,"%s [%d/%d/%d]", ptype->name, ptype->attack_strength,
-	    ptype->defense_strength, ptype->move_rate/3);
+  if (ptype->fuel) {
+    my_snprintf(buffer, sizeof(buffer),
+		"%s [%d/%d/%d(%d)]", ptype->name, ptype->attack_strength,
+		ptype->defense_strength,
+		ptype->move_rate/3,(ptype->move_rate/3)*ptype->fuel);
+  } else {
+    my_snprintf(buffer, sizeof(buffer),
+		"%s [%d/%d/%d]", ptype->name, ptype->attack_strength,
+		ptype->defense_strength, ptype->move_rate/3);
+  }
   return buffer;
 }
 
+/***************************************************************
+...
+***************************************************************/
 struct unit *find_unit_by_id(int id)
 {
   int i;
@@ -88,7 +98,9 @@ struct unit *find_unit_by_id(int id)
   return 0;
 }
 
-
+/***************************************************************
+...
+***************************************************************/
 int unit_move_rate(struct unit *punit)
 {
   int val;
@@ -133,13 +145,10 @@ int diplomat_can_do_action(struct unit *pdiplomat,
   if(!is_tiles_adjacent(pdiplomat->x, pdiplomat->y, destx, desty))
     return 0;
 
-  if(pdiplomat->moves_left >= unit_move_rate(pdiplomat))
-    return 1;   /* Hasn't moved yet this turn */
+  if(!pdiplomat->moves_left)
+    return 0;
 
-  if(pdiplomat->moves_left >= map_move_cost(pdiplomat, destx, desty))
-    return 1;
-
-  return 0;
+  return 1;
 }
 
 /**************************************************************************
@@ -158,12 +167,13 @@ int is_diplomat_action_available(struct unit *pdiplomat,
     if(pcity->owner!=pdiplomat->owner) {
       if(action==DIPLOMAT_SABOTAGE)
         return 1;
-
       if(action==DIPLOMAT_EMBASSY &&
 	 !player_has_embassy(&game.players[pdiplomat->owner], 
 			     &game.players[pcity->owner]))
 	return 1;
-      if(action==SPY_POISON && unit_flag(pdiplomat->type, F_SPY))
+      if(action==SPY_POISON &&
+	 pcity->size>1 &&
+	 unit_flag(pdiplomat->type, F_SPY))
         return 1;
       if(action==DIPLOMAT_INVESTIGATE)
         return 1;
@@ -174,11 +184,10 @@ int is_diplomat_action_available(struct unit *pdiplomat,
       if(action==DIPLOMAT_ANY_ACTION)
         return 1;
     }
-  }
-  else {
+  } else {
     struct tile *ptile=map_get_tile(destx, desty);
     if((action==SPY_SABOTAGE_UNIT || action==DIPLOMAT_ANY_ACTION) &&
-       unit_list_size(&ptile->units)>=1 &&
+       unit_list_size(&ptile->units)==1 &&
        unit_list_get(&ptile->units, 0)->owner!=pdiplomat->owner &&
        unit_flag(pdiplomat->type, F_SPY))
       return 1;
@@ -594,7 +603,7 @@ int is_hiding_unit(struct unit *punit)
 int unit_flag(Unit_Type_id id, int flag)
 {
   assert(flag>=0 && flag<F_LAST);
-  return unit_types[id].flags & (1<<flag);
+  return BOOL_VAL(unit_types[id].flags & (1<<flag));
 }
 
 /**************************************************************************
@@ -603,7 +612,7 @@ int unit_flag(Unit_Type_id id, int flag)
 int unit_has_role(Unit_Type_id id, int role)
 {
   assert(role>=L_FIRST && role<L_LAST);
-  return unit_types[id].roles & (1<<(role-L_FIRST));
+  return BOOL_VAL(unit_types[id].roles & (1<<(role-L_FIRST)));
 }
 
 /**************************************************************************
@@ -697,7 +706,7 @@ int can_unit_build_city(struct unit *punit)
 {
   int x, y;
 
-  if(!unit_flag(punit->type, F_SETTLERS))
+  if(!unit_flag(punit->type, F_CITIES))
     return 0;
     
   if(map_get_terrain(punit->x, punit->y)==T_OCEAN)
@@ -726,7 +735,7 @@ int can_unit_add_to_city(struct unit *punit)
 {
   struct city *pcity;
 
-  if(!unit_flag(punit->type, F_SETTLERS))
+  if(!unit_flag(punit->type, F_CITIES))
     return 0;
   if(!punit->moves_left)
     return 0;
@@ -735,7 +744,7 @@ int can_unit_add_to_city(struct unit *punit)
 
   if(!pcity)
     return 0;
-  if(pcity->size>8)
+  if(pcity->size > game.add_to_size_limit)
     return 0;
 
   if(improvement_exists(B_AQUEDUCT)
@@ -803,6 +812,60 @@ int can_unit_do_auto(struct unit *punit)
   return 0;
 }
 
+/**************************************************************************
+Return whether the unit can connect with given activity (or with
+any activity if activity arg is set to ACTIVITY_IDLE)
+**************************************************************************/
+int can_unit_do_connect (struct unit *punit, enum unit_activity activity) 
+{
+  struct player  *pplayer = get_player (punit->owner);
+
+  if (!unit_flag(punit->type, F_SETTLERS))
+    return 0;
+
+  if (activity == ACTIVITY_IDLE)   /* IDLE here means "any activity" */
+    return 1;
+
+  if (activity == ACTIVITY_ROAD 
+      || activity == ACTIVITY_IRRIGATE 
+      || (activity == ACTIVITY_RAILROAD
+	  && player_knows_techs_with_flag(pplayer, TF_RAILROAD))
+      || (activity == ACTIVITY_FORTRESS 
+	  && player_knows_techs_with_flag(pplayer, TF_FORTRESS)))
+  return 1;
+
+  return 0;
+}
+
+/**************************************************************************
+Return name of activity in static buffer
+**************************************************************************/
+
+char* get_activity_text (int activity)
+{
+  char *text;
+
+  switch (activity) {
+  case ACTIVITY_IDLE:      text = _("Idle"); break;
+  case ACTIVITY_POLLUTION: text = _("Pollution"); break;
+  case ACTIVITY_ROAD:      text = _("Road"); break;
+  case ACTIVITY_MINE:      text = _("Mine"); break;
+  case ACTIVITY_IRRIGATE:  text = _("Irrigation"); break;
+  case ACTIVITY_FORTIFY:   text = _("Fortify"); break;
+  case ACTIVITY_FORTRESS:  text = _("Fortress"); break;
+  case ACTIVITY_SENTRY:    text = _("Sentry"); break;
+  case ACTIVITY_RAILROAD:  text = _("Railroad"); break;
+  case ACTIVITY_PILLAGE:   text = _("Pillage"); break;
+  case ACTIVITY_GOTO:      text = _("Goto"); break;
+  case ACTIVITY_EXPLORE:   text = _("Explore"); break;
+  case ACTIVITY_TRANSFORM: text = _("Transform"); break;
+  case ACTIVITY_AIRBASE:   text = _("Airbase"); break;
+  default:                 text = _("Unknown"); break;
+  }
+
+  return text;
+}
+
 /**************************************************************************
 Return whether the unit can be paradropped.
 That is if the unit is in a friendly city or on an Airbase
@@ -865,7 +928,7 @@ int can_unit_do_activity_targeted(struct unit *punit,
     return (is_ground_unit(punit) || is_sailing_unit(punit));
 
   /* can't go directly from non-idle to non-idle activity */
-  if(punit->activity!=ACTIVITY_IDLE) return 0;
+  if(punit->activity!=ACTIVITY_IDLE && punit->activity != activity) return 0;
 
   switch(activity)  {
   case ACTIVITY_POLLUTION:
@@ -899,7 +962,7 @@ int can_unit_do_activity_targeted(struct unit *punit,
 	unit_flag(punit->type, F_SETTLERS) && punit->moves_left &&
 	(!(ptile->special&S_IRRIGATION) ||
 	 (!(ptile->special&S_FARMLAND) &&
-	  player_knows_improvement_tech(pplayer, B_SUPERMARKET))) &&
+	  player_knows_techs_with_flag(pplayer, TF_FARMLAND))) &&
 	( (ptile->terrain==type->irrigation_result && 
 	   is_water_adjacent_to_tile(punit->x, punit->y)) ||
 	  (ptile->terrain!=type->irrigation_result &&
@@ -932,7 +995,11 @@ int can_unit_do_activity_targeted(struct unit *punit,
     /* if the tile has road, the terrain must be ok.. */
     return terrain_control.may_road &&
            unit_flag(punit->type, F_SETTLERS) && punit->moves_left &&
-           (ptile->special&S_ROAD) && !(ptile->special&S_RAILROAD) &&
+           ((ptile->special&S_ROAD) || (punit->connecting
+	    && (ptile->terrain!=T_OCEAN &&
+		((ptile->terrain!=T_RIVER && !(ptile->special&S_RIVER))
+		 || player_knows_techs_with_flag(pplayer, TF_BRIDGE)))))
+           && !(ptile->special&S_RAILROAD) &&
 	   player_knows_techs_with_flag(pplayer, TF_RAILROAD);
 
   case ACTIVITY_PILLAGE:
@@ -971,6 +1038,7 @@ void set_unit_activity(struct unit *punit, enum unit_activity new_activity)
   punit->activity=new_activity;
   punit->activity_count=0;
   punit->activity_target=0;
+  punit->connecting = 0;
 }
 
 /**************************************************************************
@@ -982,6 +1050,7 @@ void set_unit_activity_targeted(struct unit *punit,
   punit->activity=new_activity;
   punit->activity_count=0;
   punit->activity_target=new_target;
+  punit->connecting = 0;
 }
 
 /**************************************************************************
@@ -1019,7 +1088,7 @@ char *unit_description(struct unit *punit)
 
   pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity);
 
-  sprintf(buffer, "%s%s\n%s\n%s", 
+  my_snprintf(buffer, sizeof(buffer), "%s%s\n%s\n%s", 
 	  get_unit_type(punit->type)->name, 
 	  punit->veteran ? _(" (veteran)") : "",
 	  unit_activity_text(punit), 
@@ -1044,64 +1113,57 @@ char *unit_activity_text(struct unit *punit)
        rate=get_unit_type(punit->type)->move_rate/3;
        f=((punit->fuel)-1);
        if(punit->moves_left%3) {
-        if(punit->moves_left/3>0)
-          sprintf(text, "%s: (%d)%d %d/3", moves_str,
-                  ((rate*f)+(punit->moves_left/3)),
-                  punit->moves_left/3, punit->moves_left%3);
-        else
-          sprintf(text, "%s: (%d)%d/3", moves_str,
-                  ((rate*f)+(punit->moves_left/3)),
-                  punit->moves_left%3);
+	 if(punit->moves_left/3>0) {
+	   my_snprintf(text, sizeof(text), "%s: (%d)%d %d/3", moves_str,
+		       ((rate*f)+(punit->moves_left/3)),
+		       punit->moves_left/3, punit->moves_left%3);
+	 } else {
+	   my_snprintf(text, sizeof(text), "%s: (%d)%d/3", moves_str,
+		       ((rate*f)+(punit->moves_left/3)),
+		       punit->moves_left%3);
+	 }
+       } else {
+	 my_snprintf(text, sizeof(text), "%s: (%d)%d", moves_str,
+		     rate*f+punit->moves_left/3,
+		     punit->moves_left/3);
        }
-       else
-        sprintf(text, "%s: (%d)%d", moves_str,
-		rate*f+punit->moves_left/3,
-                punit->moves_left/3);
-     }
-     else {
+     } else {
        if(punit->moves_left%3) {
-        if(punit->moves_left/3>0)
-          sprintf(text, "%s: %d %d/3", moves_str,
-		  punit->moves_left/3, punit->moves_left%3);
-        else
-          sprintf(text, "%s: %d/3", moves_str, punit->moves_left%3);
+	 if(punit->moves_left/3>0) {
+	   my_snprintf(text, sizeof(text), "%s: %d %d/3", moves_str,
+		       punit->moves_left/3, punit->moves_left%3);
+	 } else {
+	   my_snprintf(text, sizeof(text),
+		       "%s: %d/3", moves_str, punit->moves_left%3);
+	 }
+       } else {
+	 my_snprintf(text, sizeof(text),
+		     "%s: %d", moves_str, punit->moves_left/3);
        }
-       else
-        sprintf(text, "%s: %d", moves_str, punit->moves_left/3);
      }
      return text;
    case ACTIVITY_POLLUTION:
-    return _("Pollution");
    case ACTIVITY_ROAD:
-    return _("Road");
    case ACTIVITY_RAILROAD:
-    return _("Railroad");
    case ACTIVITY_MINE: 
-    return _("Mine");
-    case ACTIVITY_IRRIGATE:
-    return _("Irrigation");
+   case ACTIVITY_IRRIGATE:
    case ACTIVITY_TRANSFORM:
-    return _("Transform");
    case ACTIVITY_FORTIFY:
-    return _("Fortify");
    case ACTIVITY_AIRBASE:
-    return _("Airbase");
    case ACTIVITY_FORTRESS:
-    return _("Fortress");
    case ACTIVITY_SENTRY:
-    return _("Sentry");
+   case ACTIVITY_GOTO:
+   case ACTIVITY_EXPLORE:
+     return get_activity_text (punit->activity);
    case ACTIVITY_PILLAGE:
      if(punit->activity_target == 0) {
-       return _("Pillage");
+       return get_activity_text (punit->activity);
      } else {
-       sprintf(text, "%s: %s", _("Pillage"),
-	       map_get_infrastructure_text(punit->activity_target));
+       my_snprintf(text, sizeof(text), "%s: %s",
+		   get_activity_text (punit->activity),
+		   map_get_infrastructure_text(punit->activity_target));
        return (text);
      }
-   case ACTIVITY_GOTO:
-    return _("Goto");
-   case ACTIVITY_EXPLORE:
-    return _("Explore");
    default:
     freelog(LOG_FATAL, "Unknown unit activity:%d in unit_activity_text()", punit->activity);
     exit(0);
