@@ -54,7 +54,8 @@ enum unit_activity {
   ACTIVITY_IDLE, ACTIVITY_POLLUTION, ACTIVITY_ROAD, ACTIVITY_MINE, 
   ACTIVITY_IRRIGATE, ACTIVITY_FORTIFY, ACTIVITY_FORTRESS, ACTIVITY_SENTRY,
   ACTIVITY_RAILROAD, ACTIVITY_PILLAGE, ACTIVITY_GOTO, ACTIVITY_EXPLORE,
-  ACTIVITY_TRANSFORM, ACTIVITY_UNKNOWN, ACTIVITY_AIRBASE
+  ACTIVITY_TRANSFORM, ACTIVITY_UNKNOWN, ACTIVITY_AIRBASE, 
+  ACTIVITY_LAST   /* leave this one last */
 };
 
 enum unit_move_type {
@@ -108,6 +109,7 @@ struct unit {
      and city.units_supported; they are only used for save/reload */
   int moved;
   int paradropped;
+  int connecting;
 };
 
 /* Unit "special effects" flags:
@@ -132,7 +134,7 @@ enum unit_flag_id {
   F_FIGHTER,     
   F_MARINES,     
   F_SUBMARINE,   
-  F_SETTLERS,    
+  F_SETTLERS,         /* Does not include ability to found cities */
   F_DIPLOMAT,    
   F_TRIREME,          /* Trireme sinking effect */
   F_NUCLEAR,          /* Nuclear attack effect */
@@ -140,6 +142,7 @@ enum unit_flag_id {
   F_TRANSFORM,        /* Can transform terrain types (Engineers) */
   F_PARATROOPERS,
   F_AIRBASE,          /* Can build Airbases */
+  F_CITIES,           /* Can build cities */
   F_LAST
 };
 
@@ -149,6 +152,7 @@ enum unit_flag_id {
    in various circumstances, or "roles".
    Note that in some cases flags can act as roles, eg, we don't need
    a role for "settlers", because we can just use F_SETTLERS.
+   (Now have to consider F_CITIES too)
    So we make sure flag values and role values are distinct,
    so some functions can use them interchangably.
 */
@@ -164,6 +168,14 @@ enum unit_role_id {
   L_ATTACK_FAST,        /* quick attacking unit (Horse..Armor) (unused)*/
   L_ATTACK_STRONG,      /* powerful attacking unit (Catapult..) (unused) */
   L_FERRYBOAT,	        /* is useful for ferrying (AI) */
+  L_BARBARIAN,          /* barbarians unit, land only */
+  L_BARBARIAN_TECH,     /* barbarians unit, global tech required */
+  L_BARBARIAN_BOAT,     /* barbarian boat */
+  L_BARBARIAN_BUILD,    /* what barbarians should build */
+  L_BARBARIAN_BUILD_TECH, /* barbarians build when global tech */
+  L_BARBARIAN_LEADER,   /* barbarian leader */
+  L_BARBARIAN_SEA,      /* sea raider unit */
+  L_BARBARIAN_SEA_TECH, /* sea raider unit, global tech required */
   L_LAST
 };
 
@@ -241,6 +253,8 @@ int unit_can_airlift_to(struct unit *punit, struct city *pcity);
 
 int can_unit_paradropped(struct unit *punit);
 int can_unit_change_homecity(struct unit *punit);
+int can_unit_do_connect(struct unit *punit, enum unit_activity activity);
+char* get_activity_text (int activity);
 int can_unit_do_activity(struct unit *punit, enum unit_activity activity);
 int can_unit_do_activity_targeted(struct unit *punit,
 				  enum unit_activity activity, int target);
