@@ -19,22 +19,37 @@
 #endif
 #endif
 
-/* The following is for the benefit (?) of non-configure make methods: */
+/* The following is for the benefit (?) of non-configure make methods. */
+/* !! These must be the same as their counterparts in configure.in. !! */
 #ifndef MAJOR_VERSION
-#define MAJOR_VERSION  1
+#define MAJOR_VERSION		1
 #endif
 #ifndef MINOR_VERSION
-#define MINOR_VERSION  9
+#define MINOR_VERSION		10
 #endif
 #ifndef PATCH_VERSION
-#define PATCH_VERSION  0
+#define PATCH_VERSION		0
 #endif
 #ifndef VERSION_LABEL
-#define VERSION_LABEL  ""
+#define VERSION_LABEL		""
+#endif
+#ifndef IS_DEVEL_VERSION
+#define IS_DEVEL_VERSION	0
 #endif
 #ifndef IS_BETA_VERSION
-#define IS_BETA_VERSION 0
+#define IS_BETA_VERSION		0
 #endif
+
+/* This is only used if IS_BETA_VERSION is true. */
+#ifndef NEXT_STABLE_VERSION
+#define NEXT_STABLE_VERSION	"1.10.0"
+#endif
+/* This is only used in version.c, and only if IS_BETA_VERSION is true.
+   The month[] array is defined in version.c (index: 1==Jan, 2==Feb, ...). */
+#ifndef NEXT_RELEASE_MONTH
+#define NEXT_RELEASE_MONTH	(month[2])
+#endif
+
 #ifndef VERSION_STRING
 #define VER_STRINGIFY1(x) #x
 #define VER_STRINGIFY(x) VER_STRINGIFY1(x)
@@ -52,4 +67,7 @@
 #define WORD_VERSION "version"
 #endif
 
+/* If returns NULL, not a beta version. */
+char *beta_message (void);
+
 #endif  /* FC__VERSION_H */
