@@ -25,6 +25,7 @@
 #include "map.h"
 #include "mem.h"
 #include "player.h"
+#include "rand.h"
 #include "shared.h"
 #include "tech.h"
 #include "unit.h"
@@ -42,6 +43,8 @@
 #include "unithand.h"
 #include "unittools.h"
 
+#include "advmilitary.h"	/* assess_danger_player() */
+
 #include "citytools.h"
 
 /****************************************************************
@@ -92,9 +95,13 @@ int content_citizens(struct player *pplayer)
   int cities =  city_list_size(&pplayer->cities);
   int content = game.unhappysize;
   int basis   = game.cityfactor + get_gov_pplayer(pplayer)->empire_size_mod;
+  int step    = get_gov_pplayer(pplayer)->empire_size_inc;
 
-  if (cities > basis) 
+  if (cities > basis) {
     content--;
+    if (step)
+      content -= (cities - basis) / step;
+  }
   return content;
 }
 
@@ -125,6 +132,8 @@ int get_cathedral_power(struct city *pcity)
    power--;
   if (get_invention(p, game.rtech.cathedral_plus/*A_THEOLOGY*/) == TECH_KNOWN)
    power++;
+  if (improvement_variant(B_MICHELANGELO)==1 && city_affected_by_wonder(pcity, B_MICHELANGELO))
+   power*=2;
   return power;
 }
 
@@ -312,7 +321,8 @@ void eval_buildings(struct city *pcity,int *values)
     if (can_build_improvement(pcity, B_GRANARY)) 
       values[B_GRANARY]=pcity->food_surplus*50;
   }
-  if (can_build_improvement(pcity, B_SUPERMARKET))
+  if (can_build_improvement(pcity, B_SUPERMARKET) &&
+      player_knows_techs_with_flag(plr, TF_FARMLAND))
     values[B_SUPERMARKET]=pcity->size*55;
 
   if (can_build_improvement(pcity, B_AQUEDUCT)
@@ -567,6 +577,7 @@ int wants_to_be_bigger(struct city *pcity)
   return 0;
 }
 
+
 /*
  *  Units in a bought city are transferred to the new owner, units 
  * supported by the city, but held in other cities are updated to
@@ -580,6 +591,15 @@ int wants_to_be_bigger(struct city *pcity)
  *
  * If verbose is true, send extra messages to players detailing what
  * happens to all the units.  --dwp
+ *
+ * If statement added to see if the owner of pcity is the same as vcity. If so,
+ * then the city is disbanded and not bought, and we only need to evaluate the
+ * units supported by the city, and not those actually present.
+ * - Thue <thue.kristensen@get2net.dk>
+ *
+ * Interpretation of kill_outside changed to mean the radius outside
+ * of which supported units are killed.  If 0, all supported units not
+ * in the city are killed.  If -1, no supported units are killed.  --jjm
  */
 void transfer_city_units(struct player *pplayer, struct player *pvictim, 
 			 struct city *pcity, struct city *vcity, 
@@ -588,27 +608,30 @@ void transfer_city_units(struct player *pplayer, struct player *pvictim,
   int x = vcity->x;
   int y = vcity->y;
 
-  /* Transfer units in the city to the new owner */
-  unit_list_iterate(map_get_tile(x, y)->units, vunit)  {
-    freelog(LOG_VERBOSE, "Transfered %s in %s from %s to %s",
-	    unit_name(vunit->type), vcity->name, pvictim->name, pplayer->name);
-    if (verbose) {
-      notify_player(pvictim, _("Game: Transfered %s in %s from %s to %s."),
-		    unit_name(vunit->type), vcity->name,
-		    pvictim->name, pplayer->name);
-    }
-    create_unit_full(pplayer, x, y, vunit->type, vunit->veteran,
-		pcity->id, vunit->moves_left, vunit->hp);
-    wipe_unit(0, vunit);
-  } unit_list_iterate_end;
+  if (city_owner(pcity)!=city_owner(vcity)) { /* true=>bought, false =>disbanded */
+    /* Transfer units in the city to the new owner */
+    unit_list_iterate(map_get_tile(x, y)->units, vunit)  {
+      freelog(LOG_VERBOSE, "Transfered %s in %s from %s to %s",
+	      unit_name(vunit->type), vcity->name, pvictim->name, pplayer->name);
+      if (verbose) {
+	notify_player(pvictim, _("Game: Transfered %s in %s from %s to %s."),
+		      unit_name(vunit->type), vcity->name,
+		      pvictim->name, pplayer->name);
+      }
+      create_unit_full(pplayer, x, y, vunit->type, vunit->veteran,
+		       pcity->id, vunit->moves_left, vunit->hp);
+      wipe_unit(0, vunit);
+    } unit_list_iterate_end;
+  }
 
   /* Any remaining units supported by the city are either given new home
    * cities or maybe destroyed */
   unit_list_iterate(vcity->units_supported, vunit) {
     struct city* new_home_city = map_get_city(vunit->x, vunit->y);
-    if(new_home_city)  {
-      /* unit is in another city: make that the new homecity */
-      
+    if(new_home_city) {
+      /* unit is in another city: make that the new homecity,
+	 unless that city is actually the same city (happens if disbanding) */
+
       if (pvictim == city_owner(new_home_city)) {
 	freelog(LOG_VERBOSE, "Changed homecity of %s's %s in %s",
 	     pvictim->name, unit_name(vunit->type), new_home_city->name);
@@ -626,24 +649,33 @@ void transfer_city_units(struct player *pplayer, struct player *pvictim,
 			pvictim->name, pplayer->name);
 	}
       }
-      create_unit_full(city_owner(new_home_city), vunit->x, vunit->y,
-		       vunit->type, vunit->veteran, new_home_city->id,
-		       vunit->moves_left, vunit->hp);
+      if(new_home_city != vcity) {
+	create_unit_full(city_owner(new_home_city), vunit->x, vunit->y,
+			 vunit->type, vunit->veteran, new_home_city->id,
+			 vunit->moves_left, vunit->hp);
+      } else {
+	create_unit_full(pplayer, vunit->x, vunit->y,
+			 vunit->type, vunit->veteran, pcity->id,
+			 vunit->moves_left, vunit->hp);
+      }
+    }else if((kill_outside < 0) ||
+	     ((kill_outside > 0) &&
+	      (real_map_distance(vunit->x, vunit->y, x, y) <= kill_outside))) {
 
-    }else if(!kill_outside){
-      
       freelog(LOG_VERBOSE, "Transfered %s at (%d, %d) from %s to %s",
-	      unit_name(vunit->type), x, y, pvictim->name, pplayer->name);
+	      unit_name(vunit->type), vunit->x, vunit->y,
+	      pvictim->name, pplayer->name);
       if (verbose) {
 	notify_player(pvictim,
 		      _("Game: Transfered %s at (%d, %d) from %s to %s."),
-		      unit_name(vunit->type), x, y,
+		      unit_name(vunit->type), vunit->x, vunit->y,
 		      pvictim->name, pplayer->name);
       }
       create_unit_full(pplayer, vunit->x, vunit->y, vunit->type, 
 		       vunit->veteran, pcity->id, vunit->moves_left,
 		       vunit->hp);
       lighten_area(pplayer, vunit->x,vunit->y);
+
     }
     wipe_unit_spec_safe(0, vunit, NULL, 0);
   } unit_list_iterate_end;
@@ -685,11 +717,11 @@ struct city *find_closest_owned_city(struct player *pplayer, int x, int y)
 
 static struct player *split_player(struct player *pplayer)
 {
-  int *nations_used, i, num_nations_avail=game.nation_count, pick;
+  int *nations_used, i, num_nations_avail=game.playable_nation_count, pick;
   int newplayer = game.nplayers;
   struct player *cplayer = &game.players[newplayer];
 
-  nations_used = fc_calloc(game.nation_count,sizeof(int));
+  nations_used = fc_calloc(game.playable_nation_count,sizeof(int));
   
   /* make a new player */
 
@@ -697,18 +729,20 @@ static struct player *split_player(struct player *pplayer)
   
   /* select a new name and nation for the copied player. */
 
-  for(i=0; i<game.nation_count;i++){ 
+  for(i=0; i<game.playable_nation_count;i++){
     nations_used[i]=i;
   }
 
   for(i = 0; i < game.nplayers; i++){
-    nations_used[game.players[i].nation] = -1;
-    num_nations_avail--;
+    if( game.players[i].nation < game.playable_nation_count ) {
+      nations_used[game.players[i].nation] = -1;
+      num_nations_avail--;
+    }
   }
 
   pick = myrand(num_nations_avail);
 
-  for(i=0; i<game.nation_count; i++){ 
+  for(i=0; i<game.playable_nation_count; i++){ 
     if(nations_used[i] != -1)
       pick--;
     if(pick < 0) break;
@@ -768,12 +802,23 @@ static struct player *split_player(struct player *pplayer)
   pplayer->turn_done = 1; /* Have other things to think about - paralysis*/
   pplayer->embassy = 0; /* all embassys destroyed */
 
+  player_limit_to_government_rates(pplayer);
+
   /* copy the maps */
 
   give_map_from_player_to_player(pplayer, cplayer);
 
   game.nplayers++;
   game.max_players = game.nplayers;
+  
+  /* Not sure if this is necessary, but might be a good idea
+     to avoid doing some ai calculations with bogus data:
+  */
+  assess_danger_player(cplayer);
+  if (pplayer->ai.control) {
+    assess_danger_player(pplayer);
+  }
+		    
   return cplayer;
 }
 
@@ -909,7 +954,7 @@ void civil_war(struct player *pplayer)
 	notify_player(pplayer, _("Game: %s declares allegiance to %s."),
 		      pnewcity->name,cplayer->name);
 	map_set_city(pnewcity->x, pnewcity->y, pnewcity);   
-	transfer_city_units(cplayer, pplayer, pnewcity, pcity, 0, 0);
+	transfer_city_units(cplayer, pplayer, pnewcity, pcity, -1, 0);
 	remove_city(pcity); /* don't forget this! */
 	map_set_city(pnewcity->x, pnewcity->y, pnewcity);
 
@@ -971,12 +1016,12 @@ struct city *transfer_city(struct player *pplayer, struct player *cplayer,
 
   if (!pcity)
     return NULL;
-  
-  if(pcity->owner != cplayer->player_no)
-    return NULL;
 
   if (cplayer==pplayer || cplayer==NULL) 
     return NULL;
+  
+  if(pcity->owner != cplayer->player_no)
+    return NULL;
 
   pnewcity=fc_malloc(sizeof(struct city));
   *pnewcity=*pcity;
@@ -992,6 +1037,8 @@ struct city *transfer_city(struct player *pplayer, struct player *cplayer,
   unit_list_init(&pnewcity->units_supported);
   city_list_insert(&pplayer->cities, pnewcity);
 
+  give_citymap_from_player_to_player(pnewcity, cplayer, pplayer);
+
   return pnewcity;
 }
 
