@@ -16,6 +16,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "city.h"
 #include "events.h"
@@ -25,6 +26,7 @@
 #include "log.h"
 #include "map.h"
 #include "player.h"
+#include "rand.h"
 #include "shared.h"
 #include "tech.h"
 #include "unit.h"
@@ -67,8 +69,13 @@ static void city_populate(struct city *pcity);
 static void city_increase_size(struct city *pcity);
 static void city_reduce_size(struct city *pcity);
 
+static int worklist_change_build_target(struct player *pplayer, 
+					struct city *pcity);
+
 static int city_build_stuff(struct player *pplayer, struct city *pcity);
+static int upgrade_improvement(struct city *pcity, int imp);
 static void upgrade_building_prod(struct city *pcity);
+static Unit_Type_id upgrade_unit(struct city *pcity, Unit_Type_id id);
 static void upgrade_unit_prod(struct city *pcity);
 static void obsolete_building_test(struct city *pcity, int b1, int b2);
 static void pay_for_buildings(struct player *pplayer, struct city *pcity);
@@ -362,7 +369,7 @@ static void set_food_trade_shields(struct city *pcity)
     pcity->trade_prod+=pcity->trade_value[i];
   }
   pcity->corruption = city_corruption(pcity, pcity->trade_prod);
-  pcity->ai.trade_want = 8 - city_corruption(pcity, 8);
+  pcity->ai.trade_want = TRADE_WEIGHTING - city_corruption(pcity, TRADE_WEIGHTING);
 /* AI would calculate this 1000 times otherwise; better to do it once -- Syela */
   pcity->trade_prod -= pcity->corruption;
 }
@@ -766,6 +773,8 @@ static void city_increase_size(struct city *pcity)
 {
   int have_square, x, y;
   int has_granary = city_got_effect(pcity, B_GRANARY);
+  int rapture_grow = city_rapture_grow(pcity); /* check before size increase! */
+  int new_food;
   
   if (!city_got_building(pcity, B_AQUEDUCT)
       && pcity->size>=game.aqueduct_size) {/* need aqueduct */
@@ -780,8 +789,9 @@ static void city_increase_size(struct city *pcity)
 		       pcity->name, improvement_types[B_AQUEDUCT].name);
     }
     /* Granary can only hold so much */
-    pcity->food_stock = ((pcity->size+1) * game.foodbox *
-			 (100 - game.aqueductloss/(1+has_granary))) / 100;
+    new_food = ((pcity->size+1) * game.foodbox *
+		(100 - game.aqueductloss/(1+has_granary))) / 100;
+    pcity->food_stock = MIN(pcity->food_stock, new_food);
     return;
   }
 
@@ -798,16 +808,23 @@ static void city_increase_size(struct city *pcity)
 		       pcity->name, improvement_types[B_SEWER].name);
     }
     /* Granary can only hold so much */
-    pcity->food_stock = ((pcity->size+1) * game.foodbox *
-			 (100 - game.aqueductloss/(1+has_granary))) / 100; 
+    new_food = ((pcity->size+1) * game.foodbox *
+		(100 - game.aqueductloss/(1+has_granary))) / 100;
+    pcity->food_stock = MIN(pcity->food_stock, new_food);
     return;
   }
 
   pcity->size++;
-  if (has_granary)
-    pcity->food_stock = ((pcity->size+1) * game.foodbox) / 2;
-  else
-    pcity->food_stock = 0;
+  /* Do not empty food stock if city is growing by celebrating */
+  if (rapture_grow) {
+    new_food = (pcity->size+1) * game.foodbox;
+  } else {
+    if (has_granary)
+      new_food = ((pcity->size+1) * game.foodbox) / 2;
+    else
+      new_food = 0;
+  }
+  pcity->food_stock = MIN(pcity->food_stock, new_food);
 
   /* If there is enough food, and the city is big enough,
    * make new citizens into scientists or taxmen -- Massimo */
@@ -862,16 +879,24 @@ static void city_reduce_size(struct city *pcity)
 }
  
 /**************************************************************************
-...
+  Check whether the population can be increased or
+  if the city is unable to support a 'settler'...
 **************************************************************************/
 static void city_populate(struct city *pcity)
 {
   pcity->food_stock+=pcity->food_surplus;
-  if(pcity->food_stock >= (pcity->size+1)*game.foodbox) 
+  if(pcity->food_stock >= (pcity->size+1)*game.foodbox || city_rapture_grow(pcity))
     city_increase_size(pcity);
   else if(pcity->food_stock<0) {
+    /* FIXME: should this depend on units with ability to build
+     * cities or on units that require food in uppkeep?
+     * I'll assume citybuilders (units that 'contain' 1 pop) -- sjolie
+     * The above may make more logical sense, but in game terms
+     * you want to disband a unit that is draining your food
+     * reserves.  Hence, I'll assume food upkeep > 0 units. -- jjm
+     */
     unit_list_iterate(pcity->units_supported, punit) {
-      if (unit_flag(punit->type, F_SETTLERS)) {
+      if (get_unit_type(punit->type)->food_cost > 0) {
 	char *utname = get_unit_type(punit->type)->name;
 	wipe_unit(0, punit);
 	notify_player_ex(city_owner(pcity), pcity->x, pcity->y, E_UNIT_LOST,
@@ -895,7 +920,7 @@ static void city_populate(struct city *pcity)
 /**************************************************************************
 ...
 **************************************************************************/
-int advisor_choose_build(struct city *pcity)
+int advisor_choose_build(struct player *pplayer, struct city *pcity)
 { /* Old stuff that I obsoleted deleted. -- Syela */
   struct ai_choice choice;
   int i;
@@ -912,14 +937,7 @@ int advisor_choose_build(struct city *pcity)
   want = choice.want;
 
   if (id!=-1 && id != B_LAST && want > 0) {
-    if(is_wonder(id)) {
-      notify_player_ex(0, pcity->x, pcity->y, E_WONDER_STARTED,
-		    _("Game: The %s have started building The %s in %s."),
-		    get_nation_name_plural(city_owner(pcity)->nation),
-		    get_imp_name_ex(pcity, id), pcity->name);
-    }
-    pcity->currently_building=id;
-    pcity->is_building_unit=0;
+    change_build_target(pplayer, pcity, id, 0, E_IMP_AUTO);
     return 1; /* making something.  return value = 1 */
   }
 
@@ -932,6 +950,140 @@ int advisor_choose_build(struct city *pcity)
   return 0;
 }
 
+/**************************************************************************
+  Examine the worklist and change the build target.  Return 0 if no
+  targets are available to change to.  Otherwise return non-zero.  Has
+  the side-effect of removing from the worklist any no-longer-available
+  targets as well as the target actually selected, if any.
+**************************************************************************/
+int worklist_change_build_target(struct player *pplayer, struct city *pcity)
+{
+  int success = 0;
+  int i;
+
+  if (worklist_is_empty(pcity->worklist))
+    /* Nothing in the worklist; bail now. */
+    return 0;
+
+  i = 0;
+  while (1) {
+    int target, is_unit;
+
+    /* What's the next item in the worklist? */
+    if (!worklist_peek_ith(pcity->worklist, &target, &is_unit, i))
+      /* Nothing more in the worklist.  Ah, well. */
+      break;
+
+    i++;
+
+    /* Sanity checks */
+    if (is_unit &&
+	!can_build_unit(pcity, target)) {
+      int new_target = upgrade_unit(pcity, target);
+
+      /* If the city can never build this unit or its descendants, drop it. */
+      if (!can_eventually_build_unit(pcity, new_target)) {
+	/* Nope, never in a million years. */
+	notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_CANTBUILD,
+			 _("Game: %s can't build %s from the worklist.  "
+			   "Purging..."),
+			 pcity->name,
+			 /* Yes, warn about the targets that's actually
+			    in the worklist, not its obsolete-closure
+			    new_target. */
+			 get_unit_type(target)->name);
+	/* Purge this worklist item. */
+	worklist_remove(pcity->worklist, i-1);
+	/* Reset i to index to the now-next element. */
+	i--;
+	
+	continue;
+      }
+
+      /* Maybe we can just upgrade the target to what the city /can/ build. */
+      if (new_target == target) {
+	/* Nope, we're stuck.  Dump this item from the worklist. */
+	notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_CANTBUILD,
+			 _("Game: %s can't build %s from the worklist; "
+			   "tech not yet available.  Postponing..."),
+			 pcity->name,
+			 get_unit_type(target)->name);
+	continue;
+      } else {
+	/* Yep, we can go after new_target instead.  Joy! */
+	notify_player_ex(pplayer, pcity->x, pcity->y, E_WORKLIST,
+			 _("Game: Production of %s is upgraded to %s in %s."),
+			 get_unit_type(target)->name, 
+			 get_unit_type(new_target)->name,
+			 pcity->name);
+	target = new_target;
+      }
+    } else if (!is_unit && !can_build_improvement(pcity, target)) {
+      int new_target = upgrade_improvement(pcity, target);
+
+      /* If the city can never build this improvement, drop it. */
+      if (!can_eventually_build_improvement(pcity, new_target)) {
+	/* Nope, never in a million years. */
+	notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_CANTBUILD,
+			 _("Game: %s can't build %s from the worklist.  "
+			   "Purging..."),
+			 pcity->name,
+			 get_imp_name_ex(pcity, target));
+
+	/* Purge this worklist item. */
+	worklist_remove(pcity->worklist, i-1);
+	/* Reset i to index to the now-next element. */
+	i--;
+	
+	continue;
+      }
+
+
+      /* Maybe this improvement has been obsoleted by something that
+	 we can build. */
+      if (new_target == target) {
+	/* Nope, no use.  *sigh*  */
+	notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_CANTBUILD,
+			 _("Game: %s can't build %s from the worklist; "
+			   "tech not yet available.  Postponing..."),
+			 pcity->name,
+			 get_imp_name_ex(pcity, target));
+	continue;
+      } else {
+	/* Hey, we can upgrade the improvement!  */
+	notify_player_ex(pplayer, pcity->x, pcity->y, E_WORKLIST,
+			 _("Game: Production of %s is upgraded to %s in %s."),
+			 get_imp_name_ex(pcity, target), 
+			 get_imp_name_ex(pcity, new_target),
+			 pcity->name);
+	target = new_target;
+      }
+    }
+
+    /* All okay.  Switch targets. */
+    change_build_target(pplayer, pcity, target, is_unit, E_WORKLIST);
+
+    success = 1;
+    break;
+  }
+
+  if (success) {
+    /* i is the index immediately _after_ the item we're changing to.
+       Remove the (i-1)th item from the worklist. */
+    worklist_remove(pcity->worklist, i-1);
+  }
+
+  if (worklist_is_empty(pcity->worklist)) {
+    /* There *was* something in the worklist, but it's empty now.  Bug the
+       player about it. */
+    notify_player_ex(pplayer, pcity->x, pcity->y, E_WORKLIST,
+		     _("Game: %s's worklist is now empty."),
+		     pcity->name);
+  }
+
+  return success;
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -942,6 +1094,26 @@ static void obsolete_building_test(struct city *pcity, int b1, int b2)
     pcity->currently_building=b2;
 }
 
+/**************************************************************************
+  If imp is obsolete, return the improvement that _can_ be built that
+  lead to imp's obsolesence.
+  !!! Note:  I hear that the building ruleset code is going to be
+  overhauled soon.  If this happens, then this function should be updated
+  to follow the new model.  This function will probably look a lot like
+  upgrade_unit().
+**************************************************************************/
+static int upgrade_improvement(struct city *pcity, int imp)
+{
+  if (imp == B_BARRACKS && can_build_improvement(pcity, B_BARRACKS3))
+    return B_BARRACKS3;
+  else if (imp == B_BARRACKS && can_build_improvement(pcity, B_BARRACKS2))
+    return B_BARRACKS2;
+  else if (imp == B_BARRACKS2 && can_build_improvement(pcity, B_BARRACKS3))
+    return B_BARRACKS3;
+  else
+    return imp;
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -952,6 +1124,23 @@ static void upgrade_building_prod(struct city *pcity)
   obsolete_building_test(pcity, B_BARRACKS2,B_BARRACKS3);
 }
 
+/**************************************************************************
+  Follow the list of obsoleted_by units until we hit something that
+  we can build.  Return id if we can't upgrade at all.  NB:  returning
+  id doesn't guarantee that pcity really _can_ build id; just that
+  pcity can't build whatever _obsoletes_ id.
+**************************************************************************/
+static Unit_Type_id upgrade_unit(struct city *pcity, Unit_Type_id id)
+{
+  while(can_build_unit_direct(pcity, id) &&
+	can_build_unit_direct(pcity, unit_types[id].obsoleted_by))
+  {
+    id = unit_types[id].obsoleted_by;
+  }
+    
+  return id;
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -959,13 +1148,7 @@ static void upgrade_unit_prod(struct city *pcity)
 {
   struct player *pplayer=&game.players[pcity->owner];
   int id = pcity->currently_building;
-  int id2= unit_types[id].obsoleted_by;
-
-  while(can_build_unit_direct(pcity, id2) &&
-	can_build_unit_direct(pcity, unit_types[id2].obsoleted_by))
-  {
-    id2 = unit_types[id2].obsoleted_by;
-  }
+  int id2 = upgrade_unit(pcity, unit_types[id].obsoleted_by);
     
   if (can_build_unit_direct(pcity, id2)) {
     pcity->currently_building=id2;
@@ -985,22 +1168,20 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
 {
   struct government *g = get_gov_pplayer(pplayer);
   int space_part;
-  
-  if (pcity->shield_surplus<0) {
+
+  while (pcity->shield_surplus<0) {
     unit_list_iterate(pcity->units_supported, punit) {
       if (utype_shield_cost(get_unit_type(punit->type), g)) {
 	notify_player_ex(pplayer, pcity->x, pcity->y, E_UNIT_LOST,
 			 _("Game: %s can't upkeep %s, unit disbanded."),
 			 pcity->name, get_unit_type(punit->type)->name);
-	 wipe_unit(pplayer, punit);
-	 break;
+	wipe_unit(pplayer, punit);
+	break;
       }
     }
     unit_list_iterate_end;
   }
-  
-  if(pcity->shield_surplus<=0 && !city_unhappy(pcity)) 
-    pcity->shield_surplus=1;
+
   pcity->shield_stock+=pcity->shield_surplus;
   if (!pcity->is_building_unit) {
     if (pcity->currently_building==B_CAPITAL) {
@@ -1036,6 +1217,7 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
 	pcity->improvements[pcity->currently_building]=1;
       }
       pcity->shield_stock-=improvement_value(pcity->currently_building); 
+      pcity->turn_last_built = game.year;
       /* to eliminate micromanagement */
       if(is_wonder(pcity->currently_building)) {
 	game.global_wonders[pcity->currently_building]=pcity->id;
@@ -1065,8 +1247,8 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
 		      _("Game: %s boosts research, "
 			"you gain 2 immediate advances."),
 		      improvement_types[B_DARWIN].name);
-	update_tech(pplayer, 1000000); 
-	update_tech(pplayer, 1000000); 
+	tech_researched(pplayer);
+	tech_researched(pplayer);
       }
       if (space_part && pplayer->spaceship.state == SSHIP_NONE) {
 	notify_player_ex(0, pcity->x, pcity->y, E_SPACESHIP,
@@ -1078,18 +1260,23 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
 	send_spaceship_info(pplayer, 0);
       } else {
 	city_refresh(pcity);
-	freelog(LOG_DEBUG, "Trying advisor_choose_build.");
-	advisor_choose_build(pcity);
-	freelog(LOG_DEBUG, "Advisor_choose_build didn't kill us.");
-	notify_player_ex(pplayer, pcity->x, pcity->y, E_IMP_AUTO,
-			 _("Game: %s is now building %s."), pcity->name, 
-			 improvement_types[pcity->currently_building].name);
+	/* If there's something in the worklist, change the build target. */
+	if (!worklist_change_build_target(pplayer, pcity)) {
+	  /* Fall back to the good old ways. */
+	  freelog(LOG_DEBUG, "Trying advisor_choose_build.");
+	  advisor_choose_build(pplayer, pcity);
+	  freelog(LOG_DEBUG, "Advisor_choose_build didn't kill us.");
+	}
       }
     } 
   } else {
     upgrade_unit_prod(pcity);
+    /* FIXME: F_CITIES should be changed to any unit
+     * that 'contains' 1 (or more) pop -- sjolie
+     */
     if(pcity->shield_stock>=unit_value(pcity->currently_building)) {
-      if (unit_flag(pcity->currently_building, F_SETTLERS)) {
+      pcity->turn_last_built = game.year;
+      if (unit_flag(pcity->currently_building, F_CITIES)) {
 	if (pcity->size==1) {
 
 	  /* Should we disband the city? -- Massimo */
@@ -1097,7 +1284,7 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
 	    disband_city(pcity);
 	    return 0;
 	  } else {
-	    notify_player_ex(pplayer, pcity->x, pcity->y, E_NOEVENT,
+	    notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_CANTBUILD,
 			     _("Game: %s can't build %s yet."),
 			     pcity->name, unit_name(pcity->currently_building));
 	  }
@@ -1117,6 +1304,13 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
 		       _("Game: %s is finished building %s."), 
 		       pcity->name, 
 		       unit_types[pcity->currently_building].name);
+
+      /* If there's something in the worklist, change the build target. 
+	 If there's nothing there, worklist_change_build_target won't
+	 do anything. */
+      worklist_change_build_target(pplayer, pcity);
+
+
     gamelog(GAMELOG_UNIT, "%s build %s in %s (%i,%i)",
 	    get_nation_name_plural(pplayer->nation), 
 	    unit_types[pcity->currently_building].name,
@@ -1287,13 +1481,7 @@ static int update_city_activity(struct player *pplayer, struct city *pcity)
   int turns_growth, turns_granary;
 
   city_check_workers(pplayer, pcity);
-
-  /* fill citys food box if it is in rapture -- SKi */
-  if (city_refresh(pcity) && g->rapture_size &&
-      pcity->size >= g->rapture_size &&
-      pcity->food_surplus > 0) {
-    pcity->food_stock=(pcity->size+1)*game.foodbox+1; 
-  }
+  city_refresh(pcity);
 
   if (!city_got_effect(pcity,B_GRANARY) && !pcity->is_building_unit &&
     (pcity->currently_building == B_GRANARY) && (pcity->food_surplus > 0)
@@ -1321,28 +1509,38 @@ become obsolete.  This is a quick hack to prevent this.  980805 -- Syela */
     if (!ai_make_elvis(pcity)) break;
   } /* putting this lower in the routine would basically be cheating. -- Syela */
 
-  if (city_build_stuff(pplayer, pcity)) {
-    if (!pcity->was_happy && city_happy(pcity) && pcity->size>4) {
-      notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_LOVE,
-	     	    _("Game: We Love The %s Day celebrated in %s."), 
-	            get_ruler_title(pplayer->government, pplayer->is_male, pplayer->nation),
-		    pcity->name);
-    }
-    if (!city_happy(pcity) && pcity->was_happy && pcity->size>4) {
-      notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_NORMAL,
-		    _("Game: We Love The %s Day canceled in %s."),
-	            get_ruler_title(pplayer->government, pplayer->is_male, pplayer->nation),
-		    pcity->name);
 
+  /* reporting of celebrations rewritten, copying the treatment of disorder below,
+     with the added rapture rounds count.  991219 -- Jing */
+  if (city_build_stuff(pplayer, pcity)) {
+    if (city_celebrating(pcity)) {
+      pcity->rapture++;
+      if (pcity->rapture == 1)
+	notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_LOVE,
+			 _("Game: We Love The %s Day celebrated in %s."), 
+			 get_ruler_title(pplayer->government, pplayer->is_male,
+					 pplayer->nation),
+			 pcity->name);
+    }
+    else {
+      if (pcity->rapture)
+	notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_NORMAL,
+			 _("Game: We Love The %s Day canceled in %s."),
+			 get_ruler_title(pplayer->government, pplayer->is_male,
+					 pplayer->nation),
+			 pcity->name);
+      pcity->rapture=0;
     }
     pcity->was_happy=city_happy(pcity);
-      {
-        int id=pcity->id;
-        city_populate(pcity);
-        if(!city_list_find_id(&pplayer->cities, id))
-	  return 0;
-      }
-     
+
+    /* City population updated here, after the rapture stuff above. --Jing */
+    {
+      int id=pcity->id;
+      city_populate(pcity);
+      if(!city_list_find_id(&pplayer->cities, id))
+	return 0;
+    }
+
     pcity->is_updated=1;
 
     pcity->did_sell=0;
@@ -1412,7 +1610,7 @@ static void disband_city(struct city *pcity)
 
   if (!rcity) {
     /* What should we do when we try to disband our only city? */
-    notify_player_ex(pplayer, x, y, E_NOEVENT,
+    notify_player_ex(pplayer, x, y, E_CITY_CANTBUILD,
 		     _("Game: %s can't build %s yet, "
 		     "and we can't disband our only city."),
 		     pcity->name, unit_name(pcity->currently_building));
@@ -1424,7 +1622,7 @@ static void disband_city(struct city *pcity)
 	      pcity->id, -1);
 
   /* shift all the units supported by pcity (including the new settler) to rcity */
-  transfer_city_units(pplayer, pplayer, rcity, pcity, 0, 1);
+  transfer_city_units(pplayer, pplayer, rcity, pcity, -1, 1);
 
   notify_player_ex(pplayer, x, y, E_UNIT_BUILD,
 		   _("Game: %s is disbanded into %s."), 
