@@ -20,7 +20,8 @@ struct unit;
 
 int unit_being_aggressive(struct unit *punit);
 
-int advisor_choose_build(struct city *pcity);  /* used by the AI */
+int advisor_choose_build(struct player *pplayer, 
+			 struct city *pcity);  /* used by the AI */
 int city_refresh(struct city *pcity);          /* call if city has changed */
 void global_city_refresh(struct player *pplayer); /* tax/govt changed */
 
