@@ -58,13 +58,16 @@
 #include "nation.h"
 #include "packets.h"
 #include "player.h"
+#include "rand.h"
 #include "registry.h"
 #include "shared.h"
+#include "support.h"
 #include "tech.h"
 #include "timing.h"
 #include "version.h"
 
 #include "autoattack.h"
+#include "barbarian.h"
 #include "cityhand.h"
 #include "cityturn.h"
 #include "console.h"
@@ -96,7 +99,6 @@ static void send_all_info(struct player *dest);
 static void shuffle_players(void);
 static void ai_start_turn(void);
 static int is_game_over(void);
-static void read_init_script(char *script_filename);
 static void save_game_auto(void);
 static void generate_ai_players(void);
 static int mark_nation_as_used(int nation);
@@ -124,10 +126,12 @@ enum server_states server_state;
 int force_end_of_sniff;
 char metaserver_info_line[256];
 char metaserver_addr[256];
+unsigned short int metaserver_port;
+
 /* server name for metaserver to use for us */
 char metaserver_servername[64]="";
 
-struct player *shuffled[MAX_NUM_PLAYERS];
+struct player *shuffled[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
 
 /* this counter creates all the id numbers used */
 /* use get_next_id_number()                     */
@@ -135,8 +139,6 @@ unsigned short global_id_counter=100;
 unsigned char used_ids[8192]={0};
 int is_server = 1;
 
-int is_new_game=1;
-
 /* The following is unused, AFAICT.  --dwp */
 char usage[] = 
 "Usage: %s [-fhlpgv] [--file] [--help] [--log] [--port]\n\t[--gamelog] [--version]\n";
@@ -149,8 +151,6 @@ int *nations_avail;
 int *nations_used;
 int num_nations_avail;
 
-int rand_init=0;
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -169,8 +169,9 @@ int main(int argc, char *argv[])
   init_nls();
   dont_run_as_root(argv[0], "freeciv_server");
 
-  strcpy(metaserver_info_line, DEFAULT_META_SERVER_INFO_STRING);
-  strcpy(metaserver_addr, DEFAULT_META_SERVER_ADDR);
+  sz_strlcpy(metaserver_info_line, DEFAULT_META_SERVER_INFO_STRING);
+  sz_strlcpy(metaserver_addr, DEFAULT_META_SERVER_ADDR);
+  metaserver_port = DEFAULT_META_SERVER_PORT;
 
 #ifdef GENERATING_MAC
   Mac_options(&argc, argv);
@@ -198,7 +199,9 @@ int main(int argc, char *argv[])
         no_meta=0;
     else if ((option = get_option("--Metaserver",argv,&i,argc)) != NULL)
     {
-	strcpy(metaserver_addr,argv[i]);
+	sz_strlcpy(metaserver_addr, argv[i]);
+	meta_addr_split();
+
 	no_meta = 0; /* implies --meta */
     }
     else if ((option = get_option("--port",argv,&i,argc)) != NULL)
@@ -206,7 +209,7 @@ int main(int argc, char *argv[])
     else if ((option = get_option("--read",argv,&i,argc)) != NULL)
 	script_filename=option;
     else if ((option = get_option("--server",argv,&i,argc)) != NULL)
-      	strcpy(metaserver_servername,option);
+      	sz_strlcpy(metaserver_servername, option);
     else if ((option = get_option("--debug",argv,&i,argc)) != NULL)
       {
 	loglevel = log_parse_level_str(option);
@@ -236,20 +239,21 @@ int main(int argc, char *argv[])
 	    WEBSITE_URL);
 
   if(h) {
-    fprintf(stderr, _("  -f, --file F\t\tLoad saved game F\n"));
-    fprintf(stderr, _("  -g, --gamelog F\tUse F as game logfile\n"));
+    fprintf(stderr, _("Usage: %s [option ...]\nValid options are:\n"), argv[0]);
     fprintf(stderr, _("  -h, --help\t\tPrint a summary of the options\n"));
-    fprintf(stderr, _("  -l, --log F\t\tUse F as logfile\n"));
-    fprintf(stderr, _("  -m, --meta\t\tSend info to Metaserver\n"));
-    fprintf(stderr, _("  -M, --Metaserver\tSet Metaserver address.\n"));
-    fprintf(stderr, _("  -p, --port N\t\tconnect to port N\n"));
-    fprintf(stderr, _("  -r, --read\t\tRead startup script\n"));
-    fprintf(stderr, _("  -s, --server H\tList this server as host H\n"));
+    fprintf(stderr, _("  -r, --read FILE\tRead startup script FILE\n"));
+    fprintf(stderr, _("  -f, --file FILE\tLoad saved game FILE\n"));
+    fprintf(stderr, _("  -p, --port PORT\tListen for clients on port PORT\n"));
+    fprintf(stderr, _("  -g, --gamelog FILE\tUse FILE as game logfile\n"));
+    fprintf(stderr, _("  -l, --log FILE\tUse FILE as logfile\n"));
+    fprintf(stderr, _("  -m, --meta\t\tSend info to metaserver\n"));
+    fprintf(stderr, _("  -M, --Metaserver ADDR\tSet ADDR as metaserver address\n"));
+    fprintf(stderr, _("  -s, --server HOST\tList this server as host HOST\n"));
 #ifdef DEBUG
-    fprintf(stderr, _("  -d, --debug N\t\tSet debug log level (0,1,2,3,"
+    fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (0,1,2,3,"
 	                                "or 3:file1,min,max:...)\n"));
 #else
-    fprintf(stderr, _("  -d, --debug N\t\tSet debug log level (0,1,2)\n"));
+    fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (0,1,2)\n"));
 #endif
     fprintf(stderr, _("  -v, --version\t\tPrint the version number\n"));
     fprintf(stderr, _("Report bugs to <%s>.\n"), BUG_EMAIL_ADDRESS);
@@ -269,8 +273,9 @@ int main(int argc, char *argv[])
   con_puts(C_COMMENT, "");
 #endif
 #if IS_BETA_VERSION
-  con_write(C_COMMENT, _("Freeciv 1.9 will be released "
-			 "early October, at %s"), WEBSITE_URL);
+  con_puts(C_COMMENT, "");
+  con_puts(C_COMMENT, beta_message());
+  con_puts(C_COMMENT, "");
 #endif
   
   con_flush();
@@ -292,18 +297,12 @@ int main(int argc, char *argv[])
       freelog(LOG_FATAL, _("Couldn't load savefile: %s"), load_filename);
       exit(1);
     }
-    game.scenario=game_load(&file);
+    game_load(&file);
     section_file_check_unused(&file, load_filename);
     section_file_free(&file);
 
-   /* game.scenario: 0=normal savegame, 1=everything but players,
-       2=just tile map and startpositions, 3=just tile map
-   */
-    if (game.scenario) { /* we may have a scenario here */
-      if(game.nplayers) { /* no, it's just a normal savegame */
-	is_new_game=0;
-	game.scenario=0;
-      }
+    if (!game.is_new_game) {
+      /* Is this right/necessary/the_right_place_for_this/etc? --dwp */
       while(is_id_allocated(global_id_counter++));
     }
     freelog(LOG_VERBOSE, "Load time: %g seconds (%g apparent)",
@@ -315,8 +314,8 @@ int main(int argc, char *argv[])
   init_connections(); 
   server_open_socket();
   if(no_meta==0) {
-    freelog(LOG_NORMAL, _("Sending info to metaserver[%s %d]"),
-	    metaserver_addr, METASERVER_PORT);
+    freelog(LOG_NORMAL, _("Sending info to metaserver [%s]"),
+	    meta_addr_port());
     server_open_udp(); /* open socket for meta server */ 
   }
 
@@ -334,20 +333,20 @@ int main(int argc, char *argv[])
 
   send_server_info_to_metaserver(1,0);
 
-  if(is_new_game) {
+  if(game.is_new_game) {
     load_rulesets();
     /* otherwise rulesets were loaded when savegame was loaded */
   }
 
-  nations_avail = fc_calloc( game.nation_count, sizeof(int));
-  nations_used = fc_calloc( game.nation_count, sizeof(int));
+  nations_avail = fc_calloc( game.playable_nation_count, sizeof(int));
+  nations_used = fc_calloc( game.playable_nation_count, sizeof(int));
 
 main_start_players:
 
   send_rulesets(0);
 
-  num_nations_avail = game.nation_count;
-  for(i=0; i<game.nation_count; i++) {
+  num_nations_avail = game.playable_nation_count;
+  for(i=0; i<game.playable_nation_count; i++) {
       nations_avail[i]=i;
       nations_used[i]=i;
   }
@@ -392,38 +391,38 @@ main_start_players:
     game.randseed = time(NULL) & (MAX_UINT32 >> 1);
   }
  
-  if(!rand_init)
+  if(!myrand_is_init())
     mysrand(game.randseed);
 
-  if(is_new_game)
+  if(game.is_new_game)
     generate_ai_players();
    
   /* if we have a tile map, and map.generator==0, call map_fractal_generate
      anyway, to make the specials and huts */
-  if(map_is_empty() || (map.generator == 0 && is_new_game))
+  if(map_is_empty() || (map.generator == 0 && game.is_new_game))
     map_fractal_generate();
-  else 
-    flood_it(1);
+
+  if (map.num_continents==0)
+    assign_continent_numbers();
+
   gamelog_map();
   /* start the game */
 
   server_state=RUN_GAME_STATE;
   send_server_info_to_metaserver(1,0);
 
-  if(is_new_game) {
+  if(game.is_new_game) {
     int i;
     for(i=0; i<game.nplayers; i++) {
-      init_tech(&game.players[i], game.tech); 
+      init_tech(&game.players[i], game.tech);
+      player_limit_to_government_rates(&game.players[i]);
       game.players[i].economic.gold=game.gold;
     }
     game.max_players=game.nplayers;
 
     /* we don't want random start positions in a scenario which already
        provides them.  -- Gudy */
-    if(game.scenario==1 || game.scenario==2)
-      flood_it(1);
-    else {
-      flood_it(0);
+    if(map.num_start_positions==0) {
       create_start_positions();
     }
   }
@@ -431,29 +430,26 @@ main_start_players:
   initialize_move_costs(); /* this may be the wrong place to do this */
   generate_minimap(); /* for city_desire; saves a lot of calculations */
 
-  if (!is_new_game) {
+  if (!game.is_new_game) {
     for (i=0;i<game.nplayers;i++) {
-      civ_score(&game.players[i]);  /* if we don't, the AI gets really confused */
-      if (game.players[i].ai.control) {
-	set_ai_level_direct(&game.players[i], game.players[i].ai.skill_level);
-        city_list_iterate(game.players[i].cities, pcity)
-          assess_danger(pcity); /* a slowdown, but a necessary one */
-        city_list_iterate_end;
+      struct player *pplayer = get_player(i);
+      civ_score(pplayer);	/* if we don't, the AI gets really confused */
+      if (pplayer->ai.control) {
+	set_ai_level_direct(pplayer, pplayer->ai.skill_level);
+	assess_danger_player(pplayer); /* a slowdown, but a necessary one */
       }
     }
   }
   
   send_all_info(0);
 
-  if(is_new_game) 
+  if(game.is_new_game) 
     init_new_game();
-    
+
+  game.is_new_game = 0;
+
   send_game_state(0, CLIENT_GAME_RUNNING_STATE);
   
-  /* from here on, treat scenarios as normal games, as this is what
-     they have become (nothing special about them anymore)*/
-  game.scenario=0;
-  
   while(server_state==RUN_GAME_STATE) {
     force_end_of_sniff=0;
     freelog(LOG_DEBUG, "Shuffleplayers");
@@ -466,6 +462,9 @@ main_start_players:
     
     for(i=0;i<game.nplayers;i++)
       connection_do_buffer(game.players[i].conn);
+    freelog(LOG_DEBUG, "Season of native unrests");
+    summon_barbarians(); /* wild guess really, no idea where to put it, but
+                            I want to give them chance to move their units */
     freelog(LOG_DEBUG, "Autosettlers");
     auto_settlers(); /* moved this after ai_start_turn for efficiency -- Syela */
     /* moved after sniff_packets for even more efficiency.
@@ -479,7 +478,7 @@ main_start_players:
     freelog(LOG_DEBUG, "Endturn");
     end_turn();
     freelog(LOG_DEBUG, "Gamenextyear");
-    game_next_year();
+    game_advance_year();
     check_spaceship_arrivals();
     freelog(LOG_DEBUG, "Sendplayerinfo");
     send_player_info(0, 0);
@@ -514,15 +513,25 @@ main_start_players:
   return 0;
 }
 
+/**************************************************************************
+...
+**************************************************************************/
 static int is_game_over(void)
 {
+  int barbs = 0;
   int alive = 0;
   int i;
-  if (game.nplayers == 1)
-    return 0;
+
   for (i=0;i<game.nplayers; i++) {
-    if (game.players[i].is_alive)
-      alive ++;
+    if (is_barbarian(&(game.players[i])))
+      barbs++;
+  }
+  if (game.nplayers == (barbs + 1))
+    return 0;
+
+  for (i=0;i<game.nplayers; i++) {
+    if (game.players[i].is_alive && !(is_barbarian(&(game.players[i]))))
+      alive++;
   }
   return (alive <= 1);
 }
@@ -530,27 +539,6 @@ static int is_game_over(void)
 /**************************************************************************
 ...
 **************************************************************************/
-static void read_init_script(char *script_filename)
-{
-  FILE *script_file;
-  char buffer[512];
-
-  script_file = fopen(script_filename,"r");
-  if (script_file)
-    {
-      /* the size 511 is set as to not overflow buffer in handle_stdin_input */
-      while(fgets(buffer,511,script_file))
-	handle_stdin_input((struct player *)NULL, buffer);
-      fclose(script_file);
-    }
-  else
-    freelog(LOG_NORMAL, _("Could not open script file '%s'."), script_filename);
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-
 int send_server_info_to_metaserver(int do_send,int reset_timer)
 {
   static struct timer *time_since_last_send = NULL;
@@ -577,41 +565,43 @@ int send_server_info_to_metaserver(int do_send,int reset_timer)
   /* build description block */
   desc[0]='\0';
   
-  sprintf(desc+strlen(desc), "Freeciv\n");
-  sprintf(desc+strlen(desc), VERSION_STRING"\n");
+  cat_snprintf(desc, sizeof(desc), "Freeciv\n");
+  cat_snprintf(desc, sizeof(desc), VERSION_STRING"\n");
   switch(server_state) {
   case PRE_GAME_STATE:
-    sprintf(desc+strlen(desc), "Pregame\n");
+    cat_snprintf(desc, sizeof(desc), "Pregame\n");
     break;
   case RUN_GAME_STATE:
-    sprintf(desc+strlen(desc), "Running\n");
+    cat_snprintf(desc, sizeof(desc), "Running\n");
     break;
   case GAME_OVER_STATE:
-    sprintf(desc+strlen(desc), "Game over\n");
+    cat_snprintf(desc, sizeof(desc), "Game over\n");
     break;
   default:
-    sprintf(desc+strlen(desc), "Waiting\n");
+    cat_snprintf(desc, sizeof(desc), "Waiting\n");
   }
-  sprintf(desc+strlen(desc), "%s\n",metaserver_servername);
-  sprintf(desc+strlen(desc), "%d\n", port);
-  sprintf(desc+strlen(desc), "%d\n", game.nplayers);
-  sprintf(desc+strlen(desc), "%s", metaserver_info_line);
+  cat_snprintf(desc, sizeof(desc), "%s\n",metaserver_servername);
+  cat_snprintf(desc, sizeof(desc), "%d\n", port);
+  cat_snprintf(desc, sizeof(desc), "%d\n", game.nplayers);
+  cat_snprintf(desc, sizeof(desc), "%s", metaserver_info_line);
     
   /* now build the info block */
   info[0]='\0';
-  sprintf(info+strlen(info), 
+  cat_snprintf(info, sizeof(info),
 	  "Players:%d  Min players:%d  Max players:%d\n",
 	  game.nplayers, game.min_players, game.max_players);
-  sprintf(info+strlen(info), 
+  cat_snprintf(info, sizeof(info),
 	  "Timeout:%d  Year: %s\n",
 	  game.timeout, textyear(game.year));
     
     
-  sprintf(info+strlen(info), "NO:  NAME:               HOST:\n");
-  sprintf(info+strlen(info), "----------------------------------------\n");
+  cat_snprintf(info, sizeof(info),
+	       "NO:  NAME:               HOST:\n");
+  cat_snprintf(info, sizeof(info),
+	       "----------------------------------------\n");
   for(i=0; i<game.nplayers; ++i) {
-    sprintf(info+strlen(info), "%2d   %-20s %s\n", 
-	    i, game.players[i].name, game.players[i].addr);
+    cat_snprintf(info, sizeof(info), "%2d   %-20s %s\n", 
+		 i, game.players[i].name, game.players[i].addr);
   }
 
   clear_timer_start(time_since_last_send);
@@ -741,6 +731,9 @@ static void shuffle_players(void)
   }
 }
 
+/**************************************************************************
+...
+**************************************************************************/
 static void ai_start_turn(void)
 {
   int i;
@@ -814,7 +807,8 @@ static void save_game_auto(void)
 
   assert(strlen(game.save_name)<256);
   
-  sprintf(filename, "%s%d.sav", game.save_name, game.year);
+  my_snprintf(filename, sizeof(filename),
+	      "%s%d.sav", game.save_name, game.year);
   save_game(filename);
   gamelog_save();		/* should this be in save_game()? --dwp */
 }
@@ -931,6 +925,10 @@ void handle_packet_input(struct connection *pconn, char *packet, int type)
     handle_city_change(pplayer, (struct packet_city_request *)packet);
     break;
 
+  case PACKET_CITY_WORKLIST:
+    handle_city_worklist(pplayer, (struct packet_city_request *)packet);
+    break;
+
   case PACKET_CITY_MAKE_SPECIALIST:
     handle_city_make_specialist(pplayer, (struct packet_city_request *)packet);
     break;
@@ -967,6 +965,10 @@ void handle_packet_input(struct connection *pconn, char *packet, int type)
     handle_player_tech_goal(pplayer, (struct packet_player_request *)packet);
     break;
 
+  case PACKET_PLAYER_WORKLIST:
+    handle_player_worklist(pplayer, (struct packet_player_request *)packet);
+    break;
+
   case PACKET_UNIT_BUILD_CITY:
     handle_unit_build_city(pplayer, (struct packet_unit_request *)packet);
     break;
@@ -1050,6 +1052,9 @@ void handle_packet_input(struct connection *pconn, char *packet, int type)
   case PACKET_UNIT_PARADROP_TO:
     handle_unit_paradrop_to(pplayer, (struct packet_unit_request *)packet);
     break;
+  case PACKET_UNIT_CONNECT:
+    handle_unit_connect(pplayer, (struct packet_unit_connect *)packet);
+    break;
   default:
     freelog(LOG_NORMAL, "uh got an unknown packet from %s", game.players[i].name);
   }
@@ -1120,10 +1125,10 @@ int check_for_full_turn_done(void)
 static void handle_turn_done(int player_no)
 {
   game.players[player_no].turn_done=1;
-  
-  /* if someone is still moving, then let the other know that this play moved */ 
-  if(!check_for_full_turn_done())
-    send_player_info(&game.players[player_no], 0);
+
+  check_for_full_turn_done();
+
+  send_player_info(&game.players[player_no], 0);
 }
 
 /**************************************************************************
@@ -1157,7 +1162,7 @@ static void handle_alloc_nation(int player_no, struct packet_alloc_nation *packe
                              &select_nation);
 
   game.players[player_no].nation=packet->nation_no;
-  strcpy(game.players[player_no].name, packet->name);
+  sz_strlcpy(game.players[player_no].name, packet->name);
   game.players[player_no].is_male=packet->is_male;
   game.players[player_no].city_style=packet->city_style;
 
@@ -1172,7 +1177,7 @@ static void handle_alloc_nation(int player_no, struct packet_alloc_nation *packe
   mark_nation_as_used(packet->nation_no);
 
   /* if there's no nation left, reject remaining players, sorry */
-  if( nations_used == game.nation_count ) {
+  if( nations_used == game.playable_nation_count ) {   /* barb */
     for(i=0; i<game.nplayers; i++) {
       if( game.players[i].nation == MAX_NUM_NATIONS ) {
         reject_new_player(_("Sorry, you can't play."
@@ -1225,9 +1230,9 @@ static void join_game_accept(struct player *pplayer, int rejoin)
   assert(pplayer);
   assert(pplayer->conn);
   packet.you_can_join = 1;
-  strcpy(packet.capability, our_capability);
-  sprintf(packet.message, "%s %s.", (rejoin?"Welcome back":"Welcome"),
-				     pplayer->name);
+  sz_strlcpy(packet.capability, our_capability);
+  my_snprintf(packet.message, sizeof(packet.message),
+	      "%s %s.", (rejoin?"Welcome back":"Welcome"), pplayer->name);
   send_packet_join_game_reply(pplayer->conn, &packet);
   if (rejoin) {
     freelog(LOG_NORMAL, _("<%s@%s> has rejoined the game."),
@@ -1258,7 +1263,7 @@ static void introduce_game_to_player(struct player *pplayer)
 {
   char hostname[512];
   struct player *cplayer;
-  int i, nconn, nother;
+  int i, nconn, nother, nbarb;
   
   if (!pplayer->conn)
     return;
@@ -1295,8 +1300,9 @@ static void introduce_game_to_player(struct player *pplayer)
     /* if the game is running, players can just view the Players menu?  --dwp */
     return;
   }
-  for(nconn=0, i=0; i<game.nplayers; ++i) {
+  for(nbarb=0, nconn=0, i=0; i<game.nplayers; ++i) {
     nconn += (game.players[i].is_connected);
+    nbarb += is_barbarian(&game.players[i]);
   }
   if (nconn != 1) {
     notify_player(pplayer, _("There are currently %d players connected:"),
@@ -1312,7 +1318,7 @@ static void introduce_game_to_player(struct player *pplayer)
 		     :cplayer->ai.control ? _(" (AI mode)") : ""));
     }
   }
-  nother = game.nplayers - nconn;
+  nother = game.nplayers - nconn - nbarb;
   if (nother > 0) {
     if (nother == 1) {
       notify_player(pplayer, _("There is 1 other player:"));
@@ -1321,6 +1327,7 @@ static void introduce_game_to_player(struct player *pplayer)
     }
     for(i=0; i<game.nplayers; ++i) {
       cplayer = &game.players[i];
+      if( is_barbarian(cplayer) ) continue;
       if (!cplayer->is_connected) {
 	notify_player(pplayer, "  %s%s", cplayer->name, 
 		    ((!cplayer->is_alive) ? _(" (R.I.P.)")
@@ -1347,12 +1354,12 @@ void accept_new_player(char *name, struct connection *pconn)
    * inheriting stale AI status etc
    */
   
-  strcpy(pplayer->name, name);
-  strcpy(pplayer->username, name);
+  sz_strlcpy(pplayer->name, name);
+  sz_strlcpy(pplayer->username, name);
   pplayer->conn = pconn;
   pplayer->is_connected = (pconn!=NULL);
   if (pconn) {
-    strcpy(pplayer->addr, pconn->addr); 
+    sz_strlcpy(pplayer->addr, pconn->addr); 
     join_game_accept(pplayer, 0);
   } else {
     freelog(LOG_NORMAL, _("%s has been added as an AI-controlled player."),
@@ -1377,8 +1384,8 @@ static void reject_new_player(char *msg, struct connection *pconn)
   struct packet_join_game_reply packet;
   
   packet.you_can_join=0;
-  strcpy(packet.capability, our_capability);
-  strcpy(packet.message, msg);
+  sz_strlcpy(packet.capability, our_capability);
+  sz_strlcpy(packet.message, msg);
   send_packet_join_game_reply(pconn, &packet);
 }
   
@@ -1397,11 +1404,12 @@ static void handle_request_join_game(struct connection *pconn,
 	  req->patch_version, req->version_label);
   freelog(LOG_VERBOSE, "Client caps: %s", req->capability);
   freelog(LOG_VERBOSE, "Server caps: %s", our_capability);
-  strcpy(pconn->capability, req->capability);
+  sz_strlcpy(pconn->capability, req->capability);
   
   /* Make sure the server has every capability the client needs */
   if (!has_capabilities(our_capability, req->capability)) {
-    sprintf(msg, _("The client is missing a capability that this server needs.\n"
+    my_snprintf(msg, sizeof(msg),
+		_("The client is missing a capability that this server needs.\n"
 		   "Server version: %d.%d.%d%s Client version: %d.%d.%d%s."
 		   "  Upgrading may help!"),
 	    MAJOR_VERSION, MINOR_VERSION, PATCH_VERSION, VERSION_LABEL,
@@ -1416,7 +1424,8 @@ static void handle_request_join_game(struct connection *pconn,
 
   /* Make sure the client has every capability the server needs */
   if (!has_capabilities(req->capability, our_capability)) {
-    sprintf(msg, _("The server is missing a capability that the client needs.\n"
+    my_snprintf(msg, sizeof(msg),
+		_("The server is missing a capability that the client needs.\n"
 		   "Server version: %d.%d.%d%s Client version: %d.%d.%d%s."
 		   "  Upgrading may help!"),
 	    MAJOR_VERSION, MINOR_VERSION, PATCH_VERSION, VERSION_LABEL,
@@ -1434,7 +1443,7 @@ static void handle_request_join_game(struct connection *pconn,
     if(!pplayer->conn) {
       pplayer->conn=pconn;
       pplayer->is_connected=1;
-      strcpy(pplayer->addr, pconn->addr); 
+      sz_strlcpy(pplayer->addr, pconn->addr); 
       join_game_accept(pplayer, 1);
       introduce_game_to_player(pplayer);
       if(server_state==RUN_GAME_STATE) {
@@ -1468,8 +1477,9 @@ static void handle_request_join_game(struct connection *pconn,
       }
     }
 
-    sprintf(msg, _("You can't join the game.  %s is already connected."), 
-	    pplayer->name);
+    my_snprintf(msg, sizeof(msg),
+		_("You can't join the game.  %s is already connected."), 
+		pplayer->name);
     reject_new_player(msg, pconn);
     freelog(LOG_NORMAL, _("%s was rejected."), pplayer->name);
     close_connection(pconn);
@@ -1512,12 +1522,12 @@ void lost_connection_to_player(struct connection *pconn)
     if(game.players[i].conn==pconn) {
       game.players[i].conn=NULL;
       game.players[i].is_connected=0;
-      strcpy(game.players[i].addr, "---.---.---.---");
+      sz_strlcpy(game.players[i].addr, "---.---.---.---");
       freelog(LOG_NORMAL, _("Lost connection to %s."), game.players[i].name);
       send_player_info(&game.players[i], 0);
       notify_player(0, _("Game: Lost connection to %s."), game.players[i].name);
 
-      if(is_new_game && (server_state==PRE_GAME_STATE ||
+      if(game.is_new_game && (server_state==PRE_GAME_STATE ||
 			 server_state==SELECT_RACES_STATE))
 	 server_remove_player(&game.players[i]);
       check_for_full_turn_done();
@@ -1567,7 +1577,7 @@ static void generate_ai_players(void)
       continue;
     }
 
-    for (nation = 0; nation < game.nation_count; nation++) {
+    for (nation = 0; nation < game.playable_nation_count; nation++) {
       if ( check_nation_leader_name( nation, game.players[player].name ) )
         if(nations_used[nation] != -1) {
            game.players[player].nation=mark_nation_as_used(nation);
@@ -1577,7 +1587,7 @@ static void generate_ai_players(void)
         }
     }
 
-    if(nation == game.nation_count) {
+    if(nation == game.playable_nation_count) {
       game.players[player].nation=
               mark_nation_as_used(nations_avail[myrand(num_nations_avail)]);
       game.players[player].is_male=myrand(2);
@@ -1590,11 +1600,11 @@ static void generate_ai_players(void)
   /* Create and pick nation and name for AI players needed to bring the
      total number of players == game.aifill */
 
-  if( game.nation_count < game.aifill ) {
-    game.aifill = game.nation_count;
+  if( game.playable_nation_count < game.aifill ) {
+    game.aifill = game.playable_nation_count;
     freelog( LOG_NORMAL,
 	     _("Nation count smaller than aifill; aifill reduced to %d."),
-             game.nation_count);
+             game.playable_nation_count);
   }
 
   for(;game.nplayers < game.aifill;) {
@@ -1624,6 +1634,16 @@ static void generate_ai_players(void)
 }
 
 
+/*************************************************************************
+ Used in pick_ai_player_name() below; buf has size at least MAX_LEN_NAME;
+*************************************************************************/
+static int good_name(char *try, char *buf) {
+  if (!find_player_by_name(try)) {
+     mystrlcpy(buf, try, MAX_LEN_NAME);
+     return 1;
+  }
+  return 0;
+}
 
 /*************************************************************************
  pick_ai_player_name() - Returns a random ruler name picked from given nation
@@ -1631,31 +1651,31 @@ static void generate_ai_players(void)
      taken, iterates through all leader names to find unused one. If it fails
      it iterates through "Player 1", "Player 2", ... until an unused name
      is found.
+ newname should point to a buffer of size at least MAX_LEN_NAME.
 *************************************************************************/
-void pick_ai_player_name (Nation_Type_id nation, char *newname) 
+void pick_ai_player_name(Nation_Type_id nation, char *newname) 
 {
-   int i, playernumber=1, names_count;
-   char tempname[50];
+   int i, names_count;
    char **names;
 
    names = get_nation_leader_names(nation, &names_count);
 
-   /* first try random name, then all available */
-   strcpy( tempname, names[myrand(names_count)] );
-
-   if( find_player_by_name(tempname) ) {
-     for(i=0; i<names_count; i++) {
-       sprintf( tempname, names[i] );
-       if( !find_player_by_name(tempname) ) {
-         strcpy(newname,tempname);
-         return;
-       }
-     }
-     while(find_player_by_name(tempname)) {
-       sprintf(tempname, _("Player %d"), playernumber++);
-     }
+   /* Try random names (scattershot), then all available,
+    * then "Player 1" etc:
+    */
+   for(i=0; i<names_count; i++) {
+     if (good_name(names[myrand(names_count)], newname)) return;
+   }
+   
+   for(i=0; i<names_count; i++) {
+     if (good_name(names[i], newname)) return;
+   }
+   
+   for(i=1; /**/; i++) {
+     char tempname[50];
+     my_snprintf(tempname, sizeof(tempname), _("Player %d"), i);
+     if (good_name(tempname, newname)) return;
    }
-   strcpy(newname,tempname);
 }
 
 /*************************************************************************
