@@ -25,7 +25,9 @@
 #include "map.h"
 #include "mem.h"
 #include "packets.h"
+#include "rand.h"
 #include "registry.h"
+#include "support.h"
 #include "version.h"
 
 #include "cityturn.h"
@@ -40,11 +42,9 @@
 
 extern char metaserver_info_line[];
 extern char metaserver_addr[];
-extern RANDOM_TYPE RandomState[];
-extern int iRandJ, iRandK, iRandX; 
-extern int rand_init;
 
-#define SAVEFILE_OPTIONS "1.7 startoptions unirandom spacerace2 rulesets"
+#define SAVEFILE_OPTIONS "1.7 startoptions unirandom spacerace2 rulesets \
+diplchance_percent"
 
 /**************************************************************************
 ...
@@ -54,7 +54,7 @@ void init_new_game(void)
   int i, j, x, y;
   int start_pos[MAX_NUM_PLAYERS]; /* indices into map.start_positions[] */
   
-  if (game.scenario!=1 && game.scenario!=2) {
+  if (!map.fixed_start_positions) {
     /* except in a scenario which provides them,
        shuffle the start positions around... */
     assert(game.nplayers==map.num_start_positions);
@@ -74,7 +74,7 @@ void init_new_game(void)
      to random from remainder.  (Would be better to label start
      positions by nation etc, but this will do for now. --dwp)
   */
-  if(game.scenario!=1 && game.scenario!=2) {
+  if(!map.fixed_start_positions) {
     for(i=0; i<game.nplayers; i++) {
       start_pos[i] = i;
     } 
@@ -132,7 +132,7 @@ void init_new_game(void)
       light_square(&game.players[i], x, y+1, 1);
     }
     for (j=0;j<game.settlers;j++) 
-      create_unit(&game.players[i], x, y, get_role_unit(F_SETTLERS,0), 0, 0, -1);
+      create_unit(&game.players[i], x, y, get_role_unit(F_CITIES,0), 0, 0, -1);
     for (j=0;j<game.explorer;j++) 
       create_unit(&game.players[i], x, y, get_role_unit(L_EXPLORER,0), 0, 0, -1);
   }
@@ -268,23 +268,28 @@ static void apply_unit_ordering(void)
 /***************************************************************
 ...
 ***************************************************************/
-int game_load(struct section_file *file)
+void game_load(struct section_file *file)
 {
   int i;
   enum server_states tmp_server_state;
   char *savefile_options=" ";
-  char name[20],*string;
+  char *string;
 
   game.version = secfile_lookup_int_default(file, 0, "game.version");
   tmp_server_state = (enum server_states)
     secfile_lookup_int_default(file, RUN_GAME_STATE, "game.server_state");
-  strcpy(metaserver_info_line,
-	 secfile_lookup_str_default(file, DEFAULT_META_SERVER_INFO_STRING,
-				    "game.metastring"));
-  strcpy(metaserver_addr,
-	 secfile_lookup_str_default(file, DEFAULT_META_SERVER_ADDR,
-				    "game.metaserver"));
-  
+
+  /* grr, hardcoded sizes --dwp */
+  mystrlcpy(metaserver_info_line,
+	    secfile_lookup_str_default(file, DEFAULT_META_SERVER_INFO_STRING,
+				       "game.metastring"),
+	    256);
+  mystrlcpy(metaserver_addr,
+	    secfile_lookup_str_default(file, DEFAULT_META_SERVER_ADDR,
+				       "game.metaserver"),
+	    256);
+  meta_addr_split();
+
   game.gold          = secfile_lookup_int(file, "game.gold");
   game.tech          = secfile_lookup_int(file, "game.tech");
   game.skill_level   = secfile_lookup_int(file, "game.skill_level");
@@ -339,48 +344,71 @@ int game_load(struct section_file *file)
 
   game.scorelog = secfile_lookup_int_default(file, 0, "game.scorelog");
   
-  game.diplchance = secfile_lookup_int_default(file, game.diplchance,
-					       "game.diplchance");
+  if(has_capability("diplchance_percent", savefile_options)) {
+    game.diplchance = secfile_lookup_int_default(file, game.diplchance,
+						 "game.diplchance");
+  } else {
+    game.diplchance = secfile_lookup_int_default(file, 3, /* old default */
+						 "game.diplchance");
+    if (game.diplchance < 2) {
+      game.diplchance = GAME_MAX_DIPLCHANCE;
+    } else if (game.diplchance > 10) {
+      game.diplchance = GAME_MIN_DIPLCHANCE;
+    } else {
+      game.diplchance = 100 - (10 * (game.diplchance - 1));
+    }
+  }
   game.aqueductloss = secfile_lookup_int_default(file, game.aqueductloss,
 						 "game.aqueductloss");
   game.turnblock = secfile_lookup_int_default(file,game.turnblock,
                          "game.turnblock");
+  game.barbarianrate = secfile_lookup_int_default(file, game.barbarianrate,
+						  "game.barbarians");
+  game.onsetbarbarian = secfile_lookup_int_default(file, game.onsetbarbarian,
+						   "game.onsetbarbs");
+  game.occupychance = secfile_lookup_int_default(file, game.occupychance,
+  						 "game.occupychance");
   
   if(has_capability("unirandom", savefile_options)) {
+    RANDOM_STATE rstate;
     game.randseed = secfile_lookup_int(file, "game.randseed");
-    iRandJ = secfile_lookup_int(file,"random.index_J");
-    iRandK = secfile_lookup_int(file,"random.index_K");
-    iRandX = secfile_lookup_int(file,"random.index_X");
+    rstate.j = secfile_lookup_int(file,"random.index_J");
+    rstate.k = secfile_lookup_int(file,"random.index_K");
+    rstate.x = secfile_lookup_int(file,"random.index_X");
     for(i=0;i<8;i++) {
-      sprintf(name,"random.table%d",i);
+      char name[20];
+      my_snprintf(name, sizeof(name), "random.table%d",i);
       string=secfile_lookup_str(file,name);
-      sscanf(string,"%8X %8X %8X %8X %8X %8X %8X",&RandomState[7*i],
-	     &RandomState[7*i+1],&RandomState[7*i+2],&RandomState[7*i+3],
-	     &RandomState[7*i+4],&RandomState[7*i+5],&RandomState[7*i+6]);
+      sscanf(string,"%8X %8X %8X %8X %8X %8X %8X", &rstate.v[7*i],
+	     &rstate.v[7*i+1], &rstate.v[7*i+2], &rstate.v[7*i+3],
+	     &rstate.v[7*i+4], &rstate.v[7*i+5], &rstate.v[7*i+6]);
     }
-    rand_init=1;
+    rstate.is_init = 1;
+    set_myrand_state(rstate);
   }
 
-  strcpy(game.ruleset.techs,
-	 secfile_lookup_str_default(file, "default", "game.ruleset.techs"));
-  strcpy(game.ruleset.units,
-	 secfile_lookup_str_default(file, "default", "game.ruleset.units"));
-  strcpy(game.ruleset.buildings,
-	 secfile_lookup_str_default(file, "default", "game.ruleset.buildings"));
-  strcpy(game.ruleset.terrain,
-	 secfile_lookup_str_default(file, "classic", "game.ruleset.terrain"));
-  strcpy(game.ruleset.governments,
-	 secfile_lookup_str_default(file, "default",
-				    "game.ruleset.governments"));
-  strcpy(game.ruleset.nations,
-         secfile_lookup_str_default(file, "default", "game.ruleset.nations"));
-  strcpy(game.ruleset.cities,
-         secfile_lookup_str_default(file, "default", "game.ruleset.cities"));
+  sz_strlcpy(game.ruleset.techs,
+	     secfile_lookup_str_default(file, "default", "game.ruleset.techs"));
+  sz_strlcpy(game.ruleset.units,
+	     secfile_lookup_str_default(file, "default", "game.ruleset.units"));
+  sz_strlcpy(game.ruleset.buildings,
+	     secfile_lookup_str_default(file, "default",
+					"game.ruleset.buildings"));
+  sz_strlcpy(game.ruleset.terrain,
+	     secfile_lookup_str_default(file, "classic",
+					"game.ruleset.terrain"));
+  sz_strlcpy(game.ruleset.governments,
+	     secfile_lookup_str_default(file, "default",
+					"game.ruleset.governments"));
+  sz_strlcpy(game.ruleset.nations,
+	     secfile_lookup_str_default(file, "default",
+					"game.ruleset.nations"));
+  sz_strlcpy(game.ruleset.cities,
+	     secfile_lookup_str_default(file, "default", "game.ruleset.cities"));
 
-
-  strcpy(game.demography,
-	 secfile_lookup_str_default(file, GAME_DEFAULT_DEMOGRAPHY,
-				    "game.demography"));
+  sz_strlcpy(game.demography,
+	     secfile_lookup_str_default(file, GAME_DEFAULT_DEMOGRAPHY,
+					"game.demography"));
 
   game.spacerace = secfile_lookup_int_default(file, game.spacerace,
 					      "game.spacerace");
@@ -414,12 +442,15 @@ int game_load(struct section_file *file)
       }
 
       if (tmp_server_state==PRE_GAME_STATE &&
-	  map.generator == 0) { /* generator 0 = map done with map editor */
+	  map.generator == 0) {
+	/* generator 0 = map done with map editor */
+	/* aka a "scenario" */
         if (has_capability("specials",savefile_options)) {
           map_load(file);
+	  map.fixed_start_positions = 1;
           section_file_check_unused(file, NULL);
           section_file_free(file);
-          return 1; /* make this a type 1 scenario */
+          return;
         }
         map_tiles_load(file);
         if (has_capability("riversoverlay",savefile_options)) {
@@ -427,16 +458,19 @@ int game_load(struct section_file *file)
 	}
         if (has_capability("startpos",savefile_options)) {
           map_startpos_load(file);
-          return 2; /* make this a type 2 scenario */
+	  map.fixed_start_positions = 1;
+          return;
         }
-	return 3; /* make this a type 3 scenario */
+	return;
       }
     }
     if(tmp_server_state==PRE_GAME_STATE) {
-      return 0;
+      return;
     }
   }
 
+  game.is_new_game = 0;
+  
   load_rulesets();
 
   map_load(file);
@@ -466,7 +500,7 @@ int game_load(struct section_file *file)
   game.player_idx=0;
   game.player_ptr=&game.players[0];  
 
-  return 1;
+  return;
 }
 
 
@@ -477,35 +511,33 @@ void game_save(struct section_file *file)
 {
   int i;
   int version;
-  char name[20],temp[100], temp1[100], *temp2;
+  char temp[100], temp1[100], *temp2;
 
   version = MAJOR_VERSION *10000 + MINOR_VERSION *100 + PATCH_VERSION; 
   secfile_insert_int(file, version, "game.version");
   secfile_insert_int(file, (int) server_state, "game.server_state");
   secfile_insert_str(file, metaserver_info_line, "game.metastring");
-  secfile_insert_str(file, metaserver_addr, "game.metaserver");
+  secfile_insert_str(file, meta_addr_port(), "game.metaserver");
 
   if(server_state!=PRE_GAME_STATE) {
     secfile_insert_str(file, SAVEFILE_OPTIONS, "savefile.options");
   } else { /* cut out unirandom, and insert startpos if necessary */
-    strcpy(temp, SAVEFILE_OPTIONS);
+    sz_strlcpy(temp, SAVEFILE_OPTIONS);
     temp2=strtok(temp," ");
     *temp1='\0';
     while(temp2 != NULL) {
       /* we don't have unirandom in settings and scenarios */
       if(strcmp(temp2,"unirandom")!=0) {
-        strcat(temp1," ");
-        strcat(temp1,temp2);
+        sz_strlcat(temp1, " ");
+        sz_strlcat(temp1, temp2);
       }
       temp2=strtok(NULL," ");
     }
-    if(game.scenario==2) {
-      /* we have start positions in type 2 scenarios */
-      strcat(temp1," startpos");
-    } else
-    if(game.scenario==1) {
-      /* we have start positions and specials in type 1 scenarios */
-      strcat(temp1," specials");
+    if(map.num_start_positions>0) {
+      sz_strlcat(temp1, " startpos");
+    }
+    if(map.have_specials) {
+      sz_strlcat(temp1, " specials");
     }
     secfile_insert_str(file, temp1+1, "savefile.options");
   }
@@ -538,8 +570,10 @@ void game_save(struct section_file *file)
   secfile_insert_int(file, game.spacerace, "game.spacerace");
   secfile_insert_int(file, game.diplchance, "game.diplchance");
   secfile_insert_int(file, game.aqueductloss, "game.aqueductloss");
-  secfile_insert_int(file, game.randseed, "game.randseed");
   secfile_insert_int(file, game.turnblock, "game.turnblock");
+  secfile_insert_int(file, game.barbarianrate, "game.barbarians");
+  secfile_insert_int(file, game.onsetbarbarian, "game.onsetbarbs");
+  secfile_insert_int(file, game.occupychance, "game.occupychance");
   secfile_insert_str(file, game.ruleset.techs, "game.ruleset.techs");
   secfile_insert_str(file, game.ruleset.units, "game.ruleset.units");
   secfile_insert_str(file, game.ruleset.buildings, "game.ruleset.buildings");
@@ -570,20 +604,27 @@ void game_save(struct section_file *file)
     secfile_insert_int(file, map.huts, "map.huts");
     secfile_insert_int(file, map.generator, "map.generator");
   } 
-  if ((server_state==PRE_GAME_STATE) && (game.scenario==0)) {
+  if ((server_state==PRE_GAME_STATE) && game.is_new_game) {
     return; /* want to save scenarios as well */
   }
   if (server_state!=PRE_GAME_STATE) {
-    secfile_insert_int(file,iRandJ,"random.index_J");
-    secfile_insert_int(file,iRandK,"random.index_K");
-    secfile_insert_int(file,iRandX,"random.index_X");
+    RANDOM_STATE rstate = get_myrand_state();
+    assert(rstate.is_init);
+
+    secfile_insert_int(file, game.randseed, "game.randseed");
+
+    secfile_insert_int(file, rstate.j, "random.index_J");
+    secfile_insert_int(file, rstate.k, "random.index_K");
+    secfile_insert_int(file, rstate.x, "random.index_X");
 
     for(i=0;i<8;i++) {
-      sprintf(name,"random.table%d",i);
-      sprintf(temp,"%8X %8X %8X %8X %8X %8X %8X",RandomState[7*i],
-              RandomState[7*i+1],RandomState[7*i+2],RandomState[7*i+3],
-              RandomState[7*i+4],RandomState[7*i+5],RandomState[7*i+6]);
-      secfile_insert_str(file,temp,name);
+      char name[20], vec[100];
+      my_snprintf(name, sizeof(name), "random.table%d", i);
+      my_snprintf(vec, sizeof(vec),
+		  "%8X %8X %8X %8X %8X %8X %8X", rstate.v[7*i],
+		  rstate.v[7*i+1], rstate.v[7*i+2], rstate.v[7*i+3],
+		  rstate.v[7*i+4], rstate.v[7*i+5], rstate.v[7*i+6]);
+      secfile_insert_str(file, vec, name);
     }
      
   }
