@@ -15,7 +15,12 @@
 
 #include "map.h"		/* MAP_MAX_ */
 
-void do_unit_goto(struct player *pplayer, struct unit *punit);
+enum goto_move_restriction {
+  GOTO_MOVE_ANY, GOTO_MOVE_CARDINAL_ONLY, GOTO_MOVE_STRAIGHTEST
+};
+
+void do_unit_goto(struct player *pplayer, struct unit *punit,
+		  enum goto_move_restriction restriction);
 void generate_warmap(struct city *pcity, struct unit *punit);
 void really_generate_warmap(struct city *pcity, struct unit *punit,
 			    enum unit_move_type which);
@@ -31,9 +36,9 @@ int could_unit_move_to_tile(struct unit *punit, int x0, int y0, int x, int y);
 int goto_is_sane(struct player *pplayer, struct unit *punit, int x, int y, int omni);
 
 struct move_cost_map {
-  unsigned char cost[MAP_MAX_WIDTH][MAP_MAX_HEIGHT];
-  unsigned char seacost[MAP_MAX_WIDTH][MAP_MAX_HEIGHT];
-  unsigned char vector[MAP_MAX_WIDTH][MAP_MAX_HEIGHT];
+  unsigned char *cost[MAP_MAX_WIDTH];
+  unsigned char *seacost[MAP_MAX_WIDTH];
+  unsigned char *vector[MAP_MAX_WIDTH];
   struct city *warcity; /* so we know what we're dealing with here */
   struct unit *warunit; /* so we know what we're dealing with here */
   int orig_x, orig_y;
