@@ -24,9 +24,11 @@
 #include "map.h"
 #include "mem.h"
 #include "packets.h"
+#include "rand.h"
 #include "registry.h"
 
 #include "cityhand.h"
+#include "mapgen.h"		/* assign_continent_numbers */
 #include "plrhand.h"           /* notify_player */
 #include "unitfunc.h"
 
@@ -54,7 +56,7 @@ void global_warming(int effect)
 	  effect--;
 	  map_set_terrain(x, y, T_JUNGLE);
           reset_move_costs(x, y);
-          relight_square_if_known(0, x, y);
+          relight_square_if_known(x, y);
 	  break;
 	case T_DESERT:
 	case T_PLAINS:
@@ -64,7 +66,7 @@ void global_warming(int effect)
 	  map_clear_special(x, y, S_FARMLAND);
 	  map_clear_special(x, y, S_IRRIGATION);
           reset_move_costs(x, y);
-          relight_square_if_known(0, x, y);
+          relight_square_if_known(x, y);
 	  break;
 	default:
 	  break;
@@ -77,7 +79,7 @@ void global_warming(int effect)
 	  effect--;
 	  map_set_terrain(x, y, T_DESERT);
           reset_move_costs(x, y);
-          relight_square_if_known(0, x, y);
+          relight_square_if_known(x, y);
 	  break;
 	default:
 	  break;
@@ -116,6 +118,23 @@ void give_seamap_from_player_to_player(struct player *pfrom, struct player *pdes
       }
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void give_citymap_from_player_to_player(struct city *pcity,
+					struct player *pfrom, struct player *pdest)
+{
+  int cx, cy, mx, my;
+
+  city_map_iterate(cx, cy) {
+    mx = map_adjust_x(pcity->x+cx-(CITY_MAP_SIZE/2));
+    my = map_adjust_y(pcity->y+cy-(CITY_MAP_SIZE/2));
+    if(map_get_known(mx, my, pfrom) && !map_get_known(mx, my, pdest)) {
+      light_square(pdest, mx, my, 0);
+    }
+  }
+}
+
 /**************************************************************************
 dest can be NULL meaning all players
 **************************************************************************/
@@ -174,7 +193,7 @@ void send_tile_info(struct player *dest, int x, int y, enum known_type type)
 /**************************************************************************
 ...
 **************************************************************************/
-void relight_square_if_known(struct player *pplayer, int x, int y)
+void relight_square_if_known(int x, int y)
 {
   int o;
   for(o=0; o<game.nplayers; o++) {
@@ -332,8 +351,7 @@ void map_save(struct section_file *file)
    */
   secfile_insert_int(file, map.is_earth, "map.is_earth");
 
-  /* don't save start positions in a type 3 scenario */
-  if(game.scenario!=3) {
+  if(map.num_start_positions>0) {
     for(i=0; i<game.nation_count; i++) {
       secfile_insert_int(file, map.start_positions[i].x, "map.r%dsx", i);
       secfile_insert_int(file, map.start_positions[i].y, "map.r%dsy", i);
@@ -349,8 +367,7 @@ void map_save(struct section_file *file)
     secfile_insert_str(file, pbuf, "map.t%03d", y);
   }
 
-  /* don't save specials in scenarios of types 2 and 3 */
-  if((game.scenario==3) || (game.scenario==2)) {
+  if(!map.have_specials) {
     free(pbuf);
     return;
   }
@@ -472,6 +489,8 @@ void map_tiles_load(struct section_file *file)
       map_get_tile(x, y)->terrain=pch-terrain_chars;
     }
   }
+
+  assign_continent_numbers();
 }
 
 /***************************************************************
@@ -606,6 +625,7 @@ void map_load(struct section_file *file)
       }
     }
   }
+  map.have_specials = 1;
 }
 
 /***************************************************************
