@@ -35,6 +35,8 @@ The info string should look like this:
 
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
+#include <errno.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -61,24 +63,59 @@ The info string should look like this:
 #endif
 
 #include "fcintl.h"
+#include "log.h"
 #include "packets.h"
+#include "support.h"
+
 #include "console.h"
+
 #include "meta.h"
 
 #ifndef INADDR_NONE
 #define INADDR_NONE     0xffffffff
 #endif
 
+int server_is_open=0;
+
 #ifdef GENERATING_MAC    /* mac network globals */
 TEndpointInfo meta_info;
 EndpointRef meta_ep;
 InetAddress serv_addr;
 #else /* Unix network globals */
-static int			sockfd,n,in_size;
+static int			sockfd=0;
 static struct sockaddr_in	cli_addr,serv_addr;
 #endif /* end network global selector */
 
 extern char metaserver_addr[];
+extern unsigned short int metaserver_port;
+
+
+void meta_addr_split(void)
+{
+  char *metaserver_port_separator;
+  int specified_port;
+
+  if ((metaserver_port_separator = strchr(metaserver_addr,':'))) {
+    metaserver_port_separator[0] = '\0';
+    if ((specified_port=atoi(&metaserver_port_separator[1]))) {
+      metaserver_port = (unsigned short int)specified_port;
+    }
+  }
+}
+
+char *meta_addr_port(void)
+{
+  static char retstr[300];
+
+  if (metaserver_port == DEFAULT_META_SERVER_PORT) {
+    sz_strlcpy(retstr, metaserver_addr);
+  } else {
+    my_snprintf(retstr, sizeof(retstr),
+		"%s:%d", metaserver_addr, metaserver_port);
+  }
+
+  return retstr;
+}
 
 int send_to_metaserver(char *desc, char *info)
 {
@@ -89,7 +126,7 @@ int send_to_metaserver(char *desc, char *info)
   xmit.udata.maxlen = MAX_LEN_PACKET;
   xmit.udata.buf=buffer;
 #else  
-  if(sockfd==0)
+  if(sockfd<=0)
     return 0;
 #endif
   cptr=put_uint16(buffer+2,  PACKET_UDP_PCKT);
@@ -100,18 +137,23 @@ int send_to_metaserver(char *desc, char *info)
   xmit.udata.len=strlen((const char *)buffer);
   err=OTSndUData(meta_ep, &xmit);
 #else
-  n=sendto(sockfd, buffer, cptr-buffer,0, 
-	   (struct sockaddr *) &serv_addr, sizeof(serv_addr) );
+  sendto(sockfd, buffer, cptr-buffer,0, 
+	 (struct sockaddr *) &serv_addr, sizeof(serv_addr) );
 #endif
   return 1;
 }
 
 void server_close_udp(void)
 {
+  server_is_open=0;
+
 #ifdef GENERATING_MAC  /* mac networking */
   OTUnbind(meta_ep);
 #else
+  if(sockfd<=0)
+    return;
   close(sockfd);
+  sockfd=0;
 #endif
 }
 
@@ -125,6 +167,7 @@ void server_open_udp(void)
   InetSvcRef ref=OTOpenInternetServices(kDefaultInternetServicesPath, 0, &err1);
   InetHostInfo hinfo;
 #else
+  int in_size;
   struct hostent *hp;
   u_int bin;
 #endif
@@ -149,7 +192,8 @@ void server_open_udp(void)
 	 && ((hp = gethostbyname(servername)) == NULL));
 #endif
   if (bad) {
-    perror(_("Metaserver: address error"));
+    freelog(LOG_NORMAL, _("Metaserver: bad address: [%s]."),
+	    servername);
     con_puts(C_METAERROR, _("Not reporting to the metaserver in this game."));
     con_flush();
     return;
@@ -163,12 +207,13 @@ void server_open_udp(void)
 #else  
   memset(&serv_addr, 0, sizeof(serv_addr));
   serv_addr.sin_family      = AF_INET;
-  serv_addr.sin_port        = htons(METASERVER_PORT);
+  serv_addr.sin_port        = htons(metaserver_port);
   memcpy(&serv_addr.sin_addr, hp->h_addr, hp->h_length); 
   bad = ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0);
 #endif
   if (bad) {
-    perror(_("Metaserver: can't open datagram socket"));
+    freelog(LOG_DEBUG, "Metaserver: can't open datagram socket: %s",
+	    mystrerror(errno));
     con_puts(C_METAERROR, _("Not reporting to the metaserver in this game."));
     con_flush();
     return;
@@ -188,9 +233,12 @@ void server_open_udp(void)
   bad = (bind(sockfd, (struct sockaddr *) &cli_addr, sizeof(cli_addr)) < 0);
 #endif
   if (bad) {
-    perror(_("Metaserver: can't bind local address"));
+    freelog(LOG_DEBUG, "Metaserver: can't bind local address: %s",
+	    mystrerror(errno));
     con_puts(C_METAERROR, _("Not reporting to the metaserver in this game."));
     con_flush();
     return;
   }
+
+  server_is_open=1;
 }
