@@ -19,6 +19,7 @@
 #include "log.h"
 #include "map.h"
 #include "packets.h"
+#include "support.h"
 #include "timing.h"
 
 #include "cityhand.h"
@@ -40,6 +41,14 @@ signed short int minimap[MAP_MAX_WIDTH][MAP_MAX_HEIGHT];
 static unsigned short int territory[MAP_MAX_WIDTH][MAP_MAX_HEIGHT];
 /* negative: in_city_radius, 0: unassigned, positive: city_des */
 
+/*************************************************************************/
+
+static int auto_settler_findwork(struct player *pplayer, struct unit *punit); 
+static void auto_settlers_player(struct player *pplayer); 
+
+static int is_already_assigned(struct unit *myunit, struct player *pplayer, 
+			       int x, int y);
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -47,8 +56,8 @@ const char *get_a_name(struct player *pplayer)
 {
   static char buf[80];
   static int x=0;
-  sprintf(buf, "%s", city_name_suggestion(pplayer)); /* Not a Number -- Syela */
-  if (!buf[0]) sprintf(buf, "city%d", x++);
+  sz_strlcpy(buf, city_name_suggestion(pplayer));
+  if (!buf[0]) my_snprintf(buf, sizeof(buf), "city%d", x++);
   return buf;
 }
 
@@ -61,7 +70,7 @@ static int ai_do_build_city(struct player *pplayer, struct unit *punit)
   struct packet_unit_request req;
   struct city *pcity;
   req.unit_id=punit->id;
-  strcpy(req.name, get_a_name(pplayer));
+  sz_strlcpy(req.name, get_a_name(pplayer));
   x = punit->x; y = punit->y; /* Trevor Pering points out that punit gets freed */
   handle_unit_build_city(pplayer, &req);        
   pcity=map_get_city(x, y); /* so we need to cache x and y for a very short time */
@@ -195,7 +204,7 @@ int city_desirability(struct player *pplayer, int x, int y)
   ptile = map_get_tile(x, y);
   if (ptile->terrain == T_OCEAN) return(0);
   pcity = map_get_city(x, y);
-  if (pcity && pcity->size > 7) return(0);
+  if (pcity && pcity->size > (game.add_to_size_limit - 1)) return(0);
   if (!pcity && minimap[x][y] < 0) return(0);
   if (!pcity && minimap[x][y] > 0) return(minimap[x][y]);
 
@@ -490,9 +499,15 @@ int ai_calc_pollution(struct city *pcity, struct player *pplayer, int i, int j)
 
 static int is_wet(struct player *pplayer, int x, int y)
 {
-  if (!map_get_known(x, y, pplayer) && !pplayer->ai.control) return 0;
-  if (map_get_terrain(x,y) == T_OCEAN || map_get_terrain(x,y) == T_RIVER ||
-      map_get_special(x,y)&S_RIVER || map_get_special(x,y)&S_IRRIGATION) return 1;
+  enum tile_terrain_type t;
+  enum tile_special_type s;
+
+  if (!pplayer->ai.control && !map_get_known(x, y, pplayer)) return 0;
+
+  t=map_get_terrain(x,y);
+  if (t == T_OCEAN || t == T_RIVER) return 1;
+  s=map_get_special(x,y);
+  if ((s & S_RIVER) || (s & S_IRRIGATION)) return 1;
   return 0;
 }
 
@@ -523,7 +538,7 @@ int ai_calc_irrigate(struct city *pcity, struct player *pplayer, int i, int j)
     return(m);
   } else if((ptile->terrain==type->irrigation_result &&
      (ptile->special&S_IRRIGATION) && !(ptile->special&S_FARMLAND) &&
-     player_knows_improvement_tech(pplayer, B_SUPERMARKET) &&
+     player_knows_techs_with_flag(pplayer, TF_FARMLAND) &&
      !(ptile->special&S_MINE) && !(ptile->city) &&
      (is_wet(pplayer,x,y) || is_wet(pplayer,x,y-1) || is_wet(pplayer,x,y+1) ||
      is_wet(pplayer,x-1,y) || is_wet(pplayer,x+1,y)))) {
@@ -735,7 +750,7 @@ int auto_settler_do_goto(struct player *pplayer, struct unit *punit, int x, int
   punit->goto_dest_y=map_adjust_y(y);
   set_unit_activity(punit, ACTIVITY_GOTO);
   send_unit_info(0, punit, 0);
-  do_unit_goto(pplayer, punit);
+  do_unit_goto(pplayer, punit, GOTO_MOVE_ANY);
   return 1;
 }
 
@@ -1008,9 +1023,10 @@ and the prioritization of useless (b <= 0) activities are his. -- Syela
 
   /** Decide whether to build a new city:
    ** if so, modify: gx, gy, best_newv, best_act */
-  
-  if (pplayer->ai.control && 
-		ai_fuzzy(pplayer,1)) { /* don't want to make cities otherwise */
+
+  if (unit_flag(punit->type, F_CITIES) &&
+      pplayer->ai.control &&
+      ai_fuzzy(pplayer,1)) {    /* don't want to make cities otherwise */
     if (punit->ai.ai_role == AIUNIT_BUILD_CITY) {
       remove_city_from_minimap(punit->goto_dest_x, punit->goto_dest_y);
     }
@@ -1157,7 +1173,8 @@ improving those tiles, and then immigrating shortly thereafter. -- Syela
 
   /** We've now worked out what to do; go to it! **/
   if (gx!=-1 && gy!=-1) {
-    if (best_act == ACTIVITY_UNKNOWN /* flag */) {
+    if (unit_flag(punit->type, F_CITIES) &&
+	(best_act == ACTIVITY_UNKNOWN /* flag */)) {
       punit->ai.ai_role = AIUNIT_BUILD_CITY;
       add_city_to_minimap(gx, gy);
     } else {
@@ -1259,10 +1276,9 @@ void auto_settlers_player(struct player *pplayer)
   freelog(LOG_DEBUG, "Warmth = %d, game.globalwarming=%d",
 	  pplayer->ai.warmth, game.globalwarming);
   unit_list_iterate(pplayer->units, punit) {
-    freelog(LOG_DEBUG, "%s's settler at (%d, %d)",
-	    pplayer->name, punit->x, punit->y); 
     if (punit->ai.control && unit_flag(punit->type, F_SETTLERS)) {
-      freelog(LOG_DEBUG, "Is ai controlled.");
+      freelog(LOG_DEBUG, "%s's settler at (%d, %d) is ai controlled.",
+	      pplayer->name, punit->x, punit->y); 
       if(punit->activity == ACTIVITY_SENTRY)
 	set_unit_activity(punit, ACTIVITY_IDLE);
       if (punit->activity == ACTIVITY_GOTO && punit->moves_left)
@@ -1281,7 +1297,7 @@ void auto_settlers_player(struct player *pplayer)
 
 static void assign_settlers_player(struct player *pplayer)
 {
-  short i = 1<<pplayer->player_no;
+  int i = 1<<pplayer->player_no;
   struct tile *ptile;
   unit_list_iterate(pplayer->units, punit)
     if (unit_flag(punit->type, F_SETTLERS)) {
@@ -1378,7 +1394,9 @@ void contemplate_settling(struct player *pplayer, struct city *pcity)
 {
   struct unit virtualunit;
   int want;
+
 /* used to use old crappy formulas for settler want, but now using actual want! */
+
   memset(&virtualunit, 0, sizeof(struct unit));
   virtualunit.id = 0;
   virtualunit.owner = pplayer->player_no;
@@ -1392,4 +1410,22 @@ void contemplate_settling(struct player *pplayer, struct city *pcity)
     if (qpass->ai.ferryboat == pcity->id) want = -199;
   unit_list_iterate_end;
   pcity->ai.settler_want = want;
+
+  if (unit_flag(virtualunit.type, F_CITIES)) {
+    pcity->ai.founder_want = pcity->ai.settler_want;
+  } else {
+    memset(&virtualunit, 0, sizeof(struct unit));
+    virtualunit.id = 0;
+    virtualunit.owner = pplayer->player_no;
+    virtualunit.x = pcity->x;
+    virtualunit.y = pcity->y;
+    virtualunit.type = best_role_unit(pcity, F_CITIES);
+    virtualunit.moves_left = unit_types[virtualunit.type].move_rate;
+    virtualunit.hp = unit_types[virtualunit.type].hp;  
+    want = auto_settler_findwork(pplayer, &virtualunit);
+    unit_list_iterate(pplayer->units, qpass)
+      if (qpass->ai.ferryboat == pcity->id) want = -199;
+    unit_list_iterate_end;
+    pcity->ai.founder_want = want;
+  }
 }
