@@ -18,6 +18,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "barbarian.h"
 #include "city.h"
 #include "events.h"
 #include "fcintl.h"
@@ -27,6 +28,8 @@
 #include "mem.h"
 #include "packets.h"
 #include "player.h"
+#include "rand.h"
+#include "support.h"
 #include "unit.h"
 
 #include "cityhand.h"
@@ -34,6 +37,7 @@
 #include "cityturn.h"
 #include "civserver.h"
 #include "gamelog.h"
+#include "gotohand.h"
 #include "maphand.h"
 #include "plrhand.h"
 #include "settlers.h"
@@ -47,8 +51,6 @@
 
 #include "unithand.h"
 
-void do_unit_goto(struct player *pplayer, struct unit *punit);
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -65,7 +67,36 @@ void handle_unit_goto_tile(struct player *pplayer,
 
     send_unit_info(0, punit, 0);
       
-    do_unit_goto(pplayer, punit);  
+    do_unit_goto(pplayer, punit, GOTO_MOVE_ANY);  
+  }
+}
+
+/**************************************************************************
+Handler for PACKET_UNIT_CONNECT request
+The unit is send on way and will build something (roads only for now)
+along the way
+**************************************************************************/
+void handle_unit_connect(struct player *pplayer, 
+		          struct packet_unit_connect *req)
+{
+  struct unit *punit;
+
+  if((punit=unit_list_find(&pplayer->units, req->unit_id))) {
+    if (can_unit_do_connect (punit, req->activity_type)) {
+      punit->goto_dest_x=req->dest_x;
+      punit->goto_dest_y=req->dest_y;
+
+      set_unit_activity(punit, req->activity_type);
+      punit->connecting = 1;
+
+      send_unit_info(0, punit, 0);
+
+      /* avoid wasting first turn if unit cannot do the activity
+	 on the starting tile */
+      if (! can_unit_do_activity (punit, req->activity_type)) 
+	do_unit_goto (pplayer, punit,
+		      get_activity_move_restriction(req->activity_type));
+    }
   }
 }
 
@@ -185,122 +216,60 @@ void handle_diplomat_action(struct player *pplayer,
   struct unit *pdiplomat=unit_list_find(&pplayer->units, packet->diplomat_id);
   struct unit *pvictim=find_unit_by_id(packet->target_id);
   struct city *pcity=find_city_by_id(packet->target_id);
-  int x,y, move_cost;  
 
   if (!pdiplomat) return;
   if (!unit_flag(pdiplomat->type, F_DIPLOMAT)) 
     return;
 
-  if(pcity){
-    x = pcity->x;
-    y = pcity->y;
-  }else{
-    x = pvictim->x;
-    y = pvictim->y;
-  }
-  move_cost = tile_move_cost(pdiplomat,pdiplomat->x,pdiplomat->y, x, y);
-  
-  if(pdiplomat->moves_left >= move_cost ||
-     pdiplomat->moves_left >= unit_move_rate(pdiplomat)) {
-
+  if(pdiplomat->moves_left) {
     switch(packet->action_type) {
     case DIPLOMAT_BRIBE:
       if(pvictim && diplomat_can_do_action(pdiplomat, DIPLOMAT_BRIBE,
-					   pvictim->x, pvictim->y)){
-	pdiplomat->moves_left -= move_cost;    
-	send_unit_info(pplayer, pdiplomat, 0);
+					   pvictim->x, pvictim->y)) {
 	diplomat_bribe(pplayer, pdiplomat, pvictim);
       }
       break;
     case SPY_SABOTAGE_UNIT:
       if(pvictim && diplomat_can_do_action(pdiplomat, SPY_SABOTAGE_UNIT,
-					   pvictim->x, pvictim->y)){
-	pdiplomat->moves_left -= move_cost;    
-	send_unit_info(pplayer, pdiplomat, 0);
+					   pvictim->x, pvictim->y)) {
 	spy_sabotage_unit(pplayer, pdiplomat, pvictim);
       }
       break;
      case DIPLOMAT_SABOTAGE:
-      if(pcity && diplomat_can_do_action(pdiplomat, DIPLOMAT_SABOTAGE, 
-					 pcity->x, pcity->y)){
-	pdiplomat->moves_left -= move_cost;    
-	send_unit_info(pplayer, pdiplomat, 0);
-	diplomat_sabotage(pplayer, pdiplomat, pcity, packet->value);
+      if(pcity && diplomat_can_do_action(pdiplomat, DIPLOMAT_SABOTAGE,
+					 pcity->x, pcity->y)) {
+	/* packet value is improvement ID + 1 (or some special codes) */
+	diplomat_sabotage(pplayer, pdiplomat, pcity, packet->value - 1);
       }
       break;
     case SPY_POISON:
       if(pcity && diplomat_can_do_action(pdiplomat, SPY_POISON,
-					 pcity->x, pcity->y)){
-	pdiplomat->moves_left -= move_cost;    
-	send_unit_info(pplayer, pdiplomat, 0);
+					 pcity->x, pcity->y)) {
 	spy_poison(pplayer, pdiplomat, pcity);
       }
       break;
     case DIPLOMAT_INVESTIGATE:
-
-      /* Note: this is free (no movement cost) for spies */
-      
-      if(pcity && diplomat_can_do_action(pdiplomat,DIPLOMAT_INVESTIGATE ,
-					 pcity->x, pcity->y)){
-	send_unit_info(pplayer, pdiplomat, 0);
+      if(pcity && diplomat_can_do_action(pdiplomat,DIPLOMAT_INVESTIGATE,
+					 pcity->x, pcity->y)) {
 	diplomat_investigate(pplayer, pdiplomat, pcity);
       }
       break;
     case DIPLOMAT_EMBASSY:
-      if(pcity && diplomat_can_do_action(pdiplomat, DIPLOMAT_EMBASSY, 
+      if(pcity && diplomat_can_do_action(pdiplomat, DIPLOMAT_EMBASSY,
 					 pcity->x, pcity->y)) {
-	if(pdiplomat->foul){
-	  notify_player_ex(pplayer, pcity->x, pcity->y, E_NOEVENT,
-			   _("Game: Your %s was executed in %s on suspicion"
-			     " of spying.  The %s welcome future diplomatic"
-			     " efforts providing the Ambassador is reputable."),
-			   unit_name(pdiplomat->type),
-			   pcity->name, get_nation_name_plural(pplayer->nation));
-	  
-	  notify_player_ex(&game.players[pcity->owner], pcity->x, pcity->y,
-			   E_DIPLOMATED,
-			   _("You executed a %s the %s had sent to establish"
-			     " an embassy in %s"),
-			   unit_name(pdiplomat->type),
-			   get_nation_name_plural(pplayer->nation),
-			   pcity->name);
-	  
-	  wipe_unit(0, pdiplomat);
-	  break;
-	}
-	
-	pplayer->embassy|=(1<<pcity->owner);
-	send_player_info(pplayer, pplayer);
-	notify_player_ex(pplayer, pcity->x, pcity->y, E_MY_DIPLOMAT,
-			 _("Game: You have established an embassy in %s."),
-			 pcity->name);
-	
-	notify_player_ex(&game.players[pcity->owner], pcity->x, pcity->y,
-			 E_DIPLOMATED, 
-			 _("Game: The %s have established an embassy in %s."),
-		         get_nation_name_plural(pplayer->nation), pcity->name);
-        gamelog(GAMELOG_EMBASSY,"%s establish an embassy in %s (%s) (%i,%i)\n",
-                get_nation_name_plural(pplayer->nation),
-                pcity->name,
-                get_nation_name(game.players[pcity->owner].nation),
-		pcity->x,pcity->y);
-
+	diplomat_embassy(pplayer, pdiplomat, pcity);
       }
-      wipe_unit(0, pdiplomat);
       break;
     case DIPLOMAT_INCITE:
-      if(pcity && diplomat_can_do_action(pdiplomat, DIPLOMAT_INCITE, 
-					 pcity->x, pcity->y)){
-	pdiplomat->moves_left -= move_cost;    
-	send_unit_info(pplayer, pdiplomat, 0);
+      if(pcity && diplomat_can_do_action(pdiplomat, DIPLOMAT_INCITE,
+					 pcity->x, pcity->y)) {
 	diplomat_incite(pplayer, pdiplomat, pcity);
       }
       break;
     case DIPLOMAT_STEAL:
-      if(pcity && diplomat_can_do_action(pdiplomat, DIPLOMAT_STEAL, 
-					 pcity->x, pcity->y)){
-	pdiplomat->moves_left -= move_cost;    
-	send_unit_info(pplayer, pdiplomat, 0);
+      if(pcity && diplomat_can_do_action(pdiplomat, DIPLOMAT_STEAL,
+					 pcity->x, pcity->y)) {
+	/* packet value is technology ID (or some special codes) */
 	diplomat_get_tech(pplayer, pdiplomat, pcity, packet->value);
       }
       break;
@@ -377,8 +346,8 @@ void handle_unit_build_city(struct player *pplayer,
   unit_name = get_unit_type(punit->type)->name;
   pcity = map_get_city(punit->x, punit->y);
   
-  if (!unit_flag(punit->type, F_SETTLERS)) {
-    char *us = get_units_with_flag_string(F_SETTLERS);
+  if (!unit_flag(punit->type, F_CITIES)) {
+    char *us = get_units_with_flag_string(F_CITIES);
     notify_player_ex(pplayer, punit->x, punit->y, E_NOEVENT,
 		     _("Game: Only %s can build or add to a city."), us);
     free(us);
@@ -403,7 +372,7 @@ void handle_unit_build_city(struct player *pplayer,
 		       _("Game: %s added to aid %s in growing."), 
 		       unit_name, pcity->name);
     } else {
-      if(pcity->size > 8) {
+      if(pcity->size > game.add_to_size_limit) {
 	notify_player_ex(pplayer, punit->x, punit->y, E_NOEVENT, 
 			 _("Game: %s is too big to add %s."),
 			 pcity->name, unit_name);
@@ -471,9 +440,10 @@ void handle_unit_info(struct player *pplayer, struct packet_unit_info *pinfo)
   if(punit) {
     if (!same_pos(punit->x, punit->y, pinfo->x, pinfo->y)) {
       punit->ai.control=0;
-      handle_unit_move_request(pplayer, punit, pinfo->x, pinfo->y);
+      handle_unit_move_request(pplayer, punit, pinfo->x, pinfo->y, FALSE);
     }
-    else if(punit->activity!=pinfo->activity || punit->activity_target!=pinfo->activity_target ||
+    else if(punit->activity!=pinfo->activity ||
+	    punit->activity_target!=pinfo->activity_target ||
 	    punit->ai.control==1) {
       /* Treat change in ai.control as change in activity, so
        * idle autosettlers behave correctly when selected --dwp
@@ -493,7 +463,7 @@ void handle_move_unit(struct player *pplayer, struct packet_move_unit *pmove)
 
   punit=unit_list_find(&pplayer->units, pmove->unid);
   if(punit)  {
-    handle_unit_move_request(pplayer, punit, pmove->x, pmove->y);
+    handle_unit_move_request(pplayer, punit, pmove->x, pmove->y, FALSE);
   }
 }
 /**************************************************************************
@@ -505,9 +475,10 @@ void handle_unit_attack_request(struct player *pplayer, struct unit *punit,
   int o;
   struct packet_unit_combat combat;
   struct unit *plooser, *pwinner;
+  struct unit old_punit = *punit;	/* Used for new ship algorithm. -GJW */
   struct city *pcity;
-  struct city *incity, *nearcity1, *nearcity2;
-  incity = map_get_city(pdefender->x, pdefender->y);
+  struct city *nearcity1, *nearcity2;
+  int def_x = pdefender->x, def_y = pdefender->y;
   
   freelog(LOG_DEBUG, "Start attack: %s's %s against %s's %s.",
 	  pplayer->name, unit_types[punit->type].name, 
@@ -517,14 +488,14 @@ void handle_unit_attack_request(struct player *pplayer, struct unit *punit,
   if(unit_flag(punit->type, F_NUCLEAR)) {
     struct packet_nuke_tile packet;
     
-    packet.x=pdefender->x;
-    packet.y=pdefender->y;
-    if ((pcity=sdi_defense_close(punit->owner, pdefender->x, pdefender->y))) {
+    packet.x=def_x;
+    packet.y=def_y;
+    if ((pcity=sdi_defense_close(punit->owner, def_x, def_y))) {
       notify_player_ex(pplayer, punit->x, punit->y, E_UNIT_LOST_ATT,
 		       _("Game: Your Nuclear missile was shot down by"
 			 " SDI defences, what a waste."));
       notify_player_ex(&game.players[pcity->owner], 
-		       pdefender->x, pdefender->y, E_UNIT_WIN, 
+		       def_x, def_y, E_UNIT_WIN, 
 		       _("Game: The nuclear attack on %s was avoided by"
 			 " your SDI defense."),
 		       pcity->name); 
@@ -535,7 +506,7 @@ void handle_unit_attack_request(struct player *pplayer, struct unit *punit,
     for(o=0; o<game.nplayers; o++)
       send_packet_nuke_tile(game.players[o].conn, &packet);
     
-    do_nuclear_explosion(pdefender->x, pdefender->y);
+    do_nuclear_explosion(def_x, def_y);
     return;
   }
   
@@ -543,12 +514,23 @@ void handle_unit_attack_request(struct player *pplayer, struct unit *punit,
 
   /* Adjust attackers moves_left _after_ unit_versus_unit() so that
    * the movement attack modifier is correct! --dwp
+   *
+   * And for greater Civ2 compatibility (and ship balance issues), ships
+   * use a different algorithm.  Recompute a new total MP based on the HP
+   * the ship has left after being damaged, and subtract out all of the MP
+   * that had been used so far this turn (plus the points used in the attack
+   * itself). -GJW
    */
-  punit->moves_left-=3;
+  if (is_sailing_unit (punit)) {
+    int moves_used = unit_move_rate (&old_punit) - old_punit.moves_left;
+    punit->moves_left = unit_move_rate (punit) - moves_used - 3;
+  } else {
+    punit->moves_left-=3;
+  }
   if(punit->moves_left<0)
     punit->moves_left=0;
 
-  if (punit->hp && (pcity=map_get_city(pdefender->x, pdefender->y)) && pcity->size>1 && !city_got_citywalls(pcity) && is_ground_unit(punit)) {
+  if (punit->hp && (pcity=map_get_city(def_x, def_y)) && pcity->size>1 && !city_got_citywalls(pcity) && is_ground_unit(punit)) {
     pcity->size--;
     city_auto_remove_worker(pcity);
     city_refresh(pcity);
@@ -567,11 +549,11 @@ void handle_unit_attack_request(struct player *pplayer, struct unit *punit,
   
   for(o=0; o<game.nplayers; o++)
     if(map_get_known(punit->x, punit->y, &game.players[o]) ||
-       map_get_known(pdefender->x, pdefender->y, &game.players[o]))
+       map_get_known(def_x, def_y, &game.players[o]))
       send_packet_unit_combat(game.players[o].conn, &combat);
 
-  nearcity1 = dist_nearest_city(get_player(pwinner->owner), pdefender->x, pdefender->y);
-  nearcity2 = dist_nearest_city(get_player(plooser->owner), pdefender->x, pdefender->y);
+  nearcity1 = dist_nearest_city(get_player(pwinner->owner), def_x, def_y, 0, 0);
+  nearcity2 = dist_nearest_city(get_player(plooser->owner), def_x, def_y, 0, 0);
   
   if(punit==plooser) {
     /* The attacker lost */
@@ -591,7 +573,7 @@ void handle_unit_attack_request(struct player *pplayer, struct unit *punit,
 		     unit_name(plooser->type));
     
     notify_player_ex(get_player(plooser->owner),
-		     pdefender->x, pdefender->y, E_UNIT_LOST_ATT, 
+		     def_x, def_y, E_UNIT_LOST_ATT, 
 		     _("Game: Your attacking %s failed against %s's %s%s!"),
 		     unit_name(plooser->type),
 		     get_player(pwinner->owner)->name,
@@ -611,7 +593,7 @@ void handle_unit_attack_request(struct player *pplayer, struct unit *punit,
 
     notify_player_ex(get_player(pwinner->owner), 
 		     punit->x, punit->y, E_UNIT_WIN_ATT, 
-		     _("Game: Your attacking %s was successful"
+		     _("Game: Your attacking %s succeeded"
 		       " against %s's %s%s!"),
 		     unit_name(pwinner->type),
 		     get_player(plooser->owner)->name,
@@ -625,6 +607,31 @@ void handle_unit_attack_request(struct player *pplayer, struct unit *punit,
     wipe_unit(pplayer, pwinner);
     return;
   }
+
+  /* If attacker wins, and occupychance > 0, it might move in.  Don't move in
+     if there are enemy units in the tile (a fortress or city with multiple
+     defenders and unstacked combat).  Note that this could mean capturing (or
+     destroying) a city. -GJW */
+
+  if (pwinner == punit && myrand(100) < game.occupychance &&
+      !is_enemy_unit_tile (def_x, def_y, punit->owner)) {
+
+    /* Hack: make sure the unit has enough moves_left for the move to succeed,
+       and adjust moves_left to afterward (if successful). */
+
+    int old_moves = punit->moves_left;
+    int full_moves = unit_move_rate (punit);
+    punit->moves_left = full_moves;
+    if (handle_unit_move_request (pplayer, punit, def_x, def_y, FALSE)) {
+      punit->moves_left = old_moves - (full_moves - punit->moves_left);
+      if (punit->moves_left < 0) {
+	punit->moves_left = 0;
+      }
+    } else {
+      punit->moves_left = old_moves;
+    }
+  }
+
   send_unit_info(0, pwinner, 0);
 }
 
@@ -682,36 +689,38 @@ int handle_unit_enter_hut(struct unit *punit)
     {
       int res=pplayer->research.researched;
       int wasres=pplayer->research.researching;
+      int new_tech;
 
       choose_random_tech(pplayer);
+      new_tech=pplayer->research.researching;
+      pplayer->research.researched=res;
+      pplayer->research.researching=wasres;
  
-      pplayer->research.researchpoints++;
-      if (pplayer->research.researching!=A_NONE) {
-       notify_player_ex(pplayer, punit->x, punit->y, E_NOEVENT,
-                        _("Game: You gain knowledge about %s."), 
-                        advances[pplayer->research.researching].name);
-       if (tech_flag(pplayer->research.researching,TF_RAILROAD)) {
-	 upgrade_city_rails(pplayer, 1);
-       }
-       set_invention(pplayer, pplayer->research.researching, TECH_KNOWN);
+      if (new_tech!=A_NONE) {
+	notify_player_ex(pplayer, punit->x, punit->y, E_NOEVENT,
+			 _("Game: You gain knowledge about %s."), 
+			 advances[new_tech].name);
+
+	gamelog(GAMELOG_TECH,"%s discover %s (Hut)",
+		get_nation_name_plural(pplayer->nation),advances[new_tech].name
+		);
+
+	notify_embassies(pplayer, (struct player *)0,
+		  _("Game: The %s have aquired %s from ancient scrolls of wisdom"),
+		  get_nation_name_plural(pplayer->nation),
+		  advances[new_tech].name);
+
+	found_new_tech(pplayer,new_tech,0,1);
       }
       else {
        pplayer->future_tech++;
        notify_player(pplayer,
                      _("Game: You gain knowledge about Future Tech. %d."),
                      pplayer->future_tech);
+       pplayer->research.researchpoints++; /* don't call found_new_tech() */
       }
-      remove_obsolete_buildings(pplayer);
       
-      if (get_invention(pplayer,wasres)==TECH_KNOWN) {
-	if (!choose_goal_tech(pplayer))
-	  choose_random_tech(pplayer);
-	pplayer->research.researched=res;
-      }  else {
-	pplayer->research.researched=res;
-	pplayer->research.researching=wasres;
-      }
-     do_free_cost(pplayer);
+      do_free_cost(pplayer);
     }
     break;
   case 8:
@@ -719,7 +728,7 @@ int handle_unit_enter_hut(struct unit *punit)
     notify_player_ex(pplayer, punit->x, punit->y, E_NOEVENT,
 		     _("Game: A band of friendly mercenaries"
 		       " joins your cause."));
-    create_unit(pplayer, punit->x, punit->y, find_a_unit_type(),
+    create_unit(pplayer, punit->x, punit->y, find_a_unit_type(L_HUT,L_HUT_TECH),
 		0, punit->homecity, -1);
     break;
   case 10:
@@ -727,11 +736,11 @@ int handle_unit_enter_hut(struct unit *punit)
       notify_player_ex(pplayer, punit->x, punit->y, E_NOEVENT,
 		       _("Game: An abandoned village is here."));
     else {
+      ok = unleash_barbarians(pplayer, punit->x, punit->y);
       notify_player_ex(pplayer, punit->x, punit->y, E_NOEVENT,
-		       _("Game: Your unit has been slaughtered by a band"
-			 " of cowardly barbarians."));
-      wipe_unit(pplayer, punit);
-      ok = 0;
+		       ok ?
+		         _("Game: You have unleashed a horde of barbarians!") :
+		         _("Game: Your unit has been killed by barbarians."));
     }
     break;
   case 11:
@@ -748,7 +757,7 @@ int handle_unit_enter_hut(struct unit *punit)
       notify_player_ex(pplayer, punit->x, punit->y, E_NOEVENT,
 		       _("Game: Friendly nomads are impressed by you,"
 			 " and join you."));
-      create_unit(pplayer, punit->x, punit->y, get_role_unit(F_SETTLERS,0),
+      create_unit(pplayer, punit->x, punit->y, get_role_unit(F_CITIES,0),
 		  0, punit->homecity, -1);
     }
     break;
@@ -788,13 +797,18 @@ static void wakeup_neighbor_sentries(struct player *pplayer,
   could be done, false if it couldn't for some reason.
 **************************************************************************/
 int handle_unit_move_request(struct player *pplayer, struct unit *punit,
-			      int dest_x, int dest_y)
+			      int dest_x, int dest_y, int igzoc)
 {
   int unit_id, transport_units = 1, ok;
   struct unit *pdefender, *ferryboat, *bodyguard, *passenger;
   struct unit_list cargolist;
   struct city *pcity;
 
+  /* barbarians shouldn't enter huts */
+  if(is_barbarian(pplayer) && (map_get_tile(dest_x, dest_y)->special&S_HUT)) {
+    return 0;
+  }
+
   if (same_pos(punit->x, punit->y, dest_x, dest_y)) return 0;
 /* this occurs often during lag, and to the AI due to some quirks -- Syela */
  
@@ -834,8 +848,8 @@ is the source of the problem.  Hopefully we won't abort() now. -- Syela */
           passenger = unit_list_find(&(map_get_tile(punit->x, punit->y)->units),
               punit->ai.passenger);
           if (passenger) {
-            if (get_transporter_capacity(punit)) abort();
-            else {
+/* removed what seemed like a very bad abort() -- JMC/jjm */
+            if (!get_transporter_capacity(punit)) {
 	      freelog(LOG_NORMAL, "%s#%d@(%d,%d) thinks %s#%d is a passenger?",
 		      unit_name(punit->type), punit->id, punit->x, punit->y,
 		      unit_name(passenger->type), passenger->id);
@@ -844,8 +858,9 @@ is the source of the problem.  Hopefully we won't abort() now. -- Syela */
         }
         if (get_unit_type(punit->type)->attack_strength == 0) {
           char message[MAX_LEN_NAME + 64];
-          sprintf(message, _("Game: A %s cannot attack other units."),
-                           unit_name(punit->type));
+          my_snprintf(message, sizeof(message),
+		      _("Game: A %s cannot attack other units."),
+		      unit_name(punit->type));
           notify_player_ex(pplayer, punit->x, punit->y, E_NOEVENT,
                            message);
           return 0;
@@ -858,8 +873,12 @@ is the source of the problem.  Hopefully we won't abort() now. -- Syela */
 		       _("Game: You can't attack there."));
       return 0;
     };
-  } else if (punit->ai.bodyguard > 0 && (bodyguard = unit_list_find(&map_get_tile(punit->x,
-      punit->y)->units, punit->ai.bodyguard)) && !bodyguard->moves_left) {
+  } else if (pplayer->ai.control &&
+	     punit->ai.bodyguard > 0 &&
+	     (bodyguard =
+	      unit_list_find(&map_get_tile(punit->x, punit->y)->units,
+			     punit->ai.bodyguard)) &&
+	     !bodyguard->moves_left) {
     notify_player_ex(pplayer, punit->x, punit->y, E_NOEVENT,
 		     _("Game: %s doesn't want to leave its bodyguard."),
 		     unit_types[punit->type].name);
@@ -873,9 +892,10 @@ is the source of the problem.  Hopefully we won't abort() now. -- Syela */
     notify_player_ex(pplayer, punit->x, punit->y, E_NOEVENT,
 		     _("Game: %s ending move early to stay out of trouble."),
 		     unit_types[punit->type].name);
-  } else if(can_unit_move_to_tile(punit, dest_x, dest_y) && try_move_unit(punit, dest_x, dest_y)) {
+  } else if(can_unit_move_to_tile(punit, dest_x, dest_y, igzoc) &&
+	    try_move_unit(punit, dest_x, dest_y)) {
     int src_x, src_y;
-    
+
     if((pcity=map_get_city(dest_x, dest_y))) {
       if (pcity->owner!=punit->owner &&  
 	  (is_air_unit(punit) || !is_military_unit(punit))) {
@@ -963,7 +983,8 @@ is the source of the problem.  Hopefully we won't abort() now. -- Syela */
     if (!ok) return 1;
     
     /* bodyguard code */
-    if(unit_list_find(&pplayer->units, unit_id)) {
+    if(pplayer->ai.control &&
+       unit_list_find(&pplayer->units, unit_id)) {
       if (punit->ai.bodyguard > 0) {
         bodyguard = unit_list_find(&(map_get_tile(src_x, src_y)->units),
                     punit->ai.bodyguard);
@@ -972,7 +993,7 @@ is the source of the problem.  Hopefully we won't abort() now. -- Syela */
           handle_unit_activity_request(pplayer, bodyguard, ACTIVITY_IDLE);
 	  /* may be fortifying, have to FIX this eventually -- Syela */
 	  success = handle_unit_move_request(pplayer, bodyguard,
-					     dest_x, dest_y);
+					     dest_x, dest_y, igzoc);
 	  freelog(LOG_DEBUG, "Dragging %s from (%d,%d)->(%d,%d) (Success=%d)",
 		  unit_types[bodyguard->type].name, src_x, src_y,
 		  dest_x, dest_y, success);
@@ -1091,11 +1112,13 @@ int handle_unit_establish_trade(struct player *pplayer,
   revenue = establish_trade_route(pcity_homecity, pcity_dest);
   connection_do_buffer(pplayer->conn);
   notify_player_ex(pplayer, pcity_dest->x, pcity_dest->y, E_NOEVENT,
-		   _("Game: Your %s has arrived in %s,"
-		   " and revenues amount to %d in gold."), 
-		   unit_name(punit->type), pcity_dest->name, revenue);
+		   _("Game: Your %s from %s has arrived in %s,"
+		   " and revenues amount to %d in gold and research."), 
+		   unit_name(punit->type), pcity_homecity->name,
+		   pcity_dest->name, revenue);
   wipe_unit(0, punit);
   pplayer->economic.gold+=revenue;
+  update_tech(pplayer, revenue);
   send_player_info(pplayer, pplayer);
   city_refresh(pcity_homecity);
   city_refresh(pcity_dest);
@@ -1110,7 +1133,7 @@ int handle_unit_establish_trade(struct player *pplayer,
 **************************************************************************/
 void handle_unit_enter_city(struct player *pplayer, struct city *pcity)
 {
-  int i, x, y, old_id;
+  int i, n, x, y, old_id;
   int coins;
   struct player *cplayer;
   if(pplayer->player_no!=pcity->owner) {
@@ -1124,17 +1147,25 @@ void handle_unit_enter_city(struct player *pplayer, struct city *pcity)
     */
 
     if(city_got_building(pcity, B_PALACE)
-       && cplayer->spaceship.state == SSHIP_LAUNCHED) {
+       && ((cplayer->spaceship.state == SSHIP_STARTED)
+	   || (cplayer->spaceship.state == SSHIP_LAUNCHED))) {
       spaceship_lost(cplayer);
     }
-       
+
     if(city_got_building(pcity, B_PALACE) 
        && city_list_size(&cplayer->cities) >= game.civilwarsize 
        && game.nplayers < game.nation_count
-       && game.nplayers < MAX_NUM_PLAYERS
-       && game.civilwarsize < GAME_MAX_CIVILWARSIZE
-       && civil_war_triggered(cplayer))
-      civil_war(cplayer);
+       && game.civilwarsize < GAME_MAX_CIVILWARSIZE) {
+      n = 0;
+      for( i = 0; i < game.nplayers; i++ ) {
+	if(!is_barbarian(&game.players[i])) {
+	  n++;
+	}
+      }
+      if(n < MAX_NUM_PLAYERS && civil_war_triggered(cplayer)) {
+	civil_war(cplayer);
+      }
+    }
 
     if (pcity->size<1) {
       notify_player_ex(pplayer, pcity->x, pcity->y, E_NOEVENT,
