@@ -18,6 +18,8 @@
 
 void handle_unit_goto_tile(struct player *pplayer, 
 			   struct packet_unit_request *req);
+void handle_unit_connect(struct player *pplayer, 
+		          struct packet_unit_connect *req);
 void handle_upgrade_unittype_request(struct player *pplayer, 
 				     struct packet_unittype_info *packet);
 void handle_unit_upgrade_request(struct player *pplayer,
@@ -38,7 +40,7 @@ void handle_unit_attack_request(struct player *pplayer, struct unit *punit,
 				struct unit *pdefender);
 int handle_unit_enter_hut(struct unit *punit);
 int handle_unit_move_request(struct player *pplayer, struct unit *punit,
-			      int dest_x, int dest_y);
+			      int dest_x, int dest_y, int igzoc);
 void handle_unit_help_build_wonder(struct player *pplayer, 
 				   struct packet_unit_request *req);
 int handle_unit_establish_trade(struct player *pplayer, 
