@@ -27,6 +27,7 @@
 #include "map.h"
 #include "packets.h"
 #include "player.h"
+#include "rand.h"
 #include "shared.h"
 #include "unit.h"
 
@@ -52,14 +53,14 @@
      with free capacity
   5) marines are the only units that can attack from a ocean square
   6) naval units can only be moved to ocean squares or city squares
-  7) if there is no enemy units blocking (zoc)
+  7) if there is no enemy units blocking (zoc) [or igzoc is true]
 **************************************************************************/
-int can_unit_move_to_tile(struct unit *punit, int x, int y)
+int can_unit_move_to_tile(struct unit *punit, int x, int y, int igzoc)
 {
   struct tile *ptile,*ptile2;
   int zoc;
 
-  if(punit->activity!=ACTIVITY_IDLE && punit->activity!=ACTIVITY_GOTO)
+  if(punit->activity!=ACTIVITY_IDLE && punit->activity!=ACTIVITY_GOTO && !punit->connecting)
     return 0;
   
   if(x<0 || x>=map.xsize || y<0 || y>=map.ysize)
@@ -102,7 +103,7 @@ int can_unit_move_to_tile(struct unit *punit, int x, int y)
       if(!is_friendly_city_tile(x,y,punit->owner))
 	return 0;
   } 
-  zoc = zoc_ok_move(punit, x, y);
+  zoc = igzoc || zoc_ok_move(punit, x, y);
   if (!zoc) 
     notify_player_ex(&game.players[punit->owner], punit->x, punit->y, E_NOEVENT,
 		     _("Game: %s can only move into your own zone of control."),
@@ -254,7 +255,8 @@ int unit_bribe_cost(struct unit *punit)
   if (g->fixed_corruption_distance)
     dist = MIN(g->fixed_corruption_distance, dist);
   cost=(cost/(dist+2))*(get_unit_type(punit->type)->build_cost/10);
-  if (unit_flag(punit->type, F_SETTLERS)) 
+  /* FIXME: This is a weird one - should be replaced */
+  if (unit_flag(punit->type, F_CITIES)) 
     cost/=2;
 
   /* Cost now contains the basic bribe cost.  We now reduce it by:
@@ -269,15 +271,17 @@ int unit_bribe_cost(struct unit *punit)
 }
 
 /**************************************************************************
- return whether or not there is a diplomat on this square
+ return number of diplomats on this square.  AJS 20000130
 **************************************************************************/
-int diplomat_on_tile(int x, int y)
+int count_diplomats_on_tile(int x, int y)
 {
+  int count = 0;
+
   unit_list_iterate(map_get_tile(x, y)->units, punit)
     if (unit_flag(punit->type, F_DIPLOMAT))
-      return 1;
+      count++;
   unit_list_iterate_end;
-  return 0;
+  return count;
 }
 
 /**************************************************************************
@@ -533,33 +537,44 @@ struct city *sdi_defense_close(int owner, int x, int y)
 }
 
 /**************************************************************************
-  returns a unit type for the goodie huts
+  returns a unit type with a given role, use -1 if you don't want a tech 
+  role. Always try tech role and only if not available, return role unit.
 **************************************************************************/
-int find_a_unit_type(void)
+int find_a_unit_type(int role, int role_tech)
 {
   int which[U_LAST];
   int i, num=0, iunit;
 
-  for(i=0; i<num_role_units(L_HUT); i++) {
-    which[num++] = get_role_unit(L_HUT, i);
+  if( role_tech == -1 ) {
+    for(i=0; i<num_role_units(role); i++) {
+      which[num++] = get_role_unit(role, i);
+    }
+    return which[myrand(num)];
   }
-  for(i=0; i<num_role_units(L_HUT_TECH); i++) {
-    iunit = get_role_unit(L_HUT_TECH, i);
-    if (game.global_advances[get_unit_type(iunit)->tech_requirement]) {
+
+  for(i=0; i<num_role_units(role_tech); i++) {
+    iunit = get_role_unit(role_tech, i);
+    if (game.global_advances[get_unit_type(iunit)->tech_requirement] >= 2) {
       which[num++] = iunit;
     }
   }
+  if(num==0) {
+    for(i=0; i<num_role_units(role); i++) {
+      which[num++] = get_role_unit(role, i);
+    }
+  }
   return which[myrand(num)];
 }
 
 /**************************************************************************
   unit can't attack if :
  1) it don't have any attack power
- 2) it's not a fighter and the defender is a flying unit
+ 2) it's not a fighter and the defender is a flying unit (except city/airbase)
  3) if it's not a marine (and ground unit) and it attacks from ocean
- 4) a ground unit can't attack a ship on an ocean square
+ 4) a ground unit can't attack a ship on an ocean square (except marines)
 **************************************************************************/
-int can_unit_attack_unit_at_tile(struct unit *punit, struct unit *pdefender, int dest_x, int dest_y)
+int can_unit_attack_unit_at_tile(struct unit *punit, struct unit *pdefender,
+				 int dest_x, int dest_y)
 {
   int fromtile=map_get_terrain(punit->x, punit->y);
   int totile=map_get_terrain(dest_x, dest_y);
@@ -567,11 +582,12 @@ int can_unit_attack_unit_at_tile(struct unit *punit, struct unit *pdefender, int
   if(!is_military_unit(punit))
     return 0;
 
-  /*only fighters can attack planes, except for city attacks */
-  if (!unit_flag(punit->type, F_FIGHTER) && is_air_unit(pdefender) && !map_get_city(dest_x, dest_y)) {
+  /* only fighters can attack planes, except for city or airbase attacks */
+  if (!unit_flag(punit->type, F_FIGHTER) && is_air_unit(pdefender) &&
+      !(map_get_city(dest_x, dest_y) || map_get_special(dest_x, dest_y)&S_AIRBASE)) {
     return 0;
   }
-  /* can't attack with ground unit from ocean */
+  /* can't attack with ground unit from ocean, except for marines */
   if(fromtile==T_OCEAN && is_ground_unit(punit) && !unit_flag(punit->type, F_MARINES)) {
     return 0;
   }
@@ -622,14 +638,15 @@ int can_place_partisan(int x, int y)
 
 int enemies_at(struct unit *punit, int x, int y)
 {
-  int i, j, a = 0, d;
-  int db;
+  int i, j, a = 0, d, db;
   struct player *pplayer = get_player(punit->owner);
+
+  if (is_friendly_city_tile(x, y, punit->owner)) return 0;
+
   db = get_tile_type(map_get_terrain(x, y))->defense_bonus;
   if (map_get_special(x, y) & S_RIVER)
     db += (db * terrain_control.river_defense_bonus) / 100;
   d = unit_vulnerability_virtual(punit) * db;
-  if (is_friendly_city_tile(x, y, punit->owner)) return 0;
   for (j = y - 1; j <= y + 1; j++) {
     if (j < 0 || j >= map.ysize) continue;
     for (i = x - 1; i <= x + 1; i++) {
@@ -668,6 +685,7 @@ struct unit *is_enemy_unit_on_tile(int x, int y, int owner)
 
 /**************************************************************************
 Teleport punit to city at cost specified.  Returns success.
+(If specified cost is <0, then teleportation costs all movement.)
                          - Kris Bubendorfer
 **************************************************************************/
 
@@ -677,7 +695,7 @@ int teleport_unit_to_city(struct unit *punit, struct city *pcity, int mov_cost)
     unit_list_unlink(&map_get_tile(punit->x, punit->y)->units, punit);
     punit->x = pcity->x;
     punit->y = pcity->y;
-    if(punit->moves_left < mov_cost)
+    if((mov_cost < 0) || (punit->moves_left < mov_cost))
       punit->moves_left = 0;
     else
       punit->moves_left -= mov_cost;
