@@ -16,7 +16,7 @@
 #include "packets.h"
 #include "unit.h"
 
-int can_unit_move_to_tile(struct unit *punit, int x, int y);
+int can_unit_move_to_tile(struct unit *punit, int x, int y, int igzoc);
 int is_enemy_city_tile(int x, int y, int owner);
 int is_friendly_city_tile(int x, int y, int owner);
 int is_enemy_unit_tile(int x, int y, int owner);
@@ -25,7 +25,7 @@ int is_my_zoc(struct unit *myunit, int x0, int y0);
 int zoc_ok_move(struct unit *punit,int x, int y);
 int zoc_ok_move_gen(struct unit *punit, int x1, int y1, int x2, int y2);
 int unit_bribe_cost(struct unit *punit);
-int diplomat_on_tile(int x, int y);
+int count_diplomats_on_tile(int x, int y);
 int hp_gain_coord(struct unit *punit);
 int rate_unit_d(struct unit *punit, struct unit *against);
 int rate_unit_a(struct unit *punit, struct unit *against);
@@ -44,7 +44,7 @@ int unit_behind_coastal(struct unit *punit);
 int unit_behind_sam(struct unit *punit);
 int unit_behind_sdi(struct unit *punit);
 struct city *sdi_defense_close(int owner, int x, int y);
-int find_a_unit_type(void);
+int find_a_unit_type(int role, int role_tech);
 int can_unit_attack_unit_at_tile(struct unit *punit, struct unit *pdefender, int dest_x, int dest_y);
 int can_unit_attack_tile(struct unit *punit, int dest_x, int dest_y);
 int build_points_left(struct city *pcity);
